#ifndef defs_H
#define defs_H

typedef unsigned char BYTE;
typedef unsigned short WORD;
#ifndef _x64
typedef unsigned long DWORD;
#else
#pragma pack(push)
#pragma pack(8)
#ifndef _WIN32 //win LLP64
typedef unsigned int DWORD;
#else
typedef unsigned long DWORD;
#endif
#endif

//   
#define BYTE_START			0x00		//   
#define BYTE_STOP			0xFF		//   

//  
#define ADR_CPU				0		//  
#define ADR_HEAD			1		//  
#define ADR_STAGE			2		//  
#define ADR_DACS			3		//  
#define ADR_LOCKIN			4		// Lock In Amplifier
#define ADR_STEP_MOTORS			5		//   
#define ADR_AIB				6		//   /
#define ADR_BOOTLOADER			7		// USB Boot Loader
#define ADR_THERMO			8		// thermocontroller

//Boot Loader: control commands
#define BL_RD_STATS		0x01	// read max memory & flashgood
#define BL_ER_FLASH		0x02	// erase flash
#define BL_RD_FLASH		0x03	// read flash
#define BL_WT_FLASH		0x04	// write flash
#define BL_WT_FLASH_FF          0x05	// write allFF flash
#define BL_GO_BOOT		0x06	// boot
#define BL_VERIFY		0x07	// verify = calc checksum

//Boot Loader: response commands
#define BL_BOOT_OK		0x08
#define BL_ERROR		0x09
#define BL_STATS		0x0A
#define BL_FLASH		0x0B    //from read flash
#define BL_NO_FLASH		0x0C

//Boot Loader: ext
#define BL_ER_FLASH_EXT	0x0D
#define BL_EXT_LOCK 	0x0E

#define ERR_FLASH_PROG		0x11
#define ERR_UNKNOWN_OPCODE	0x12

//CPU:  
//free: 43-49, 53-70, 93-100, 100-230/{130, 131, 132, 135, 140, 145, 150, 160, 170, 180, 190}
#define CPU_CAP_BITS     	239		// capabilities bits dword
#define CPU_DEVICE_CAP_BITS 232		// csb/motors/other capabilities bits dword
#define CPU_GO_REBOOT		250		// soft reset, to bootloader, if present
#define CPU_LOOPBACK		52		//   -   
#define CPU_STATUS_ON_OFF	18		//    (  STATUS_SEND_RATE)
#define CPU_USB_BUF_RST 	90		//

#define CPU_MOTOR_CSB_COUNT	82		//1-4 bytes, per-channel
#define CPU_MOTOR_REMAP		83  	//
#define CPU_MOTOR_CSB_ADDR	84		//

#define CPU_READ_CALDATA	1		//     EEPROM      ()
#define	CPU_READ_CALDATA_STAGE 71      //     EEPROM     ()
#define	CPU_SET_CALDATA		2		//      ()
#define	CPU_SET_CALDATA_STAGE 72      //     ()
#define	CPU_SET_CALDATA_Z3	73      //     ()
#define CPU_SAVE_CALDATA	3		//     EEPROM
#define	CPU_SET_PARTCALDATA	4		// set partial caldata struct
#define CPU_HW_LVM         	88		// hw lvm
#define CPU_HV_SPLIT_EN    	32		// hv split - dwords: axis idx, on/off, kfwd(float), kbwd(float), lpf freq(float)

#define CPU_SET_CSB_RATE	249		// select one of precompiled settings
#define CPU_CSB_DUALSPEED	248		// sets dual speed setting

#define CPU_SETPOINT		5		//    XYZ
#define CPU_SETPOINT_STAGE	75	    //    XYZ ()
#define CPU_SETPOINT_Z3 	74	    //    XYZ ()
#define CPU_SETPOINT_SPM	12		//      (  Z)
#define CPU_HV_ZERO         81		//    
#define CPU_SET_FEEDBACK	6		//   
#define CPU_SET_HW_CALDATA	19		//   
#define CPU_SET_SPM_CALDATA	11		//      
#define CPU_SET_HEAD		14      //    ( ,    .)

#define CPU_TIP_CAP_MODE	251		// set tip cap mode
#define CPU_SETKELVINMODE	252		// kelvin mode settings
#define CPU_EFM_PLUS       	231		// single pass
#define CPU_PIEZO_TIP_2 	253		// piezo/tip switch for lift mode
#define CPU_TIP_CAP_MODE_2	254		// tip cap mode for lift mode
#define CPU_HEAD_DAC_2  	255		// head dac for lift mode
#define CPU_DFL_XTALK   	234		// float, full-range xtalk range
#define CPU_MAG_XTALK   	235		// float, full-range xtalk range

#define CPU_JM_LIFT     	240		// jumping mode lift height
#define CPU_JM_PREC     	241		// jumping mode z-step
#define CPU_JM_PREC_LSB     233		// jumping mode z-step
#define CPU_JM_DIV       	238		// jumping mode points divider
#define CPU_LAND_M4_Z     	242		//

#define CPU_SET_SCAN		7		//   
#define CPU_SET_SCAN_STAGE	78		//    
#define CPU_SET_SCAN_CCD        76      //    CCD 
#define CPU_SET_SCAN_VELOCITY   9		//    (   )
#define CPU_SET_SCAN_LM_HEIGHT   51		// (   )
#define CPU_CCD_READY           77      //       CCD 

#define CPU_SCAN_WAIT_SYNC  236      // sets BYTE scan sync in wait flag, 1\2-polarity
#define CPU_FAST_RETURN  	243		// sets bool scan fast return
#define CPU_SCAN_POINT_DELAY 237	// sets WORD ms scan point delay, enables sync out
#define CPU_LINE_DELAY  	244		// sets WORD ms scan line start delay
#define CPU_FIRST_DELAY 	245		// sets WORD ms scan first point delay
#define CPU_SPM_PRECISION	246		// sets precision for SPM prec mode, WORD
#define CPU_CCD_READOUT_TIME 247		// sets delay for ccd fast scan, WORD in ms

#define CPU_SCAN_ON_OFF		8		// / 
#define CPU_SCAN_ON_OFF_STAGE 79		// /  
#define CPU_SCAN_ON_OFF_CCD	89		// /  CCD 

#define CPU_FREQ_SCAN		10		//    
#define CPU_FORCE_CURVE		13		//   ( )
#define CPU_GET_DATA    	80		// request current value of data channel

#define CPU_SCOPE_ON_OFF	15		//   /
#define CPU_SCOPE_BUFFER_CLEAR	16		//   
#define CPU_SCOPE_SET_PARAMS	17		//     

#define CPU_LITHO_START		85	//   ( )
#define CPU_LITHO_STOP		86	//  
#define CPU_LITHO_START2	87	//  2 ( )

//     ,    
#define CPU_SET_FREQUENCY	20		//   
#define CPU_SET_EXCITATION_A 21	//   
#define CPU_SET_EXCITATION_B 22  //lift mode voltage
#define CPU_SET_PHASE_SHIFT	23
#define CPU_SET_LPF_FREQ	24		//    
#define CPU_SET_GAINS		25		//    PGA
#define CPU_SET_DAC_A		26		//  
#define CPU_SET_DAC_B		27
#define CPU_SET_FREQUENCY_2	28      //lift mode frequency
#define CPU_SET_GAINS_2		29	//    PGA, lift mode
#define CPU_SET_QCONTROL	31  //gain doubles as on/off

//   /
#define CPU_SET_DACHR1		91
#define CPU_SET_DACHR2		92

#define CPU_SET_MOTOR		30		//   
#define CPU_LANDING         35		//     
#define CPU_LANDING_MKM     34      // 1mkm in steps
#define CPU_ADC_A           40		//   ,   
#define CPU_ADC_B           41		//   
#define CPU_SYNC            50		//  

//CPU:   
#define CPU_TIMEDATA		132		// 2DWORDs elapsed:expected ms
#define CPU_SCANDATA		130		//  
#define CPU_SCANDATA_PART	131		//  , part
#define CPU_CCD_START       145    	//  CCD   
#define CPU_FORCE_CURVE_DATA 135		//   
#define CPU_OSCDATA         140		//  
#define CPU_FREQSCAN_DATA	150		//          
#define CPU_STATUS          160     //  DFL, LFL, MAG, DInt, Zpos
#define CPU_STEP_MOTOR_STATUS 170		//   
#define CPU_LANDING_STATUS	180		//  
#define CPU_LITHO_DONE      190     //    

#define CPU_DAC_JUMP_CHANNELS 33 // /     

#define CPU_AUTO_MAG_FREE_AMPLITUDE 36 //       mag
#define CPU_AUTO_MAG_START 37       // /  mag
#define CPU_AUTO_MAG_ENDED 38       //        mag
#define CPU_AUTO_MAG_MOVE_STEP 39   //   
#define CPU_AUTO_MAG_STEP_WAIT 42   //  

//    
//  .       ...
#define SMD_MOTOR_ITOA(idx) ((idx) << 4) //to addr
#define SMD_MOTOR_ATOI(idx) (((idx) & 0xF0) >> 4) //from addr
#define SMD_MOTOR1 SMD_MOTOR_ITOA(1)
#define SMD_MOTOR2 SMD_MOTOR_ITOA(2)
#define SMD_MOTOR3 SMD_MOTOR_ITOA(3)
#define SMD_MOTOR4 SMD_MOTOR_ITOA(4)
//...     
#define REG_CONTROL	0		//   (, ,  ,  )
#define REG_STEPS_HW 1		//   :  
#define REG_STEPS_LW 2		//  
#define REG_SPEED	3		// 
#define REG_INV_END	4		//  
#define REG_PWM_DC 	5		// current duty cycle, 0x0-0xFF
#define REG_BL_SPEED 	6	// backlash speed
#define REG_BL_STEPS_H 	7	// backlash steps
#define REG_BL_STEPS_L 	8	// backlash steps
#define REG_FLASH_READ 9
#define REG_FLASH_WRITE 10
#define REG_FLASH_IDX 11
#define REG_FLASH_SET 12
//   
#define RC_RUN			0x0001
#define RC_DIR_CW		0x0000
#define RC_DIR_CCW		0x0002
#define RC_ENABLED		0x0004
#define RC_FOREVER		0x0008
#define RC_LEAVE_VOLTAGE 0x0010
#define RC_NO_BACKLASH	0x0020
#define RC_GO_BACKLASH_MIDDLE 0x0040 //go to middle of deadband
#define RC_MS1			0x0100
#define RC_MS2			0x0200
//    
#define DIR_CW	0
#define DIR_CCW	1
//  
#define STEP_1          0x0000
#define STEP_2          0x0100
#define STEP_4          0x0200
#define STEP_16         0x0300
#define RC_STEP_SIZE    0x0300
//  
#define SMD_STOPPED     0x00
#define SMD_LIMIT_1     0x01
#define SMD_LIMIT_2     0x02
#define SMD_RUNNING     0x03


enum egMode {
    mdIdle, mdScan, mdFreqSweep, mdForceCurve, mdLitho, mdScanCorrection, mdWaitPositionReady
}; //   
enum Scan_Axes {
    AxisX, AxisY, AxisZ, AxisX2, AxisY2, AxisZ2, AxisX3, AxisY3, AxisZ3
}; //   .     ,    
enum Scanner {
    scHead, scStage, scZ3
}; //  

enum DACChannels {
    dc_vTip, dc_dac1, dc_dac2, dc_dacAUX1, dc_dacAUX2, dc_axisX, dc_axisY, dc_axisZ, dc_axisX2, dc_axisY2, dc_axisZ2, dc_axisZ3
};

//    
enum EG_DataSources {
    srcNone, srcHeight, srcDFL, srcLFL, srcMag1, srcMagSin1, srcMagCos1, srcPhase1, srcIntensity,
    srcADC1, srcADC2, srcADC3, srcADC4, srcADC5, srcCX, srcCY, srcCZ, srcDACX, srcDACY, srcDACZ,
    srcCX2, srcCY2, srcCZ2, srcDACX2, srcDACY2, srcDACZ2, srcADCHR1, srcADCHR2, srcCCD, srcVTip, srcMag2,
    srcDACZ3, srcCZ3
};

//      
//    
typedef struct {
    float sx;
    float sy;
    float sz;
} EG_SetPoint;

//      (   SPM)
//      
typedef struct {
    union { //  PID-
        struct {
            float PID_px, PID_ix, PID_dx,
                    PID_py, PID_iy, PID_dy,
                    PID_pz, PID_iz, PID_dz;
        };
        float PID_k[3][3]; // (  - ,  - P,I,D)
    };
    float K_XYZ_Piezo[3][6]; //  (, )
    float K_XYZ_Sensors[3][6]; //    
    float Range[3]; //      
    BYTE NLCStatus; //XY, CL
    BYTE CLStatus; //XY
    DWORD CSB_Mult[3], //    
            CSB_Shift[3];

    float DefaultRateOL;
    BYTE CLStatusZ; //Z
    BYTE NLCStatusZ; //Z, CL
    BYTE NLCStatusOL; //XY, OL
    BYTE NLCStatusOLZ; //Z, OL
} EG_CalData;

enum EG_CalDataID {
    cdSensor, cdPID, cdNLC_V, cdNLC_C, cdNLC_CPID
};
typedef struct {
    BYTE caldataID;
    BYTE deviceID; //head or stage, axis, other ids
    WORD data;
    union {
        float fcaldata[6];
        DWORD icaldata[6];
    };
} EG_PartCalData;

//      SPM 
typedef struct {
    float PID_pz_spm, PID_iz_spm, PID_dz_spm; //  PID    Z   SPM
    BYTE FB_SPM_Signal;
    BYTE Sign; //  ,    Z ( . EG_DataSources)
    BYTE Scanner;
} EG_SPM_CalData;

//  
enum EG_StartPoint {
    spTopLeft, spTopRight, spBottomRight, spBottomLeft
};
enum EG_StartPoint_ScanDirection {
    spTopLeftAxisX, spTopLeftAxisY,
    spTopRightAxisX, spTopRightAxisY,
    spBottomLeftAxisX, spBottomLeftAxisY,
    spBottomRightAxisX, spBottomRightAxisY
};

//    
#define MAX_CHANNELS_NUMBER		16	//    (    )
typedef struct {
    WORD NX; //    
    WORD NY;
    float StepX; //    
    float StepY;
    float x_0; //    
    float y_0;

    BYTE StartPoint; //   (. EG_StartPoint)
    BYTE FastAxis; //    (. EG_ScanDirection)

    BYTE DataSource[MAX_CHANNELS_NUMBER]; //  ,     .   16 
    BYTE DataSourceBack[MAX_CHANNELS_NUMBER]; // . EG_DataSources

    float LinesPerSecond; //      

    BYTE LiftMode;
    BYTE LiftModeEnabled[MAX_CHANNELS_NUMBER];
    BYTE LiftModeEnabledBack[MAX_CHANNELS_NUMBER];
    float LiftModeHeight;

    DWORD unused; // unused, replace with payload
    BYTE CorrectScanSize; //        
    BYTE PrecisionMode; // Enable variable scan speed - until PID done or min speed reached

} EG_ScanParams;

//  CCD
enum EG_ccdMode {
    ccdSingleScanMode, ccdDualScanMode, ccdDualScanMode2
};
typedef struct {
    WORD NX; //    
    WORD NY;
    float StepX; //    
    float StepY;
    float x_0; //    
    float y_0;
    BYTE StartPoint; //   (. EG_StartPoint)
    BYTE FastAxis; //    (. EG_ScanDirection)

    BYTE Scanner; //  (  )
    BYTE Mode; //   (. EG_ccdMode)
    BYTE FastMode; // fast mode enabled flag

} EG_ccdScanParams;

typedef struct {
    BYTE Scanner;
    float Speed;
} EG_ScanSpeed;

//   : /
enum EG_Direction {
    dirFW, dirBW
};

// ,   
#define MAX_LINE_SIZE	1024	//     
typedef struct {
    BYTE Scanner;
    BYTE Channel; //  
    BYTE Axis; //     
    BYTE Direction; //  
    WORD Size; //    
    WORD Line; //   
    WORD ScanSize; //     
    BYTE Source; //  
    WORD Data[MAX_LINE_SIZE]; //  
} EG_ScanData;

//  
#define OSC_LINE_SIZE	256	//   
typedef struct {
    BYTE Channel; //  
    WORD Size; //    
    BYTE Source; //  
    WORD Data[OSC_LINE_SIZE]; //  
} EG_OscData;

// current data value request
typedef struct {
    DWORD Data; //  
    BYTE Source; //  
} EG_GetData;

//      
#define LOCKIN_LINE_SIZE	1000	//       
typedef struct {
    WORD Size; //    
    BYTE Source; //  
    float res_point; //  
    WORD Data[LOCKIN_LINE_SIZE]; //  
} EG_LockInScanData;

//  
//  : DFL, LFL, MAG, INT, Zpos
typedef struct {
    BYTE Scanning;
    BYTE ErrorCode;
    WORD DFL;
    WORD LFL;
    WORD DInt;
    WORD MAG;
    WORD Zpos;
} EG_Status;

//   
typedef struct {
    float Frequency;
    float Voltage_A;
    float Voltage_B;
    float Phase;
} EG_ExcitationData;

//      
typedef struct {
    float Fmin;
    float Fmax;
    BYTE Source;
} EG_FreqScanData;

// fbZ_SPM -      SPM 
enum EG_FeedbackMode {
    fbOff = 0, fbZ_SPM = 1, fbZ_SPM_j = 2
};

//  
enum ecCodes {
    ecOK = 0,
    ecHeadNotFound = 1,
    ecStageNotFound = 2,
    ecScanErrorPID = 4,
    ecAxisErrorPID = 8,
    ecAxisErrorPID2 = 16
};

//     
enum smModes {
    smStop, smUp, smDown, smUpToLimit, smDownToLimit, smLandUp, smLandDown
};

//  
enum smStatus {
    smStopped, smRunningUp, smRunningDown, smLimitUp, smLimitDown
};

//  
enum lndMode {
    lmOff, lm01, lm02, lm03, lm04,
    lm05 //extended/configurable
};

//  
enum lndStatus {
    lndOff, lndLanding, lndDone, lndError, lndWaitDrift
};


//    
#define STEP_MOTORS_123 	250		//       ,       
typedef struct {
    BYTE Mode; // smModes
    DWORD Steps;
    WORD Speed;
    WORD StepSize;
    BYTE MotorN; //  
} EG_StepMotorData;

//  
typedef struct {
    BYTE Idx;
    BYTE Dir;
    WORD Speed;
    WORD Speed_near;
    DWORD Steps_per_mkm;
} EG_LandingMCfg;

#define LANDING_EX_CFG 4
typedef struct {
    BYTE Mode;
    BYTE Direction;
    WORD Speed;
    WORD Speed_near;
    float near_setp;
    EG_LandingMCfg MCfg[LANDING_EX_CFG]; //mode 5 config
} EG_LandingData;

//   
typedef struct {
    BYTE MotorN;
    BYTE Status;
} EG_StepMotorStatus;


//  :  
typedef struct {
    BYTE State;
    BYTE Source; //  
    float z1; //  
    float z2; //  
    WORD Size; //  
    float Speed; //   

} EG_ForceCurve;

//  :  
#define FORCE_CURVE_LINE_SIZE	1000 //      
typedef struct {
    BYTE Source;
    WORD Size;
    WORD Data[FORCE_CURVE_LINE_SIZE];
    WORD DataBack[FORCE_CURVE_LINE_SIZE];
} EG_ForceCurveData;


//   
typedef struct {
    BYTE LaserOn;
    BYTE AC_DC;
    BYTE Preamp;
} EG_HeadParams;


// 
enum EG_LithoMode {
    ltmNone, ltmSetPoint, ltmVoltage, ltmSync
};

#define LITHO_DATA_SIZE		100	//      ;

typedef struct {
    BYTE Scanner; //   
    BYTE LithoMode; //     :  SetPoint    -
    BYTE DACIndex; //        ( 0    )
    BYTE isContinuous; //1 = do not remove force between points
    BYTE Size; //     
    BYTE reserved[3];
    float V; //  
    float T; //    , 
    float Speed; //    
    //    
    float XY[LITHO_DATA_SIZE*2]; //maximum size, could be less than LITHO_DATA_SIZE*2
} EG_LithoParams2;

typedef struct {
    BYTE DACIndex;
    BYTE StartJumping; //1 -   . 0 - 
    WORD MSPeriod; //   ,  StartJumping = 1
    BYTE direction; //  0 - , 1 - 
    
    float BoundJump; //      /
} EG_DacJumpChannel;

#ifdef _x64
#pragma pack(pop)
#endif

#endif
