﻿#include "i_eg3000.h"

#ifndef NO_QT
#include <qglobal.h>
#endif
#ifndef Q_CC_CLANG
#include <malloc.h>
#else
#include <cstdlib>
#endif

#include <string.h>

#if !defined(PRINT_PRINTF) && !defined(PRINT_NO)
#include <QDebug>
#endif
#ifdef PRINT_PRINTF
#include <stdlib.h>
#define qWarning printf
#define qCritical printf
#endif
#ifdef PRINT_NO
#define qWarning(...)
#define qCritical(...)
#endif

#include "i_ftdi.h"
#include "i_libusb.h"

#if _MSC_VER
#define __inline inline
#else
#define __inline
#endif

#if defined __cplusplus
extern "C" {
#endif

//low-level interface wrappers
static int _eg3000_backend = 0;
BYTE eg3000_set_backend(WORD id)
{
    if (id > 1)
        return -1;
    _eg3000_backend = id;
    return 0;
}

static void * _eg3000_last_err_hndl = 0;
BYTE eg3000_is_ok(void *handle)
{
    if (_eg3000_last_err_hndl == handle)
        return RET_FALSE;
    return RET_TRUE;
}

BYTE eg3000_get_number(WORD vid, WORD pid, WORD *size)
{
#ifndef _NO_FTDI
    if (_eg3000_backend == 0)
        return ftdi_list(vid, pid, size, NULL, 0);
    else
#endif
#ifndef _NO_LIBUSB
        return libusb_list(vid, pid, size, NULL, 0);
#else
        ;
#endif
}
BYTE eg3000_list(WORD vid, WORD pid, WORD *size, struct s_info *arr_flags, WORD array_size)
{
#ifndef _NO_FTDI
    if (_eg3000_backend == 0)
        return ftdi_list(vid, pid, size, arr_flags, array_size);
    else
#endif
#ifndef _NO_LIBUSB
        return libusb_list(vid, pid, size, arr_flags, array_size);
#else
        ;
#endif
}
BYTE eg3000_open(void *handle, WORD vid, WORD pid, void **ret_handle)
{
#ifndef _NO_FTDI
    if (_eg3000_backend == 0)
        return ftdi_open(handle, vid, pid, ret_handle);
    else
#endif
#ifndef _NO_LIBUSB
        return libusb_open_(handle, vid, pid, ret_handle);
#else
        ;
#endif
}
BYTE eg3000_close(void *handle)
{
    if (_eg3000_last_err_hndl == handle)
        _eg3000_last_err_hndl = 0;

#ifndef _NO_FTDI
    if (_eg3000_backend == 0)
        return ftdi_close(handle);
    else
#endif
#ifndef _NO_LIBUSB
        return libusb_close_(handle);
#else
        ;
#endif
}
BYTE eg3000_clear(void *handle)
{
    if (_eg3000_last_err_hndl == handle)
        return RET_FALSE;

    BYTE st;
#ifndef _NO_FTDI
    if (_eg3000_backend == 0)
        st = ftdi_clear(handle);
    else
#endif
#ifndef _NO_LIBUSB
        st = libusb_clear(handle);
#else
        ;
#endif

    if (st == RET_FALSE && !_eg3000_last_err_hndl)
        _eg3000_last_err_hndl = handle;
    return st;
}
BYTE eg3000_set_timeout(void *handle, WORD timeout)
{
    if (_eg3000_last_err_hndl == handle)
        return RET_FALSE;

    BYTE st;
#ifndef _NO_FTDI
    if (_eg3000_backend == 0)
        st = ftdi_set_timeout(handle, timeout);
    else
#endif
#ifndef _NO_LIBUSB
        st = libusb_set_timeout(handle, timeout);
#else
        ;
#endif

    if (st == RET_FALSE && !_eg3000_last_err_hndl)
        _eg3000_last_err_hndl = handle;
    return st;
}
__inline
BYTE eg3000_raw_send(const void *handle, const void *buffer, WORD data_size)
{
    if (_eg3000_last_err_hndl == handle)
        return RET_FALSE;

    BYTE st;
#ifndef _NO_FTDI
    if (_eg3000_backend == 0)
        st = ftdi_write((void *)handle, buffer, data_size);
    else
#endif
#ifndef _NO_LIBUSB
        st = libusb_write((void *)handle, buffer, data_size);
#else
        ;
#endif

    if (st == RET_FALSE && !_eg3000_last_err_hndl)
        _eg3000_last_err_hndl = (void *)handle;
    return st;
}
BYTE eg3000_send_packet(const void *handle, BYTE Address, BYTE SubAddress, const void *buffer, WORD data_size)
{
    const int Overhead = 6;
    const int static_buf_size = 16;
    BYTE static_buf[static_buf_size];
    BYTE is_static = static_buf_size - Overhead >= data_size;
    BYTE status;

    BYTE *buf = is_static ? static_buf : (BYTE *)malloc(data_size + Overhead);
    if (!buf) return RET_FALSE;

    buf[0] = BYTE_START;
    buf[1] = data_size >> 8;
    buf[2] = data_size;
    buf[3] = Address;
    buf[4] = SubAddress;

    if (data_size != 0)
        memcpy(buf+Overhead-1, buffer, data_size);

    buf[data_size+Overhead-1] = BYTE_STOP;

    status = eg3000_raw_send(handle, buf, data_size + Overhead);
    if (status != RET_TRUE)
        qWarning("eg3000_send_packet: error: address %d %d", Address, SubAddress);

    if (!is_static)
        free(buf);
    return status;
}
__inline
BYTE eg3000_send_word_packet(const void *handle, BYTE Address, BYTE SubAddress, WORD Data)
{
    return eg3000_send_packet(handle, Address, SubAddress, &Data, sizeof(WORD));
}
__inline
BYTE eg3000_send_empty_packet(const void *handle, BYTE Address, BYTE SubAddress)
{
    return eg3000_send_packet(handle, Address, SubAddress, NULL, 0);
}
__inline
BYTE eg3000_read(void *handle, void *buffer, WORD buf_size, DWORD *p_bytes_read)
{
    if (_eg3000_last_err_hndl == handle)
        return RET_FALSE;

    BYTE st;
#ifndef _NO_FTDI
    if (_eg3000_backend == 0)
        st = ftdi_read(handle, buffer, buf_size, p_bytes_read);
    else
#endif
#ifndef _NO_LIBUSB
        st = libusb_read(handle, buffer, buf_size, p_bytes_read);
#else
        ;
#endif

    if (st == RET_FALSE && !_eg3000_last_err_hndl)
        _eg3000_last_err_hndl = handle;
    return st;
}

//packet recv & parsing
__inline
void eg3000_init_read_handle(struct eg3000_read_handle *p_read_handle)
{
    p_read_handle->buffer = malloc(2048);
    p_read_handle->buffer_size = 2048;

    if (p_read_handle->buffer == NULL) {
        qCritical("eg3000_init_read_handle: memory allocation failed");
        p_read_handle->buffer_size = 0;
    }

    eg3000_clear_read_handle(p_read_handle);
}
__inline
void eg3000_close_read_handle(struct eg3000_read_handle *p_read_handle)
{
    free(p_read_handle->buffer);
    p_read_handle->buffer = NULL;
    p_read_handle->buffer_size = 0;

    eg3000_clear_read_handle(p_read_handle);
}
__inline
void eg3000_clear_read_handle(struct eg3000_read_handle *p_read_handle)
{
    p_read_handle->buf_data_length = 0;
    p_read_handle->address = 0;
    p_read_handle->sub_address = 0;
    p_read_handle->header_read = 0;
    p_read_handle->packet_size = 0;
    p_read_handle->packet_recieved = 0;
    p_read_handle->packet_malformed = 0;
}
__inline
BYTE eg3000_remove_read_handle(struct eg3000_read_handle *p_read_handle, WORD size)
{
    BYTE *ptr;

    if (p_read_handle->buffer == NULL) {
        qCritical("eg3000_remove_read_handle: read handle not initialized");
        return RET_FALSE;
    }

    BYTE stat = RET_TRUE;
    if (size > p_read_handle->buf_data_length) {
        if (size > p_read_handle->buffer_size) {
            size = p_read_handle->buffer_size;
            stat = RET_FALSE;
        } else {
            size = p_read_handle->buf_data_length;
            stat = RET_TRUE_ERR;
        }
    }

    ptr = (BYTE *)p_read_handle->buffer;
    ptr += size;
    memmove(p_read_handle->buffer, ptr, p_read_handle->buf_data_length - size);
    p_read_handle->buf_data_length -= size;

    return stat;
}

BYTE eg3000_update_parse(void *handle, struct eg3000_read_handle *p_read_handle)
{
    DWORD bytes_read;
    BYTE *ptr;

    if (p_read_handle->buffer == NULL) {
        qCritical("eg3000_update_parse: read handle not initialized");
        return RET_TRUE_ERR;
    }

    if (p_read_handle->packet_recieved) {
        qWarning("eg3000_update_parse: you should get packet before update!");
        return RET_TRUE_ERR;
    }

    WORD size = 0xFFFF;
    if ((p_read_handle->buffer_size - p_read_handle->buf_data_length) < size)
        size = p_read_handle->buffer_size - p_read_handle->buf_data_length;

    //realloc more memory
    const int min_data_sz = 2048; //needs buffer >= transfer,packet size, max 1024 for usb3
    if (size < min_data_sz) {
        void *new_memory = realloc(p_read_handle->buffer, p_read_handle->buffer_size*2);
        if (new_memory == NULL) {
            qCritical("eg3000_update_parse: memory reallocation failed");
            return RET_FALSE;
        } else {
            p_read_handle->buffer = new_memory;
            p_read_handle->buffer_size *= 2;

            size = 0xFFFF;
            if ((p_read_handle->buffer_size - p_read_handle->buf_data_length) < size)
                size = p_read_handle->buffer_size - p_read_handle->buf_data_length;
        }
    }

    BYTE read_skipped = 0;
redo_read:
    ptr = (BYTE *)p_read_handle->buffer;
    ptr += p_read_handle->buf_data_length;

    if ((p_read_handle->header_read == 0) && (p_read_handle->buf_data_length >= 5)) {
        read_skipped = 1;
        goto skip_read;
    }

    //read
{
    BYTE st = eg3000_read(handle, ptr, size, &bytes_read);
    if (st != RET_TRUE)
        return st;
}
    if (bytes_read == 0)
        return RET_TRUE;

    //qDebug() << "dbg" << QByteArray((char *)ptr, bytes_read).toHex();
    p_read_handle->buf_data_length += bytes_read;


skip_read: //parse
    if ((p_read_handle->header_read == 0) && (p_read_handle->buf_data_length >= 5)) {
        ptr = (BYTE *)p_read_handle->buffer;
        if (ptr[0] == BYTE_START) {
            p_read_handle->packet_size = ptr[1]*256;
            p_read_handle->packet_size += ptr[2];
            p_read_handle->address = ptr[3];
            p_read_handle->sub_address = ptr[4];
            p_read_handle->header_read = 1;

            eg3000_remove_read_handle(p_read_handle, 5);
        } else {
            qCritical("eg3000_update_parse: wrong packet start, state reset: %d %d %d %d %d",
                      ptr[0], ptr[1], ptr[2], ptr[3], ptr[4]);

            eg3000_clear_read_handle(p_read_handle);
            p_read_handle->packet_malformed = 1;
        }
    }
    if (p_read_handle->header_read && (p_read_handle->buf_data_length >= p_read_handle->packet_size+1)) {
        p_read_handle->header_read = 0;
        ptr = (BYTE *)p_read_handle->buffer;

        p_read_handle->packet_recieved = (ptr[p_read_handle->packet_size] == BYTE_STOP);
        if (!p_read_handle->packet_recieved) {
            qCritical("eg3000_update_parse: wrong packet end, state reset: %d %ld %d %d",
                      ptr[p_read_handle->packet_size], p_read_handle->packet_size, p_read_handle->address, p_read_handle->sub_address);

            eg3000_clear_read_handle(p_read_handle);
            p_read_handle->packet_malformed = 1;
        }
    } else if (p_read_handle->header_read && read_skipped) {
        read_skipped = 0;
        goto redo_read;
    }

    return RET_TRUE;
}

BYTE eg3000_get_packet(struct eg3000_read_handle *p_read_handle, BYTE *buffer, DWORD buffersize)
{
    if (p_read_handle->buffer == NULL) {
        qCritical("eg3000_get_packet: read handle not initialized");
        return RET_FALSE;
    }

    BYTE stat = RET_TRUE;
    if (p_read_handle->packet_recieved) {
        if (buffer != NULL) {
            DWORD size = p_read_handle->packet_size;
            if (size > p_read_handle->buf_data_length) {
                if (size > p_read_handle->buffer_size) {
                    size = p_read_handle->buffer_size;
                    stat = RET_FALSE;
                } else {
                    size = p_read_handle->buf_data_length;
                    stat = RET_TRUE_ERR;
                }
            }
            if (size > buffersize) {
                size = buffersize;
                if (stat == RET_TRUE)
                    stat = RET_TRUE_ERR;
            }
            memcpy(buffer, p_read_handle->buffer, size);
        }

        BYTE stat2 = eg3000_remove_read_handle(p_read_handle, p_read_handle->packet_size + 1);
        if (stat == RET_TRUE)
            stat = stat2;

        p_read_handle->header_read = 0;
        p_read_handle->packet_size = 0;
        p_read_handle->packet_recieved = 0;
    }

    return stat;
}
BYTE eg3000_discard_packet(struct eg3000_read_handle *p_read_handle)
{
    if (p_read_handle->buffer == NULL) {
        qCritical("eg3000_discard_packet: read handle not initialized");
        return RET_FALSE;
    }

    BYTE stat = RET_TRUE;
    if (p_read_handle->packet_recieved) {
        stat = eg3000_remove_read_handle(p_read_handle, p_read_handle->packet_size + 1);

        p_read_handle->header_read = 0;
        p_read_handle->packet_size = 0;
        p_read_handle->packet_recieved = 0;
    }

    return stat;
}

//bootloader
BYTE eg3000_bl_program(void *handle, BYTE *buffer, DWORD length, struct eg3000_read_handle *p_read_handle, BYTE type)
{
    if (type != 0) {
        DWORD lock = 1;
        if (!eg3000_send_packet(handle, ADR_BOOTLOADER, BL_EXT_LOCK, &lock, 4)) return RET_FALSE;
        eg3000_set_timeout(handle, 10000);
    }

    {
        BYTE buf[512+4+4];
        DWORD *buf_dw = (DWORD *)buf;
        int nPacket = 0;
        DWORD base_addr = 0x9D000000;
        int packet_size = 512;
        DWORD size = length;
        if (type == 1) { //ext epcs
            base_addr = 0xFF000000;
            packet_size = 256;
        }

        //todo check for max flash size, now 0x40000 max;
        if (type == 0 && length > 0x40000) {
            qWarning("eg3000_bl_program: firmware is too big %lx %lx", length, 0x40000lu);
            return RET_TRUE_ERR;
        }

        while (size > 0) {
            DWORD sz = packet_size;
            if (sz > size)
                sz = size;

            buf_dw[0] = base_addr + (nPacket*packet_size);
            buf_dw[1] = sz/4;

            BYTE page_blank = true;
            for (DWORD i = 0; i < sz; i++) {
                if (buffer[nPacket*packet_size + i] != 0xFF) {
                    page_blank = false;
                    break;
                }
            }

            if (page_blank) {
                if (!eg3000_send_packet(handle, ADR_BOOTLOADER, BL_WT_FLASH_FF, buf, 8)) return RET_FALSE;
            } else {
                for (DWORD i = 0; i < sz; i++)
                    buf[8+i] = buffer[nPacket*packet_size + i];

                if (!eg3000_send_packet(handle, ADR_BOOTLOADER, BL_WT_FLASH, buf, sz+8)) return RET_FALSE;
            }

            size -= sz;
            nPacket++;
        }
    }

    BYTE ok = eg3000_bl_verify(handle, buffer, length, p_read_handle, type);

    if (type != 0) {
        DWORD lock = 0;
        if (!eg3000_send_packet(handle, ADR_BOOTLOADER, BL_EXT_LOCK, &lock, 4)) return RET_FALSE;
        eg3000_set_timeout(handle, 1000);
    }

    return ok;
}

BYTE eg3000_bl_verify(void *handle, BYTE *buffer, DWORD length, struct eg3000_read_handle *p_read_handle, BYTE type)
{
    BYTE buf[512+4+4];
    DWORD *buf_dw = (DWORD *)buf;
    int nPacket = 0;
    DWORD base_addr = 0x9D000000;
    int packet_size = 512;
    DWORD size = length;
    if (type == 1) { //ext epcs
        base_addr = 0xFF000000;
        packet_size = 256;
    }

    DWORD errors = 0;
    DWORD recv_size = 0;

    //todo check for max flash size, now 0x40000 max;
    if (type == 0 && length > 0x40000) {
        qWarning("eg3000_bl_verify: firmware is too big %lx %lx", length, 0x40000lu);
        return RET_TRUE_ERR;
    }

    qint64 start, prev = -1, ecount = 0; //err range
    while (size > 0) {
        DWORD sz = packet_size;
        if (sz > size)
            sz = size;

        buf_dw[0] = base_addr + (nPacket*packet_size);
        buf_dw[1] = sz/4;
        if (!eg3000_send_packet(handle, ADR_BOOTLOADER, BL_RD_FLASH, buf, 8)) return RET_FALSE;

        //read
        BYTE need_response = true;
        while (need_response) {
            if (eg3000_update_parse(handle, p_read_handle) != RET_TRUE) {
                qWarning("eg3000_bl_verify: read error");
                return RET_FALSE;
            }

            if (p_read_handle->packet_recieved) {
                if (p_read_handle->address == ADR_BOOTLOADER && p_read_handle->sub_address == BL_FLASH &&
                        p_read_handle->packet_size <= 512+4+4) {
                    recv_size = p_read_handle->packet_size;
                    eg3000_get_packet(p_read_handle, buf, 512+4+4);
                    need_response = false;
                } else
                    eg3000_discard_packet(p_read_handle);
            }
            //TODO sleep?
        }

        //compare
        DWORD adr = buf_dw[0];
        if (adr != base_addr + (nPacket*packet_size)) {
            errors++;
            qWarning("eg3000_bl_verify: wrong address %lx %lx", adr, (base_addr + (nPacket*packet_size)));
        }

        for (unsigned int i = 0; i < recv_size-4; i++) {
            if (buf[4+i] != buffer[nPacket*packet_size + i]) {
                errors++;
                ecount++;
                if ((prev + 32 < adr + i) && //32byte gap max, likely due to 0xFF sequence
                    (prev + 1 != adr + i))
                {
                    if (prev >= 0 && start != prev)
                        qWarning("eg3000_bl_verify: wrong data from %lx to %lx, %d%%", (DWORD)start, (DWORD)prev, (BYTE)(float(ecount)/(prev-start)*100));

                    qWarning("eg3000_bl_verify: wrong data at %lx have %x need %x", (adr + i), buf[4+i], buffer[nPacket*packet_size + i]);
                    start = adr + i;
                    ecount = 1;
                }
                prev = adr + i;
            }
        }

        size -= sz;
        nPacket++;
    }

    if (!errors) {
        if (type == 0)
            if (!eg3000_send_packet(handle, ADR_BOOTLOADER, BL_VERIFY, &length, 4)) return RET_FALSE;
    } else {
        if (prev >= 0 && start != prev)
            qWarning("eg3000_bl_verify: wrong data from %lx to %lx, %d%%", (DWORD)start, (DWORD)prev, (BYTE)(float(ecount)/(prev-start)*100));
        qWarning("eg3000_bl_verify: firmware verification failed, %ld errors", errors);
        return RET_TRUE_ERR;
    }

    return RET_TRUE;
}

BYTE eg3000_bl_erase(void *handle)
{
    return eg3000_send_empty_packet(handle, ADR_BOOTLOADER, BL_ER_FLASH);
}

BYTE eg3000_bl_stats(void *handle)
{
    return eg3000_send_empty_packet(handle, ADR_BOOTLOADER, BL_RD_STATS);
}

BYTE eg3000_bl_run(void *handle)
{
    return eg3000_send_empty_packet(handle, ADR_BOOTLOADER, BL_GO_BOOT);
}

//simple functions
BYTE eg3000_cpu_loopback(void *handle)
{
    return eg3000_send_empty_packet(handle, ADR_CPU, CPU_LOOPBACK);
}
BYTE eg3000_fb_control(void *handle, BYTE val)
{
    return eg3000_send_word_packet(handle, ADR_CPU, CPU_SET_FEEDBACK, val);
}
BYTE eg3000_set_head_dac(void *handle, WORD val)
{
#define CSB_DAC 0x40
    return eg3000_send_word_packet(handle, ADR_HEAD, CSB_DAC, val);
}

#define CSB_SETTINGS 0x60
#define CSB_AFM_STM 0x01
#define CSB_I_STM 0x02
#define CSB_PIEZO_TIP 0x03
#define CSB_OF_ACTION 0x06
#define CSB_DOUBLE_OUTPUT 0x07
BYTE eg3000_set_head_piezotip(void *handle, BYTE val)
{
    return eg3000_send_word_packet(handle, ADR_HEAD, CSB_SETTINGS + CSB_PIEZO_TIP, val);
}
BYTE eg3000_set_head_lfl_current(void *handle, BYTE val)
{
    return eg3000_send_word_packet(handle, ADR_HEAD, CSB_SETTINGS + CSB_AFM_STM, val);
}
BYTE eg3000_set_head_ext_current(void *handle, BYTE val)
{
    return eg3000_send_word_packet(handle, ADR_HEAD, CSB_SETTINGS + CSB_I_STM, val);
}
BYTE eg3000_csb_overflow(void *handle, BYTE scanner, BYTE mode)
{
    return eg3000_send_word_packet(handle, (scanner == scHead) ? ADR_HEAD : ADR_STAGE, CSB_SETTINGS + CSB_OF_ACTION, mode);
}
BYTE eg3000_csb_double_c_out(void *handle, BYTE scanner, BYTE mode)
{
    return eg3000_send_word_packet(handle, (scanner == scHead) ? ADR_HEAD : ADR_STAGE, CSB_SETTINGS + CSB_DOUBLE_OUTPUT, mode);
}

BYTE eg3000_hv_zero(void *handle)
{
    return eg3000_send_empty_packet(handle, ADR_CPU, CPU_HV_ZERO);
}
BYTE eg3000_osc_startstop(void *handle, BYTE on)
{
    return eg3000_send_word_packet(handle, ADR_CPU, CPU_SCOPE_ON_OFF, on);
}
BYTE eg3000_status_onoff(void *handle, BYTE on)
{
    return eg3000_send_word_packet(handle, ADR_CPU, CPU_STATUS_ON_OFF, on);
}

//struct sending functions
BYTE eg3000_osc_set_data(void *handle, BYTE ch_a, BYTE ch_b, BYTE ch_c, BYTE ch_d)
{
    BYTE ch[4];
    ch[0] = ch_a;
    ch[1] = ch_b;
    ch[2] = ch_c;
    ch[3] = ch_d;
    if (!eg3000_send_packet(handle, ADR_CPU, CPU_SCOPE_SET_PARAMS, ch, 4)) return RET_FALSE;

    //compatibility hackfix!!! old fw < 1.7 uses size==2 for data assignment
    return eg3000_send_packet(handle, ADR_CPU, CPU_SCOPE_SET_PARAMS, ch, 2);
}
BYTE eg3000_send_caldata(void *handle, int scanner,
                         float arr[48], DWORD mult[3], long shift[3],
                         BYTE nlc_status_cl_xy, BYTE cl_status_xy, float free_positioning_speed,
                         BYTE nlc_status_cl_z, BYTE cl_status_z,
                         BYTE nlc_status_ol_xy, BYTE nlc_status_ol_z)
{
    BYTE Address = CPU_SET_CALDATA;
    if (scanner == scStage)
        Address = CPU_SET_CALDATA_STAGE;
    if (scanner == scZ3)
        Address = CPU_SET_CALDATA_Z3;

    EG_CalData CD;

    //first 48 floats in arrays
    memcpy((void *)&CD, (void *)arr, 48*sizeof(float));

    CD.NLCStatus = nlc_status_cl_xy;
    CD.CLStatus  = cl_status_xy;
    CD.NLCStatusZ = nlc_status_cl_z;
    CD.CLStatusZ  = cl_status_z;
    CD.NLCStatusOL = nlc_status_ol_xy;
    CD.NLCStatusOLZ = nlc_status_ol_z;

    CD.CSB_Mult[0] = mult[0];
    CD.CSB_Mult[1] = mult[1];
    CD.CSB_Mult[2] = mult[2];

    CD.CSB_Shift[0] = shift[0]+32768;
    CD.CSB_Shift[1] = shift[1]+32768;
    CD.CSB_Shift[2] = shift[2]+32768;

    CD.DefaultRateOL = free_positioning_speed;

    return eg3000_send_packet(handle, ADR_CPU, Address, &CD, sizeof(EG_CalData));
}

BYTE eg3000_set_point_xyz(void *handle, int scanner, float x_sp, float y_sp, float z_sp)
{
    BYTE Address = CPU_SETPOINT;
    if (scanner == scStage)
        Address = CPU_SETPOINT_STAGE;
    if (scanner == scZ3)
        Address = CPU_SETPOINT_Z3;

    EG_SetPoint SP;
    SP.sx = x_sp;
    SP.sy = y_sp;
    SP.sz = z_sp;

    return eg3000_send_packet(handle, ADR_CPU, Address, &SP, sizeof(EG_SetPoint));
}

BYTE eg3000_scan_speed(void *handle, int scanner, float v)
{
    EG_ScanSpeed SD;
    SD.Scanner = (BYTE)scanner;
    SD.Speed = v;

    return eg3000_send_packet(handle, ADR_CPU, CPU_SET_SCAN_VELOCITY, &SD, sizeof(EG_ScanSpeed));
}
BYTE eg3000_scan_params(void *handle, int scanner,
                        WORD NX, WORD NY, float V,
                        float stepx, float stepy, float start_point_x, float start_point_y,
                        BYTE dir_point_type, BYTE scan_a, BYTE scan_b, BYTE scan_aback, BYTE scan_bback,
                        BYTE LmEn, BYTE LmEnA, BYTE LmEnB, BYTE LmEnC, BYTE LmEnD, float LmHeight,
                        BYTE fast_scan_correction, BYTE precise_mode)
{
    BYTE Address = CPU_SET_SCAN;
    if (scanner == scStage)
        Address = CPU_SET_SCAN_STAGE;

    EG_ScanParams SD;

    SD.NX = NX;
    SD.NY = NY;
    SD.StepX = stepx;
    SD.StepY = stepy;
    SD.x_0 = start_point_x;
    SD.y_0 = start_point_y;

    SD.DataSource[0] = scan_a;
    SD.DataSource[1] = scan_b;
    SD.DataSourceBack[0] = scan_aback;
    SD.DataSourceBack[1] = scan_bback;

    SD.DataSource[2] = 0;//fast scan correction x reserved
    SD.DataSourceBack[2] = 0;//fast scan correction x reserved
    SD.DataSource[3] = 0;//fast scan correction y reserved
    SD.DataSourceBack[3] = 0;//fast scan correction y reserved
    SD.DataSource[4] = 0;
    SD.DataSourceBack[4] = 0;
    SD.DataSource[5] = 0;
    SD.DataSourceBack[5] = 0;
    SD.DataSource[6] = 0;
    SD.DataSourceBack[6] = 0;
    SD.DataSource[7] = 0;
    SD.DataSourceBack[7] = 0;

    SD.unused = 0;
    SD.CorrectScanSize = fast_scan_correction != 3 ? fast_scan_correction : 0; //3-voltage
    SD.PrecisionMode = precise_mode;

    if (fast_scan_correction == 3) { //voltage
        switch (scanner)
        {
        case scHead:
            if (scan_a != srcNone || scan_b != srcNone) {
                if (scan_a != srcDACX && scan_b != srcDACX)
                    SD.DataSource[2] = srcDACX;
                if (scan_a != srcDACY && scan_b != srcDACY)
                    SD.DataSource[3] = srcDACY;
            }
            if (scan_aback != srcNone || scan_bback != srcNone) {
                if (scan_aback != srcDACX && scan_bback != srcDACX)
                    SD.DataSourceBack[2] = srcDACX;
                if (scan_aback != srcDACY && scan_bback != srcDACY)
                    SD.DataSourceBack[3] = srcDACY;
            }
            break;
        case scStage:
            if (scan_a != srcNone || scan_b != srcNone) {
                if (scan_a != srcDACX2 && scan_b != srcDACX2)
                    SD.DataSource[2] = srcDACX2;
                if (scan_a != srcDACY2 && scan_b != srcDACY2)
                    SD.DataSource[3] = srcDACY2;
            }
            if (scan_aback != srcNone || scan_bback != srcNone) {
                if (scan_aback != srcDACX2 && scan_bback != srcDACX2)
                    SD.DataSourceBack[2] = srcDACX2;
                if (scan_aback != srcDACY2 && scan_bback != srcDACY2)
                    SD.DataSourceBack[3] = srcDACY2;
            }
            break;
        }
    } else
    if (fast_scan_correction) {
        switch (scanner)
        {
        case scHead:
            if (scan_a != srcNone || scan_b != srcNone) {
                if (scan_a != srcCX && scan_b != srcCX)
                    SD.DataSource[2] = srcCX;
                if (scan_a != srcCY && scan_b != srcCY)
                    SD.DataSource[3] = srcCY;
            }
            if (scan_aback != srcNone || scan_bback != srcNone) {
                if (scan_aback != srcCX && scan_bback != srcCX)
                    SD.DataSourceBack[2] = srcCX;
                if (scan_aback != srcCY && scan_bback != srcCY)
                    SD.DataSourceBack[3] = srcCY;
            }
            break;
        case scStage:
            if (scan_a != srcNone || scan_b != srcNone) {
                if (scan_a != srcCX2 && scan_b != srcCX2)
                    SD.DataSource[2] = srcCX2;
                if (scan_a != srcCY2 && scan_b != srcCY2)
                    SD.DataSource[3] = srcCY2;
            }
            if (scan_aback != srcNone || scan_bback != srcNone) {
                if (scan_aback != srcCX2 && scan_bback != srcCX2)
                    SD.DataSourceBack[2] = srcCX2;
                if (scan_aback != srcCY2 && scan_bback != srcCY2)
                    SD.DataSourceBack[3] = srcCY2;
            }
            break;
        }
    }

    SD.LinesPerSecond = V;

    SD.LiftMode = LmEn;
    SD.LiftModeEnabled[0] = LmEnA;
    SD.LiftModeEnabled[1] = LmEnB;
    SD.LiftModeEnabledBack[0] = LmEnC;
    SD.LiftModeEnabledBack[1] = LmEnD;
    SD.LiftModeHeight = LmHeight;

    switch (dir_point_type) {
    case spTopLeftAxisX:
        SD.StartPoint = spTopLeft;
        SD.FastAxis  =  AxisX;
        break;
    case spTopLeftAxisY:
        SD.StartPoint = spTopLeft;
        SD.FastAxis  =  AxisY;
        break;
    case spTopRightAxisX:
        SD.StartPoint = spTopRight;
        SD.FastAxis  =  AxisX;
        break;
    case spTopRightAxisY:
        SD.StartPoint = spTopRight;
        SD.FastAxis  =  AxisY;
        break;
    case spBottomRightAxisX:
        SD.StartPoint = spBottomRight;
        SD.FastAxis  =  AxisX;
        break;
    case spBottomRightAxisY:
        SD.StartPoint = spBottomRight;
        SD.FastAxis  =  AxisY;
        break;
    case spBottomLeftAxisX:
        SD.StartPoint = spBottomLeft;
        SD.FastAxis  =  AxisX;
        break;
    case spBottomLeftAxisY:
        SD.StartPoint = spBottomLeft;
        SD.FastAxis  =  AxisY;
        break;
    default:
        SD.StartPoint = spBottomLeft;
        SD.FastAxis  =  AxisX;
        break;
    }

    return eg3000_send_packet(handle, ADR_CPU, Address, &SD, sizeof(EG_ScanParams));
}

BYTE eg3000_ccd_scan_params(void *handle, int scanner,
                            int mode, WORD NX, WORD NY,
                            float stepx, float stepy, float start_point_x, float start_point_y,
                            BYTE dir_point_type)
{
    EG_ccdScanParams SD;
    SD.Scanner = scanner;
    SD.Mode = mode;
    SD.FastMode = false;

    SD.NX = NX;
    SD.NY = NY;
    SD.StepX = stepx;
    SD.StepY = stepy;
    SD.x_0 = start_point_x;
    SD.y_0 = start_point_y;

    switch (dir_point_type) {
    case spTopLeftAxisX:
        SD.StartPoint = spTopLeft;
        SD.FastAxis  =  AxisX;
        break;
    case spTopLeftAxisY:
        SD.StartPoint = spTopLeft;
        SD.FastAxis  =  AxisY;
        break;
    case spTopRightAxisX:
        SD.StartPoint = spTopRight;
        SD.FastAxis  =  AxisX;
        break;
    case spTopRightAxisY:
        SD.StartPoint = spTopRight;
        SD.FastAxis  =  AxisY;
        break;
    case spBottomRightAxisX:
        SD.StartPoint = spBottomRight;
        SD.FastAxis  =  AxisX;
        break;
    case spBottomRightAxisY:
        SD.StartPoint = spBottomRight;
        SD.FastAxis  =  AxisY;
        break;
    case spBottomLeftAxisX:
        SD.StartPoint = spBottomLeft;
        SD.FastAxis  =  AxisX;
        break;
    case spBottomLeftAxisY:
        SD.StartPoint = spBottomLeft;
        SD.FastAxis  =  AxisY;
        break;
    default:
        SD.StartPoint = spBottomLeft;
        SD.FastAxis  =  AxisX;
        break;
    }

    return eg3000_send_packet(handle, ADR_CPU, CPU_SET_SCAN_CCD, &SD, sizeof(EG_ccdScanParams));
}

BYTE eg3000_head_params(void *handle, BYTE laser_on_off, BYTE ac_dc)
{
    EG_HeadParams RMD;

    RMD.LaserOn = laser_on_off;
    RMD.AC_DC   = ac_dc;
    RMD.Preamp  = 0;

    return eg3000_send_packet(handle, ADR_CPU, CPU_SET_HEAD, &RMD, sizeof(EG_HeadParams));
}
BYTE _Dll eg3000_kelvin_params(void *handle, BYTE on, float p)
{
    EG_SPM_CalData SD;
    SD.Scanner = on;
    SD.PID_pz_spm = p;
    SD.PID_iz_spm = 0;
    SD.PID_dz_spm = 0;
    SD.FB_SPM_Signal = 0;
    SD.Sign = 0;

    return eg3000_send_packet(handle, ADR_CPU, CPU_SETKELVINMODE, &SD, sizeof(EG_SPM_CalData));
}
BYTE eg3000_fb_pid_params(void *handle, BYTE scanner, float p, float i, float d, BYTE src_channel, BYTE sign)
{
    EG_SPM_CalData SD;
    SD.Scanner = scanner;
    SD.PID_pz_spm = p;
    SD.PID_iz_spm = i;
    SD.PID_dz_spm = d;
    SD.FB_SPM_Signal = src_channel;
    SD.Sign = sign;

    return eg3000_send_packet(handle, ADR_CPU, CPU_SET_SPM_CALDATA, &SD, sizeof(EG_SPM_CalData));
}
BYTE eg3000_fb_pid_setpoint(void *handle, float val)
{
    return eg3000_send_packet(handle, ADR_CPU, CPU_SETPOINT_SPM, &val, sizeof(float));
}

BYTE eg3000_frequency_scan_start(void *handle, float f_start, float f_end, BYTE channel)
{
    EG_FreqScanData FSD;
    FSD.Fmin = f_start;
    FSD.Fmax = f_end;
    FSD.Source = channel;

    return eg3000_send_packet(handle, ADR_CPU, CPU_FREQ_SCAN, &FSD, sizeof(EG_FreqScanData));
}
BYTE eg3000_set_frequency(void *handle, float freq)
{
    return eg3000_send_packet(handle, ADR_CPU, CPU_SET_FREQUENCY, &freq, sizeof(float));
}
BYTE eg3000_set_frequency2(void *handle, float freq)
{
    return eg3000_send_packet(handle, ADR_CPU, CPU_SET_FREQUENCY_2, &freq, sizeof(float));
}
BYTE eg3000_set_cutoff_frequency(void *handle, float cutoff_freq)
{
    return eg3000_send_packet(handle, ADR_CPU, CPU_SET_LPF_FREQ, &cutoff_freq, sizeof(float));
}
BYTE eg3000_set_ampl_gains(void *handle, int gain1, int gain2, int gain3)
{
    /* 1-x1
       2-x2
       3-x4
       4-x5
       5-x10
       6-x20
       7-x25
       8-x50
       9-x100 */
    //first+second amps=2+2bits, high=*5, low=*2
    unsigned int gains_word;
    unsigned int g1 = 0;
    unsigned int g2 = 0;
    unsigned int g3 = 0;
    switch(gain1)
    {
    case 1:
        g1 = 0;
        break;
    case 2:
        g1 = 4;
        break;
    case 3:
        g1 = 5;
        break;
    case 4:
        g1 = 8;
        break;
    case 5:
        g1 = 12;
        break;
    case 6:
        g1 = 13;
        break;
    case 7:
        g1 = 10;
        break;
    case 8:
        g1 = 14;
        break;
    case 9:
        g1 = 15;
        break;
    }
    switch(gain2)
    {
    case 1:
        g2 = 0;
        break;
    case 2:
        g2 = 64;
        break;
    case 3:
        g2 = 80;
        break;
    case 4:
        g2 = 128;
        break;
    case 5:
        g2 = 192;
        break;
    case 6:
        g2 = 208;
        break;
    case 7:
        g2 = 160;
        break;
    case 8:
        g2 = 224;
        break;
    case 9:
        g2 = 240;
        break;
    }
    switch(gain3)
    {
    case 1:
        g3 = 0;
        break;
    case 2:
        g3 = 256;
        break;
    case 3:
        g3 = 512;
        break;
    case 4:
        g3 = 768;
        break;
    }
    gains_word = g1 | g2 | g3;
    return eg3000_send_word_packet(handle, ADR_CPU, CPU_SET_GAINS, gains_word);
}
BYTE eg3000_set_ampl_gains2(void *handle, int gain1, int gain2, int gain3)
{
    /* 1-x1
       2-x2
       3-x4
       4-x5
       5-x10
       6-x20
       7-x25
       8-x50
       9-x100 */

    //first+second amps=2+2bits, high=*5, low=*2
    unsigned int gains_word;
    unsigned int g1 = 0;
    unsigned int g2 = 0;
    unsigned int g3 = 0;
    switch(gain1)
    {
    case 1:
        g1 = 0;
        break;
    case 2:
        g1 = 4;
        break;
    case 3:
        g1 = 5;
        break;
    case 4:
        g1 = 8;
        break;
    case 5:
        g1 = 12;
        break;
    case 6:
        g1 = 13;
        break;
    case 7:
        g1 = 10;
        break;
    case 8:
        g1 = 14;
        break;
    case 9:
        g1 = 15;
        break;
    }
    switch(gain2)
    {
    case 1:
        g2 = 0;
        break;
    case 2:
        g2 = 64;
        break;
    case 3:
        g2 = 80;
        break;
    case 4:
        g2 = 128;
        break;
    case 5:
        g2 = 192;
        break;
    case 6:
        g2 = 208;
        break;
    case 7:
        g2 = 160;
        break;
    case 8:
        g2 = 224;
        break;
    case 9:
        g2 = 240;
        break;
    }
    switch(gain3)
    {
    case 1:
        g3 = 0;
        break;
    case 2:
        g3 = 256;
        break;
    case 3:
        g3 = 512;
        break;
    case 4:
        g3 = 768;
        break;
    }
    gains_word = g1 | g2 | g3;
    return eg3000_send_word_packet(handle, ADR_CPU, CPU_SET_GAINS_2, gains_word);
}
BYTE eg3000_set_phase(void *handle, float phase)
{
    return eg3000_send_packet(handle, ADR_CPU, CPU_SET_PHASE_SHIFT, &phase, sizeof(float));
}
BYTE eg3000_set_piezo_drive_voltage(void *handle, float voltage)
{
    return eg3000_send_packet(handle, ADR_CPU, CPU_SET_EXCITATION_A, &voltage, sizeof(float));
}
BYTE eg3000_set_piezo_drive_voltage_lift(void *handle, float voltage)
{
    return eg3000_send_packet(handle, ADR_CPU, CPU_SET_EXCITATION_B, &voltage, sizeof(float));
}
BYTE eg3000_set_cap_mode(void *handle, BYTE isOn)
{
    return eg3000_send_word_packet(handle, ADR_CPU, CPU_TIP_CAP_MODE, isOn);
}
BYTE eg3000_set_cap_mode2(void *handle, BYTE isOn)
{
    return eg3000_send_word_packet(handle, ADR_CPU, CPU_TIP_CAP_MODE_2, isOn);
}
BYTE eg3000_set_extra_DAC1(void *handle, float v)
{
    return eg3000_send_packet(handle, ADR_CPU, CPU_SET_DAC_A, &v, sizeof(float));
}
BYTE eg3000_set_extra_DAC2(void *handle, float v)
{
    return eg3000_send_packet(handle, ADR_CPU, CPU_SET_DAC_B, &v, sizeof(float));
}
BYTE eg3000_set_extra_DAC3(void *handle, float v)
{
    return eg3000_send_packet(handle, ADR_CPU, CPU_SET_DACHR1, &v, sizeof(float));
}
BYTE eg3000_set_extra_DAC4(void *handle, float v)
{
    return eg3000_send_packet(handle, ADR_CPU, CPU_SET_DACHR2, &v, sizeof(float));
}

BYTE eg3000_set_spectroscopy_params(void *handle, BYTE on_off,
                                    float z1, float z2, float Speed,
                                    BYTE Channel, WORD Size)
{
    EG_ForceCurve SMD;
    SMD.State = on_off;
    SMD.Source = Channel;
    SMD.z1 = z1;
    SMD.z2 = z2;
    SMD.Size = Size;
    SMD.Speed = Speed;

    return eg3000_send_packet(handle, ADR_CPU, CPU_FORCE_CURVE, &SMD, sizeof(EG_ForceCurve));
}

//step motors functions
BYTE eg3000_set_motor_leveling(void *handle, BYTE motor_number, BYTE cont_byte, DWORD steps, float speed)
{
    EG_StepMotorData SMD;
    SMD.MotorN = motor_number;
    SMD.Mode = cont_byte;
    SMD.Steps = steps;
    SMD.Speed = speed;
    SMD.StepSize = STEP_16;

    return eg3000_send_packet(handle, ADR_CPU, CPU_SET_MOTOR, &SMD, sizeof(EG_StepMotorData));
}
BYTE eg3000_set_landing(void *handle, BYTE mode, BYTE direction, float speed, float speed_slow, float close_perc)
{
    EG_LandingData SMD;
    SMD.Mode = mode;
    SMD.Direction = direction;
    SMD.Speed = speed;
    SMD.Speed_near = speed_slow;
    SMD.near_setp = close_perc;

    return eg3000_send_packet(handle, ADR_CPU, CPU_LANDING, &SMD, sizeof(EG_LandingData));
}

BYTE _Dll eg3000_set_landing_ex(void *handle, BYTE mode, BYTE direction, float speed, float speed_slow, float close_perc,
                                EG_LandingMCfg *mcfg, int mcfg_elems)
{
    EG_LandingData SMD;
    SMD.Mode = mode;
    SMD.Direction = direction;
    SMD.Speed = speed;
    SMD.Speed_near = speed_slow;
    SMD.near_setp = close_perc;

    for (int i = 0; i < mcfg_elems && i < LANDING_EX_CFG; i++)
        SMD.MCfg[i] = mcfg[i];
    for (int i = mcfg_elems; i < LANDING_EX_CFG; i++)
        SMD.MCfg[i].Idx = 0;

    return eg3000_send_packet(handle, ADR_CPU, CPU_LANDING, &SMD, sizeof(EG_LandingData));
}


#if defined __cplusplus
} // extern "C" {
#endif
