#include "i_libusb.h"

#undef _WIN32_WINNT
#define _WIN32_WINNT 0x0502
#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#ifndef NO_QT
#include <qglobal.h>
#endif

//#include <string.h>

#if !defined(PRINT_PRINTF) && !defined(PRINT_NO)
#include <QDebug>
#endif
#ifdef PRINT_PRINTF
#include <stdlib.h>
#define qWarning printf
#define qCritical printf
#endif
#ifdef PRINT_NO
#define qWarning(...)
#define qCritical(...)
#endif

#ifndef _WIN32
#include <usb.h>
//#include <libusb.h>
//libusb_context *ctx;
#else
#include "lusb0_usb.h"
#endif

#if defined __cplusplus
extern "C" {
#endif

BYTE libusb_get_number(WORD vid, WORD pid, WORD *size)
{
    return libusb_list(vid, pid, size, NULL, 0);
}

BYTE libusb_list(WORD vid, WORD pid, WORD *size, struct s_info *arr_flags, WORD array_size)
{
    usb_init();
    usb_find_busses();
    usb_find_devices();

    WORD devices = 0;
    usb_bus *busses = usb_get_busses();
    usb_bus *bus;
    for (bus = busses; bus; bus = bus->next) {
        struct usb_device *dev;
        for (dev = bus->devices; dev; dev = dev->next) {
            if (arr_flags != NULL && devices >= array_size)
                break;

            if (dev->descriptor.idVendor == vid && dev->descriptor.idProduct == pid) {
                if (arr_flags != NULL) {
                    arr_flags[devices].handle = (void *)devices;
                    arr_flags[devices].flags = F_NO;
                }
                devices++;
            }
        }
    }

    *size = devices;
    return RET_TRUE;
}
BYTE libusb_open_(void *handle, WORD vid, WORD pid, void **ret_handle)
{
    usb_init();
    usb_find_busses();
    usb_find_devices();

    usb_bus *busses = usb_get_busses();

    DWORD devices = 0;

    usb_dev_handle *h = 0;
    usb_bus *bus;
    for (bus = busses; bus; bus = bus->next) {
        struct usb_device *dev;
        for (dev = bus->devices; dev; dev = dev->next) {
            if (dev->descriptor.idVendor == vid && dev->descriptor.idProduct == pid) {
                if ((void *)devices == handle) {
                    h = usb_open(dev);
                    break;
                }
                devices++;
            }
        }
    }

    *ret_handle = h;
    if (h == 0)
        return RET_FALSE;

    int ret = usb_set_configuration(h, 2); //1-fs, 2-hs
    if (ret != 0) {
        int ret1 = usb_set_configuration(h, 1); //not found... EG5000? try 1!
        if (ret1 != 0) {
            qWarning("libusb_open: usb_set_configuration error %d %d", ret, ret1);
            return RET_FALSE;
        }
    }

    ret = usb_claim_interface(h, 0);
    if (ret != 0) {
        qWarning("libusb_open: usb_claim_interface error %d", ret);
        return RET_FALSE;
    }
    return RET_TRUE;
}

BYTE libusb_close_(void *handle)
{
    usb_dev_handle *h = (usb_dev_handle *)handle;

    int ret = usb_release_interface(h, 0);
    if (ret != 0) {
        qWarning("usb_close: usb_release_interface error %d", ret);
        return RET_FALSE;
    }

    ret = usb_close(h);
    if (ret != 0) {
        qWarning("usb_close: usb_close error %d", ret);
        return RET_FALSE;
    }
    return RET_TRUE;
}

BYTE libusb_clear(void *)
{
    //usb_dev_handle *h = (usb_dev_handle *)handle;
    return RET_TRUE;
}

static WORD timeout = 1000;
BYTE libusb_set_timeout(void *, WORD _timeout)
{
    timeout = _timeout;
    return RET_TRUE;
}

//write
BYTE libusb_write(void *handle, const void *buffer, WORD data_size)
{
    usb_dev_handle *h = (usb_dev_handle *)handle;

    int sent = 0;
    int last_sent;

    while (sent < data_size) {
        last_sent = usb_bulk_write(h, 0x01, (char*)buffer + sent, data_size - sent, timeout);
        if (last_sent == -5) //EIO
            usb_resetep(h, 0x01); //clear USBD_STATUS_XACT_ERROR
        //if (last_sent == -116) //timeout
            //continue;

        if (last_sent < 0) {
            qWarning("libusb_write: error %d, %d/%d sent", last_sent, sent, data_size);
            return RET_FALSE;
        }
        if (last_sent == 0) {
            qWarning("libusb_write: sent 0 bytes, returning");
            return RET_FALSE;
        }
        sent += last_sent;
    }

    return RET_TRUE;
}

//read
BYTE libusb_read(void *handle, void *buffer, WORD buf_size, DWORD *p_bytes_read)
{
    usb_dev_handle *h = (usb_dev_handle *)handle;

    *p_bytes_read = 0;

    int read = usb_bulk_read(h, 0x81, (char*)buffer, buf_size, timeout);
    if (read == -5) //EIO
        usb_resetep(h, 0x81); //clear USBD_STATUS_XACT_ERROR
    if (read == -116) //timeout
        return RET_TRUE_ERR;
    if (read < 0) {
        qWarning("libusb_read: error %d %d", read, buf_size);
        return RET_FALSE;
    }

    *p_bytes_read = read;
    return RET_TRUE;
}

#if defined __cplusplus
} // extern "C" {
#endif
