#pragma once

#include "2d/plot_2dtypes.h"
#include "core/plot_core.h"
#include "widgets/plot_widgets.h"

#include <QComboBox>
#include <QMimeData>
#include <QHBoxLayout>

#include "widgets/colorrect2.h"

namespace plot {

class TComboBox: public QComboBox //todo move from file
{
    void hidePopup() override { QComboBox::hidePopup(); emit activated(currentIndex()); }
    void showPopup() override { QComboBox::showPopup(); emit highlighted(currentIndex()); }
};

class T2dPlotControl: public TCommonPlotControl //todo move from file
{
    void CreateActionsAndMenus();

public:
    void GUICreate();
    ~T2dPlotControl() override;
    void ResetAllFilters();

    QPushButton  *BResetMCCurve;

    QComboBox    *CBChooseActiveCurve;
    QPushButton  *BAutoScaleX;
    QPushButton  *BAutoScaleY;
    QActionGroup *AGFitOrders;

    QToolButton  *TBCurveColour;
    QToolButton  *TBCurveThickness;
    QActionGroup *AGCurveColours;
    QActionGroup *AGCurveThickness;

};

class T2dMime : public QMimeData
{
    Q_OBJECT
public:
    TCurvesData data;
};

class TPure2dPlotPrivate;
class T2dPlot;
class TwoDOverlayButtons;
namespace math { class TMathProcessing; }
class TPure2dPlot;

class T2dPlot: public QWidget
{
    Q_OBJECT
public slots:
    void on_BFixX_toggled();
    void on_BFixY_toggled();
    void on_FontSize_changed();
    void on_CurveThickness_changed();

    void on_BSave_clicked();
    void on_FitOrder_changed();
    void setFitOrder(int i);
    void on_BMarkers_clicked(bool);
    void UpdateAllMarkersInfo();
    void on_BAutoScale_toggled();
    void on_CurveColour_changed();
    void on_BSavePlotPixmap_clicked();
    void on_BDetailedMarkersInfo_clicked();
    void on_CBChooseActiveCurve_highlighted(int Index);
    void on_SyncTimer_timeout();
    void on_CBChooseActiveCurve_activated(int Index);
    void sync_active_curve(int);
    void setFiltering(bool filter);
    void resetAllRilters();
    void on_BSelectByRect_clicked();
    void on_BCancel_clicked();
    void StartMCCurveConstructing();
    void on_BApply_clicked();
    void SetSelectionModeEnabled(bool Enabled);
    void HideSelectionIfAny();
    void SetTitle(const QString &Ttl);
signals:
    void AddDataToContainer(plot::TCurvesDataOwner, plot::TMetaStorage);
    void ManualConstructingDone();
    void CurveColourChanged(QColor Colour, int AC);


public:
    ~T2dPlot() override;
    void GUICreate(bool child = false, bool vertical = false);
    void ConnectPlotControlSlots();
    void FillCBChooseActiveCurve();
    void PreparePlotControlForMC(bool MCStarts);
    void RestoreEnvironmentState();
    bool event(QEvent *Event) override;
    void setCurveColor(QColor color, bool emitEvent = true);
    void savePlot(TCurvesDataOwner &src, TMetaStorage &attributes);

    QColor lineColors[4] = {Qt::black};
    int lineWidth = 1;

    QHBoxLayout *MainLayout;
    QObject *Parent;
    QTimer *SyncTimer;
    TPure2dPlot *Pure2dPlot;
    TMarkersInfo *MarkersInfo;
    T2dPlotControl *PlotControl;
    bool showPlotControl = true;
    T2dPlot *depent2dPlot; //vertical/slave-dublicate plot
    T2dPlot *master2dPlot;

    TwoDOverlayButtons *overlayButtons = nullptr;


    T2dPlot(QObject *Parent = nullptr, bool child = false, QWidget *wparent = nullptr, bool vertical = false, TwoDOverlayButtons *ovrlBtns = nullptr);
    void Refresh();
    void CreateOverlayButtons();

    void SetGridVisible(const bool Visible);
    void SetAutoScalingEnabled(const bool AutoScaling);
    QImage MakePreview();
};

}
