#include "plot_2dpure.h"
#include "plot_2dplot_p.h"
#include "math/plot_MathProcessing.h"
#include <qwt_plot_directpainter.h>
#include <qwt_plot_layout.h>
#include <qwt_plot_marker.h>
#include <qwt_plot_canvas.h>
#include <algorithm>
#include <limits>

#include <QApplication>
#include <QDesktopWidget>
#include <QtEvents>
#include <QColorDialog>
#include <QDrag>
#include <QTimer>
#include <QFileDialog>
#include <QMessageBox>
#include <QSettings>
#include <QGestureEvent>
#include <QDebug>
#include <QWeakPointer>

namespace plot {

void TCalcLimitsByFrameContent::operator ()(TInfoForAllPlots *InfoForAll, TCurves *Src, const int ApplyTo, int N)
{
    PlotInfo = InfoForAll;
    Curves = Src;
    MinPlotsX = PlotInfo->CurrentXLimits[0];
    MaxPlotsX = PlotInfo->CurrentXLimits[1];
    MinPlotsY = PlotInfo->CurrentYLimits[0];
    MaxPlotsY = PlotInfo->CurrentYLimits[1];

    Clear();
    CalcScrLimits(N);

    if (ApplyTo == TPlotEnums::A_Both || ApplyTo == TPlotEnums::A_XAxis) {
        PlotInfo->CurrentXLimits[0] = min(XMins);
        PlotInfo->CurrentXLimits[1] = max(XMaxs);
    }

    if (ApplyTo == TPlotEnums::A_Both || ApplyTo == TPlotEnums::A_YAxis) {
        PlotInfo->CurrentYLimits[0] = min(YMins);
        PlotInfo->CurrentYLimits[1] = max(YMaxs);
    }
}
inline void TCalcLimitsByFrameContent::Clear()
{
    XMins.clear(); XMaxs.clear();
    YMins.clear(); YMaxs.clear();
}
void TCalcLimitsByFrameContent::CalcScrLimits(int N)
{
    if (N < 0)
        for (auto QwtCurve: Curves->QwtCurves)
            CalcScrLimits(*QwtCurve);
    else
        CalcScrLimits(*Curves->QwtCurves.at(N));
}
inline bool TCalcLimitsByFrameContent::IsIn(const QwtPlotCurve &CurveData, const int Index) const
{
    return CurveData.sample(Index).x() >= MinPlotsX && CurveData.sample(Index).x() <= MaxPlotsX;
}
void TCalcLimitsByFrameContent::CalcScrLimits(const QwtPlotCurve &CurveData)
{
    //TODO in-place
    QVector<double> Y;
    for (int i = 0; i < CurveData.dataSize(); i++) {
        if (IsIn(CurveData, i)) {
            Y.push_back(CurveData.sample(i).y());
        }
    }

    if (CurveData.dataSize() == 0)
        return; //todo supposed to be an error

    XMins.push_back(CurveData.sample(0).x()); // min(X);
    XMaxs.push_back(CurveData.sample(CurveData.dataSize() - 1).x()); // max(X);
    QPair<double, double> _minmax = minmax(Y);
    YMins.push_back(_minmax.first);
    YMaxs.push_back(_minmax.second);
}

void TInfoFor2dPlot::UpdateAxisLegends()
{
    if (AutoLegendEnabled)
    {
        XLegend = XAxisUnit.Legend;
        YLegend = YAxisUnit.Legend;
    }
}

void TPure2dPlot::on_BExpandSelectRect_clicked()
{
    ExpandSelectionRect();
    ReDrawSelectedRectangle(TSelectEvent::SRO_Mouse);
}

TPure2dPlotPrivate::TPure2dPlotPrivate()
{
    q_ptr = nullptr;
    AutoScaleEnabled = true;
    ZoomBySelection = true;
    SelectedPointForMC = -1;
    State = TPure2dPlot::Mode_Normal;
}
QPointF TPure2dPlotPrivate::FindAltitude(const QPointF &Click, const QPointF &Closest, const QPointF &p) const
{
    QPointF Clk = Click - Closest;
    QPointF P = p - Closest;
    double Alpha = (Clk.x()*P.x() + Clk.y()*P.y()) / (P.x()*P.x() + P.y()*P.y());
    return P*Alpha + Closest;
}
bool TPure2dPlotPrivate::PointBelongToLine(const QPointF &P, const QLineF &Line) const
{
    if (P.x() > qMax(Line.x1(), Line.x2()) || P.x() < qMin(Line.x1(), Line.x2()) ||
       P.y() > qMax(Line.y1(), Line.y2()) || P.y() < qMin(Line.y1(), Line.y2())) return false;
    else return true;
}
QPointF TPure2dPlotPrivate::GetIntersectPoint(const QwtPlotCurve *Curve, const QPoint &ScrPos, int &Idx) const
{
    if (!Curve) return QPointF();
    if (Curve->dataSize() < 2) return QPointF();

    Q_Q(const TPure2dPlot);
    double minv = std::numeric_limits<double>::max();
    int minidx = -1;
    for (int i = 0; i < Curve->dataSize() - 1; i++)
    {
        QPointF p1 = q->PlotToScreen(Curve->sample(i));
        QPointF p2 = q->PlotToScreen(Curve->sample(i+1));
        const QPointF AltIntersection = FindAltitude(ScrPos, p1, p2);
        if (PointBelongToLine(AltIntersection, QLineF(p1, p2))) {
            double v = GetDist(ScrPos, AltIntersection);
            if (v < minv) {
                minv = v;
                minidx = i;
            }
        }
    }
    if (minidx == -1) return QPointF();

    Idx = minidx;
    QPointF p1 = q->PlotToScreen(Curve->sample(Idx));
    QPointF p2 = q->PlotToScreen(Curve->sample(Idx+1));
    QPointF p3 = FindAltitude(ScrPos, p1, p2);
    return q->ScreenToPlot(p3.toPoint());
}

bool TPure2dPlot::isMathReady()
{
    return !MathProcessing->isRunning();
}
TPure2dPlot::~TPure2dPlot()
{
    delete d_ptr;
    delete MathProcessing;
}
TPure2dPlot::TPure2dPlot(bool _vertical, QObject *_Parent, QWidget *parent) : TCommonPlot(parent), TCurves(QPlot), d_ptr(new TPure2dPlotPrivate)
{
    setMouseTracking(true);
    incRefreshGuard = false;
    UseMathOnly = false;
    tParent = static_cast<plot::T2dPlot*>(parent);
    this->vertical = _vertical;
    Parent = _Parent;

    Q_D(TPure2dPlot);
    d->q_ptr = this;
    FitOrder = TPlotEnums::FO_None;

    MathProcessing = new math::TMathProcessing(&MathData, vertical);
    MathProcessing->MCCurve = MCCurve;
    connect(MathProcessing, SIGNAL(finished()), this, SLOT(s_Refresh()), Qt::QueuedConnection);

    setAcceptDrops(true);
    ((QWidget *)QPlot->titleLabel())->setToolTip(tr("Drag & Drop as data to plots, as image"));
}

void TPure2dPlot::paintEvent(QPaintEvent *e)
{
    TCommonPlot::paintEvent(e);
    incRefreshGuard = true;
}
void TPure2dPlot::RefreshInc()
{
    if (blockRefresh)
        return;

    Q_D(TPure2dPlot);
    QwtPlotDirectPainter p;
    bool full_refresh = d->AutoScaleEnabled ||
            (FitOrder != TPlotEnums::FO_None && isFiltering ) ||
            MathProcessing->Work[plot::math::TMathProcessing::Scale2d].arg.toInt() >= 0;
    incRefreshGuard = false;
    for (int i = 0; i < QwtCurves.size() && !full_refresh && !incRefreshGuard; i++) {
        if (PaintData.at(i)) {
            int from = LastPaintSize.at(i);
            int size = PaintData.at(i).data()->size();
            LastPaintSize[i] = size;
            if (from > size) {
                full_refresh = true;
                break;
            }
            if (from != size)
                p.drawSeries(QwtCurves.at(i), qMax(0, from-1), size);
        }
    }
    if (full_refresh)
        Refresh();
    else if (incRefreshGuard)
        //QPlot->canvas()->replot(); ///Replot();
        QPlot->canvas()->update(); //update including frame, not only contentrect
}

void TPure2dPlot::Refresh()
{
    if (blockRefresh)
        return;

    if (vertical && tParent->master2dPlot)
    {
        CurrentXLimits[0] = tParent->master2dPlot->Pure2dPlot->CurrentYLimits[0];
        CurrentXLimits[1] = tParent->master2dPlot->Pure2dPlot->CurrentYLimits[1];
        CurrentYLimits[0] = tParent->master2dPlot->Pure2dPlot->CurrentXLimits[0];
        CurrentYLimits[1] = tParent->master2dPlot->Pure2dPlot->CurrentXLimits[1];
        YAxisUnit = tParent->master2dPlot->Pure2dPlot->XAxisUnit;
        XAxisUnit = tParent->master2dPlot->Pure2dPlot->YAxisUnit;
    }

    if ((FitOrder == TPlotEnums::FO_None || !isFiltering) &&
        MathProcessing->Work[plot::math::TMathProcessing::Scale2d].arg.toInt() < 0 &&
        !UseMathOnly)
    {
        PaintData = SourceData;

        ReSyncDataAndQwtCurves();

        MathProcessing->wait();
        MathData.clear();

        s_Refresh();
    }
    else if (!MathProcessing->isRunning())
    {
        MathData.clear();
        ResizeOwner(MathData, SourceData.size());
        PaintData = getUnLock(MathData);

        for (int i = 0; i < SourceData.size(); i++)
            if (SourceData[i]) {
                *MathData[i] = *SourceData[i].data();
                MathData[i]->detach();
            }

        ReSyncDataAndQwtCurves();

        if (FitOrder != TPlotEnums::FO_None && isFiltering)
            MathProcessing->DoFit(FitOrder);

        if (MathProcessing->Work[plot::math::TMathProcessing::Scale2d].arg.toInt() >= 0)
            MathProcessing->DoScale(MathProcessing->Work[plot::math::TMathProcessing::Scale2d].arg.toInt());

        MathProcessing->start();
    }
}
void TPure2dPlot::s_Refresh()
{
    if (blockRefresh)
        return;

    ReSetAxisScales();
    ReAssignPlotOptions();
    Replot();

    PlotWasChanged();
    emit MathUpdated();
}

void TPure2dPlot::setVertLineVisible(int N, bool v)
{
    VertLines[N]->setVisible(v);
    Replot();
}
void TPure2dPlot::MoveVertLine(double XPos, int N)
{
    VertLines[N]->setXValue(XPos);
    Replot();
}
void TPure2dPlot::SetVertLinePen(QPen &Pen, int N)
{
    VertLines[N]->setLinePen(Pen);
    Replot();
}
void TPure2dPlot::SetNumOfVertLines(int N)
{
    VertLines.resize(N);

    for (auto &VertLne: VertLines) {
        VertLne = new QwtPlotMarker;
        VertLne->setXValue(0);
        VertLne->setLinePen(QPen(QColor(0, 0, 255), 0, Qt::SolidLine));
        VertLne->setLineStyle(QwtPlotMarker::VLine);
        VertLne->attach(QPlot);
        VertLne->hide();
    }
}
void TPure2dPlot::DefineClickInfo(const QPoint &Pos)
{
    Q_D(TPure2dPlot);
    if (d->State == TPure2dPlot::Mode_ManualConstruct) {
        d->SelectedPointForMC = -1;

        if (MCCurve->dataSize() <= 1)
            return;

        if ((d->SelectedPointForMC = MCCurve->closestPoint(Pos, nullptr)) == -1)
            return;

#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)) && !defined(_ADROID)
    double scale = qApp->desktop()->logicalDpiX();
#else
    double scale = qApp->desktop()->physicalDpiX();
#endif

    int Range = 1.5*qMax<int>(20, 0.25 * scale);
        if ((PlotToScreen(MCCurve->sample(d->SelectedPointForMC)) - Pos).manhattanLength() > Range)
        {
            int Idx;
            QPointF p2 = d->GetIntersectPoint(MCCurve, Pos, Idx);
            if ((PlotToScreen(p2) - Pos).manhattanLength() < Range)
                AddPointToMCCurve(Idx, p2);
            else { d->SelectedPointForMC = -1; TCommonPlot::DefineClickInfo(Pos); }
        }
    }
    else TCommonPlot::DefineClickInfo(Pos);
}
void TPure2dPlot::dragEnterEvent(QDragEnterEvent *event)
{
    const T2dMime *data = qobject_cast<const T2dMime *>(event->mimeData());
    if (data && event->source() != this)
        event->acceptProposedAction();
}
void TPure2dPlot::dropEvent(QDropEvent *event)
{
    const T2dMime *data = qobject_cast<const T2dMime *>(event->mimeData());
    if (!data || event->source() == this) return;
    SourceData << data->data;
    Refresh();
}
bool TPure2dPlot::eventFilter(QObject *Object, QEvent *Event)
{
    auto me = static_cast<QMouseEvent *>(Event);
    if (me != nullptr)
        emit mouseEvent(me, this);

    if (Event->type() == QEvent::Resize)
        emit resized(size(), this);

    if (!isEnabled()) {
        Action = ExternalAct;
    } else
    if (Enabled)
    {
        Range PlotXLimits, PlotYLimits;
        PlotXLimits[0] = qMin(DefaultXLimits[0], PlotsXLimits[0]);
        PlotXLimits[1] = qMax(DefaultXLimits[1], PlotsXLimits[1]);
        PlotYLimits[0] = qMin(DefaultYLimits[0], PlotsYLimits[0]);
        PlotYLimits[1] = qMax(DefaultYLimits[1], PlotsYLimits[1]);

        PlotXLimits[0] -= (PlotXLimits[1] - PlotXLimits[0]);
        PlotXLimits[1] += (PlotXLimits[1] - PlotXLimits[0])/2.;
        PlotYLimits[0] -= (PlotYLimits[1] - PlotYLimits[0]);
        PlotYLimits[1] += (PlotYLimits[1] - PlotYLimits[0])/2.;

        if (Event->type() == QEvent::Wheel) {
            QPointF Pos = mapToCanvas(((QWheelEvent *)Event)->pos());
            Pos = ScreenToPlot(Pos.toPoint());

            auto pos2 = mapFromGlobal(QCursor::pos());
            BFixXIsChecked = QPlot->plotLayout()->scaleRect(QwtPlot::yLeft).contains(pos2);
            BFixYIsChecked = QPlot->plotLayout()->scaleRect(QwtPlot::xBottom).contains(pos2);

            ZoomPlot(Pos, ((QWheelEvent *)Event)->delta(), &PlotXLimits, &PlotYLimits);
        } else if (Event->type() == QEvent::MouseButtonPress) {
            QPoint Pos = mapToCanvas(me->pos());
            DefineClickInfo(Pos);

            if (me->button() == Qt::LeftButton)
                BFixXIsChecked = true;
            else if (me->button() == Qt::RightButton)
                BFixYIsChecked = true;

            if (KeyFixXPressed)
                BFixXIsChecked = true;
            if (KeyFixYPressed)
                BFixYIsChecked = true;

            if (Action == Panning && QPlot->plotLayout()->titleRect().contains(me->pos())) {
                Action = ExternalAct;

                QDrag *drag = new QDrag(this);
                T2dMime *mimeData = new T2dMime;
                mimeData->data = ActiveCurve < 0 || ActiveCurve >= SourceData.size() ?
                            SourceData : (TCurvesData() << SourceData.at(ActiveCurve));
                mimeData->setImageData(QPixmap::grabWidget(this).toImage());
                drag->setMimeData(mimeData);
                drag->setPixmap(QPixmap::fromImage(MakePreview()));
                drag->exec(Qt::CopyAction);
            }
        } else if (Event->type() == QEvent::MouseButtonRelease)
        {
            Action = ExternalAct;
            BFixXIsChecked = false;
            BFixYIsChecked = false;
        }
        else if (Event->type() == QEvent::MouseMove) {
            QPoint Pos = mapToCanvas(me->pos());

            Q_D(TPure2dPlot);
            if (d->State == TPure2dPlot::Mode_ManualConstruct && d->SelectedPointForMC != -1)
                MoveSelectedPointInMCCurve(Pos);
            else if (Action == TCommonPlot::MoveC)
                MoveMarker(Pos.x(), Pos.y(), false);
            else if (Action == MoveA || Action == MoveB)
                TraceCurve(Pos, ActiveCurve);
            else if (Action == Panning) {
                double kx = (ScreenXToPlotX(20) - ScreenXToPlotX(0)) / 20.;
                double ky = (ScreenYToPlotY(20) - ScreenYToPlotY(0)) / 20.;

                QPointF Delta((Pos.x() - ClickPos.x())*kx,
                              (Pos.y() - ClickPos.y())*ky);
                PanPlot(Delta, &PlotXLimits, &PlotYLimits);
            }
        }

        if (Event->type() == QEvent::MouseButtonPress)
        {
            if (static_cast<T2dPlot*>(parent())->master2dPlot != nullptr)
                emit static_cast<T2dPlot*>(parent())->master2dPlot->Pure2dPlot->ShowPlotControl();
            else
                emit ShowPlotControl();
        }

        if (Event->type() == QEvent::Gesture) {
            return gestureEvent(static_cast<QGestureEvent*>(Event));
        }
    }


    return QWidget::eventFilter(Object, Event);
}
void TPure2dPlot::mouseDoubleClickEvent(QMouseEvent * e)
{
    emit doubleClicked();

    QWidget::mouseDoubleClickEvent(e);
}
bool TPure2dPlot::gestureEvent(QGestureEvent *event)
{
    if (event->gesture(Qt::TapAndHoldGesture))
        mouseDoubleClickEvent(nullptr);

    return true;
}
void TPure2dPlot::MoveSelectedPointInMCCurve(const QPoint &Pos)
{
    Q_D(TPure2dPlot);

    int sz = MCCurve->dataSize();
    if (sz <= 1)
        return;
//((QwtPointSeriesData *)MCCurve->data())->samples()[d->SelectedPointForMC] = Pos;

    auto x = new double [sz];
    auto y = new double [sz];

    for (int i = 0; i < sz; i++) {
        if (i == d->SelectedPointForMC) {
            x[i] = ScreenXToPlotX(Pos.x());
            y[i] = ScreenYToPlotY(Pos.y());
        } else {
            x[i] = MCCurve->sample(i).x();
            y[i] = MCCurve->sample(i).y();
        }
    }

    MCCurve->setSamples(x, y, sz);

    delete [] x;
    delete [] y;

    Replot();
}
void TPure2dPlot::SetActiveCurve(int Active)
{
    TCurves::SetActiveCurve(Active);
    emit sync_active_curve(ActiveCurve);
}
void TPure2dPlot::TraceCurve(const QPoint &Pos, int NOfCurve)
{
    if (NOfCurve < 0)
        MoveMarker(Pos.x(), Pos.y(), false);
    else {
        double x = 0., y = 0.;
        GetClosestPoint(&x, &y, Pos, NOfCurve);
        MoveMarker(x, y, true);
    }
}

void TPure2dPlot::on_BAutoScaleX_clicked()
{
    //if (tParent->isHidden())
    //    return;


    if (vertical)
    {
        //LimitsStretcher(this, this, TPlotEnums::A_YAxis, ActiveCurve);
        //SetCurrentYScales();
        TPlotCore::ReSetAxisScales(TPlotEnums::A_XAxis);
    }
    else
    {
        LimitsStretcher(this, this, TPlotEnums::A_XAxis, ActiveCurve);
        SetCurrentXScales();
    }

    HandleAxisMultiplier();
    ReAssignPlotOptions();
    Replot();
}
void TPure2dPlot::on_BAutoScaleY_clicked()
{
    LimitsStretcher(this, this, TPlotEnums::A_YAxis, ActiveCurve);
    SetCurrentYScales();
    HandleAxisMultiplier();
    ReAssignPlotOptions();
    Replot();
}
void TPure2dPlot::ReSetAxisScales()
{
    Q_D(TPure2dPlot);
    if (d->AutoScaleEnabled)
    {
        if (ActiveCurve < PaintData.size()) {
            QVector<double> MaxX, MaxY, MinX, MinY;
            DefineLimits(MaxX, MaxY, MinX, MinY, ActiveCurve);
            if (MinX.size() && MaxX.size()) {
                Range TmpXLimits;
                TmpXLimits[0] = *std::min_element(MinX.begin(), MinX.end());
                TmpXLimits[1] = *std::max_element(MaxX.begin(), MaxX.end());

                if (TmpXLimits[0] >= std::numeric_limits<double>::max() &&
                    TmpXLimits[1] <= -std::numeric_limits<double>::max())
                {
                    HandleAxisMultiplier();
                    return;
                }

                HandleZeroDiff(TmpXLimits[1], TmpXLimits[0]);
                if (vertical && CurrentXLimits[1] < 1.0)
                    PlotsYLimits = TmpXLimits;
                else
                    PlotsXLimits = TmpXLimits;
            }

            if (MinY.size() && MaxY.size()) {
                Range TmpYLimits;
                TmpYLimits[0] = *std::min_element(MinY.begin(), MinY.end());
                TmpYLimits[1] = *std::max_element(MaxY.begin(), MaxY.end());

                if (TmpYLimits[0] >= std::numeric_limits<double>::max() &&
                    TmpYLimits[1] <= -std::numeric_limits<double>::max())
                {
                    HandleAxisMultiplier();
                    return;
                }

                HandleZeroDiff(TmpYLimits[1], TmpYLimits[0]);
                if (vertical && CurrentYLimits[1] < 1.0)
                    PlotsXLimits = TmpYLimits;
                else
                    PlotsYLimits = TmpYLimits;
            }
        }

        if (vertical)
        {
            tParent->master2dPlot->Pure2dPlot->ReSetAxisScales();
            tParent->master2dPlot->Pure2dPlot->Replot();
            PlotsXLimits[0] = tParent->master2dPlot->Pure2dPlot->PlotsYLimits[0];
            PlotsXLimits[1] = tParent->master2dPlot->Pure2dPlot->PlotsYLimits[1];
            PlotsYLimits[0] = tParent->master2dPlot->Pure2dPlot->PlotsXLimits[0];
            PlotsYLimits[1] = tParent->master2dPlot->Pure2dPlot->PlotsXLimits[1];

            CurrentXLimits[0] = tParent->master2dPlot->Pure2dPlot->CurrentYLimits[0];
            CurrentXLimits[1] = tParent->master2dPlot->Pure2dPlot->CurrentYLimits[1];
            CurrentYLimits[0] = tParent->master2dPlot->Pure2dPlot->CurrentXLimits[0];
            CurrentYLimits[1] = tParent->master2dPlot->Pure2dPlot->CurrentXLimits[1];

            //qDebug()<<1<<"---"<<PlotsXLimits[0]<<PlotsXLimits[1];

            TPlotCore::ReSetAxisScales();

            HandleAxisMultiplier();
        }

        TPlotCore::ReSetAxisScales();
    }
    else
        HandleAxisMultiplier();
}
void TPure2dPlot::SetFitOrder(int Order)
{
    FitOrder = Order;
}

bool TPure2dPlot::IsAutoScaleEnabled()
{
    Q_D(TPure2dPlot);
    return d->AutoScaleEnabled;
}

void TPure2dPlot::SetAutoScaleEnabled(bool enabled, bool refresh)
{
    Q_D(TPure2dPlot);
    d->AutoScaleEnabled = enabled;
    //TODO plotslimits = defaultplotslimits
    if (refresh)
        Refresh();
}
void TPure2dPlot::SelectionApplied()
{
    Q_D(TPure2dPlot);
    if (d->ZoomBySelection)
    {
        QRectF Rect = GetSelectedRect();
        SetCurrentXScales(Rect.x(), Rect.right());
        SetCurrentYScales(Rect.y(), Rect.bottom());
        Replot();
    }
}
void TPure2dPlot::EnableSelectMode(const bool st)
{
    Q_D(TPure2dPlot);
    QPlot->setVisible(!st);
    SetSelectionModeEnabled(st);
    setFocus(Qt::MouseFocusReason);
    d->State = st ? Mode_Select : Mode_Normal;
}

void TPure2dPlot::clearSouceData()
{
    for(auto &dt : SourceData)
    {
        if (dt.data())
            dt.data()->clear();
    }
    SourceData.clear();
}

TCurvesData* TPure2dPlot::cloneSourceData(QPen **pen)
{
    return cloneSourceData(SourceData, pen);
}

TCurvesData* TPure2dPlot::cloneSourceData(const TCurvesData &data, QPen **pen)
{
    TCurvesData* c = new TCurvesData;
    int i = 0;
    for (auto &d:data)
    {
        if (d.data())
        {
            TCurveData* curve = d.data()->clone(pen ? pen[i]:nullptr);
            QWeakPointer<TCurveData> weak(*(new QSharedPointer<TCurveData>(curve)));
            c->append(weak);
        }
        else
        {
            QWeakPointer<TCurveData> weak(*(new QSharedPointer<TCurveData>(d.data())));
            c->append(weak);
        }
        i++;
    }

    return c;
}


void TPure2dPlot::SelectRectChanged(const QRectF &Rect)
{
    QRectF tRect;
    tRect.setX(Rect.y());
    tRect.setY(Rect.x());
    tRect.setWidth(Rect.height());
    tRect.setHeight(Rect.width());

    if (isVisible())
    {
        if (Parent) QCoreApplication::postEvent(Parent,
                      new TSelectEvent(vertical ? tRect : Rect, TSelectEvent::St_SelectChanged,
                                       (TSelectEvent::TSelectRectChangedOrigins)SelectRectEventOrigin));
    }
    SelectRectEventOrigin = TSelectEvent::SRO_Unknown;
}


void TPure2dPlot::HandleAxisMultiplier()
{
    XAxisUnit.UpdateLegend(CurrentXLimits[0], CurrentXLimits[1]);
    YAxisUnit.UpdateLegend(CurrentYLimits[0], CurrentYLimits[1]);
    UpdateAxisLegends();
}
void TPure2dPlot::UpdateAxisLegends()
{
    UpdateLabels();
    TInfoFor2dPlot::UpdateAxisLegends();

    QPlot->axisScaleDraw(QwtPlot::xBottom)->Multiplier = AutoLegendEnabled ? XAxisUnit.Multiplier : 1.;
    QPlot->axisScaleDraw(QwtPlot::yLeft)->Multiplier = AutoLegendEnabled ? YAxisUnit.Multiplier : 1.;
}
void TPure2dPlot::UpdateLabels(bool IsDetailedInfoVisible)
{
    Q_D(TPure2dPlot);
    if (vertical)
    {
        d->AMarkerInfo(AutoLegendEnabled, YAxisUnit, XAxisUnit, MarkerA->yValue(), MarkerA->xValue());
        d->BMarkerInfo(AutoLegendEnabled, YAxisUnit, XAxisUnit, MarkerB->yValue(), MarkerB->xValue());
    }
    else
    {
        d->AMarkerInfo(AutoLegendEnabled, XAxisUnit, YAxisUnit, MarkerA->xValue(), MarkerA->yValue());
        d->BMarkerInfo(AutoLegendEnabled, XAxisUnit, YAxisUnit, MarkerB->xValue(), MarkerB->yValue());
    }
    TValueInfoCreator difx, dify;

    QString AText, BText, MText;
    if (IsDetailedInfoVisible) { AText = "A"; BText = "B"; }
    else
    {
        AText = d->AMarkerInfo.FormUniteString();
        BText = d->BMarkerInfo.FormUniteString();

        if (vertical)
        {
            dify(AutoLegendEnabled, XAxisUnit, d->BMarkerInfo.XInfo.SrcValue - d->AMarkerInfo.XInfo.SrcValue);
            difx(AutoLegendEnabled, YAxisUnit, d->BMarkerInfo.YInfo.SrcValue - d->AMarkerInfo.YInfo.SrcValue);
            MText = QString(QChar(0x2206)) + "X = " + dify.Value + dify.Unit + '\n' +
                    QString(QChar(0x2206)) + "Y = " + difx.Value + difx.Unit;
        }
        else
        {
            difx(AutoLegendEnabled, XAxisUnit, d->BMarkerInfo.XInfo.SrcValue - d->AMarkerInfo.XInfo.SrcValue);
            dify(AutoLegendEnabled, YAxisUnit, d->BMarkerInfo.YInfo.SrcValue - d->AMarkerInfo.YInfo.SrcValue);
            MText = QString(QChar(0x2206)) + "X = " + difx.Value + difx.Unit + '\n' +
                    QString(QChar(0x2206)) + "Y = " + dify.Value + dify.Unit;
        }
    }

    QwtText QwtAText(AText),
            QwtBText(BText),
            QwtMText(MText);

    QFont font = qApp->font();
    font.setPointSize(FontSize);
    QwtAText.setFont(font);
    QwtBText.setFont(font);
    QwtMText.setFont(font);

    QwtAText.setColor(QColor(190, 255, 190));
    QwtBText.setColor(QColor(190, 255, 190));
    QwtMText.setColor(QColor(190, 255, 190));

    MarkerA->setLabel(QwtAText);
    MarkerB->setLabel(QwtBText);
    MarkerM->setLabel(QwtMText);
}
void TPure2dPlot::ResetMCCurve()
{
    int idx = qMax(0, ActiveCurve);
    if (!QwtCurves.size() || QwtCurves.at(idx)->dataSize() <= 1)
        return;

	auto curve = QwtCurves.at(idx);
    MCCurve->show();
    MCCurve->setSamples(QVector<QPointF>() <<
		curve->sample(0) <<
		curve->sample(
			curve->dataSize() - 1));

    Replot();
}
void TPure2dPlot::StartMCMode()
{
    Q_D(TPure2dPlot);
    d->State = TPure2dPlot::Mode_ManualConstruct;

    if (MCCurve->dataSize()) {
        MCCurve->show();
        Replot();
    }
    else
        ResetMCCurve();
}
void TPure2dPlot::SetZoomBySelection(const bool State)
{
    Q_D(TPure2dPlot);
    d->ZoomBySelection = State;
}
void TPure2dPlot::AddPointToMCCurve(const int After, const QPointF &p2)
{
    Q_D(TPure2dPlot);

    int i = 0;
    double *x = new double [MCCurve->dataSize() + 1], *y = new double [MCCurve->dataSize() + 1];

    //TODO QVector.insert?
    for (; i <= After; i++) {
		auto sample = MCCurve->sample(i);
        x[i] = sample.x();
        y[i] = sample.y();
    }
    x[i] = p2.x();
    y[i] = p2.y();
    d->SelectedPointForMC = i;

    for (; i < MCCurve->dataSize(); i++) {
		auto sample = MCCurve->sample(i);
        x[i + 1] = sample.x();
        y[i + 1] = sample.y();
    }

    MCCurve->setSamples(x, y, MCCurve->dataSize() + 1);
    delete [] x; delete [] y;

    Replot();
}
void TPure2dPlot::ExitMCMode()
{
    tParent->PreparePlotControlForMC(false);
    MCCurve->hide();
    Replot();

    Q_D(TPure2dPlot);
    d->State = TPure2dPlot::Mode_Normal;
}

TCurveData TPure2dPlot::GetMCCurve() const
{
    TCurveData Result;

    for (int i = 0; i < MCCurve->dataSize(); i++)
        Result.AddPoint(MCCurve->sample(i).x(), MCCurve->sample(i).y());

    return Result;
}

}
