#ifndef PLOT_2DPURE_H
#define PLOT_2DPURE_H

#include <QObject>
#include "2d/plot_2dtypes.h"
#include "core/plot_core.h"
#include "widgets/plot_widgets.h"

namespace plot {

namespace math { class TMathProcessing; }
class TPure2dPlotPrivate;

class TCalcLimitsByFrameContent
{
    inline void Clear();
    void CalcScrLimits(int N);
    void CalcScrLimits(const QwtPlotCurve &CurveData);
    inline bool IsIn(const QwtPlotCurve &CurveData, const int Index) const;

    TCurves *Curves;
    TInfoForAllPlots *PlotInfo;
    QVector<double> XMins, YMins, XMaxs, YMaxs;
    double MinPlotsX, MaxPlotsX, MinPlotsY, MaxPlotsY;
public:
    void operator ()(TInfoForAllPlots *InfoForAll, TCurves *Src, const int ApplyTo, int N = -1);
};

class TInfoFor2dPlot : virtual public TInfoForAllPlots
{
public:
    TSIUnit XAxisUnit;
    TSIUnit YAxisUnit;

    void UpdateAxisLegends() override;
};

class T2dPlot;

class TPure2dPlot : public TCommonPlot, public TInfoFor2dPlot, public TCurves
{
    Q_OBJECT

    friend class T2dPlot;

public slots:
    void s_Refresh();
    void ResetMCCurve();

    void on_BAutoScaleX_clicked();
    void on_BAutoScaleY_clicked();
    void on_BExpandSelectRect_clicked();


signals:
    void MathUpdated();
    void ShowPlotControl();
    void sync_active_curve(int);
    void doubleClicked();
    void mouseEvent(QMouseEvent* mouse, TPure2dPlot* plotPure);
    void resized(QSize newSize, TPure2dPlot* plotPure);

public:
    enum TModes {Mode_ManualConstruct, Mode_Select, Mode_Normal};

protected:
    TPure2dPlotPrivate * const d_ptr;
    void dragEnterEvent(QDragEnterEvent *event) override;
    void dropEvent(QDropEvent *event) override;

public:
    Q_DECLARE_PRIVATE(TPure2dPlot);

    void ReSetAxisScales();
    using TCommonPlot::UpdateLabels;
    void UpdateLabels(bool IsDetailedInfoVisible);
    void HandleAxisMultiplier() override;
    void UpdateAxisLegends() override;
    bool IsAutoScaleEnabled();
    void SetAutoScaleEnabled(bool enabled, bool refresh);
    void SetZoomBySelection(const bool State);

    bool eventFilter(QObject *Object, QEvent *Event) override;
    void mouseDoubleClickEvent(QMouseEvent * Event) override;
    bool gestureEvent(QGestureEvent *event);

    void TraceCurve(const QPoint &Pos, int NOfCurve = 0);
    void SelectionApplied();
    void SetFitOrder(int Order);
    void EnableSelectMode(const bool State);
    void SelectRectChanged(const QRectF &Rect) override;

    void StartMCMode();
    void AddPointToMCCurve(const int After, const QPointF &p2);
    void MoveSelectedPointInMCCurve(const QPoint &Pos);
    void ExitMCMode();
    TCurveData GetMCCurve() const;

    void DefineClickInfo(const QPoint &Pos) override;

    virtual void SetActiveCurve(int Active) override;

    math::TMathProcessing *MathProcessing;
    bool isMathReady();
    int FitOrder;
    bool UseMathOnly;
    TCalcLimitsByFrameContent LimitsStretcher;

    TPure2dPlot(bool vertical, QObject *Parent = nullptr, QWidget *parent = nullptr);
    ~TPure2dPlot() override;
    T2dPlot *tParent;
    QObject *Parent; //for postEvent(Parent, blablabla);
    bool isFiltering = true;

    void Refresh() override;
    void RefreshInc();
    void paintEvent(QPaintEvent *e) override;
    bool incRefreshGuard;

    void setVertLineVisible(int N, bool v);
    void SetNumOfVertLines(int N);
    void MoveVertLine(double XPos, int N);
    void SetVertLinePen(QPen &Pen, int N = 0);
    QVector<QwtPlotMarker *> VertLines;

    TCurvesData* cloneSourceData(QPen **pen = nullptr);
    static TCurvesData* cloneSourceData(const TCurvesData &data, QPen **pen = nullptr);
    void clearSouceData();
};

}

#endif // PLOT_2DPURE_H
