#pragma once

#include "2d/plot_2dplot.h"
#include "plot_3dtypes.h"
#include "widgets/plot_DoubleSlider.h"
#include "plot_3dWidgets.h"
#include <QLayout>
#include "widgets/fixedsplitter.h"
#include "widgets/plot_colorchooser.h"
#include <QSettings>

class QSplitter;


namespace plot
{

template <int N>
void ColorMapHistogramEqualization(const THistogram<N> &zhistogram, QwtLinearColorMap *map, double from, double to);

class ThreeDOverlayButtons;

class T4dMime : public QMimeData
{
    Q_OBJECT
public:
    T3dData data3D;
    TCurvesData data2D;
};

class T3dMime : public QMimeData
{
    Q_OBJECT
public:
    T3dData data;
};

class TPure3dPlot;

class T3dPlot: public QWidget
{
    Q_OBJECT
public slots:
    void appendAttrib(plot::TMetaStorage &, int N = 0); //customization/filters
    void restoreAttributes(const plot::TMetaStorage&, int N = 0);

    void UpdateAllMarkersInfo();
    void on_BGrid_clicked();
    void on_BFixX_toggled();
    void on_BFixY_toggled();
    void on_BSave_clicked();
    void on_BCross_clicked();
    void on_BAngle_clicked();
    void on_BMarkers_clicked(bool);
    void on_BFixOXDir_clicked();
    void on_BFixOYDir_clicked();
    void on_GwyXFit_triggered(bool Checked);
    void on_GwyYFit_triggered(bool Checked);
    void MakeGaussian(int Size);
    void MakeMeanValue(int Size);
    void MakeMedianValue(int Size);
    void on_MeanValue_triggered(bool Checked);
    void on_MedianValue_triggered(bool Checked);
    void on_Gaussian_triggered(bool Checked);
    void on_Histogram_triggered(bool Checked);
    void on_MPalettes_trigered();
    void on_BCrossection_clicked();
    void on_BAreaZzoom_clicked();
    void on_BSelectByRect_clicked();
    void on_BRestoreScale_clicked();
    void on_BApply_clicked();
    void on_BSavePlotPixmap_clicked();
    void on_BCancel_clicked();
    void on_BDetailedMarkersInfo_clicked();
    void on_Sliders_ValuesChanged(double Min, double Max);
    void on_BExpandSelectRect_clicked();
    void on_FontSize_changed();

    void on_FitXOrder_changed();
    void on_FitYOrder_changed();
    void on_SubtrOrder_changed();
    void on_AutoZScale_changed(bool on);

public slots:
    void Show2dPlotControl();
    void Set2dPlotVisible(bool Visible);
    void HideSelectionIfAny();
    void plotDoubleClicked();
    void plot2dDoubleClicked();

signals:
    void AddDataToContainer(plot::T3dDataOwner, plot::TMetaStorage);
    void AddDataToContainer(plot::TCurvesDataOwner, plot::TMetaStorage);
    void ActivePlotChanged(int N);
    void PaletteChanged(QVector<QColor>);

private:
    void ConnectPlot2dWithVertical2d();

public:
    T3dPlot(int N = 1, QObject *Parent = nullptr, QWidget *wparent = nullptr);
    void Refresh();
    void AutoSave(int Ind);

    void SetActivePlot(int N);
    void RestoreEnvironmentState(int N);


    void GUICreate(int N);
    void ConnectSlots();
    void ConnectPlotControlSlots();
    void AdjustTwoDPlot();
    void MakeCrossection();
    void AdjustAxisLegends();

    bool SavePlot(int plotId, T3dDataOwner *src, TMetaStorage *attributes);

    void InitPalettes();
    static TPalette GetSpectralPalette(const Range &ZLimits, double X1, double X2);
    void ResizeSliders();
    void UpdateMinMaxInSlider();
    template <int Size>
    void UpdateSliderHistogram(const THistogram<Size> &zhistogram);
    bool eventFilter(QObject *Object, QEvent *Event) override;

    QVector<TPalette> Palettes;

    void SyncPan(int N, const QPointF &Delta);
    void SyncDefineClickInfo(int N, const QPoint &Pos);
    void SyncZoom(int N, const QPointF &Pos, int Increment, bool BFixX_isChecked, bool BFixY_isChecked);


    void SetTitle(const QString &Title, int N = 0);
    void SetZoomBySelection(bool State);
    const QImage MakePreview(int N = 0);

    void SetPlotsXScales(double Min, double Max);
    void SetPlotsYScales(double Min, double Max);
    void SetAllXScales(double Min, double Max);
    void SetAllYScales(double Min, double Max);
    void SetDefaultXLimits(double Min, double Max);
    void SetDefaultYLimits(double Min, double Max);

    void SetXYDescAndSIUnit(const QString &Desc, const QString &Unit, int N = 0);
    void SetZDescAndSIUnit(const QString &Desc, const QString &Unit, int N = 0);

    void HideCross();
    void SetCrossCoords(double x, double y);

    void SetAllFilterSizesTextInAction();
    static void SetFilterSizeTextInAction(QAction *Act, const int Size, const QString &FilterName);

    void SetSelectWidthRelCenter(qreal W);
    void SetSelectHeightRelCenter(qreal H);
    void SetSelectWidth(qreal W);
    void SetSelectHeight(qreal H);

    void SwapPlots(int p1, int p2);
    void SetFullWindowPlot(int plot, bool full);
    void setScanSPM2DPlot(bool setScanPlot);

    void ResetAllFilters();

    QVector<TPure3dPlot *> ThreeDPlots;
    T2dPlot *TwoDPlot;
    T2dPlot *TwoDPlotVertical;
    //QVector<QSplitter *> Containers;
    QHBoxLayout *MainLayout;
    QVBoxLayout *MarkersInfoLayout;
    QVBoxLayout *PlotVerticalLay;
    TCurvesDataOwner curveData;
    QVector<int> *orderChannel; // orderChannel[x]=y, where x-position plot in ThreeDPlots, y - his position on the screen
    QVector<int> *positionChannel; // same as prev, but reversed
    int currFullWindPlot = -1;
    bool plot2dFullWind = false;
    QVBoxLayout *plots2D3DLay;
    QVBoxLayout *ControlLayout;

    bool blockVisibility2DPlot = false;

    int lastSplitPlotSizes[2];

    ThreeDOverlayButtons *overlayButtons;

    int ActivePlot;
    bool showingScan2DPlot = true;
    T2dPlot *TwoDPlotSPM;
    bool showPlotControl = true;

    QObject *Parent;
    TDoubleSlider *Slider;
    bool showSlider = true;

    QSplitter *SplitPlotTab;
    FixedSplitter *fixedSplit;
    QTabWidget *mainTab;

    T3dMarkersInfo *MarkersInfo;
    T3dPlotControl *PlotControl;

    uint numPlotByX = 0;
    uint numPlotByY = 0;
    uint numTabsFor3DPlot = 0;

    QList<int> last2D3DPlotSplitSizes;
};

}
