#include "plot_3dpure.h"
#include "math/plot_MathProcessing.h"

#include <qwt_plot_layout.h>
#include <qwt_plot_marker.h>
#include <qwt_plot_canvas.h>

#include <QtEvents>
#include <QDrag>
#include <QApplication>
#include <QSplitter>
#include "widgets/fixedsplitter.h"
#include <QVBoxLayout>
#include <QFileDialog>
#include <QInputDialog>
#include <QMessageBox>
#include <QGestureEvent>
#include <QDebug>
#include "widgets/threedoverlaybuttons.h"
#include "3d/plot_3dplot.h"
#include "2d/plot_2dplot.h"
#include "2d/plot_2dpure.h"

namespace plot
{

void T3dMarkerInfoHandler::operator()
    (bool AutoLegendEnabled, const TSIUnit &XYUnitInfo, const TSIUnit &ZUnitInfo, double XVal, double YVal, double ZVal)
{
    TGeneralMarkerInfoHandler::operator ()(AutoLegendEnabled, XYUnitInfo, XYUnitInfo, XVal, YVal);
    ZInfo(AutoLegendEnabled, ZUnitInfo, ZVal);
}
QString T3dMarkerInfoHandler::FormUniteString()
{
    return TGeneralMarkerInfoHandler::FormUniteString() + "\nZ = " + ZInfo.Value + ZInfo.Unit;
}

bool TPure3dPlot::isMathReady()
{
    return !MathProcessing->isRunning();
}
TPure3dPlot::~TPure3dPlot()
{
    delete MathProcessing;
}
TPure3dPlot::TPure3dPlot(int Num, T3dPlot *Chld, QObject *_Parent, QWidget *parent)
    : TCommonPlot(parent), Child(Chld), ZoomBySelection(true)
{
    last_resize_x = 0;
    isResizing = false;
    Parent = _Parent;
    colorChooser = new plot_colorChooser(this);
    colorChooser->hide();
    connect(colorChooser, SIGNAL(colorChoosed(QColor, int)), this, SLOT(PaletteChoosed(QColor, int)));

    MathProcessing = new math::TMathProcessing(&MathData, false);
    N = Num;
    QSpectrogram->attach(QPlot);
    QSpectrogram->setRenderThreadCount(QThread::idealThreadCount());

    Cross = new QwtPlotMarker;
    Cross->setValue(0, 0);
    Cross->setSymbol(new QwtSymbol(QwtSymbol::Cross, QColor(255, 100, 255), QColor(255, 100, 255), QSize(15, 15)));
    Cross->setLineStyle(QwtPlotMarker::NoLine);
    Cross->attach(QPlot);
    Cross->hide();

    areaRB = new QRubberBand(QRubberBand::Rectangle, QPlot);
    areaRB->hide();

    AreaZzoom = false;
    zoom_active = 0;

    singleInTabPage = false;

    wait_for_math = false;
    math_delayed_refresh = false;

    connect(MathProcessing, SIGNAL(finished()), this, SLOT(s_Refresh()), Qt::QueuedConnection);

    setAcceptDrops(true);
    ((QWidget *)QPlot->titleLabel())->setToolTip(tr("Drag & Drop as data to plots, as image"));

    QPlot->canvas()->installEventFilter(this);
}
void TPure3dPlot::SelectDone()
{
    if (ZoomBySelection)
    {
        QRectF Rect = GetSelectedRect();
        if (!Rect.isValid() || Rect.width() < 9e-9 || Rect.height() < 9e-9) {
            //qDebug() << Rect;
            return;
        }
        SetPlotsXScales(Rect.left(), Rect.right());
        SetPlotsYScales(Rect.top(),  Rect.bottom());
        ResizeQPlot();
    }
}
inline void TPure3dPlot::MakeCrossection()
{
    if (BCrossectionIsChecked && BMarkersIsChecked)
        Child->MakeCrossection();
}

void TPure3dPlot::AreaZzoomST(double x, double y, double xo, double yo, int st)
{
    if (st == 1) {
        areazA = areazB = QPointF(x, y);
        areazAo = QPointF(xo, yo);
        areaRB->show();
        areaRB->setGeometry(xo, yo, 1, 1);
    } else if (st == 2) {
        areazB = QPointF(x, y);

        double xA = qMin<qreal>(areazAo.x(), xo);
        double xB = qMax<qreal>(areazAo.x(), xo);
        double yA = qMin<qreal>(areazAo.y(), yo);
        double yB = qMax<qreal>(areazAo.y(), yo);
        areaRB->setGeometry(xA, yA, xB-xA, yB-yA);
    } else if (st == 3) {
        areazB = QPointF(x, y);

        if (fabs(xo - areazAo.x()) <= 4 || fabs(yo - areazAo.y()) <= 4 ) {
            areaRB->hide();
            return;
        }

        double xA = qMin<qreal>(areazA.x(), areazB.x());
        double xB = qMax<qreal>(areazA.x(), areazB.x());
        double yA = qMin<qreal>(areazA.y(), areazB.y());
        double yB = qMax<qreal>(areazA.y(), areazB.y());
        double DeltaX = xB - xA, DeltaY = yB - yA;
        double NumberOfSteps = 1000; //GetMaxNumberOfPointsXY(xA, yA, xB, yB);
        double StepX, StepY;

        double min = 1e100;
        double max = -1e100;

        StepX = DeltaX / NumberOfSteps;
        StepY = DeltaY / NumberOfSteps;

        for (int i = 0; i < NumberOfSteps; i++) {
            for (int j = 0; j < NumberOfSteps; j++) { //fixme j is not used
                double val = value(xA + i*StepX, yA + i*StepY); //fixme is i*StepY should be j*StepY ?
                min = qMin(min, val);
                max = qMax(max, val);
            }
        }

        if (!qFuzzyCompare(1. + min, 1. + max)) {
            Child->Slider->Sliders->SetMinMaxSlidersValues(
                        min / ZAxisUnit.Multiplier,
                        max / ZAxisUnit.Multiplier);
            AutoZScaleOn = false;
        }

        areaRB->hide();
    }
}

void TPure3dPlot::dragEnterEvent(QDragEnterEvent *event)
{
    const T3dMime *data = qobject_cast<const T3dMime *>(event->mimeData());
    if (data && event->source() != this)
        event->acceptProposedAction();
}
void TPure3dPlot::dropEvent(QDropEvent *event)
{
    const T3dMime *data = qobject_cast<const T3dMime *>(event->mimeData());
    if (!data || event->source() == this) return;

    SourceData = data->data + SourceData; //new on top
    //TODO limits bounding rect
    Refresh();
}
int TPure3dPlot::Height() const
{
    return QPlot->size().height();
}
void TPure3dPlot::BRestoreScale_clicked()
{
    //if (ZoomBySelection) {
    PlotsXLimits = DefaultXLimits;
    PlotsYLimits = DefaultYLimits;
    //}

    TCommonPlot::BRestoreScale_clicked();
//    if (isVisible())
//        ResizeQPlot();
    emit onRestoredScales();
}
void TPure3dPlot::SetZoomBySelection(bool State)
{
    ZoomBySelection = State;
}
bool TPure3dPlot::eventFilter(QObject *Object, QEvent *Event)
{
    auto mouse = static_cast<QMouseEvent *>(Event);
    emit MouseEventSignal(Event, this, N);

    if (stopMouseEvent)
    {
        if (Event->type() == QEvent::MouseButtonPress) {
            Child->SetActivePlot(N); //2d->3d plotcontrol
        }
        return QWidget::eventFilter(Object, Event);
    }

    if (Object == QPlot->canvas()) {
        if (Event->type() == QEvent::Paint) {
            wait_for_math = MathProcessing->isRunning();
            if (math_delayed_refresh && !wait_for_math) {
                math_delayed_refresh = false;
                QMetaObject::invokeMethod(this, "Refresh", Qt::QueuedConnection);
            }
            return wait_for_math;
        }
        return false;
    }

    if (!isEnabled()) {
        Action = ExternalAct;
    } else
    if (Enabled && !SelectionModeEnabled && AreaZzoom && Child && (int)Child->ActivePlot == N) {
        Action = ExternalAct;

        int AreaZzoom_state = 0;

        if (Event->type() == QEvent::MouseButtonPress && zoom_active == 0) {
            AreaZzoom_state = 1;
            zoom_active = 1;
            Child->SetActivePlot(N); //2d->3d plotcontrol
        }
        if (Event->type() == QEvent::MouseMove && zoom_active == 1)
            AreaZzoom_state = 2;
        if (Event->type() == QEvent::MouseButtonRelease && zoom_active == 1) {
            AreaZzoom_state = 3;
            zoom_active = 0;
        }

        if (AreaZzoom_state != 0) {
            QPoint Pos = mapToCanvas(mouse->pos());
            double X = ScreenXToPlotX(Pos.x());
            double Y = ScreenYToPlotY(Pos.y() - 5);// y rounding error

            X = qBound(PlotsXLimits[0], X, PlotsXLimits[1]);
            Y = qBound(PlotsYLimits[0], Y, PlotsYLimits[1]);

            AreaZzoomST(X, Y, mouse->pos().x(), mouse->pos().y(), AreaZzoom_state);
        }
        return false;
    }

    if (!isEnabled()) {
        Action = ExternalAct;
    } else
    if (Enabled)
    {
#ifndef _ADROID
        if (Event->type() == QEvent::MouseButtonPress && mouse->buttons() == Qt::RightButton)
        {
            if (Child && Child->TwoDPlot->Pure2dPlot->SelectionModeEnabled)
            {
                auto twoDSelecRect = Child->TwoDPlot->Pure2dPlot->GetSelectedRect();
                QRectF r2;
                r2.setX(twoDSelecRect.y());
                r2.setY(twoDSelecRect.x());
                r2.setWidth(twoDSelecRect.width());
                r2.setHeight(twoDSelecRect.height());
                QRectF r = Child->TwoDPlotVertical->Pure2dPlot->TransformForeignRectToRectangle(twoDSelecRect);

                Child->TwoDPlotVertical->Pure2dPlot->Rectangle = r;
                Child->TwoDPlotVertical->Pure2dPlot->ReDrawSelectedRectangle();
            }
        }
#endif
        if (Event->type() == QEvent::Wheel) {
            if (Child) Child->SetActivePlot(N);
            QPointF Pos = mapToCanvas(((QWheelEvent *)Event)->pos());
            Pos = ScreenToPlot(Pos.toPoint());

            int Increment = ((QWheelEvent *) Event)->delta();
            ZoomPlot(Pos, Increment);
            MakeCrossection();
            if (Child) Child->SyncZoom(N, Pos, Increment, BFixXIsChecked, BFixYIsChecked); //TODO sync only curxylims
        } else if (Event->type() == QEvent::MouseButtonPress) {
            QPoint Pos = mapToCanvas(mouse->pos());
            DefineClickInfo(Pos);
            if (Child) Child->SyncDefineClickInfo(N, Pos);

            if (Action == Panning && QPlot->plotLayout()->titleRect().contains(mouse->pos())) {
                Action = ExternalAct;

                QDrag *drag = new QDrag(this);
                T3dMime *mimeData = new T3dMime;
                mimeData->data = SourceData;
                if (Child) {
                    QImage graph = QPixmap::grabWidget(this).toImage();
                    QImage slider = QPixmap::grabWidget(Child->Slider).toImage().scaledToHeight(graph.height(), Qt::SmoothTransformation);

                    QImage composition(graph.width() + (Child->showSlider ? slider.width():0), graph.height(), graph.format());
                    QPainter painter(&composition);
                    painter.drawImage(0, 0, graph);
                    if (Child->showSlider)
                        painter.drawImage(graph.width(), 0, slider);

                    mimeData->setImageData(composition);
                }
                drag->setMimeData(mimeData);
                drag->setPixmap(QPixmap::fromImage(MakePreview()));
                drag->exec(Qt::CopyAction);
            }
            if (Child) {
                if (BCrossIsChecked && (mouse->buttons() == Qt::LeftButton &&
                        (int)Child->ActivePlot == N)) {
                    Action = CrossAct;
                    double X = ScreenXToPlotX(Pos.x());
                    double Y = ScreenYToPlotY(Pos.y());

                    X = qBound(CurrentXLimits[0], X, CurrentXLimits[1]);
                    Y = qBound(CurrentYLimits[0], Y, CurrentYLimits[1]);

                    SetCrossCoords(X, Y);
                    if (Parent) QCoreApplication::postEvent(Parent, new TCrossEvent(QPointF(X, Y)));
                }
                Child->SetActivePlot(N);
            }
        } else if (Event->type() == QEvent::MouseMove) {
            QPoint Pos = mapToCanvas(mouse->pos());
            if (Action == CrossAct && mouse->buttons() == Qt::LeftButton) {
                double X = ScreenXToPlotX(Pos.x());
                double Y = ScreenYToPlotY(Pos.y());

                X = qBound(CurrentXLimits[0], X, CurrentXLimits[1]);
                Y = qBound(CurrentYLimits[0], Y, CurrentYLimits[1]);

                SetCrossCoords(X, Y);
                if (Parent) QCoreApplication::postEvent(Parent, new TCrossEvent(QPointF(X, Y)));
            }
            if (Action == MoveA || Action == MoveB || Action == MoveC) {
                if (!BFixOXDirIsChecked && !BFixOYDirIsChecked)
                    MoveMarker(Pos.x(), Pos.y(), false);
                else {
                    QPointF APos, BPos;
                    if (Action == MoveA) {
                        APos = ScreenToPlot(Pos);
                        if (BFixOXDirIsChecked)      { BPos.setX(APos.x()); BPos.setY(MarkerB->value().y()); }
                        else if (BFixOYDirIsChecked) { BPos.setY(APos.y()); BPos.setX(MarkerB->value().x()); }
                    }
                    else if (Action == MoveB) {
                        BPos = ScreenToPlot(Pos);
                        if (BFixOXDirIsChecked)      { APos.setX(BPos.x()); APos.setY(MarkerA->value().y()); }
                        else if (BFixOYDirIsChecked) { APos.setY(BPos.y()); APos.setX(MarkerA->value().x()); }
                    }
                    HandleMoveMarkers(APos, BPos);
                }
                MakeCrossection();
            }
            if (Action == Panning) {
                double kx = (ScreenXToPlotX(20) - ScreenXToPlotX(0)) / 20.;
                double ky = (ScreenYToPlotY(20) - ScreenYToPlotY(0)) / 20.;
                QPointF Delta((Pos.x() - ClickPos.x())*kx,
                              (Pos.y() - ClickPos.y())*ky);

                PanPlot(Delta);
                MakeCrossection();
                if (Child) Child->SyncPan(N, Delta); //TODO sync only curxylims
            }
        }
        else if (Event->type() == QEvent::MouseButtonRelease) {
            Action = ExternalAct;

        }
    }
    if (Event->type() == QEvent::Gesture) {
        return gestureEvent(static_cast<QGestureEvent*>(Event));
    }
    return QWidget::eventFilter(Object, Event);
}
void TPure3dPlot::mouseDoubleClickEvent(QMouseEvent * /*Event*/)
{
    emit doubleClicked();
}
bool TPure3dPlot::gestureEvent(QGestureEvent *event)
{
    if (event->gesture(Qt::TapAndHoldGesture))
        mouseDoubleClickEvent(nullptr);

    return true;
}
void TPure3dPlot::SelectRectChanged(const QRectF &Rect)
{
    if (Parent)
        QCoreApplication::postEvent(Parent, new TSelectEvent(Rect, TSelectEvent::St_SelectChanged));
}

void TPure3dPlot::ResizeQPlot(bool)
{
    if (!isVisible()) return;
    if (isResizing) return;
    isResizing = true;

    setMinimumSize(QSize(0,0));

    //Init various indents
    QPlot->plotLayout()->activate(QPlot, QPlot->rect());

    int UpSpace = QPlot->plotLayout()->titleRect().height();
    int DownSpace = QPlot->plotLayout()->scaleRect(QwtPlot::xBottom).height();
    int LeftSpace = QPlot->plotLayout()->scaleRect(QwtPlot::yLeft).width();
    int RightSpace = 0;

    //fit plot
    double NewDeltaX, NewDeltaY;
    const QSize AvailableSpace = FindAvailableSpaceForQPlot();

    FindSizeWithAspectRatio(AvailableSpace.width() - LeftSpace - RightSpace,
                            AvailableSpace.height() - UpSpace - DownSpace,
                            &NewDeltaX, &NewDeltaY);

    int xsize = NewDeltaX + LeftSpace + RightSpace - 10;
    int ysize = NewDeltaY + UpSpace + DownSpace - 20;
    //QPlot->resize(AvailableSpace);

    QPlot->resize(xsize, ysize);


    //set minimum size
    /*
    FindSizeWithAspectRatio(1, 1, &NewDeltaX, &NewDeltaY);
    setMinimumSize(qMin<int>(xsize+10, 100/NewDeltaY + LeftSpace + RightSpace - 10),
                   qMin<int>(ysize+10, 100/NewDeltaX + UpSpace + DownSpace - 10));
                   */

    //scale division update to fit all in size
    UpdateScalesDiv();

    if (Child && Child->ActivePlot == N)
        Child->ResizeSliders();

    //Resize ArthurFrame:
    QSize NewSizeForArtFrame = QPlot->rect().size();


    if (AvailableSpace.height() < NewSizeForArtFrame.height())
        NewSizeForArtFrame.setHeight(AvailableSpace.height());

    if (AvailableSpace.width() < NewSizeForArtFrame.width())
        NewSizeForArtFrame.setWidth(AvailableSpace.width());

    resize(NewSizeForArtFrame);

#ifndef _ADROID
    if (Child->TwoDPlotVertical && Child->TwoDPlotVertical->isVisible())
    {
        int secondWidSplitWidth = Child->fixedSplit->width();
        secondWidSplitWidth -= NewSizeForArtFrame.width();
        secondWidSplitWidth -= 35;

        Child->fixedSplit->setSizes(QList<int>() << NewSizeForArtFrame.width()+20 <<
                                                    secondWidSplitWidth);

        if (Child->Slider->isVisible())
            secondWidSplitWidth -= Child->Slider->width();
        Child->TwoDPlotVertical->setFixedSize(secondWidSplitWidth, Child->TwoDPlotVertical->height());
        Child->TwoDPlotVertical->Refresh();
        Child->TwoDPlotVertical->Pure2dPlot->SetSelectWidth(
                    Child->TwoDPlotVertical->Pure2dPlot->CurrentXLimits[1] - Child->TwoDPlotVertical->Pure2dPlot->CurrentXLimits[0]);
    }
#endif

    isResizing = false;

    emit onResized(N, NewSizeForArtFrame);
}
QSize TPure3dPlot::FindAvailableSpaceForQPlot() const
{
    int width;
    if (secondParent == nullptr)
    {
        width = parentWidget()->width();
        if (!singleInTabPage)
            width /= 2;
        if (fixedSize > 0)
            width = fixedSize;
    }
    else
    {
        width = Child->width();
        if (Child->TwoDPlot->PlotControl->isVisible())
            width -= Child->TwoDPlot->PlotControl->width();
        if (Child->TwoDPlotSPM && Child->TwoDPlotSPM->PlotControl->isVisible())
            width -= Child->TwoDPlotSPM->PlotControl->width();
        if (Child->PlotControl->isVisible())
            width -= Child->PlotControl->width();
    }

    return QSize(width, parentWidget()->height());
}
void TPure3dPlot::UpdateLabels(bool IsDetailedInfoVisible)
{
    AMarkerInfo(AutoLegendEnabled, XYAxisUnit, ZAxisUnit, MarkerA->xValue(), MarkerA->yValue(), value(MarkerA->xValue(), MarkerA->yValue()));
    BMarkerInfo(AutoLegendEnabled, XYAxisUnit, ZAxisUnit, MarkerB->xValue(), MarkerB->yValue(), value(MarkerB->xValue(), MarkerB->yValue()));
    TValueInfoCreator difx, dify, difz;

    QString AText, BText, MText;
    if (IsDetailedInfoVisible) { AText = "A"; BText = "B"; }
    else
    {
        AText = AMarkerInfo.FormUniteString();
        BText = BMarkerInfo.FormUniteString();

        difx(AutoLegendEnabled, XYAxisUnit, BMarkerInfo.XInfo.SrcValue - AMarkerInfo.XInfo.SrcValue);
        dify(AutoLegendEnabled, XYAxisUnit, BMarkerInfo.YInfo.SrcValue - AMarkerInfo.YInfo.SrcValue);
        difz(AutoLegendEnabled, ZAxisUnit, BMarkerInfo.ZInfo.SrcValue - AMarkerInfo.ZInfo.SrcValue);
        MText = QString(QChar(0x2206)) + "X = " + difx.Value + difx.Unit + '\n' +
                QString(QChar(0x2206)) + "Y = " + dify.Value + dify.Unit + '\n' +
                QString(QChar(0x2206)) + "Z = " + difz.Value + difz.Unit;
    }

//    QwtText QwtAText(AText),
//            QwtBText(BText),
//            QwtMText(MText);

//    QFont font = qApp->font(); font.setPointSize(FontSize);
//    QwtAText.setFont(font);
//    QwtBText.setFont(font);
//    QwtMText.setFont(font);

//    QwtAText.setColor(QColor(255, 160, 160));
//    QwtBText.setColor(QColor(255, 160, 160));
//    QwtMText.setColor(QColor(255, 160, 160));

//    MarkerA->setLabel(QwtAText);
//    MarkerB->setLabel(QwtBText);
//    MarkerM->setLabel(QwtMText);
}

void TPure3dPlot::Refresh()
{
    if (blockRefresh)
        return;

    if (!IsMathEnabled()) {
        PaintData = SourceData;

        MathProcessing->wait();
        MathData.clear();

        s_Refresh();
    } else {
        if ((math_delayed_refresh = (MathProcessing->isRunning() || wait_for_math)))
            return;
        qApp->sendPostedEvents(this, QEvent::MetaCall); //flush all previous s_Refresh,Refresh events

        MathData.clear();
        ResizeOwner(MathData, SourceData.size());
        PaintData = getUnLock(MathData);

        for (int i = 0; i < SourceData.size(); i++)
            if (SourceData[i]) {
                *MathData[i] = *SourceData[i].data();
                MathData[i]->detach();
            }

        if (isFiltering)
        {
            if (CurrentXFitOrder >= 0 && (xScanning == -1 || xScanning == 1))  MathProcessing->DoFitX(CurrentXFitOrder);
            if (CurrentYFitOrder >= 0 && (xScanning == -1 || xScanning == 0))  MathProcessing->DoFitY(CurrentYFitOrder);
            if (CurrentSubtrOrder >= 0) MathProcessing->DoSubtr(CurrentSubtrOrder);
            if (MakeCorrectLineByMedianaXDirIsChecked && (xScanning == -1 || xScanning == 1)) MathProcessing->DoLineCorrectByMedianaX();
            if (MakeCorrectLineByMedianaYDirIsChecked && (xScanning == -1 || xScanning == 0)) MathProcessing->DoLineCorrectByMedianaY();
            if (MeanValueFilterEnabled && MeanValueFilterSize > 0) MathProcessing->DoMeanValue(MeanValueFilterSize);
            if (MedianValueFilterEnabled && MedianValueFilterSize > 0) MathProcessing->DoMedianValue(MedianValueFilterSize);
            if (GaussianValueFilterEnabled && GaussianFilterSize > 0) MathProcessing->DoGaussian(GaussianFilterSize);
        }
        ShowHistogram(isFiltering && HistogramEqualization);
        MathProcessing->start();
    }
}
void TPure3dPlot::UpdateHistogram()
{
    UpdateHistogram(zhistogram);
}
template <int Size>
void TPure3dPlot::UpdateHistogram(const THistogram<Size> &zhistogram)
{
    if (!AutoZScaleOn)
        return;

    int sum = 0;
    for (int i = 0; i < Size; i++)
        sum += zhistogram.bins[i];
    if (!sum)
        return;

    //autoscale by throwing out outermost values
#define IGNORE_PART 0.01
    int acc;
    int low = 0;
    acc = 0;
    while (low < Size-1-1) {
        acc += zhistogram.bins[low];
        if (acc > int(sum * IGNORE_PART))
            break;
        low++;
    }

    int high = Size-1;
    acc = 0;
    while (high > low+1) {
        acc += zhistogram.bins[high];
        if (acc > int(sum * IGNORE_PART))
            break;
        high--;
    }

    MinSliderVal = low / (double)(Size-1);
    MaxSliderVal = high / (double)(Size-1);
    SetPalette(active_palette, MinSliderVal, MaxSliderVal);
}
void TPure3dPlot::s_Refresh()
{
    if (blockRefresh || (IsMathEnabled() && MathProcessing->isRunning()))
        return;

    UpdatePlotsZLimits();
    //if (0/*AutoScale*/) {plotsxylims = topdata.size(); ReSetAxisScales();} else
        HandleAxisMultiplier(); //for labels
    ReAssignPlotOptions();

    UpdateHistogram(zhistogram); //oplkill 10.12.19 todo
    if (isFiltering && HistogramEqualization) {
        SetPalette(active_palette, MinSliderVal, MaxSliderVal);
        blockRefresh = true;
        ColorMapHistogramEqualization(zhistogram, colormap, MinSliderVal, MaxSliderVal);
        blockRefresh = false;
        if (N == Child->ActivePlot)
            Child->Slider->update();
    }

    Replot();

    if (N == Child->ActivePlot) {
        Child->UpdateMinMaxInSlider();
        Child->UpdateSliderHistogram(zhistogram);
        Child->UpdateAllMarkersInfo();
        MakeCrossection();
    }

    PlotWasChanged();
    emit MathUpdated();
}

void TPure3dPlot::SetCrossCoords(double x, double y)
{
    Cross->setValue(x, y);
    Cross->show();
    Replot();
}
void TPure3dPlot::HideCross()
{
    Cross->hide();
    Replot();
}
void TPure3dPlot::setFiltering(bool on)
{
    isFiltering = on;
    Refresh();
}
bool TPure3dPlot::IsMathEnabled() const
{
    return  isFiltering &&
            (CurrentXFitOrder >= 0 ||
            CurrentYFitOrder >= 0 ||
            CurrentSubtrOrder >= 0 ||
            MakeCorrectLineByMedianaXDirIsChecked ||
            MakeCorrectLineByMedianaYDirIsChecked ||
            MeanValueFilterSize > 0 ||
            MedianValueFilterSize > 0 ||
            GaussianFilterSize > 0);
}
void TPure3dPlot::MakeXFit(const int Order) { if (CurrentXFitOrder != Order && (xScanning == -1 || xScanning == 1)) { CurrentXFitOrder = Order; Refresh(); } }
void TPure3dPlot::MakeYFit(const int Order) { if (CurrentYFitOrder != Order && (xScanning == -1 || xScanning == 0)) { CurrentYFitOrder = Order; Refresh(); } }
void TPure3dPlot::MakeSubtr(const int Order) { if (CurrentSubtrOrder != Order) { CurrentSubtrOrder = Order; Refresh(); } }
void TPure3dPlot::MakeCorrectLineByMedianaXDir(const bool Make)
{
    if (isFiltering && MakeCorrectLineByMedianaXDirIsChecked != Make && (xScanning == -1 || xScanning == 1))
    { MakeCorrectLineByMedianaXDirIsChecked = Make; Refresh(); }
}
void TPure3dPlot::MakeCorrectLineByMedianaYDir(const bool Make)
{
    if (isFiltering && MakeCorrectLineByMedianaYDirIsChecked != Make && (xScanning == -1 || xScanning == 0))
    { MakeCorrectLineByMedianaYDirIsChecked = Make; Refresh(); }
}
void TPure3dPlot::MakeMeanValue(const int Size) { if (MeanValueFilterSize != Size) { MeanValueFilterSize = Size; Refresh(); } }
void TPure3dPlot::MakeMedianValue(const int Size) { if (MedianValueFilterSize != Size) { MedianValueFilterSize = Size; Refresh(); } }
void TPure3dPlot::MakeGaussian(const int Size) { if (GaussianFilterSize != Size) { GaussianFilterSize = Size; Refresh(); } }
void TPure3dPlot::MakeHistogram(bool Make) {
    if (!isFiltering)
        Make = false;
    if (HistogramEqualization != Make) {
        HistogramEqualization = Make;

        ShowHistogram(HistogramEqualization);
    }
}
void TPure3dPlot::ShowHistogram(bool show)
{
    SetPalette(active_palette, MinSliderVal, MaxSliderVal);
    if (show)
        ColorMapHistogramEqualization(zhistogram, colormap, MinSliderVal, MaxSliderVal);
    if (N == Child->ActivePlot)
        Child->Slider->update();
    ReplotChanged();
}
void TPure3dPlot::PaletteChoosed(QColor /*color*/, int id)
{
    Child->SetActivePlot(N);
    if (id == 0)
        NOfPalette = 1;
    else if (id == 1)
        NOfPalette = 4;
    else if (id == 2)
        NOfPalette = 2;
    else if (id == 3)
        NOfPalette = 3;
    else if (id == 4)
        NOfPalette = 7;
    else if (id == 5)
        NOfPalette = 11;
    else if (id == 6)
        NOfPalette = 5;
    else if (id == 7)
        NOfPalette = 6;
    else if (id == 8)
        NOfPalette = Child->PlotControl->AGPalettes->actions().size() - 1;
    Child->on_Sliders_ValuesChanged(MinSliderVal, MaxSliderVal);

    Child->overlayButtons->BBackButton->click();
}
void TPure3dPlot::HandleAxisMultiplier()
{
    XYAxisUnit.UpdateLegend(CurrentXLimits[1], CurrentYLimits[1]);
    UpdateAxisLegends();
}
void TPure3dPlot::ResetAllFilters()
{
    MakeCorrectLineByMedianaXDir(false);
    MakeCorrectLineByMedianaYDir(false);
    MakeSubtr(-1);
    MakeXFit(-1);
    MakeYFit(-1);
    MeanValueFilterEnabled = false;
    MedianValueFilterEnabled = false;
    GaussianValueFilterEnabled = false;
    MeanValueFilterSize = -1;
    MedianValueFilterSize = -1;
    GaussianFilterSize = -1;
    HistogramEqualization = false;
    ShowHistogram(false);
    Refresh();
}
void TPure3dPlot::UpdateAxisLegends()
{
    UpdateLabels();
    TInfoFor3dPlot::UpdateAxisLegends();
    if (AutoLegendEnabled)
        QPlot->axisScaleDraw(QwtPlot::yLeft)->Multiplier = QPlot->axisScaleDraw(QwtPlot::xBottom)->Multiplier = XYAxisUnit.Multiplier;
}
void TPure3dPlot::HandleMoveMarkers(const QPointF &APos, const QPointF &BPos)
{
    MoveMarker(MarkerA, APos.x(), APos.y());
    MoveMarker(MarkerB, BPos.x(), BPos.y());

    ReSyncMarkers();

    UpdateLabels();
    Replot();
}
void TPure3dPlot::on_BFixOXDir_clicked()
{
    if (BFixOXDirIsChecked)
    {
        if (MarkerA->value().y() == MarkerB->value().y()) {
            MoveMarker(MarkerA, MarkerA->value().x(), PlotsYLimits[0]);
            MoveMarker(MarkerB, MarkerB->value().x(), PlotsYLimits[1]);
        };
        double C = (MarkerA->value().x() + MarkerB->value().x()) / 2.;
        HandleMoveMarkers(QPointF(C, MarkerA->value().y()), QPointF(C, MarkerB->value().y()));
        MakeCrossection();
    }
}
void TPure3dPlot::on_BFixOYDir_clicked()
{
    if (BFixOYDirIsChecked)
    {
        if (MarkerA->value().x() == MarkerB->value().x()) {
            MoveMarker(MarkerA, PlotsXLimits[0], MarkerA->value().y());
            MoveMarker(MarkerB, PlotsXLimits[1], MarkerB->value().y());
        };
        double C = (MarkerA->value().y() + MarkerB->value().y()) / 2.;
        HandleMoveMarkers(QPointF(MarkerA->value().x(), C), QPointF(MarkerB->value().x(), C));
        MakeCrossection();
    }
}

}
