#pragma once

#include "core/plot_core.h"
#include <qwt_color_map.h>
#include <qwt_raster_data.h>
#include <qwt_plot_spectrogram.h>

namespace plot {

typedef QVector<QColor> TPalette;
typedef QVector<QVector<double> > T2dArray;

QRectF ConstructRectF(Range XLims, Range YLims);
double DoubleToArrInd(double Pos, int TotalN, const Range &Lims);

class TInfoFor3dPlot: virtual public TInfoForAllPlots
{
public:
    QString ZLegend;

    TSIUnit XYAxisUnit;
    TSIUnit ZAxisUnit;

    void UpdateAxisLegends() override { if (AutoLegendEnabled) { XLegend = YLegend = XYAxisUnit.Legend; ZLegend = ZAxisUnit.Legend; } }
};

class TMin3dData
{
public:
    TMin3dData();

    void detach();
    void setSize(int nx, int ny);
    void UpdateDxDy();
    void UpdateNxNy() { (NYElems = Array.size()) ? NXElems = Array[0].size() : NXElems = 0; }
    void clear() { Array.clear(); UpdateNxNy(); resetMathLims(); }
    void resetMathLims();

    bool IsIn(double x, double y) const;
    void AddPoint(int X, int Y, double Z);
    TMin3dData *Crop(const QRectF &SelectedRect, const Range &XLims, const Range &YLims) const;
    TMin3dData *CropIndex(int minx, int miny, int maxx, int maxy) const;
    bool LimsForMathCorrect() const;

    T2dArray Array;
    double dx, dy;
    int NXElems, NYElems;
    Range XLimits, YLimits;
    Range LimsForMathX;
    Range LimsForMathY;
};

typedef QVector<QWeakPointer<TMin3dData> > T3dData;
typedef QVector<QSharedPointer<TMin3dData> > T3dDataOwner;
int FindDataWithPlotsLimits(const T3dDataOwner &data, const Range &XLimits, const Range &YLimits);

#define DEFAULT_HIST_SIZE 100
template<int N = DEFAULT_HIST_SIZE>
class THistogram
{
public:
    THistogram() {
        for (int i = 0; i < N; i++)
            bins[i] = 0;
    }
    int bins[N];
};

class TCompleteInfoFor3dPlot: public TInfoFor3dPlot, public QwtRasterData
{
public:
    TCompleteInfoFor3dPlot();
    virtual ~TCompleteInfoFor3dPlot() override { QSpectrogram->detach(); }

    T3dData SourceData;
    T3dDataOwner MathData;
    T3dData PaintData;
    QwtPlotSpectrogram *QSpectrogram;

    QwtLinearColorMap *colormap;
    TPalette active_palette;
    void SetPalette(const TPalette &palette, double min, double max);

    //qwt render interface
    inline int FindNecessaryData(double x, double y, int start = 0) const;
    double value(double x, double y) const override;
    double valueBiLinear(double XPos, double YPos, const TMin3dData *Temp3dData) const;

    Range PlotsZLimits;
    THistogram<> zhistogram;

    void UpdatePlotsZLimits();
    const QwtInterval interval(Qt::Axis ax) const override {
        if (ax == Qt::XAxis) return QwtInterval(PlotsXLimits[0], PlotsXLimits[1]);
        if (ax == Qt::YAxis) return QwtInterval(PlotsYLimits[0], PlotsYLimits[1]);
        if (ax == Qt::ZAxis) return QwtInterval(PlotsZLimits[0], PlotsZLimits[1]);
        return QwtInterval(0, 1); }
};

}
