#include "plot_MathProcessing.h"
#include "plot_SubtPlane.h"
#include "plot_MathFilters.h"

#include <QDebug>

namespace plot {
namespace math {

TMathProcessing::TMathProcessing(TCurvesDataOwner *Src, bool vertical)
{
    mi = mx = 0.;
    MCCurve = nullptr;
    this->vertical = vertical;
    Curves = Src;
    Data = nullptr;
    Work[plot::math::TMathProcessing::Scale2d].arg = -1;
}

TMathProcessing::TMathProcessing(T3dDataOwner *Src, bool vertical)
{
    mi = mx = 0.;
    MCCurve = nullptr;
    this->vertical = vertical;
    Curves = nullptr;
    Data = Src;
}

void TMathProcessing::SmartAddTask(const TTask &Task)
{
    Work[Task.Action] = Task;
}
void TMathProcessing::run()
{
    while (true) {
        bool set = false;
        for (int i = 1; i < FiltersSize; i++) {
            if (Work[i].Action == i) {
                ActiveTask = Work[i];
                Work[i].Action = TMathProcessing::Nothing;
                set = true;
                break;
            }
        }
        if (!set) break;

        try {
            //TODO func ptr table
            switch (ActiveTask.Action) {
            case TMathProcessing::Fit2d:
                _Fit2d();
                break;
            case TMathProcessing::Scale2d:
                _Scale2d();
                break;
            case TMathProcessing::Fit3dX:
                _Fit3d_XDir();
                break;
            case TMathProcessing::Fit3dY:
                _Fit3d_YDir();
                break;
            case TMathProcessing::Subtr:
                _Subtr();
                break;
            case TMathProcessing::LineMedianX:
            case TMathProcessing::LineMedianY:
                _LineCorrectByMediana();
                break;
            case TMathProcessing::MeanValue:
                _DoMeanValue();
                break;
            case TMathProcessing::MedianValue:
                _DoMedianValue();
                break;
            case TMathProcessing::Gaussian:
                _DoGaussian();
                break;
            };
        } catch (...) {
            qDebug() << "math exception";
        }
    }
}
void TMathProcessing::_Fit2d()
{
    for (int i = 0; i < Curves->size(); i++)
        _Fit2d(i);
}
void TMathProcessing::_Scale2d()
{
    if (ActiveTask.arg.toInt() < 0)
        return;

    bool scale = false;

    for (int i = 0; i < Curves->size(); i++)
        if (i == ActiveTask.arg.toInt() && (*Curves)[i] && (*Curves)[i]->size()) {
            scale = true;
            QPair<double, double> _minmax;
//            if (vertical)
//                _minmax = minmax((*Curves)[i]->XSeq);
//            else
                _minmax = minmax((*Curves)[i]->YSeq);
            mi = _minmax.first;
            mx = _minmax.second;

            break;
        }

    if (!scale)
        return;

    for (int i = 0; i < Curves->size(); i++)
        if (i != ActiveTask.arg.toInt() && (*Curves)[i])
            _Scale2d(i);
}

void TMathProcessing::DoManualFitting(int N, double *YArray, long Size)
{
    if (MCCurve->dataSize() <= 1)
        return;

    int Index = 0;
    double XMin = MCCurve->sample(0).x();
    double YXMin = MCCurve->sample(0).y();
    double XMax = MCCurve->sample(MCCurve->dataSize()-1).x();
    double YXMax = MCCurve->sample(MCCurve->dataSize()-1).y();
    bool dir = (XMin > XMax);
    if (dir) {
        qSwap(XMin, XMax);
        qSwap(YXMax, YXMin);
    }

    for (long i = 0; i < Size; i++) {
        double xpos = (*Curves)[N]->getXat(i);
        if (xpos <= XMin) {
            YArray[i] -= YXMin;
        } else if (xpos >= XMax) {
            YArray[i] -= YXMax;
        } else {
            while (Index <= MCCurve->dataSize()-2 &&
                   (dir ?
                    xpos < MCCurve->sample(Index+1).x() :
                    xpos > MCCurve->sample(Index+1).x()) )
                Index++;

            double t = (xpos - MCCurve->sample(Index).x()) / (MCCurve->sample(Index + 1).x() - MCCurve->sample(Index).x());
            YArray[i] -= QLineF(MCCurve->sample(Index), MCCurve->sample(Index + 1)).pointAt(t).y();
        }
    }
}

double ajust (double min, double max, double min2, double max2, double val)
{
    if (max2 == min2)
        return val;

    return min + (max - min)*(val - min2)/(max2 - min2);
}

void TMathProcessing::_Scale2d(int N)
{
    long Size = (*Curves)[N]->size();
    double *YArray;
    YArray = (*Curves)[N]->YSeq.data();

    QPair<double, double> _minmax;
    _minmax = minmax((*Curves)[N]->YSeq);

    for (int i = 0; i < Size; i++)
        YArray[i] = ajust(mi, mx, _minmax.first, _minmax.second, YArray[i]);
}

void TMathProcessing::_Fit2d(int N)
{
    if (!(*Curves)[N] || ActiveTask.arg.toInt() == TPlotEnums::FO_None)
        return;

    long Size = (*Curves)[N]->size();
    double *YArray;
    if (vertical && (*Curves)[N]->XSeq.size() == (*Curves)[N]->YSeq.size())
    {
        YArray = (*Curves)[N]->XSeq.data();
    }
    else
        YArray = (*Curves)[N]->YSeq.data();


    if (ActiveTask.arg.toInt() == TPlotEnums::FO_Manual) DoManualFitting(N, YArray, Size);
    else {
        double *Coeffs = dataLinePartFitPolynom(YArray, ActiveTask.arg.toInt(), nullptr, 0, Size);
        dataLinePartSubtractPolynom(YArray, ActiveTask.arg.toInt(), Coeffs, 0, Size);
        delete [] Coeffs;
    }
}
void TMathProcessing::_Fit3d_XDir()
{
    if (ActiveTask.arg.toInt() < 0)
        return;

    for (int i = 0; i < Data->size(); i++)
        _Fit3d_XDir(*Data->at(i));
}
void TMathProcessing::_Fit3d_XDir(TMin3dData &Min3dData)
{
    unsigned int MinX = 0, MaxX = 0, MinY = 0, MaxY = 0;
    SetupMinsMaxs(MinX, MaxX, MinY, MaxY, Min3dData);

    for (int i = 0; i < Min3dData.Array.size(); i++)
    {
        double *Coeffs = dataLinePartFitPolynom(&*Min3dData.Array[i].begin(), ActiveTask.arg.toInt(), nullptr, MinX, MaxX);
        dataLinePartSubtractPolynom(&*Min3dData.Array[i].begin(), ActiveTask.arg.toInt(), Coeffs, MinX, MaxX);
        delete [] Coeffs;
        Coeffs = nullptr;
    }
}
void TMathProcessing::_Fit3d_YDir()
{
    if (ActiveTask.arg.toInt() < 0)
        return;

    for (int i = 0; i < Data->size(); i++)
        _Fit3d_YDir(*Data->at(i));
}
void TMathProcessing::_Fit3d_YDir(TMin3dData &Min3dData)
{
    unsigned int MinX = 0, MaxX = 0, MinY = 0, MaxY = 0;
    SetupMinsMaxs(MinX, MaxX, MinY, MaxY, Min3dData);

    double *YArray = new double [MaxY - MinY];
    for (unsigned int XCounter = MinX; XCounter < MaxX; XCounter++)
    {
        for (unsigned int YCounter = MinY; YCounter < MaxY; YCounter++)
            YArray[YCounter - MinY] = Min3dData.Array[YCounter][XCounter];

        double *Coeffs = dataLinePartFitPolynom(YArray, ActiveTask.arg.toInt(), nullptr, 0, MaxY - MinY);
        dataLinePartSubtractPolynom(YArray, ActiveTask.arg.toInt(), Coeffs, 0, MaxY - MinY);
        delete [] Coeffs;
        Coeffs = nullptr;

        for (unsigned int YCounter = MinY; YCounter < MaxY; YCounter++)
            Min3dData.Array[YCounter][XCounter] = YArray[YCounter - MinY];
    }
    delete [] YArray;
}
void TMathProcessing::_Subtr()
{
    if (ActiveTask.arg.toInt() < 0)
        return;

    for (int i = 0; i < Data->size(); i++)
        _Subtr(*Data->at(i));
}
void TMathProcessing::_Subtr(TMin3dData &Min3dData)
{
    unsigned int MinX = 0, MaxX = 0, MinY = 0, MaxY = 0;
    SetupMinsMaxs(MinX, MaxX, MinY, MaxY, Min3dData);

    double *Coeffs = dataFieldAreaFitPolynom(Min3dData.Array, MinY, MinX, MaxY - MinY, MaxX - MinX, ActiveTask.arg.toInt(), ActiveTask.arg.toInt(), nullptr);
    dataFieldAreaSubtractPolynom(Min3dData.Array, MinY, MinX, MaxY - MinY, MaxX - MinX, ActiveTask.arg.toInt(), ActiveTask.arg.toInt(), Coeffs);
    delete [] Coeffs;
}
void TMathProcessing::_LineCorrectByMediana()
{
    TPlotEnums::TAxis Direction = TPlotEnums::A_XAxis;
    if (ActiveTask.Action == TMathProcessing::LineMedianY)
        Direction = TPlotEnums::A_YAxis;

    for (int i = 0; i < Data->size(); i++) {
        unsigned int MinX = 0, MaxX = 0, MinY = 0, MaxY = 0;
        SetupMinsMaxs(MinX, MaxX, MinY, MaxY, *Data->at(i));
        TCorrectLinesByMedian(&Data->at(i)->Array, Direction, MinX, MinY, MaxX, MaxY);
    }
}
void TMathProcessing::_DoMeanValue()
{
    for (int i = 0; i < Data->size(); i++) {
        unsigned int MinX = 0, MaxX = 0, MinY = 0, MaxY = 0;
        SetupMinsMaxs(MinX, MaxX, MinY, MaxY, *Data->at(i));
        TMeanValue(&Data->at(i)->Array, ActiveTask.arg.toInt(), MinX, MinY, MaxX, MaxY);
    }
}
void TMathProcessing::_DoMedianValue()
{
    for (int i = 0; i < Data->size(); i++) {
        unsigned int MinX = 0, MaxX = 0, MinY = 0, MaxY = 0;
        SetupMinsMaxs(MinX, MaxX, MinY, MaxY, *Data->at(i));
        TMedianValue(&Data->at(i)->Array, ActiveTask.arg.toInt(), MinX, MinY, MaxX, MaxY);
    }
}
void TMathProcessing::_DoGaussian()
{
    for (int i = 0; i < Data->size(); i++) {
        unsigned int MinX = 0, MaxX = 0, MinY = 0, MaxY = 0;
        SetupMinsMaxs(MinX, MaxX, MinY, MaxY, *Data->at(i));
        TGaussian(&Data->at(i)->Array, ActiveTask.arg.toInt(), MinX, MinY, MaxX, MaxY);
    }
}
void TMathProcessing::SetupMinsMaxs(unsigned int &MinX, unsigned int &MaxX, unsigned int &MinY, unsigned int &MaxY, const TMin3dData &Min3dData)
{
    bool LimsCorrect = Min3dData.LimsForMathCorrect();
    MinX = LimsCorrect ? (unsigned int)Min3dData.LimsForMathX[0] : 0;
    MinY = LimsCorrect ? (unsigned int)Min3dData.LimsForMathY[0] : 0;
    MaxX = LimsCorrect ? (unsigned int)Min3dData.LimsForMathX[1] : Min3dData.NXElems;
    MaxY = LimsCorrect ? (unsigned int)Min3dData.LimsForMathY[1] : Min3dData.NYElems;
}
void TMathProcessing::DoFit(int N)
{
    TTask Task;
    Task.arg = N;
    Task.Action = TMathProcessing::Fit2d;
    SmartAddTask(Task);
}
void TMathProcessing::DoScale(int N)
{
    TTask Task;
    Task.arg = N;
    Task.Action = TMathProcessing::Scale2d;
    SmartAddTask(Task);
}
void TMathProcessing::DoFitX(int N)
{
    TTask Task;
    Task.arg = N;
    Task.Action = TMathProcessing::Fit3dX;
    SmartAddTask(Task);
}
void TMathProcessing::DoFitY(int N)
{
    TTask Task;
    Task.arg = N;
    Task.Action = TMathProcessing::Fit3dY;
    SmartAddTask(Task);
}
void TMathProcessing::DoSubtr(int N)
{
    TTask Task;
    Task.arg = N;
    Task.Action = TMathProcessing::Subtr;
    SmartAddTask(Task);
}
void TMathProcessing::DoLineCorrectByMedianaX()
{
    TTask Task;
    Task.Action = TMathProcessing::LineMedianX;
    SmartAddTask(Task);
}
void TMathProcessing::DoLineCorrectByMedianaY()
{
    TTask Task;
    Task.Action = TMathProcessing::LineMedianY;
    SmartAddTask(Task);
}
void TMathProcessing::DoMeanValue(int Sz)
{
    TTask Task;
    Task.Action = TMathProcessing::MeanValue;
    Task.arg = Sz;
    SmartAddTask(Task);
}
void TMathProcessing::DoMedianValue(int Sz)
{
    TTask Task;
    Task.Action = TMathProcessing::MedianValue;
    Task.arg = Sz;
    SmartAddTask(Task);
}
void TMathProcessing::DoGaussian(int Sz)
{
    TTask Task;
    Task.Action = TMathProcessing::Gaussian;
    Task.arg = Sz;
    SmartAddTask(Task);
}

}
}
