#pragma once

#include "2d/plot_2dplot.h"
#include "3d/plot_3dplot.h"
#include "2d/plot_2dpure.h"
#include "3d/plot_3dpure.h"

#include <QStackedWidget>

Q_DECLARE_METATYPE(plot::TCurveData)
Q_DECLARE_METATYPE(plot::TMin3dData)
Q_DECLARE_METATYPE(plot::TInfoForAllPlots)
Q_DECLARE_METATYPE(plot::TInfoFor2dPlot)
Q_DECLARE_METATYPE(plot::TInfoFor3dPlot)

Q_DECLARE_METATYPE(plot::TMetaStorage)
Q_DECLARE_METATYPE(QVector<double>)
Q_DECLARE_METATYPE(plot::TCurvesData)
Q_DECLARE_METATYPE(plot::TCurvesDataOwner)
Q_DECLARE_METATYPE(plot::T3dData)
Q_DECLARE_METATYPE(plot::T3dDataOwner)

void InitPlotIO();

namespace plot {

class TPlot: public QStackedWidget
{
    Q_OBJECT

signals:
    void AddDataToContainer(plot::T3dDataOwner, plot::TMetaStorage);
    void AddDataToContainer(plot::TCurvesDataOwner, plot::TMetaStorage);

public slots:
    void HideSelectionIfAny() 
    { 
        if (CurrentPlotType)
            ThreeDPlot->HideSelectionIfAny();
        else
        {
            TwoDPlot->HideSelectionIfAny(); 
#ifdef _ADROID
            TwoDPlotVertical->HideSelectionIfAny(); 
#endif
        }
    }
    void Refresh()
    {
        if (CurrentPlotType)
            ThreeDPlot->Refresh();
        else
        {
            TwoDPlot->Refresh();
#ifdef _ADROID
            TwoDPlotVertical->Refresh();
#endif
        }
    }

public:
    int CurrentPlotType;
    T2dPlot *TwoDPlot;
    #ifdef _ADROID
    T2dPlot *TwoDPlotVertical;
#endif
    T3dPlot *ThreeDPlot;

    //general
    enum TPlotType {_2d = 0, _3d};
    TPlot(QObject *Parent, QWidget *wparent);
    TPlot();
    void SwitchPlots(int PlotType)
    {
        if (PlotType < TPlot::_2d || PlotType > TPlot::_3d || CurrentPlotType == PlotType)
            return;
        setCurrentIndex(CurrentPlotType = PlotType);
        if (ThreeDPlot->showPlotControl)
            ThreeDPlot->PlotControl->setVisible(ThreeDPlot->isVisible());
    }
};

}
