#include "plot_widgets.h"

#include "../../src/gui/flowlayout.hpp"
#include <QSettings>

namespace plot {

void TMarkersInfo::GUICreate()
{
    setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);

    LXAUnit = new QLabel;
    LYAUnit = new QLabel;
    LXBUnit = new QLabel;
    LYBUnit = new QLabel;
    LDeltaXUnit = new QLabel;
    LDeltaYUnit = new QLabel;
    LDistanceUnit = new QLabel;
    LAngleUnit = new QLabel(QChar(0x00B0));

    QLabel *LXA = new QLabel(tr("x ="));
    LEdXA = new QLineEdit;
    LEdXA->setMaximumWidth(MaxWidth);
    LEdXA->setReadOnly(true);

    QLabel *LYA = new QLabel(tr("y ="));
    LEdYA = new QLineEdit;
    LEdYA->setMaximumWidth(MaxWidth);
    LEdYA->setReadOnly(true);

    QLabel *LXB = new QLabel(tr("x ="));
    LEdXB = new QLineEdit;
    LEdXB->setMaximumWidth(MaxWidth);
    LEdXB->setReadOnly(true);

    QLabel *LYB = new QLabel(tr("y ="));
    LEdYB = new QLineEdit;
    LEdYB->setMaximumWidth(MaxWidth);
    LEdYB->setReadOnly(true);

    QLabel *LDeltaX = new QLabel(QString(QChar(0x2206)) + tr("x ="));
    LEdDeltaX = new QLineEdit;
    LEdDeltaX->setMaximumWidth(MaxWidth);
    LEdDeltaX->setReadOnly(true);

    QLabel *LDeltaY = new QLabel(QString(QChar(0x2206)) + tr("y ="));
    LEdDeltaY = new QLineEdit;
    LEdDeltaY->setMaximumWidth(MaxWidth);
    LEdDeltaY->setReadOnly(true);

    LQv = new QLabel("Q =");
    LEdQ = new QLineEdit;
    LEdQ->setMaximumWidth(MaxWidth);
    LEdQ->setReadOnly(true);

    QLabel *LDistance = new QLabel(QObject::tr("Dist"));
    LEdDistance = new QLineEdit;
    LEdDistance->setMaximumWidth(MaxWidth);
    LEdDistance->setReadOnly(true);

    LAngleLbl = new QLabel(tr("Angle"));
    LEdAngle = new QLineEdit;
    LEdAngle->setMaximumWidth(MaxWidth);
    LEdAngle->setReadOnly(true);

    hide();
    setTitle(QObject::tr("Detailed markers info"));

    QGroupBox *GrBMarkerA = new QGroupBox(QObject::tr("Marker A"));
    QGroupBox *GrBMarkerB = new QGroupBox(QObject::tr("Marker B"));
    QGroupBox *GLBox = new QGroupBox(QObject::tr("Extra Info"));

    QVBoxLayout *MainLayoutForGrB = new QVBoxLayout(this);
    MainLayoutForGrB->setContentsMargins(6, 6, 6, 6);
    MainLayoutForGrB->setSpacing(6);
    MainLayoutForGrB->addWidget(GrBMarkerA);
    MainLayoutForGrB->addWidget(GrBMarkerB);
    MainLayoutForGrB->addWidget(GLBox);

    GLMarkerA = new QGridLayout(GrBMarkerA);
    GLMarkerA->setSpacing(6);

    GLMarkerA->addWidget(LXA, 0, 0, Qt::AlignRight);
    GLMarkerA->addWidget(LEdXA, 0, 1);
    GLMarkerA->addWidget(LXAUnit, 0, 2);

    GLMarkerA->addWidget(LYA, 1, 0, Qt::AlignRight);
    GLMarkerA->addWidget(LEdYA, 1, 1);
    GLMarkerA->addWidget(LYAUnit, 1, 2);

    GLMarkerB = new QGridLayout(GrBMarkerB);
    GLMarkerB->setSpacing(6);

    GLMarkerB->addWidget(LXB, 0, 0, Qt::AlignRight);
    GLMarkerB->addWidget(LEdXB, 0, 1);
    GLMarkerB->addWidget(LXBUnit, 0, 2);

    GLMarkerB->addWidget(LYB, 1, 0, Qt::AlignRight);
    GLMarkerB->addWidget(LEdYB, 1, 1);
    GLMarkerB->addWidget(LYBUnit, 1, 2);

    GLDownL = new QGridLayout(GLBox);
    GLDownL->setSpacing(10);

    GLDownL->addWidget(LDeltaX, 0, 0, Qt::AlignRight);
    GLDownL->addWidget(LEdDeltaX, 0, 1);
    GLDownL->addWidget(LDeltaXUnit, 0, 2);

    GLDownL->addWidget(LDeltaY, 1, 0, Qt::AlignRight);
    GLDownL->addWidget(LEdDeltaY, 1, 1);
    GLDownL->addWidget(LDeltaYUnit, 1, 2);

    GLDownL->addWidget(LQv, 2, 0, Qt::AlignRight);
    GLDownL->addWidget(LEdQ, 2, 1);

    GLDownL->addWidget(LDistance, 4, 0, Qt::AlignRight);
    GLDownL->addWidget(LEdDistance, 4, 1);
    GLDownL->addWidget(LDistanceUnit, 4, 2);

    GLDownL->addWidget(LAngleLbl, 5, 0, Qt::AlignRight);
    GLDownL->addWidget(LEdAngle, 5, 1);
    GLDownL->addWidget(LAngleUnit, 5, 2);
}

//================================================================================================================================
void TCommonPlotControl::GUICreate()
{
    BDetailedMarkersInfo = new QPushButton;
    BDetailedMarkersInfo->setIcon(makeHugeIcon(":/plot/markers_info_1.png"));
    BDetailedMarkersInfo->setCheckable(true);
    BDetailedMarkersInfo->setToolTip(QObject::tr("Detailed markers info"));
    BDetailedMarkersInfo->setObjectName(QString::fromUtf8("BDetailedMarkersInfo"));
    BDetailedMarkersInfo->hide();

    BMarkers = new QPushButton;
    BMarkers->setIcon(makeHugeIcon(":/plot/markers.png"));
    BMarkers->setCheckable(true);
    BMarkers->setToolTip(QObject::tr("Markers"));
    BMarkers->setObjectName(QString::fromUtf8("BMarkers"));

    BAngle = new QPushButton;
    BAngle->setIcon(makeHugeIcon(":/plot/angle.png"));
    BAngle->setCheckable(true);
    BAngle->setToolTip(QObject::tr("Angular measurement"));
    BAngle->setObjectName(QString::fromUtf8("BAngle"));
    BAngle->hide();

    BGrid = new QPushButton;
    BGrid->setIcon(makeHugeIcon(":/plot/grid.png"));
    BGrid->setCheckable(true);
    BGrid->setToolTip(QObject::tr("Show Grid"));
    BGrid->setObjectName(QString::fromUtf8("BGrid"));

    BFixX = new QPushButton;
    BFixX->setIcon(makeHugeIcon(":/plot/fix_x.png"));
    BFixX->setCheckable(true);
    BFixX->setToolTip(QObject::tr("Fix X direction at Zoom"));

    BFixY = new QPushButton;
    BFixY->setIcon(makeHugeIcon(":/plot/fix_y.png"));
    BFixY->setCheckable(true);
    BFixY->setToolTip(QObject::tr("Fix Y direction at Zoom"));

    BRestoreScale = new QPushButton;
    BRestoreScale->setIcon(makeHugeIcon(":/plot/restore_scale.png"));
    BRestoreScale->setToolTip(QObject::tr("Restore scale"));
    BRestoreScale->setObjectName(QString::fromUtf8("BRestoreScale"));

    BSelectByRect = new QPushButton;
    BSelectByRect->setIcon(makeHugeIcon(":/plot/selection.png"));
    BSelectByRect->setToolTip(QObject::tr("Area selection"));
    BSelectByRect->setCheckable(true);
    BSelectByRect->setDefault(false);

    BExpandSelectRect = new QPushButton;
    BExpandSelectRect->setIcon(makeHugeIcon(":/plot/selection_full_range.png"));
    BExpandSelectRect->setToolTip(QObject::tr("Expand selection rectangle"));
    BExpandSelectRect->hide();

    BApply = new QPushButton;
    BApply->setIcon(makeHugeIcon(":/plot/apply_selection.png"));
    BApply->setToolTip(QObject::tr("Apply area selection") );// + " (Enter)");
    //BApply->setShortcut(QKeySequence(Qt::Key_Enter) | QKeySequence(Qt::Key_Return));
    BApply->hide();

    BCancel = new QPushButton;
    BCancel->setIcon(makeHugeIcon(":/plot/cancel_selection.png"));
    BCancel->setToolTip(QObject::tr("Cancel area selection") + " (Escape)");
    BCancel->hide();

    BAutoScale = new QPushButton; //TODO move to 2d?
    BAutoScale->setIcon(makeHugeIcon(":/plot/allow_deny_autoscale.png"));
    BAutoScale->setToolTip(QObject::tr("Fit data size"));
    BAutoScale->setCheckable(true);

    TBMathProcessing = new QToolButton;
    TBMathProcessing->setIcon(makeHugeIcon(":/plot/math.png"));
    TBMathProcessing->setToolTip(QObject::tr("Filters"));
    TBMathProcessing->setPopupMode(QToolButton::InstantPopup);
    MMathProcessing = new QMenu;
    TBMathProcessing->setMenu(MMathProcessing);

    BSavePlotPixmap = new QPushButton;
    BSavePlotPixmap->setIcon(makeHugeIcon(":/snap_shot.png"));
    BSavePlotPixmap->setToolTip(QObject::tr("Image snapshot") + " (Ctrl+P)");
    BSavePlotPixmap->setShortcut(QKeySequence(Qt::CTRL | Qt::Key_P));

    BSave = new QPushButton;
    BSave->setIcon(makeHugeIcon(":/plot/Attach.png"));
    BSave->setToolTip(QObject::tr("Send to Data") + " (Ctrl+S)");
    BSave->setObjectName(QString::fromUtf8("BSave"));
    BSave->setShortcut(QKeySequence(Qt::CTRL | Qt::Key_S));

    TBFontSize = new QToolButton;
    TBFontSize->setIcon(makeHugeIcon(":/plot/fontsize.png"));
    TBFontSize->setToolTip(QObject::tr("Font size"));
    TBFontSize->setPopupMode(QToolButton::InstantPopup);
    QMenu *MFontSize = new QMenu;
    TBFontSize->setMenu(MFontSize);

    AGFontSizes = new QActionGroup(this);
    AGFontSizes->addAction(MFontSize->addAction(QString::number(6)));
    AGFontSizes->actions()[0]->setCheckable(true);
    AGFontSizes->addAction(MFontSize->addAction(QString::number(8)));
    AGFontSizes->actions()[1]->setCheckable(true);
    AGFontSizes->addAction(MFontSize->addAction(QString::number(10)));
    AGFontSizes->actions()[2]->setCheckable(true);

    for (int i = 0; i < 11; i++) {
        AGFontSizes->addAction(MFontSize->addAction(QString::number(12 + 4*i)));
        AGFontSizes->actions()[i+3]->setCheckable(true);
    }
    AGFontSizes->actions()[3]->setChecked(true);

    ControlMainLayout = new HFlowLayout(this);
    ControlMainLayout->setContentsMargins(3, 3, 3, 3);
    ControlMainLayout->spaceX = 5; ControlMainLayout->spaceY = 5;
    ControlMainLayout->addWidget(BMarkers);
    ControlMainLayout->addWidget(BDetailedMarkersInfo);
    ControlMainLayout->addWidget(BAngle);
    ControlMainLayout->addWidget(BGrid);
    addDelimeterH(ControlMainLayout, BMarkers->sizeHint().width());

    ControlMainLayout->addWidget(BFixX);
    ControlMainLayout->addWidget(BFixY);
    ControlMainLayout->addWidget(BRestoreScale);
    ControlMainLayout->addWidget(BAutoScale);
    ControlMainLayout->addWidget(BCancel);
    ControlMainLayout->addWidget(BApply);
    ControlMainLayout->addWidget(BSelectByRect);
    ControlMainLayout->addWidget(BExpandSelectRect);
    addDelimeterH(ControlMainLayout, BMarkers->sizeHint().width());

    ControlMainLayout->addWidget(TBMathProcessing);
    addDelimeterH(ControlMainLayout, BMarkers->sizeHint().width());

    ControlMainLayout->addWidget(BSavePlotPixmap);
    ControlMainLayout->addWidget(BSave);

    connect(BAutoScale, SIGNAL(toggled(bool)), this, SLOT(on_BAutoScale_toggled()));
    connect(BFixX, SIGNAL(toggled(bool)), this, SLOT(on_BFixX_toggled()));
    connect(BFixY, SIGNAL(toggled(bool)), this, SLOT(on_BFixY_toggled()));
    connect(BFixX, SIGNAL(clicked()), this, SLOT(on_BFixX_clicked()));
    connect(BFixY, SIGNAL(clicked()), this, SLOT(on_BFixY_clicked()));
}

void TCommonPlotControl::SetSelectionModeEnabled(bool State)
{
    if (!State) {//disable first, enable after. keeps button count minimum
        BExpandSelectRect->setVisible(State);
        BApplyAndCancelSetVisible(State);
    }

    BSelectByRect->setVisible(!State);

    if (State) {
        BExpandSelectRect->setVisible(State);
        BApplyAndCancelSetVisible(State);
    }
}
void TCommonPlotControl::BApplyAndCancelSetVisible(bool State)
{
    BApply->setVisible(State);
    BCancel->setVisible(State);
}
void TCommonPlotControl::on_BFixX_clicked()
{
    if (BFixY->isChecked())
        BFixY->setChecked(false);
}
void TCommonPlotControl::on_BFixX_toggled()
{
    BFixX->setIcon(makeHugeIcon(BFixX->isChecked() ? ":/plot/fix_x_on.png" :
                                                     ":/plot/fix_x.png"));
}
void TCommonPlotControl::on_BFixY_clicked()
{
    if (BFixX->isChecked())
        BFixX->setChecked(false);
}
void TCommonPlotControl::on_BFixY_toggled()
{
    BFixY->setIcon(makeHugeIcon(BFixY->isChecked() ? ":/plot/fix_y_on.png" :
                                                     ":/plot/fix_y.png"));
}
void TCommonPlotControl::on_BAutoScale_toggled()
{
    bool Enable = !BAutoScale->isChecked();

    if (BAutoScale->isChecked())
        BAutoScale->setChecked(false);


//    BAutoScale->setIcon(makeHugeIcon(Enable ? ":/plot/allow_deny_autoscale.png" :
//                                              ":/plot/allow_deny_autoscale_on.png"));
}

}
