#include <QObject>
#include <QWidget>
#include "widgets/threedoverlaybuttons.h"
#include "3d/plot_3dplot.h"
#include "3d/plot_3dpure.h"
#include "../../src/gui/controls.h"
#include "qwt_plot_layout.h"
#include "core/plot_core.h"
#include <QAbstractItemView>

namespace plot {

ThreeDOverlayButtons::ThreeDOverlayButtons() : QWidget(nullptr)
{
    QSettings settings2("Param/gui.conf", QSettings::IniFormat);
    useBlockResize = settings2.value("gui/Ratis_xyz_stage_mode", false).toBool();


    installEventFilter(this);

    BPointSelection = new TwoStateButton(":/plot/point.png", ":/plot/pointOn.png");
    BPointSelection->setToolTip(QObject::tr("Point selection"));
    BPointSelection->setFlat(true);
    setButtonStyle(BPointSelection);
    BPointSelection->hide();
    connect(BPointSelection, SIGNAL(state_changed(bool)), this, SLOT(onPointSelection_changed(bool)));


    BMarkers = new TwoStateButton(":/plot/cross_section.png", ":/plot/cross_section.png");
    BMarkers->setToolTip(QObject::tr("Markers"));
    BMarkers->setFlat(true);
    setButtonStyle(BMarkers);
    BMarkers->hide();
    connect(BMarkers, SIGNAL(state_changed(bool)), this, SLOT(onMarkers_changed(bool)));


    BFilters = new TwoStateButton(":/ScanPlot/filterPlot2D.png", ":/ScanPlot/filterPlot2D.png");
    BFilters->setToolTip(QObject::tr("Enable filters"));
    BFilters->setFlat(true);
    setButtonStyle(BFilters);
    BFilters->hide();
    connect(BFilters, SIGNAL(state_changed(bool)), this, SLOT(on_Filter_changed(bool)));

    BPalette = new QPushButton();
    BPalette->hide();
    BPalette->setFlat(true);
    setButtonStyle(BPalette);
    BPalette->setIcon(makeHugeIcon(":/plot/palettePlot.png"));
    BPalette->setToolTip(QObject::tr("Palettes"));
    connect(BPalette, SIGNAL(released()), this, SLOT(colorPickClicked()));

    BAreaZzoom = new TwoStateButton(":/plot/Pall_Zoom_Of.png", ":/plot/Pall_Zoom_On.png");
    BAreaZzoom->setToolTip(QObject::tr("Set Z range by selection"));
    BAreaZzoom->setFlat(true);
    setButtonStyle(BAreaZzoom);
    BAreaZzoom->hide();
    connect(BAreaZzoom, SIGNAL(state_changed(bool)), this, SLOT(on3DBAreaZoom_changed(bool)));


    BSaveImage = new QPushButton();
    BSaveImage->hide();
    BSaveImage->setFlat(true);
    setButtonStyle(BSaveImage);
    BSaveImage->setIcon(makeHugeIcon(":/plot/picturePlot2D.png"));
    BSaveImage->setToolTip(QObject::tr("Image snapshot"));
    connect(BSaveImage, SIGNAL(released()), this, SLOT(onSaveImage_clicked()));

    BBackButton = new QPushButton();
    BBackButton->hide();
    BBackButton->setFlat(true);
    setButtonStyle(BBackButton);
    BBackButton->setIcon(makeHugeIcon(":/ScanPlot/backPlot3D.png"));
    BBackButton->setToolTip(QObject::tr("Cancel"));
    BBackButton->setShortcut(QKeySequence(Qt::Key_Escape));
    connect(BBackButton, SIGNAL(released()), this, SLOT(onBack_clicked()));

    BApplyButton = new QPushButton();
    BApplyButton->hide();
    BApplyButton->setFlat(true);
    setButtonStyle(BApplyButton);
    BApplyButton->setIcon(makeHugeIcon(":/ScanPlot/applyPlot.png"));
    BApplyButton->setToolTip(QObject::tr("Apply"));
    //BApplyButton->setShortcut(QKeySequence(Qt::Key_Enter));
    //connect(BApplyButton, SIGNAL(released()), this, SLOT(bApply_clicked()));
    connect(BApplyButton, &QPushButton::clicked, this, &ThreeDOverlayButtons::bApply_clicked);

    BOldFilter = new QPushButton();
    BOldFilter->hide();
    BOldFilter->setFlat(true);
    setButtonStyle(BOldFilter);
    BOldFilter->setIcon(makeHugeIcon(":/plot/math.png"));
    BOldFilter->setToolTip(QObject::tr("Filters"));

    BOldFilterAddit = new QPushButton();
    BOldFilterAddit->hide();
    BOldFilterAddit->setFlat(true);
    setButtonStyle(BOldFilterAddit);
    BOldFilterAddit->setIcon(makeHugeIcon(":/plot/math_extra.png"));
    BOldFilterAddit->setToolTip(QObject::tr("Addition math"));

    LFont = new QLabel(tr("Font size:"));
    LFont->setStyleSheet("QLabel { color: rgb(100,100,100)}");
    LFont->hide();

    CBFont = new QComboBox;
    CBFont->hide();
    CBFont->setStyleSheet("QComboBox {border-width:0; background-color:rgba(255,255,255,100);}\
                          QComboBox:pressed { background-color:rgba(255,255,255,100); border-width:1;}");
    CBFont->addItem("6", 6);
    CBFont->addItem("8", 8);
    CBFont->addItem("10", 10);
    for (int i = 0; i < 11; i++) {
        CBFont->addItem(QString::number(12 + 4*i), 12 + 4*i);
    }
    CBFont->setCurrentIndex(3);
    connect(CBFont, SIGNAL(currentIndexChanged(int)), this, SLOT(fontChanged(int)));

    CreateActionsAndMenus();

    //--------------------------------------------------------------------------------

    rightBtnsList << BPalette << BAreaZzoom << BSaveImage;
    upBtnsList << BPointSelection << BMarkers << BFilters;
    allBtnsList << BBackButton << BApplyButton << rightBtnsList << upBtnsList;
}

void ThreeDOverlayButtons::fontChanged(int n)
{
    parentPlot->PlotControl->AGFontSizes->actions().at(n)->trigger();
}

void ThreeDOverlayButtons::CreateActionsAndMenus()
{
    QMenu *MXFitOrders = new QMenu(tr("X Fitting"));
    QMenu *MYFitOrders = new QMenu(tr("Y Fitting"));
    QMenu *MSubtr = new QMenu(tr("Subtract"));

    AGXFitOrders = new QActionGroup(this);
    AGYFitOrders = new QActionGroup(this);
    AGSubtr      = new QActionGroup(this);

    const QString FitOrders[] = {tr("None"), tr("0 order"), tr("1st order"), tr("2nd order"), tr("3rd order"), tr("4th order"), tr("5th order")};
    for (auto &fitOrder:FitOrders)
    {
        AGXFitOrders->addAction(MXFitOrders->addAction(fitOrder))->setCheckable(true);
        AGYFitOrders->addAction(MYFitOrders->addAction(fitOrder))->setCheckable(true);
        AGSubtr->addAction(MSubtr->addAction(fitOrder))->setCheckable(true);
    }
    AGXFitOrders->actions()[0]->setChecked(true); //CurrentXFitOrder = -1;
    AGYFitOrders->actions()[0]->setChecked(true); //CurrentYFitOrder = -1;
    AGSubtr     ->actions()[0]->setChecked(true); //CurrentSubtrOrder = -1;

    //=================================================QMenu's=================================================
    MMathProcessing = new QMenu;
    MMathProcessing->addAction(tr("Fit Lines X"))->setCheckable(true);
    MMathProcessing->addAction(tr("Fit Lines Y"))->setCheckable(true);
    MMathProcessing->addMenu(MSubtr);
    BOldFilter->setMenu(MMathProcessing);

    MMath2 = new QMenu;
    MMath2->addMenu(MXFitOrders);
    MMath2->addMenu(MYFitOrders);
    MMath2->addSeparator();
    MMath2->addAction(T3dPlotControl::NameOfMeanValueFilter())->setCheckable(true);
    MMath2->addAction(T3dPlotControl::NameOfMedianValueFilter())->setCheckable(true);
    MMath2->addAction(T3dPlotControl::NameOfGaussianFilter())->setCheckable(true);
    MMath2->addAction(T3dPlotControl::NameOfHistogramFilter())->setCheckable(true);
    BOldFilterAddit->setMenu(MMath2);
}

void ThreeDOverlayButtons::on_GwyXFit_triggered(bool Checked)
{
    int n = parentPlot->ThreeDPlots.indexOf((plot::TPure3dPlot*)BOldFilter->parent());
    if (n == -1) n = 0;
    parentPlot->SetActivePlot(n);
    parentPlot->ThreeDPlots[n]->MakeCorrectLineByMedianaXDir(Checked);
    parentPlot->PlotControl->MMathProcessing->actions()[0]->setChecked(Checked);
    if (attributes)
        attributes->insert("Fit_lines_x", parentPlot->PlotControl->MMathProcessing->actions()[0]->isChecked());
}
void ThreeDOverlayButtons::on_GwyYFit_triggered(bool Checked)
{
    int n = parentPlot->ThreeDPlots.indexOf((plot::TPure3dPlot*)BOldFilter->parent());
    if (n == -1) n = 0;
    parentPlot->SetActivePlot(n);
    parentPlot->ThreeDPlots[n]->MakeCorrectLineByMedianaYDir(Checked);
    parentPlot->PlotControl->MMathProcessing->actions()[1]->setChecked(Checked);
    if (attributes)
        attributes->insert("Fit_lines_y", parentPlot->PlotControl->MMathProcessing->actions()[1]->isChecked());
}

void ThreeDOverlayButtons::on_MeanValue_triggered(bool Checked)
{
    int n = parentPlot->ThreeDPlots.indexOf((plot::TPure3dPlot*)BOldFilter->parent());
    if (n == -1) n = 0;
    parentPlot->SetActivePlot(n);
    auto name = T3dPlotControl::NameOfMeanValueFilter();
    int id = T3dPlotControl::GetNumOfAction(name);
    auto plot = parentPlot->ThreeDPlots[n];
    auto plotControl = parentPlot->PlotControl;
    auto plotAction = plotControl->MMath2->actions()[id];
    auto overAction = MMath2->actions()[id];

    plot->MeanValueFilterEnabled = Checked;
    #ifndef _ADROID
    plotAction->setChecked(!Checked);
    #endif
    parentPlot->on_MeanValue_triggered(Checked);
    Checked = plotAction->isChecked();
    plot->MeanValueFilterEnabled = Checked;

    T3dPlot::SetFilterSizeTextInAction(overAction,
                              plot->MeanValueFilterSize, name);
    if (plot->MeanValueFilterSize == 0)
        overAction->setChecked(false);
    if (attributes)
        attributes->insert("mean_filter", plot->MeanValueFilterSize);
}
void ThreeDOverlayButtons::on_MedianValue_triggered(bool Checked)
{
    int n = parentPlot->ThreeDPlots.indexOf((plot::TPure3dPlot*)BOldFilter->parent());
    if (n == -1) n = 0;
    parentPlot->SetActivePlot(n);
    auto name = T3dPlotControl::NameOfMedianValueFilter();
    int id = T3dPlotControl::GetNumOfAction(name);
    auto plot = parentPlot->ThreeDPlots[n];
    auto plotControl = parentPlot->PlotControl;
    auto plotAction = plotControl->MMath2->actions()[id];
    auto overAction = MMath2->actions()[id];

    plot->MedianValueFilterEnabled = Checked;
    #ifndef _ADROID
    plotAction->setChecked(!Checked);
    #endif
    parentPlot->on_MedianValue_triggered(Checked);
    Checked = plotAction->isChecked();
    plot->MedianValueFilterEnabled = Checked;

    T3dPlot::SetFilterSizeTextInAction(overAction,
                              plot->MedianValueFilterSize, name);
    if (plot->MedianValueFilterSize == 0)
        overAction->setChecked(false);
    if (attributes)
        attributes->insert("median_filter", plot->MedianValueFilterSize);
}
void ThreeDOverlayButtons::on_Gaussian_triggered(bool Checked)
{
    int n = parentPlot->ThreeDPlots.indexOf((plot::TPure3dPlot*)BOldFilter->parent());
    if (n == -1) n = 0;
    parentPlot->SetActivePlot(n);
    auto name = T3dPlotControl::NameOfGaussianFilter();
    int id = T3dPlotControl::GetNumOfAction(name);
    auto plot = parentPlot->ThreeDPlots[n];
    auto plotControl = parentPlot->PlotControl;
    auto plotAction = plotControl->MMath2->actions()[id];
    auto overAction = MMath2->actions()[id];

    plot->GaussianValueFilterEnabled = Checked;
    #ifndef _ADROID
    plotAction->setChecked(!Checked);
    #endif
    parentPlot->on_Gaussian_triggered(Checked);
    Checked = plotAction->isChecked();
    plot->GaussianValueFilterEnabled = Checked;

    T3dPlot::SetFilterSizeTextInAction(overAction,
                              plot->GaussianFilterSize, name);
    if (plot->GaussianFilterSize == 0)
        overAction->setChecked(false);
    if (attributes)
        attributes->insert("gaussian_filter", plot->GaussianFilterSize);
}
void ThreeDOverlayButtons::on_Histogram_triggered(bool Checked)
{
    auto name = T3dPlotControl::NameOfHistogramFilter();
    int id = T3dPlotControl::GetNumOfAction(name);

    parentPlot->PlotControl->MMath2->actions()[id]->trigger();
    parentPlot->PlotControl->MMath2->actions()[id]->setChecked(Checked);

    T3dPlot::SetFilterSizeTextInAction(MMath2->actions()[id], 0, name);
    if (attributes)
        attributes->insert("histogram_filter", Checked);
}

void ThreeDOverlayButtons::on_FitXOrder_changed()
{
    int id = FindCheckedItemIn(AGXFitOrders);
    parentPlot->PlotControl->AGXFitOrders->actions()[id]->setChecked(true);
    parentPlot->PlotControl->AGXFitOrders->actions()[id]->trigger();
    if (attributes)
        attributes->insert("x_fitting", id);
}
void ThreeDOverlayButtons::on_FitYOrder_changed()
{
    int id = FindCheckedItemIn(AGYFitOrders);
    parentPlot->PlotControl->AGYFitOrders->actions()[id]->setChecked(true);
    parentPlot->PlotControl->AGYFitOrders->actions()[id]->trigger();
    if (attributes)
        attributes->insert("y_fitting", id);
}
void ThreeDOverlayButtons::on_SubtrOrder_changed()
{
    int id = FindCheckedItemIn(AGSubtr);
    parentPlot->PlotControl->AGSubtr->actions()[id]->trigger();
    if (attributes)
        attributes->insert("substract_order", id);
}

void ThreeDOverlayButtons::setParentPlot(T3dPlot *plot)
{
    parentPlot = plot;
    connect(parentPlot->PlotControl->BCross, SIGNAL(state_changed(bool)), BPointSelection, SLOT(change_state_man(bool)));
    connect(parentPlot->PlotControl->BMarkers, SIGNAL(toggled(bool)), BMarkers, SLOT(change_state_man(bool)));
    connect(parentPlot->PlotControl->BAreaZzoom, SIGNAL(state_changed(bool)), BAreaZzoom, SLOT(change_state_man(bool)));

    connect(MMathProcessing->actions()[0], SIGNAL(toggled(bool)), this, SLOT(on_GwyXFit_triggered(bool)));
    connect(MMathProcessing->actions()[1], SIGNAL(toggled(bool)), this, SLOT(on_GwyYFit_triggered(bool)));

    QString strMean = plot::T3dPlotControl::NameOfMeanValueFilter();
    QString strMedian = plot::T3dPlotControl::NameOfMedianValueFilter();
    QString strGauss = plot::T3dPlotControl::NameOfGaussianFilter();
    QString strHistogram = plot::T3dPlotControl::NameOfHistogramFilter();
    int idMean = plot::T3dPlotControl::GetNumOfAction(strMean);
    int idMedian = plot::T3dPlotControl::GetNumOfAction(strMedian);
    int idGauss = plot::T3dPlotControl::GetNumOfAction(strGauss);
    int idHistogram = plot::T3dPlotControl::GetNumOfAction(strHistogram);

    connect(MMath2->actions()[idMean],
            SIGNAL(triggered(bool)), this, SLOT(on_MeanValue_triggered(bool)));
    connect(MMath2->actions()[idMedian],
            SIGNAL(triggered(bool)), this, SLOT(on_MedianValue_triggered(bool)));
    connect(MMath2->actions()[idGauss],
            SIGNAL(triggered(bool)), this, SLOT(on_Gaussian_triggered(bool)));
    connect(MMath2->actions()[idHistogram],
            SIGNAL(toggled(bool)),   this, SLOT(on_Histogram_triggered(bool)));

    connect(AGXFitOrders, SIGNAL(triggered(QAction*)), this, SLOT(on_FitXOrder_changed()));
    connect(AGYFitOrders, SIGNAL(triggered(QAction*)), this, SLOT(on_FitYOrder_changed()));

    connect(AGSubtr, SIGNAL(triggered(QAction*)), this, SLOT(on_SubtrOrder_changed()));

    parentPlot->ThreeDPlots[0]->colorChooser->installEventFilter(this);
    plot->installEventFilter(this);

    connect(parentPlot->PlotControl->BCancel, SIGNAL(clicked()), BBackButton, SLOT(click()));
    connect(parentPlot->PlotControl->BApply, SIGNAL(clicked()), BApplyButton, SLOT(click()));
}

void ThreeDOverlayButtons::scanButtonsSetVisibleGUI(bool visible, plot::TPure3dPlot* plotPure, bool reSetButtonsParent)
{
    int n = 0;
    if (plotPure)
        n = plotPure->Child->ThreeDPlots.indexOf(plotPure);

    if (!enabled || (n != -1 && visibilityMode[n] == svm_nothing_show))
        visible = false;

    if (setActivePlotByMouseMove)
    {
        if (visible && plotPure != nullptr)
            parentPlot->SetActivePlot(parentPlot->ThreeDPlots.indexOf(plotPure));
    }


    if (visible && blockShowing3DPlotButtons)
        return;

    blockLeaveEvent = true;

    if (plotPure != nullptr)
    {
        if (visible && reSetButtonsParent)
        {
            for (int i=0; i<allBtnsList.size(); i++)
            {
                allBtnsList[i]->setParent(plotPure);
            }
            LFont->setParent(plotPure);
            CBFont->setParent(plotPure);
            ChannelResized(n, plotPure->size());
        }

        if (visible)
        {
            BBackButton->setVisible(showingBackButtonPlot[n]);
            BApplyButton->setVisible(showingApplyButtonPlot[n]);
            #ifndef _ADROID
            setButtonActive(BFilters, plotPure->isFiltering);
            BFilters->change_state_man(plotPure->isFiltering);
            #endif
        }
        BMarkers->change_state_man(plotPure->BMarkersIsChecked);
        BAreaZzoom->change_state_man(plotPure->AreaZzoom);

        if (visible && (
                  visibilityMode[n] != svm_none ||
                  BBackButton->isVisible()))
        {
            LFont->setVisible(plotPure->colorChooser->isVisible());
            CBFont->setVisible(plotPure->colorChooser->isVisible());
            blockLeaveEvent = false;
            emit GUIPlotSetVisible(n, visible, visibilityMode[n]);
            return;
        }
    }

    for (int i=0; i<allBtnsList.size(); i++)
    {
        allBtnsList[i]->setVisible(visible);
    }

    if (visible)
    {
        BBackButton->setVisible(showingBackButtonPlot[n]);
        BApplyButton->setVisible(showingApplyButtonPlot[n]);
    }

    BFilters->setVisible(!hideFilterButton && visible);
    emit GUIPlotSetVisible(n, visible, visibilityMode[n]);
    blockLeaveEvent = false;
}
void ThreeDOverlayButtons::ChannelResized(int n, QSize newSize)
{
    if (useBlockResize && blockResizeTimes > 0) {blockResizeTimes--; return;}
    if (replacedGUIButtons[n])
    {
        emit replacedGUIPlotResized(n, newSize);
        return;
    }

    int leftX = parentPlot->ThreeDPlots[n]->QPlot->plotLayout()->scaleRect(QwtPlot::yLeft).width()*1.15;
    int width = newSize.width() - leftX;
    if (width <= 0) width = 1;
    int height = newSize.height();
    if (height <= 0) height = 1;
    int minBtnsDistBetw = width*0.007;
    if (minBtnsDistBetw == 0)
        minBtnsDistBetw = 1;
    int upY = parentPlot->ThreeDPlots[n]->QPlot->plotLayout()->titleRect().height();
    int upYPerc = upY*0.35;
    upY += upYPerc;
    int upNum = upBtnsList.size() + ((upBtnsList.size() >= 6) ? 0 : 1);
    int UpButtonSize = width/upNum - (upNum)*minBtnsDistBetw;
    //UpButtonSize *= 1.6;//btnSizeMultip;
    int btnsDistBetw = (width - UpButtonSize*upNum)/(upNum-1);
    btnsDistBetw *= 0.95;
    if (UpButtonSize < 0) UpButtonSize = 1;
    if (UpButtonSize > 150) UpButtonSize = 150;

    bool longVertical = false;
    bool longHorizontal = false;
    bool smallWidth = false;
    if (newSize.width() > height*1.3)
        longHorizontal = true;
    if (height > newSize.width()*1.3)
        longVertical = true;
    if (height > newSize.width()*1.6)
        smallWidth = true;

    if (longVertical || longHorizontal)
    {
        UpButtonSize = 60;
    }

    if (width < UpButtonSize*upBtnsList.size())
        UpButtonSize = minBtnsSize;

    if (longHorizontal)
        btnsDistBetw = UpButtonSize*0.3;
    if (longVertical && !smallWidth)
    {
        btnsDistBetw = (width - UpButtonSize*upNum)/(upNum-1);
        btnsDistBetw *= 0.95;
    }

    for (int i=0; i<allBtnsList.size(); i++)
    {
        allBtnsList[i]->setFixedSize(UpButtonSize, UpButtonSize);
        allBtnsList[i]->setIconSize(QSize(UpButtonSize, UpButtonSize));
    }

    BBackButton->setFixedSize(UpButtonSize, UpButtonSize);
    BBackButton->setIconSize(QSize(UpButtonSize, UpButtonSize));
    BApplyButton->setFixedSize(UpButtonSize, UpButtonSize);
    BApplyButton->setIconSize(QSize(UpButtonSize, UpButtonSize));

    int downYH = parentPlot->ThreeDPlots[n]->QPlot->plotLayout()->scaleRect(QwtPlot::xBottom).height();

    int x = leftX;
    int y = upY;

    BBackButton->move(leftX, upY);
    BApplyButton->move(leftX + UpButtonSize + btnsDistBetw, upY);

    for (int i=0; i<upBtnsList.size(); i++)
    {
        upBtnsList[i]->move(x, y);
        if (smallWidth)
            y += UpButtonSize + btnsDistBetw;
        else
            x += UpButtonSize + btnsDistBetw;
    }

    y = upY;
    if (upBtnsList.size() == 6 && !smallWidth)
    {
        y += UpButtonSize + btnsDistBetw;
        x = upBtnsList.last()->x();
    }
    if (smallWidth)
        x = newSize.width() - UpButtonSize - btnsDistBetw;

    for (int i=0; i<rightBtnsList.size(); i++)
    {
        rightBtnsList[i]->move(x, y);
        if (longHorizontal)
            x += UpButtonSize + btnsDistBetw;
        else
            y += UpButtonSize + btnsDistBetw;
    } 

    int colorW = width;
    int colorH = height;
    int colorSz = (colorW < colorH) ? colorW:colorH;
	auto plot = parentPlot->ThreeDPlots[n];
    plot->colorChooser->updateRectSizes();
    plot->colorChooser->setFixedSize(colorSz*0.4, colorSz*0.4);
    plot->colorChooser->move(
                        leftX + colorW*0.5 - plot->colorChooser->width()*0.5,
                        upY + (colorH - upY - downYH)*0.5 - plot->colorChooser->height()*0.5);

    int cbSize = (newSize.height() - upY)*0.05*1.14;
    if (cbSize < 1) cbSize = 1;
    if (cbSize > 9999) cbSize = 9999;
    auto font = QFont(LFont->font().family(), cbSize*0.45);
    font.setBold(true);
    LFont->setScaledContents(true);
    LFont->setFont(font);
    LFont->adjustSize();
    int cbfontSize = plot->colorChooser->width() - LFont->width()*1.15;
    if (cbfontSize < 1) cbfontSize = 1;
    CBFont->setFixedSize(cbfontSize, cbSize);
    CBFont->setFont(font);
    auto view = CBFont->view();
    view->setFont(font);

    y = plot->colorChooser->y() + plot->colorChooser->height();

    LFont->move(plot->colorChooser->x(), y + LFont->height()*0.2);
    CBFont->move(LFont->x() + LFont->width()*1.05, y);


    emit GUIPlotResized(n, newSize);
}

void ThreeDOverlayButtons::setVisibleMenu(int n, plot::TPure3dPlot* plotPure, bool visible)
{
    if (replacedGUIButtons[n])
        emit replacedGUIPlotSetVisible(n, visible);
    else
        scanButtonsSetVisibleGUI(visible, plotPure);
}

void ThreeDOverlayButtons::setVisibleIfUnderMouse(int n)
{
    if (constantShowingMenu)
        setVisibleMenu(n, parentPlot->ThreeDPlots[n], true);
    else
        setVisibleMenu(n, parentPlot->ThreeDPlots[n], isPlotContainMouse(n));
}

bool ThreeDOverlayButtons::isPlotContainMouse(int n)
{
    auto rect = parentPlot->ThreeDPlots[n]->rect();
    auto pos = parentPlot->ThreeDPlots[n]->mapFromGlobal(QCursor::pos());
    return rect.contains(pos);
}

void ThreeDOverlayButtons::MouseEvent(QEvent* event, plot::TPure3dPlot* plotPure, int n)
{
    if (constantShowingMenu)
        return;

    if (event->type() == QEvent::MouseButtonRelease)
    {
        if (CBFont->isVisible())
        {
            BBackButton->click();
        }
    }

    if (event->type() == QEvent::Enter)
        setVisibleMenu(n, plotPure, true);
    else if (event->type() == QEvent::Leave && !blockLeaveEvent)
    {
        auto rect = plotPure->rect();
        if (setActivePlotByMouseMove && plotPure->Child->ActivePlot != plotPure->Child->ThreeDPlots.indexOf(plotPure))
        {
            event->ignore();
            return;
        }

        auto pos = plotPure->mapFromGlobal(QCursor::pos());
        if (rect.contains(pos))
        {
            event->ignore();
            return;
        }

        int boundH = rect.height()*0.01;
        int boundW = rect.width()*0.01;
        QRect smallerRect(rect.left()+boundW, rect.top()+boundH,
			rect.width()-boundW*2, rect.height()-boundH*2);
        setVisibleMenu(n, plotPure, smallerRect.contains(pos));
    }
}
void ThreeDOverlayButtons::onBack_clicked()
{
    int n = parentPlot->ThreeDPlots.indexOf((plot::TPure3dPlot*)BBackButton->parent());
    if (n == -1) n = 0;
    parentPlot->SetActivePlot(n);
    auto visMode = visibilityMode[n];
    blockResizeTimes = 20;
    if (visibilityMode[n] == svm_area_selection)
    {
        parentPlot->on_BCancel_clicked();
        if (constantShowingMenu)
        {
            blockResizeTimes = 0;
            showingBackButtonPlot[n] = false;
            showingApplyButtonPlot[n] = false;
            BBackButton->hide();
            blockShowing3DPlotButtons = false;
            scanButtonsSetVisibleGUI(true, parentPlot->ThreeDPlots.first());
            constantShowingMenu = false;
            setVisibleMenu(n, parentPlot->ThreeDPlots.first(), false);
            setVisibleIfUnderMouse(n);
            constantShowingMenu = true;
            ChannelResized(n, parentPlot->ThreeDPlots.first()->size());
            setVisibleMenu(n, parentPlot->ThreeDPlots.first(), true);
        }
        else
            setVisibleIfUnderMouse(n);
        for (int i=0; i<parentPlot->ThreeDPlots.size(); i++)
        {
            parentPlot->ThreeDPlots.at(i)->AreaZzoom = false;
            visibilityMode[i] = svm_none;
        }
    }
    else if (visibilityMode[n] == svm_point_selection)
        BPointSelection->change_state_man_with_sig(false);
    else if (visibilityMode[n] == svm_markers)
        BMarkers->change_state_man_with_sig(false);
    else if (visibilityMode[n] == svm_range_selection)
        BAreaZzoom->change_state_man_with_sig(false);
    else if (visibilityMode[n] == svm_palettes)
        colorPickClicked();
    showingBackButtonPlot[n] = false;
    showingApplyButtonPlot[n] = false;
    BBackButton->hide();
    BApplyButton->hide();

    scanButtonsSetVisibleGUI(false);

    visibilityMode[n] = svm_none;
    emit BackCancelClicked(visMode);

    setVisibleIfUnderMouse(n);
}

void ThreeDOverlayButtons::bApply_clicked()
{
    int n = parentPlot->ThreeDPlots.indexOf((plot::TPure3dPlot*)BApplyButton->parent());
    if (n == -1) n = 0;
    parentPlot->SetActivePlot(n);
    auto visMode = visibilityMode[n];

    showingBackButtonPlot[n] = false;
    showingApplyButtonPlot[n] = false;
    BApplyButton->hide();
    BBackButton->hide();
    scanButtonsSetVisibleGUI(false);
    visibilityMode[n] = svm_none;
    if (visMode == svm_area_selection)
        parentPlot->on_BApply_clicked();

    emit ApplyEnterClicked(visMode);

    setVisibleIfUnderMouse(n);
}

void ThreeDOverlayButtons::resetBackBtnWainting()
{
    BBackButton->click();
    parentPlot->ThreeDPlots[parentPlot->ActivePlot]->colorChooser->hide();
    LFont->hide();
    CBFont->hide();
    for (int i=0; i<parentPlot->ThreeDPlots.count(); i++)
    {
        showingBackButtonPlot[i] = false;
        showingApplyButtonPlot[i] = false;
        visibilityMode[i] = svm_none;
    }
    BBackButton->hide();
}

void ThreeDOverlayButtons::onPointSelection_changed(bool pressed)
{
    int n = parentPlot->ThreeDPlots.indexOf((plot::TPure3dPlot*)BPointSelection->parent());
    if (n == -1) n = 0;
    parentPlot->SetActivePlot(n);
    for (int i=0; i<parentPlot->ThreeDPlots.count(); i++)
    {
        showingBackButtonPlot[i] = pressed;
        visibilityMode[i] = pressed ? svm_point_selection : svm_none;
    }
    parentPlot->PlotControl->BCross->change_state_man_with_sig(pressed);
    if (pressed)
    {
        scanButtonsSetVisibleGUI(false);
        #ifndef _ADROID
        if (parentPlot->TwoDPlotVertical->isHidden() && parentPlot->ThreeDPlots.count() == 1)
            parentPlot->SetFullWindowPlot(0, true);
        #endif
    }
    setVisibleIfUnderMouse(n);
    blockResizeTimes = 0;
}

void ThreeDOverlayButtons::onMarkers_changed(bool pressed)
{
    int n = parentPlot->ThreeDPlots.indexOf((plot::TPure3dPlot*)BMarkers->parent());
    if (n == -1) n = 0;
    visibilityMode[n] = pressed ? svm_markers : svm_none;
    parentPlot->SetActivePlot(n);
    showingBackButtonPlot[n] = pressed;
    parentPlot->PlotControl->BMarkers->setChecked(pressed);
    if (pressed)
    {
        scanButtonsSetVisibleGUI(false);
        #ifndef _ADROID
        if (parentPlot->TwoDPlotVertical->isHidden() && parentPlot->ThreeDPlots.count() == 1)
            parentPlot->SetFullWindowPlot(0, true);
        #endif
    }
    setVisibleIfUnderMouse(n);
    blockResizeTimes = 0;
}

void ThreeDOverlayButtons::on_Filter_changed(bool pressed)
{
    int n = parentPlot->ThreeDPlots.indexOf((plot::TPure3dPlot*)BFilters->parent());
    if (n == -1) n = 0;
    parentPlot->SetActivePlot(n);
    setButtonActive(BFilters, pressed);
    parentPlot->ThreeDPlots[n]->setFiltering(pressed);
}

void ThreeDOverlayButtons::colorPickClicked()
{
    int n = parentPlot->ThreeDPlots.indexOf((plot::TPure3dPlot*)BPalette->parent());
    if (n == -1) n = 0;
    visibilityMode[n] = svm_palettes;
    parentPlot->SetActivePlot(n);
    if (CBFont->isVisible())
    {
        parentPlot->ThreeDPlots[n]->colorChooser->hide();
        LFont->hide();
        CBFont->hide();
        showingBackButtonPlot[n] = false;
        BBackButton->hide();
    }
    else
    {
        showingBackButtonPlot[n] = true;
        parentPlot->ThreeDPlots[n]->colorChooser->setVisible(true);
        LFont->setVisible(true);
        CBFont->setVisible(true);
        scanButtonsSetVisibleGUI(false);
    }
    setVisibleIfUnderMouse(n);
}

void ThreeDOverlayButtons::on3DBAreaZoom_changed(bool pressed)
{
    int n = parentPlot->ThreeDPlots.indexOf((plot::TPure3dPlot*)BAreaZzoom->parent());
    if (n == -1) n = 0;
    visibilityMode[n] = pressed ? svm_range_selection : svm_none;
    parentPlot->SetActivePlot(n);
    parentPlot->ThreeDPlots[n]->AreaZzoom = pressed;
    for (int i=0; i<parentPlot->ThreeDPlots.count(); i++)
    {
        visibilityMode[i] = pressed ? svm_range_selection : svm_none;
        showingBackButtonPlot[i] = pressed;
        parentPlot->ThreeDPlots[i]->AreaZzoom = pressed;
    }
    if (pressed)
        scanButtonsSetVisibleGUI(false);
    setVisibleIfUnderMouse(n);
}

void ThreeDOverlayButtons::onSaveImage_clicked()
{
    int n = parentPlot->ThreeDPlots.indexOf((plot::TPure3dPlot*)BSaveImage->parent());
    if (n == -1) n = 0;
    parentPlot->SetActivePlot(n);
    blockShowing3DPlotButtons = true;
    scanButtonsSetVisibleGUI(false);
    parentPlot->on_BSavePlotPixmap_clicked();
    blockShowing3DPlotButtons = false;
    setVisibleIfUnderMouse(n);
}

bool ThreeDOverlayButtons::eventFilter(QObject *Object, QEvent *Event)
{
    if (Event->type() == QEvent::KeyPress)
    {
        auto key = ((QKeyEvent *)Event)->key();
        bool enterKey = (key == Qt::Key_Return || key == Qt::Key_Enter);
        bool escKey = key == Qt::Key_Escape;
        if (escKey)
        {
            if (BBackButton->isVisible())
                BBackButton->click();
        }
        if (enterKey)
        {
            if (BApplyButton->isVisible())
                BApplyButton->click();
        }
        if (enterKey || escKey)
        {
            if (CBFont->isVisible())
            {
                BBackButton->click();
            }
        }
    }

    return false;
}

bool ThreeDOverlayButtons::event(QEvent *Event)
{
    eventFilter(nullptr, Event);

    return QWidget::event(Event);
}

void ThreeDOverlayButtons::addUpButton(QPushButton *btn, int n)
{
    if (!upBtnsList.contains(btn))
    {
        if (n == -1 || n >= upBtnsList.size())
            upBtnsList.push_back(btn);
        else
            upBtnsList.insert(n, btn);
    }
    if (!allBtnsList.contains(btn))
        allBtnsList.push_back(btn);
}

void ThreeDOverlayButtons::setVisibleFilterButton(bool visible)
{
    if (hideFilterButton == !visible)
        return;

    hideFilterButton = !visible;
    if (visible)
    {
        addUpButton(BFilters, 3);
    }
    else {
        upBtnsList.removeAll(BFilters);
        allBtnsList.removeAll(BFilters);
    }
    for (int i=0; i<parentPlot->ThreeDPlots.size(); i++)
        ChannelResized(i, parentPlot->ThreeDPlots[i]->size());
}

void ThreeDOverlayButtons::ActivateRatisMode()
{
    removeButton(BPalette);
    removeButton(BMarkers);
    removeButton(BSaveImage);
    removeButton(BFilters);
    removeButton(BAreaZzoom);
}

void ThreeDOverlayButtons::removeButton(QPushButton *btn)
{
    upBtnsList.removeOne(btn);
    rightBtnsList.removeOne(btn);
    allBtnsList.removeOne(btn);
}

void ThreeDOverlayButtons::resetOldFilters()
{
    attributes = nullptr;
    for (int i=0; i<parentPlot->ThreeDPlots.size(); i++)
    {
        parentPlot->ThreeDPlots[i]->isFiltering = true;
        parentPlot->ThreeDPlots[i]->MeanValueFilterSize = 0;
        parentPlot->ThreeDPlots[i]->MedianValueFilterSize = 0;
        parentPlot->ThreeDPlots[i]->GaussianFilterSize = 0;
    }
    AGXFitOrders->actions()[0]->setChecked(true);
    AGYFitOrders->actions()[0]->setChecked(true);
    AGSubtr     ->actions()[0]->setChecked(true);
    MMathProcessing->actions().at(0)->setChecked(false);
    MMathProcessing->actions().at(1)->setChecked(false);

    QString strMean = T3dPlotControl::NameOfMeanValueFilter();
    QString strMedian = T3dPlotControl::NameOfMedianValueFilter();
    QString strGauss = T3dPlotControl::NameOfGaussianFilter();
    QString strHistogram = T3dPlotControl::NameOfHistogramFilter();
    int idMean = T3dPlotControl::GetNumOfAction(strMean);
    int idMedian = T3dPlotControl::GetNumOfAction(strMedian);
    int idGauss = T3dPlotControl::GetNumOfAction(strGauss);
    int idHistogram = T3dPlotControl::GetNumOfAction(strHistogram);
    MMath2->actions()[idMean]->setChecked(false);
    MMath2->actions()[idMedian]->setChecked(false);
    MMath2->actions()[idGauss]->setChecked(false);
    MMath2->actions()[idHistogram]->setChecked(false);
    MMath2->actions()[idMean]->setText(strMean);
    MMath2->actions()[idMedian]->setText(strMedian);
    MMath2->actions()[idGauss]->setText(strGauss);
    parentPlot->SetAllFilterSizesTextInAction();
}

void ThreeDOverlayButtons::setVisibleOldFilters(bool visible)
{
    isOldFiltersVisible = visible;
    if (visible)
    {
        addUpButton(BOldFilter);
        addUpButton(BOldFilterAddit);

        resetOldFilters();
    }
    else
    {
        BOldFilter->hide();
        BOldFilterAddit->hide();
        removeButton(BOldFilter);
        removeButton(BOldFilterAddit);
    }
    for (int i=0; i<parentPlot->ThreeDPlots.size(); i++)
        ChannelResized(i, parentPlot->ThreeDPlots[i]->size());
}

void ThreeDOverlayButtons::setVisiblePointSelection(bool visible)
{
    if (visible)
        addUpButton(BPointSelection, 0);
    else
    {
        BPointSelection->hide();
        removeButton(BPointSelection);
    }
    for (int i=0; i<parentPlot->ThreeDPlots.size(); i++)
        ChannelResized(i, parentPlot->ThreeDPlots[i]->size());
}
void ThreeDOverlayButtons::setButtonActive(QPushButton *btn, bool active)
{
    auto sz = btn->iconSize();
    setButtonStyle(btn, active);
    btn->setIconSize(sz);
}
void ThreeDOverlayButtons::setButtonStyle(QPushButton *btn, bool activated)
{
    QString borderColor = activated ? "red" : "white";
    QString style = "";

    style += "QPushButton {background-color: rgba(255,255,255,10);";
    style += "border: 2px solid " + borderColor +";}";
    style += "QToolTip { color: black; background-color: #FFFFE1; border: 1px solid black; }";
    style += "QPushButton:pressed{ background-color: lightgray; }";
    //style += ":disabled { color: #121212; background-color: #121212 }";

    btn->setStyleSheet( style );
}

void ThreeDOverlayButtons::setComboBoxStyle(QComboBox *cb)
{
    cb->setStyleSheet("QComboBox { border-width:0; background-color:rgba(255,255,255,100); }\
               QComboBox:pressed { background-color:rgba(255,255,255,100); border-width:1; }");
}

void ThreeDOverlayButtons::setDoubleSpinBoxStyle(QDoubleSpinBox *dsp)
{
    QString str = "";
    str += "QDoubleSpinBox {border-width:0; background-color:rgba(255,255,255,100);";
    str += "selection-color: yellow;selection-background-color: blue;alternate-background-color: yellow;";
    str += "}";
    str += "QDoubleSpinBox:edit { background: red; background-color:rgba(255,255,255,100);}";
    dsp->setStyleSheet(str);
}

void ThreeDOverlayButtons::setAttributes(plot::TMetaStorage *attr)
{
    attributes = attr;

    if (attr == nullptr)
        return;

    if (attr->contains("Fit_lines_x"))
        MMathProcessing->actions()[0]->setChecked(attr->value("Fit_lines_x").toBool());
    if (attr->contains("Fit_lines_y"))
        MMathProcessing->actions()[1]->setChecked(attr->value("Fit_lines_y").toBool());
    if (attr->contains("substract_order"))
    {
        AGSubtr->actions()[attr->value("substract_order").toInt()]->trigger();
        AGSubtr->actions()[attr->value("substract_order").toInt()]->setChecked(true);
    }
    if (attr->contains("x_fitting"))
    {
        AGXFitOrders->actions()[attr->value("x_fitting").toInt()]->trigger();
        AGXFitOrders->actions()[attr->value("x_fitting").toInt()]->setChecked(true);
    }
    if (attr->contains("y_fitting"))
    {
        AGYFitOrders->actions()[attr->value("y_fitting").toInt()]->trigger();
        AGYFitOrders->actions()[attr->value("y_fitting").toInt()]->setChecked(true);
    }
    #ifndef _ADROID
    if (attr->contains("mean_filter"))
    {
        int value = attr->value("mean_filter").toInt();

        if (value > 0)
        {
            auto name = T3dPlotControl::NameOfMeanValueFilter();
            int id = T3dPlotControl::GetNumOfAction(name);
            auto plot = parentPlot->ThreeDPlots[parentPlot->ActivePlot];
            plot->MeanValueFilterEnabled = true;
            plot->MeanValueFilterSize = value;
            parentPlot->PlotControl->MMath2->actions()[id]->setChecked(true);
            MMath2->actions()[id]->setChecked(true);

            T3dPlot::SetFilterSizeTextInAction(parentPlot->PlotControl->MMath2->actions()[id],
                                               plot->MeanValueFilterSize, name);
            T3dPlot::SetFilterSizeTextInAction(MMath2->actions()[id],
                                               plot->MeanValueFilterSize, name);
        }
    }
    if (attr->contains("median_filter"))
    {
        int value = attr->value("median_filter").toInt();

        if (value > 0)
        {
            auto name = T3dPlotControl::NameOfMedianValueFilter();
            int id = T3dPlotControl::GetNumOfAction(name);
            auto plot = parentPlot->ThreeDPlots[parentPlot->ActivePlot];
            plot->MedianValueFilterEnabled = true;
            plot->MedianValueFilterSize = value;
            parentPlot->PlotControl->MMath2->actions()[id]->setChecked(true);
            MMath2->actions()[id]->setChecked(true);

            T3dPlot::SetFilterSizeTextInAction(parentPlot->PlotControl->MMath2->actions()[id],
                                               plot->MedianValueFilterSize, name);
            T3dPlot::SetFilterSizeTextInAction(MMath2->actions()[id],
                                               plot->MedianValueFilterSize, name);
        }
    }
    if (attr->contains("gaussian_filter"))
    {
        int value = attr->value("gaussian_filter").toInt();

        if (value > 0)
        {
            auto name = T3dPlotControl::NameOfGaussianFilter();
            int id = T3dPlotControl::GetNumOfAction(name);
            auto plot = parentPlot->ThreeDPlots[parentPlot->ActivePlot];
            plot->GaussianValueFilterEnabled = true;
            plot->GaussianFilterSize = value;
            parentPlot->PlotControl->MMath2->actions()[id]->setChecked(true);
            MMath2->actions()[id]->setChecked(true);

            T3dPlot::SetFilterSizeTextInAction(parentPlot->PlotControl->MMath2->actions()[id],
                                               plot->GaussianFilterSize, name);
            T3dPlot::SetFilterSizeTextInAction(MMath2->actions()[id],
                                               plot->GaussianFilterSize, name);
        }
    }
    if (attr->contains("histogram_filter"))
    {
        auto histogram = MMath2->actions()[T3dPlotControl::GetNumOfAction(T3dPlotControl::NameOfHistogramFilter())];
        histogram->trigger();
        histogram->setChecked(attr->value("histogram_filter").toBool());
    }
    #endif
}

}
