#ifndef TWODOVERLAYBUTTONS2_H
#define TWODOVERLAYBUTTONS2_H

#include <QObject>
#include <QPushButton>
#include <QActionGroup>
#include <QLabel>
#include <QComboBox>
#ifdef _ADROID
#include "widgets/verticalqmenu.h"
#include "widgets/verticalqpushbutton.h"
#include "widgets/verticallabel.h"
#include "widgets/verticalqcombobox.h"
#endif

class TwoStateButton;
class colorRect2;

namespace plot {

class T2dPlot;
class TPure2dPlot;

class TwoDOverlayButtons : public QWidget
{
    Q_OBJECT
public: signals:
    void onBackClicked();
    void on_BCancelApply_clickedSignal();
    void on_KeyEnterPressed();
    void replacedGUIPlotResized(QSize newSize);
    void replacedGUIPlotSetVisible(bool visible);
    void replacedGUIShowBackBtn();


public:
    #ifdef _ADROID
    explicit TwoDOverlayButtons(bool vertical);
#else
    explicit TwoDOverlayButtons();
#endif

    void resetOldFilters();
    void set2DPlot(T2dPlot *pl);
    void set4DScan(const QString &scan);
    void removeButton(QPushButton *btn);
    void addLeftButton(QPushButton *btn);
    void addRightButton(QPushButton *btn, int i=-1);
    void setButtonStyle(QPushButton *btn, bool activated = false);//todo make static
    void setButtonActive(QPushButton *btn, bool active);
    void setVisibleOldFilters(bool visible);
    void setVisibleDataFitSizeButton(bool visible);
    void removeFilterButton();
    bool event(QEvent *Event) override;
    bool isOldFiltersVisible = false;

    T2dPlot *plot = nullptr;
    bool showBack = false;
    bool enabled = true;
    bool verticalPlacingRightButtons = false;

    bool constantShowingMenu = false;
    float btnSizeMultip = 1.0;

    //2d buttons
    //4d
    TwoStateButton *B4DSameXYPoints;
    QPushButton *B4DSendToData;
    QPushButton *B4DClearSpectrum;
    TwoStateButton *B4DSectionPointSpectr;

    QPushButton *B4DSelectCapturedSpectr;
    QPushButton *B4DSelectConvolutionFunc;
    //2d
    QPushButton *B2DMarkers;
    TwoStateButton *B2DGrid;
    QPushButton *B2DRestoreScale;
    QPushButton *B2DSaveImage;
    QPushButton *B2DSendToData;
    QPushButton *B2DBack;
    TwoStateButton *B2DFilter;
    QPushButton *BOldFilter;
    QPushButton *BFitData;
    QMenu *MMathProcessing;
    QActionGroup *AGFitOrders;

    //font\color\width
    QPushButton *BShowLineMenu;
    QLabel *LLineSelector;
    QLabel *LLineColor;
    QLabel *LLineWidth;
    QLabel *LFont;
    QComboBox *CBLineSelector;
    QComboBox *CBLineColor;
    QComboBox *CBLineWidth;
    QComboBox *CBFont;
    colorRect2 *background;
    QString style;
    QColor CurveQColours[7];

    bool blockShowing2DPlotButtons = false;
    bool hideFilterButton = false;
    bool waitingEnterButton = false;
    bool isLeftButtonsVertical = false;

    int lastUpButtonSize = 1;

    QList<QPushButton*> allBtnsList;

    QList<QPushButton*> leftBtnsList;
    QList<QPushButton*> rightBtnsList;

    QList<QLabel*> lineChangersListLabels;
    QList<QWidget*> lineChangersListComboBox;
    
    #ifdef _ADROID
    bool vertical = false;
    VerticalLabel *vLLineSelector;
    VerticalLabel *vLLineColor;
    VerticalLabel *vLLineWidth;
    VerticalLabel *vLFont;
    verticalQComboBox *vCBLineSelector;
    verticalQComboBox *vCBLineColor;
    verticalQComboBox *vCBLineWidth;
    verticalQComboBox *vCBFont;
    
    verticalQPushButton *vFilters;
    verticalQMenu *vMenuFilters;
#endif

public slots:
    void onBShowLineMenuClicked();
    void setVisibleGUI2D(bool visible);
    void plot2DResized();
    void plot2DResized(QSize newSize, TPure2dPlot* plotPure);
    void plot2DMouseEvent(QMouseEvent* mouse, TPure2dPlot* plotPure);
    void on4DSwitchSpectra_clicked(bool pressed);
    void on4DSelectCapturedSpectr_clicked(bool pressed);
    void on4DSelectConvolutionFunc_clicked(bool pressed);

    void on2DMarkers_clicked(bool pressed);
    void on2DGrid_changed(bool pressed);
    void on2DRestoreScale_clicked();
    void on2DSaveImage_clicked();
    void on2DSendToData_clicked();
    void on2DBack();
    void on2DFilter_clicked(bool pressed);
    void setShowingSendToDataButton(bool showing);
    void onFitDataClicked();

    void on_FitOrder_changed();
    void onCbLineWidth_changed(int n);
    void onCBLineColor_changed(int n);
    void fontChanged(int n);
    #ifdef _ADROID
    void setFilterVertical(int);
    #endif

private:

    bool eventFilter(QObject *Obj, QEvent *Event) override;
    QString FourDScan = "";
    bool hideToDataButton = false;
    bool dataFitSizeVisible = false;
};

}

#endif // TWODOVERLAYBUTTONS2_H
