#pragma once

#include "ccd.h"

#include <QThread>
#include <QtScript/QScriptEngine>
#include <QtScript/QScriptValue>

#include <plot.h>

double wavelength(double central_wl, double disp, double pixelsize, int pixel_idx, int cp);
int wavelength_Inverse(double wavelength, double central_wl, double disp, double pixelsize, int cp);
double raman_shift(double laser_wl, double wl);
double raman_shift_Inverse(double raman_shift, double laser_wl);

void set_spectra_calib(plot::TCurveData &curve, plot::TMetaStorage &attrib, int scale_idx);

class I_CCD;
class I_SPECTR;
class CCDMatrix : public QThread
{
    Q_OBJECT

public:
    I_SPECTR *m_spectrInterface; //not owned by ccdmatrix thread for now
    I_CCD *m_ccdInterface;
    float pixel_Xsize;
    bool m_update_ccd;

    bool panoram;
    double panoram_min;
    double panoram_max;
    double last_wl;
    long last_idx;

    int temp;
    bool hasTemperature;
    int currentCameraTemperature;
    int getCameraTemperature();
    QWidget *getCCDInterfaceWidget();

    int gen_panoram_cycles();
    void gen_panoram_param(double *arr, int num);
    void sew_panoram(long *result, long *add);

    void initialize();
    void ObtainProccessedData(long *dataArray, long *aver, long *base_line);

    int Scale;

    double Laser_WL;
    double Cent_WL;
    double Disp;
    int Cent_Pixel;

    //for panoram
    double Disp_backup; //effective dispersion for panoram, = last disp

    //for nlc
    double Cent_WL_real;
    double Disp_real;
    int Nx_real;
    int Cent_Pixel_real;

    //sp
    int Scale_n;
    double Laser_WL_n;
    int Cent_Pixel_n;
    double Cent_WL_n, Disp_n;


    CCDMatrix();
    ~CCDMatrix() override;
    static CCDMatrix* GetIt() noexcept;

    void setOutput(plot::T2dArray* image_array_out);
    plot::T2dArray *output2dArray;
    plot::TMetaStorage *curveCalib;

    int Nx;
    int Ny;
    bool m_closeThread;
    bool m_waitMainInit;
    int imageX_min,
    imageX_max,
    imageY_min,
    imageY_max,
    imageX_min_n,
    imageX_max_n,
    imageY_min_n,
    imageY_max_n;

public slots:
    void thread_close();
    void s_setTemp(int temp);
    void startAcquisition();
    void s_abortAcq();
    void sl_set_Exp_time(double time);
    void sl_set_acqMode(int mode);
    void sl_set_readMode(int Mode);
    void sl_set_Triger(int trigger);
    void sl_set_Acc_num(int number);
    void sl_set_Acc_time(double time);
    void sl_set_Center_row(int row);
    void sl_set_Height(int height);
    void s_setXAxisScale(int scale, double laser, int CP, double wl, double disp);
    void s_isContScan(bool cheked);
    void sl_set_Kin_ser_num(int);
    void sl_set_Kin_ser_time(double);
    void sl_set_Auto_calib(bool);
    void sl_cut_pikes(bool);
    void sl_set_new_3D_Limits(const Range &XLimits, const Range &YLimits);
    void sl_set_baseline_on_off(bool);
    void sl_save_base_line(void);
    void load_data();
    void SetManualBaseLine(long *bl);

signals:
    void updateCCDStatus(int);
    void ccd_finished();
    void sig_cur_kin_ser_num(int);
    void sig_cur_ac_num(int);
    void sigUpdateNxNy();
    void sig_Update();
    void new2dData(QVector<double> ydata, QVector<double> xdata);
    void monoch_status(bool);

public:
    void run() override;
    void processData(long *dataArray, long *aver, long *base_line);
    void Fillspectrum(long *dataArray, double laser_wl, double central_wl, double nx, double disp, double px_size, double central_px, bool scale_reverse);
    void nonlinear_correction(long *dataArray);
    double wavelengthNL(double central_wl, double disp, double pixelsize, int pixel_idx, int cp);
    void nlcPrepare(double pixelsize, int nx, int central_pixel);

    QScriptEngine nlcEngine;
    QScriptValue nlcFun;
    volatile int nonlinearMode;
    QString nonlinearCalibFormula;

    bool m_ccdStart;
    bool m_runTillCancel;

    long dataSize;
    int acquisitionMode, acquisitionMode_n;
    int m_readMode;
    int m_triggerMode;
    double m_exposureTime;

    int idx_accum;
    int idx_kin_ser;
    float accumulationCycleTime;
    int numOfAccumulations;
    float Kin_ser_time;
    int Kin_ser_number;
    int m_centralRow;
    int m_trackHeight;

    bool Action1;
    bool Action6;
    bool m_update_exposure;
    bool Action8;
    bool m_update_singleTrack;
    bool Action12;
    bool Action19;
    bool Action23;
    bool Action24;
    bool Action26;

    bool Auto_Calib;
    bool Scale_Dir;

    long  *ManBaseLine;
    bool cut_pikes_on_off;
    bool base_line_on_off;
    bool base_line_first;
    bool intensity_calib_on_off, intensity_calib_pixel_on_off;
    QVector<QPair<double, double> > intensity_calib_data;
    QVector<QPair<int, double> > intensity_calib_pixel_data;

};
