#include "ccdplot.h"
#include "gui/status.hpp"
#include "flowlayout.hpp"
#include <qwt_thermo.h>

#include "modules.hpp"
#include "oscilloscope.h"
#include "gui/ttabwidget.hpp"
#include "mainwindow.h"

#include "ccdmatrix.h"
#include "i_spectr.hpp"
#include "hardware/coboltcontroller.hpp"
#include "hardware/nspectr_proxy.h"
#include "qwt/qwt_plot_layout.h"
#include "scanplot.h"

#include <QScrollArea>
#include <QComboBox>
#include <QCheckBox>
#include <QGroupBox>
#include <QLabel>
#include <QTabWidget>
#include <QTextEdit>
#include <QRadioButton>
#include <QTimer>
#include <QSettings>
#include <QCloseEvent>
#include <QProgressBar>
#include <QFormLayout>
#include <QWidgetAction>
#include "widgets/threedoverlaybuttons.h"
#include "widgets/twodoverlaybuttons.h"


bool CCDPlot::eventFilter(QObject *Obj, QEvent *Event)
{
    if (Event->type() == plot::TScalesChangedEvent::id && setReadMode->currentIndex() == 2) {
        sl_set_new_3D_Limits(((plot::TScalesChangedEvent *) Event)->XLimits, ((plot::TScalesChangedEvent *) Event)->YLimits);
        ccdMatrix->sl_set_new_3D_Limits(((plot::TScalesChangedEvent *) Event)->XLimits, ((plot::TScalesChangedEvent *) Event)->YLimits);
    }
    if (Event->type() == QEvent::KeyPress)
    {
        if (((QKeyEvent *)Event)->key() == Qt::Key_Return || ((QKeyEvent *)Event)->key() == Qt::Key_Enter)
        {
            if (((plot::TCommonPlotControl*)MyPl->ThreeDPlot->ControlLayout->itemAt(1)->widget())->BApply->isVisible())
                ((plot::TCommonPlotControl*)MyPl->ThreeDPlot->ControlLayout->itemAt(1)->widget())->BApply->animateClick();
            if (MyPl->TwoDPlot->Pure2dPlot->FitOrder == -2)
                MyPl->TwoDPlot->PlotControl->BApply->click();
            if (MyPl->ThreeDPlot->TwoDPlot->Pure2dPlot->FitOrder == -2)
                MyPl->ThreeDPlot->TwoDPlot->PlotControl->BApply->click();
        }
        if (MyPl->ThreeDPlot->overlayButtons)
            MyPl->ThreeDPlot->overlayButtons->event(Event);
    }
    return QWidget::eventFilter(Obj, Event);
}

CCDPlot::CCDPlot(QWidget *parent) : QWidget(parent)
{
    QSettings settingsGui("Param/gui.conf", QSettings::IniFormat);
    showRightMenu = settingsGui.value("gui/show_spectr_right_menu", true).toBool();
    AutoSend = false;
    kinSerDACinitialValue = 0.;
    isInterrupted = false;
    Nspec_Initalized=false;
    nspectr_mutex = nullptr;
    ccdWidget = nullptr;
    ScanCCDMode = false;
    ccdMatrix = CCDMatrix::GetIt();
    ccdMatrix->curveCalib = &curveCalib;
    PMTPlot = nullptr;

    Nx=1024;
    Ny=256;

    deviceStatus = new StatusWidget1(tr("CCD"));
    deviceStatus->label->setToolTip(tr("Andor"));
#ifdef USE_CCD_EMULATOR
    {
        QSettings settings("Param/Emulator.conf", QSettings::IniFormat);
        if (settings.value("ccd", false).toBool())
            deviceStatus->setEmulator(true);
    }
#endif

    deviceStatus2 = new StatusWidget1(tr("Optics"));

    acqUpd = new QTimer(this);
    temperatureUpd = new QTimer(this);
    refresh_limit_timer.invalidate();

    connect(acqUpd, SIGNAL(timeout()), SLOT(updateSpecBar()));
    connect(temperatureUpd, SIGNAL(timeout()), SLOT(slot_updateTemperature()));

    fillLaserConf();

    setWindowTitle(tr("CCD - NSpec"));

    createPlot();
    plot::ResizeOwner(curveData, 1);
    MyPl->TwoDPlot->Pure2dPlot->SourceData = plot::getUnLock(curveData);
    MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->UseMathOnly = true;
    MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->clearSouceData();
    MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->SourceData = *MyPl->TwoDPlot->Pure2dPlot->cloneSourceData();
    MyPl->TwoDPlot->overlayButtons->setVisibleOldFilters(true);
    MyPl->TwoDPlot->overlayButtons->isLeftButtonsVertical = true;
    MyPl->ThreeDPlot->overlayButtons->setVisibleFilterButton(false);
    MyPl->ThreeDPlot->overlayButtons->setVisibleOldFilters(true);
    MyPl->TwoDPlot->showPlotControl = showRightMenu;
    MyPl->ThreeDPlot->TwoDPlot->showPlotControl = showRightMenu;
    MyPl->ThreeDPlot->TwoDPlotVertical->showPlotControl = showRightMenu;
    MyPl->ThreeDPlot->showPlotControl = showRightMenu;
    if (!showRightMenu)
    {
        MyPl->TwoDPlot->PlotControl->hide();
        MyPl->ThreeDPlot->TwoDPlot->PlotControl->hide();
        MyPl->ThreeDPlot->TwoDPlotVertical->PlotControl->hide();
        MyPl->ThreeDPlot->PlotControl->hide();
    }

    plot::ResizeOwner(imgData, 1);
    imgData.at(0)->setSize(Nx, Ny);

    createButtons();
    createCCDAtt();
    createLayout();
    
    {
        bool show;
        QSettings settings("Param/laser.conf", QSettings::IniFormat);
        settings.beginGroup("list");
        show = settings.allKeys().size() != (settings.allKeys().contains("dummy") ? 1 : 0);
        foreach (QString key, settings.allKeys()) {
            if (key == "dummy")
                continue;

            CBPreMonochWl->addItem(key, settings.value(key, -1).toInt());
        }
        settings.endGroup();

        CBPreMonochWl->setCurrentIndex(-1);
        CBPreMonochWl->setVisible(show);
        if (show)
            connect(CBPreMonochWl, SIGNAL(currentIndexChanged(int)), SLOT(monoch_laser_wl()));
    }
    
    load_data();

    monoch_status_update(false);

    QMetaObject::connectSlotsByName(this);

    installEventFilter(this);

    connect(MyPl->TwoDPlot->overlayButtons, SIGNAL(replacedGUIPlotResized(QSize)), this, SLOT(gui2DPlotResized(QSize)));
    connect(MyPl->TwoDPlot->overlayButtons, SIGNAL(replacedGUIPlotSetVisible(bool)), this, SLOT(gui2DPlotSetVisible(bool)));

    connect(base_line_manual, SIGNAL(clicked()),MyPl->TwoDPlot, SLOT(StartMCCurveConstructing()));
    connect(base_line_manual, SIGNAL(clicked()),MyPl->ThreeDPlot->TwoDPlotVertical, SLOT(StartMCCurveConstructing()));
    connect(MyPl->TwoDPlot->Pure2dPlot, SIGNAL(MathUpdated()), this, SLOT(Send2dPlotData()));
    connect(MyPl->TwoDPlot, SIGNAL(ManualConstructingDone()), this, SLOT(SetManualBaseLine()));
    connect(MyPl->ThreeDPlot->TwoDPlotVertical, SIGNAL(ManualConstructingDone()), this, SLOT(SetManualBaseLine()));
    connect(MyPl, SIGNAL(AddDataToContainer(plot::T3dDataOwner,plot::TMetaStorage)), SLOT(AddDataToContainer_slot(plot::T3dDataOwner,plot::TMetaStorage)));
    connect(MyPl, SIGNAL(AddDataToContainer(plot::TCurvesDataOwner,plot::TMetaStorage)), SLOT(AddDataToContainer_slot(plot::TCurvesDataOwner,plot::TMetaStorage)));

    //ccdmatrix connections  //TODO module connected()?
    ccdMatrix->setOutput(&imgData.at(0)->Array);

    connect(ccdMatrix, SIGNAL(sig_Update()), SLOT(sl_Plot_Refresh()), Qt::QueuedConnection);
    connect(ccdMatrix, SIGNAL(new2dData(QVector<double>,QVector<double>)), SLOT(new2dData(QVector<double>,QVector<double>)), Qt::QueuedConnection);
    connect(ccdMatrix, SIGNAL(sigUpdateNxNy()), SLOT(UpdateNxNy()));
    connect(ccdMatrix, SIGNAL(ccd_finished()), SLOT(sl_aq_but_off()));
    connect(ccdMatrix, SIGNAL(sig_cur_kin_ser_num(int)), SLOT(sl_set_current_kin_num(int)));
    connect(ccdMatrix, SIGNAL(sig_cur_ac_num(int)), SLOT(sl_set_current_acc_num(int)));

    connect(lineSetTemp, SIGNAL(newValue(int)), ccdMatrix, SLOT(s_setTemp(int)));
    connect(lineExpTime, SIGNAL(newValue(double)), ccdMatrix, SLOT(sl_set_Exp_time(double)));
    connect(setAcqMode, SIGNAL(currentIndexChanged(int)), ccdMatrix, SLOT(sl_set_acqMode(int)));
    connect(setReadMode, SIGNAL(currentIndexChanged(int)), ccdMatrix, SLOT(sl_set_readMode(int)));

    connect(lineNumAcc, SIGNAL(newValue(int)), ccdMatrix, SLOT(sl_set_Acc_num(int)));
    connect(lineAccCycleTime, SIGNAL(newValue(double)), ccdMatrix, SLOT(sl_set_Acc_time(double)));
    connect(Num_Kin_ser, SIGNAL(newValue(int)), ccdMatrix, SLOT(sl_set_Kin_ser_num(int)));
    connect(Time_Kin_ser, SIGNAL(newValue(double)), ccdMatrix, SLOT(sl_set_Kin_ser_time(double)));

    connect(setTriggerMode, SIGNAL(currentIndexChanged(int)), ccdMatrix, SLOT(sl_set_Triger(int)));
    connect(lineCenterRowTrack, SIGNAL(newValue(int)), ccdMatrix, SLOT(sl_set_Center_row(int)));
    connect(lineHeighTrack, SIGNAL(newValue(int)), ccdMatrix, SLOT(sl_set_Height(int)));

    connect(One_Cont, SIGNAL(state_changed(bool)), ccdMatrix, SLOT(s_isContScan(bool)));
    connect(Cut_Pikes, SIGNAL(state_changed(bool)), ccdMatrix, SLOT(sl_cut_pikes(bool)));
    connect(base_line_save, SIGNAL(clicked()), ccdMatrix, SLOT(sl_save_base_line()));
    connect(Base_line, SIGNAL(state_changed(bool)), ccdMatrix, SLOT(sl_set_baseline_on_off(bool)));

    {//ccd settings
        QSettings settings("Param/ccd.conf", QSettings::IniFormat);

        if (settings.value("use_ormins", false).toBool())
            deviceStatus->label->setToolTip(tr("Ormins"));
        lineExpTime->setValue(settings.value("exposure_time", 0.1).toDouble());
    }

    devStatusSync = new QTimer(this);
    devStatusSync->setSingleShot(true);

    prevDevStatus = -1;
    syncDevStatus = EDno;
    setDeviceStatus();
    setDeviceStatus2(EDno);

    connect(ccdMatrix, SIGNAL(updateCCDStatus(int)), this, SLOT(setDeviceStatus_handler(int)), Qt::QueuedConnection);
    connect(ccdMatrix, SIGNAL(monoch_status(bool)), SLOT(monoch_status_update(bool)), Qt::QueuedConnection);
    connect(devStatusSync, SIGNAL(timeout()), this, SLOT(setDeviceStatus()));


    QSettings settings("Param/gui.conf", QSettings::IniFormat);
    bool b1on = settings.value("ccd/intensity_calib_on_off", false).toBool();
    bool b2on = settings.value("ccd/intensity_calib_pixel_on_off", false).toBool();
    QString name1 = settings.value("ccd/intensity_calib_name", QString()).toString();
    QString name2 = settings.value("ccd/intensity_calib_pixel_name", QString()).toString();
    if (!name1.isEmpty()) intensity_calib_open_wl(name1);
    if (!name2.isEmpty()) intensity_calib_open_px(name2);
    calibEn->setChecked(b1on);
    pixelCalibEn->setChecked(b2on);
}

void CCDPlot::AddDataToContainer_slot(plot::TCurvesDataOwner src, plot::TMetaStorage attributes)
{
    attributes.insert("Name", MyPl->TwoDPlot->Pure2dPlot->Title);
    attributes.insert("Source", "CCD");

    auto osc = Oscilloscope::GetIt();
    if (osc) {
        attributes.insert("Stage X, um", osc->DacsWidget->X2->value());
        attributes.insert("Stage Y, um", osc->DacsWidget->Y2->value());
    }
    attributes.insert("CCD NX", Nx);
    attributes.insert("CCD NY", Ny);
    attributes.insert("CCD Central Wavelength, nm", ccdMatrix->Cent_WL);
    attributes.insert("CCD Dispersion", ccdMatrix->Disp);
    attributes.insert("CCD Central Pixel", lineCentralPx->value());
    attributes.insert("Exposure time", lineExpTime->value());
    if (ccdMatrix->m_spectrInterface)
    attributes.insert("Grating", CBgrating_select->currentText());
    if (Nspec_Initalized && nspectr_mutex)
    attributes.insert("ND filter", NDEdit->value());

    foreach (CoboltGui *it, cobolt_gui_list)
        attributes.insert("Laser Power, mW - " + it->windowTitle(), it->cpower->text());


    attributes.unite(curveCalib);

    emit AddDataToContainer(src, attributes);
    
    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
    {
        mob->setPropIdVar(DataID::addedScanDataToContainer_2d_spectr, QVariant::fromValue(attributes));
    }
}

void CCDPlot::AddDataToContainer_slot(plot::T3dDataOwner src, plot::TMetaStorage attributes)
{
    attributes.insert("Name", MyPl->ThreeDPlot->ThreeDPlots.at(0)->Title);
    attributes.insert("Source", "CCD");

    emit AddDataToContainer(src, attributes);
}

CCDPlot::~CCDPlot()
{
    foreach (CoboltGui *it, cobolt_gui_list)
        delete it;
    cobolt_gui_list.clear();
}

void CCDPlot::closeEvent(QCloseEvent *Event)
{

}

void CCDPlot::slot_updateTemperature()
{
    int Temper = ccdMatrix->getCameraTemperature();
    if (Temper > 15) Termometer->setFillBrush(Qt::red);
    else if (Temper >= 0) {
        Termometer->setFillBrush(Qt::green);
        temp_pic->setPixmap(QPixmap(":/CCD/plus.png"));
    } else {
        Termometer->setFillBrush(Qt::blue);
        temp_pic->setPixmap(QPixmap(":/CCD/minus.png"));
    }
    Termometer->setValue(Temper);
    lb_temp->setText(QString::number(Temper));
}

void CCDPlot::UpdateNxNy()
{
    Nx = ccdMatrix->Nx;
    Ny = ccdMatrix->Ny;

    imgData.at(0)->setSize(Nx, Ny);
}
void CCDPlot::sl_set_new_3D_Limits(const Range &/*XLimits*/, const Range &YLimits)
{
    double up_limit;
    if (YLimits[1] > (Ny-1))
        up_limit = Ny-1;
    else
        up_limit = YLimits[1];

    int cent_row = (int)(0.5*(up_limit + YLimits[0]));
    int track_height = (int)(up_limit - YLimits[0] - 1);
    if (track_height < 3) track_height = 3;

    lineCenterRowTrack->setValue(cent_row);
    lineHeighTrack->setValue(track_height);
    ccdMatrix->m_centralRow = cent_row;
    ccdMatrix->m_trackHeight = track_height;
}
void CCDPlot::new2dData(QVector<double> ydata, QVector<double> xdata)
{
    if (spec_bar != nullptr) {
        auto bar = spec_bar->bar;
        bar->setUpdatesEnabled(false);
        bar->setMaximum(ccdMatrix->m_exposureTime*1000);
        bar->reset();
        bar->setUpdatesEnabled(true);
    }

    curveData[0]->XSeq = xdata;
    curveData[0]->YSeq = ydata;

    if (!refresh_limit_timer.isValid() || refresh_limit_timer.elapsed() > 100) { //10 refreshes per second max
        refresh_limit_timer.start();

        MyPl->Refresh();

        if (PMTPlot && PMTPlot->isVisibleTo(this)) {
            PMTPlot->hide();
            MyPl->show();
        }
    }
}

void CCDPlot::sl_Plot_Refresh()
{
    if (spec_bar != nullptr) {
        spec_bar->bar->setUpdatesEnabled(false);
        spec_bar->bar->setMaximum(ccdMatrix->m_exposureTime*1000);
        spec_bar->bar->reset();
        spec_bar->bar->setUpdatesEnabled(true);
    }

    if (!refresh_limit_timer.isValid() || refresh_limit_timer.elapsed() > 100) { //10 refreshes per second max
        refresh_limit_timer.start();

        auto img = imgData.at(0);
        img->XLimits[0] = ccdMatrix->imageX_min;
        img->XLimits[1] = ccdMatrix->imageX_max;
        img->YLimits[0] = ccdMatrix->imageY_min;
        img->YLimits[1] = ccdMatrix->imageY_max;
        img->NXElems = ccdMatrix->imageX_max - ccdMatrix->imageX_min + 1;
        img->NYElems = ccdMatrix->imageY_max - ccdMatrix->imageY_min + 1;
        img->UpdateDxDy();
        img->resetMathLims();

        MyPl->ThreeDPlot->SetPlotsXScales(ccdMatrix->imageX_min, ccdMatrix->imageX_max);
        MyPl->ThreeDPlot->SetPlotsYScales(ccdMatrix->imageY_min, ccdMatrix->imageY_max);
        MyPl->ThreeDPlot->ThreeDPlots[0]->SourceData = plot::getUnLock(imgData);

        MyPl->Refresh();

        if (PMTPlot && PMTPlot->isVisibleTo(this)) {
            PMTPlot->hide();
            MyPl->show();
        }
    }
}
void CCDPlot::sl_set_current_acc_num(int num)
{
    lineNumAcc->blockSignals(true);
    lineNumAcc->setValue(num);
    lineNumAcc->blockSignals(false);
}
void CCDPlot::sl_set_current_kin_num(int num)
{
    Num_Kin_ser->blockSignals(true);
    Num_Kin_ser->setValue(num);
    Num_Kin_ser->blockSignals(false);

    if (setReadMode->currentIndex() != 2) {
        bool last = num >= ccdMatrix->Kin_ser_number;
        if (!last) {
            curveData.prepend(QSharedPointer<plot::TCurveData>(new plot::TCurveData));
            curveData[0]->Pen.setColor(QColor((int)((double)qrand() / (double)RAND_MAX * 240. ), (int)((double)qrand() / (double)RAND_MAX * 240. ), (int)((double)qrand() / (double)RAND_MAX * 240. )));
            MyPl->TwoDPlot->Pure2dPlot->SourceData = plot::getUnLock(curveData);
            MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->clearSouceData();
            MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->SourceData = *MyPl->TwoDPlot->Pure2dPlot->cloneSourceData();
        }

        if (kinSerDACscan->currentIndex() > 0) {
            auto osc = Oscilloscope::GetIt();
            if (!osc)
                return;
            DoubleChanger *dc = osc->DacsWidget->get_dacs_channel(kinSerDACscan);
            if (num == 1)
                kinSerDACinitialValue = dc->value();
            if (last)
                dc->setValue(kinSerDACinitialValue);
            else
                dc->setValue(dc->value() + kinSerDACstep->value());
            dc->setNewValue(true);
        }
    }
}
void CCDPlot::enable_options_rm(int mode)
{
    box_singleTrack->setVisible(mode == 1);
    BPanoram->setEnabled(mode != 2);
    Base_line->setEnabled(mode != 2);
    base_line_save->setEnabled(mode != 2);
    base_line_manual->setEnabled(mode != 2);

    if (mode == 2) {
        MyPl->SwitchPlots(plot::TPlot::_3d);
        MyPl->ThreeDPlot->SetXYDescAndSIUnit(QObject::tr("pixels"), QString());
        MyPl->ThreeDPlot->SetZDescAndSIUnit(QObject::tr("Counts"), QString());

        static bool first = true;
        if (first) {
            first = false;
            MyPl->ThreeDPlot->SetDefaultXLimits(ccdMatrix->imageX_min, ccdMatrix->imageX_max);
            MyPl->ThreeDPlot->SetDefaultYLimits(ccdMatrix->imageY_min, ccdMatrix->imageY_max);
        }
        MyPl->ThreeDPlot->SetPlotsXScales(ccdMatrix->imageX_min, ccdMatrix->imageX_max);
        MyPl->ThreeDPlot->SetPlotsYScales(ccdMatrix->imageY_min, ccdMatrix->imageY_max);
        MyPl->ThreeDPlot->SetTitle(tr("CCD Image"));

        MyPl->ThreeDPlot->ThreeDPlots[0]->ResizeQPlot();
        sl_Plot_Refresh();

        if (setAcqMode->currentIndex() == 2)
            setAcqMode->setCurrentIndex(0);
        setAcqMode->removeItem(2);

        if (BPanoram->isOn)
            BPanoram->change_state_man_with_sig(false);
        if (Base_line->isOn)
            Base_line->change_state_man_with_sig(false);
    } else {
        MyPl->SwitchPlots(plot::TPlot::_2d);
        MyPl->TwoDPlot->Pure2dPlot->SetDefaultYLimits(0, 65535);
        MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->SetDefaultYLimits(0, 65535);
        sl_set_X_scale(currentRelativeScale);

        if (mode == 1)
        {
            MyPl->TwoDPlot->SetTitle(tr("CCD Single Track Curve"));
            MyPl->ThreeDPlot->TwoDPlotVertical->SetTitle(tr("CCD Single Track Curve"));
        }
        else
        {
            MyPl->TwoDPlot->SetTitle(tr("CCD Full V Binning Curve"));
            MyPl->ThreeDPlot->TwoDPlotVertical->SetTitle(tr("CCD Full V Binning Curve"));
        }

        auto mob = MobileServer::GetIt();
        if (mob && mob->need_send())
            mob->setPropIdVar(DataID::PlotTitle2d_Spectr, MyPl->TwoDPlot->Pure2dPlot->Title);

        if (setAcqMode->count() != 3)
            setAcqMode->addItem(tr("Kinetic Series"));
    }

    if (PMTPlot && PMTPlot->isVisibleTo(this)) {
        PMTPlot->hide();
        MyPl->show();
    }
}
void CCDPlot::enable_options_acm(int mode)
{
    box_accumulation->setVisible(mode != 0);
    Kin_series->setVisible(mode == 2);

    auto osc = Oscilloscope::GetIt();
    if (mode == 2 && osc) {
        kinSerDACscan->clear();
        kinSerDACscan->addItem(tr("None"), -1);
        osc->DacsWidget->fill_dacs_channel_CB(kinSerDACscan);
    }
}

void CCDPlot::s_triggerMode(int mode)
{
    lineAccCycleTime->setEnabled(mode == 0);
    Time_Kin_ser->setEnabled(mode == 0);
}


void CCDPlot::sl_calib_change(bool value)
{
    bool auto_spectr = value && ccdMatrix->m_spectrInterface;
    lineDispersion->setVisible(!auto_spectr);
    lb_dispersion->setVisible(!auto_spectr);
    lineCentrWL->setVisible(!auto_spectr);
    lb_centralWL->setVisible(!auto_spectr);
    ccdMatrix->sl_set_Auto_calib(auto_spectr);

    if (auto_spectr) {
        lb_auto->setText(tr("Status: auto"));
    } else {
        lb_auto->setText(tr("Status: manual"));
        s_apply_calib_Settings();
        AutoCalibration->change_state_man(false);
    }
}
void CCDPlot::sl_set_X_scale(int sc)
{
    if (setReadMode->currentIndex() == 2)
        return;

    AGScales->actions()[sc]->trigger();
    AGScales->actions()[sc]->setChecked(true);
    sl_set_X_scale();
}
void CCDPlot::sl_set_X_scale()
{
    if (setReadMode->currentIndex() == 2)
        return;

    int sc = 0;
    if (AGScales->actions()[1]->isChecked())
        sc = 1;
    if (AGScales->actions()[2]->isChecked())
        sc = 2;
    currentRelativeScale = sc;

    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
        mob->setPropIdVar(DataID::spectrSetRelativeScales, sc);

    QString icons[3] = {":/CCD/ccdRelativeScale.png", ":/CCD/ccdRelativeScaleNM.png", ":/CCD/ccdRelativeScaleCM.png"};
    BRelativeScales->setIcon(makeHugeIcon(icons[sc]));

    switch (sc) {
    case 0:
        MyPl->TwoDPlot->Pure2dPlot->XAxisUnit.SetDescAndSIUnit(QObject::tr("pixels"));
        MyPl->TwoDPlot->Pure2dPlot->SetAllXScales(0,ccdMatrix->Nx_real);
        MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->XAxisUnit.SetDescAndSIUnit(QObject::tr("pixels"));
        MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->SetAllXScales(0,ccdMatrix->Nx_real);
        lineLaserWl->hide();
        lb_laser->hide();
        break;
    case 1:
        MyPl->TwoDPlot->Pure2dPlot->XAxisUnit.SetDescAndSIUnit(QObject::tr("WL"), QObject::tr("m"));
        MyPl->TwoDPlot->Pure2dPlot->SetAllXScales(0,2000);
        MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->XAxisUnit.SetDescAndSIUnit(QObject::tr("WL"), QObject::tr("m"));
        MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->SetAllXScales(0,2000);
        lineLaserWl->hide();
        lb_laser->hide();
        break;
    case 2:
        MyPl->TwoDPlot->Pure2dPlot->XAxisUnit.SetDescAndSIUnit(QObject::tr("Raman Shift, 1/cm"));
        MyPl->TwoDPlot->Pure2dPlot->SetAllXScales(-10000,10000);
        MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->XAxisUnit.SetDescAndSIUnit(QObject::tr("Raman Shift, 1/cm"));
        MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->SetAllXScales(-10000,10000);
        lineLaserWl->show();
        lb_laser->show();
        break;
    }

    s_apply_calib_Settings();
}

void CCDPlot::s_apply_calib_Settings()
{
    save_data();
    ccdMatrix->s_setXAxisScale(currentRelativeScale, lineLaserWl->value(), lineCentralPx->value(),
                                                     lineCentrWL->value(), lineDispersion->value());

    if (ccdMatrix->Scale != currentRelativeScale)
        for (int i = 0; i < curveData.size(); i++)
            set_spectra_calib(*curveData[i].data(), curveCalib, currentRelativeScale);
    MyPl->Refresh();
}

void CCDPlot::load_data()
{
    QSettings settings("Param/ccd.conf", QSettings::IniFormat);

    int scale_ind = settings.value("scale_ind", 0).toInt();
    lineLaserWl->setValue(settings.value("laser_wl", 532).toDouble());
    lineCentrWL->setValue(settings.value("central_wl", 532).toDouble());
    lineDispersion->setValue(settings.value("dispersion", 10).toDouble());
    lineCentralPx->setValue(settings.value("central_px", 512).toDouble());

    sl_set_X_scale(scale_ind);
}

void CCDPlot::save_data()
{
    QSettings settings("Param/ccd.conf", QSettings::IniFormat);

    settings.setValue("scale_ind", currentRelativeScale);
    settings.setValue("laser_wl", lineLaserWl->text());
    settings.setValue("central_wl", lineCentrWL->text());
    settings.setValue("central_px", lineCentralPx->text());
    settings.setValue("dispersion", lineDispersion->text());
}


void CCDPlot::startAcquisition(bool value)
{
    foreach (QWidget *wid, ccdControlWidgets)
        wid->setEnabled(!value && !ScanCCDMode);

    One_Cont->setVisible(!value);
    if (value) {
        MyPl->HideSelectionIfAny();
        if (MyPl->TwoDPlot->PlotControl->BMarkers->isChecked())
            MyPl->TwoDPlot->PlotControl->BMarkers->click();

        QColor color = Qt::black;
        if (curveData.size() == 1)
            color = curveData[0]->Pen.color();

        plot::ResizeOwner(curveData, 1);
        curveData[0]->Pen.setColor(color);
        MyPl->TwoDPlot->Pure2dPlot->SourceData = plot::getUnLock(curveData);
        MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->clearSouceData();
        MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->SourceData = *MyPl->TwoDPlot->Pure2dPlot->cloneSourceData();

        ccdMatrix->startAcquisition();

        //if (ccdMatrix->m_exposureTime*1000 >= 100) {
        if (!spec_bar) {
            spec_bar = new StatusBar(tr("CCD exposure"), ccdMatrix->m_exposureTime*1000);
            spec_bar->bar->setFormat(tr("%p% of %m ms"));
            connect(spec_bar, SIGNAL(canceled()), m_BStart, SLOT(click()));
            spec_bar->add();
        } else {
            auto bar = spec_bar->bar;
            bar->setUpdatesEnabled(false);
            bar->setMaximum(ccdMatrix->m_exposureTime*1000);
            bar->reset();
            bar->setUpdatesEnabled(true);
        }
        spec_bar->cancel->setEnabled(m_BStart->isEnabled());

        acqUpd->start(100);
    } else {
        ccdMatrix->s_abortAcq();

        acqUpd->stop();
        if (!ScanCCDMode)
            destructStatus(spec_bar);

        static bool first_spectra = true; //guard auto-send filler data started in main.c
        if (!first_spectra && AutoSend && MyPl->CurrentPlotType == plot::TPlot::_2d && !ScanCCDMode) {
            MyPl->TwoDPlot->on_SyncTimer_timeout();
            MyPl->TwoDPlot->PlotControl->BSave->animateClick();
        }
        first_spectra = false;


        lineNumAcc->blockSignals(true);
        lineNumAcc->setValue(ccdMatrix->numOfAccumulations);
        lineNumAcc->blockSignals(false);
        Num_Kin_ser->blockSignals(true);
        Num_Kin_ser->setValue(ccdMatrix->Kin_ser_number);
        Num_Kin_ser->blockSignals(false);

        if (ccdMatrix->m_spectrInterface && ccdMatrix->panoram) //update central wl
            lineCentralWlMonoch->setValue(ccdMatrix->m_spectrInterface->getWl());
    }

    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
        mob->setPropIdVar(DataID::spectrStartOnOff, value);
}

void CCDPlot::updateSpecBar()
{
    if (spec_bar != nullptr && ccdMatrix->m_ccdStart) {
        spec_bar->setValue(spec_bar->bar->value() + 100);
    }
}

void CCDPlot::sl_aq_but_off()
{
    if (m_BStart->isOn)
        m_BStart->change_state_man_with_sig(false);
}


void CCDPlot::onPanoram(bool val)
{
#ifdef USE_CCD_EMULATOR
    QSettings settings("Param/Emulator.conf", QSettings::IniFormat);
    if (!settings.value("ccd", false).toBool())
#endif
    if (val && !ccdMatrix->m_spectrInterface) {
        new Notification(tr("Panoram") + ": " + tr("Initialize optics before use."), Notification::Warn, 10);
        doBeep();

        BPanoram->change_state_man(false);
        return;
    }
    ccdMatrix->panoram = val;

    //manual calib not applicable
    if (val && BNLCFile->isChecked())
        BNLCAuto->click();
    BNLCFile->setEnabled(!val);

    panoramControl->setVisible(val);
    if (One_Cont->isOn)
        One_Cont->change_state_man_with_sig(false);
    One_Cont->setEnabled(!val);
}
void CCDPlot::onPanoramMin(double val)
{
    if (val >= ccdMatrix->panoram_max) {
        val = ccdMatrix->panoram_max - 10;
        PanoramMin->setValue(val);
    }
    ccdMatrix->panoram_min = val;
}
void CCDPlot::onPanoramMax(double val)
{
    if (val <= ccdMatrix->panoram_min) {
        val = ccdMatrix->panoram_min + 10;
        PanoramMax->setValue(val);
    }
    ccdMatrix->panoram_max = val;
}

#include "math/plot_MathProcessing.h"

void CCDPlot::createPlot()
{
    MyPl = new plot::TPlot(this, this);

    MyPl->ThreeDPlot->TwoDPlot->Pure2dPlot->SetAutoLegendEnabled(false);
    MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->UseMathOnly = true;
    MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->SetAutoLegendEnabled(false);

    MyPl->TwoDPlot->Pure2dPlot->XAxisUnit.SetDescAndSIUnit(QObject::tr("pixels"));
    MyPl->TwoDPlot->Pure2dPlot->YAxisUnit.SetDescAndSIUnit(QObject::tr("Counts"));
    MyPl->TwoDPlot->SetTitle(tr("CCD Full V Binning Curve"));
    MyPl->TwoDPlot->Pure2dPlot->SetAllXScales(0, Nx - 1);
    MyPl->TwoDPlot->Pure2dPlot->SetAllYScales(0, 65535);


    MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->XAxisUnit.SetDescAndSIUnit(QObject::tr("pixels"));
    MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->YAxisUnit.SetDescAndSIUnit(QObject::tr("Counts"));
    MyPl->ThreeDPlot->TwoDPlotVertical->SetTitle(tr("CCD Full V Binning Curve"));
    MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->SetAllXScales(0, Nx - 1);
    MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->SetAllYScales(0, 65535);
    
    connect(MyPl->TwoDPlot->overlayButtons->AGFitOrders, SIGNAL(triggered(QAction*)), this, SLOT(filter2DChanged()));
}

void CCDPlot::filter2DChanged()
{
    int id = FindCheckedItemIn(MyPl->TwoDPlot->overlayButtons->AGFitOrders);
    auto scan = ScanPlot::GetIt();
    if (scan)
    {
        twodPlotFilter = id;
        if (!scan->twodPlotFilterScanMode)
        {
            scan->Plot->TwoDPlot->PlotControl->AGFitOrders->actions()[id]->setChecked(true);
            scan->Plot->TwoDPlot->PlotControl->AGFitOrders->actions()[id]->trigger();
            scan->Plot->TwoDPlot->Refresh();
            scan->Plot->TwoDPlotVertical->Refresh();
        }
    }
}

void CCDPlot::createButtons()
{
    sendTemp = new QLabel(tr("Set Temperature, ")+QChar(176)+tr("C"));

    ccdAdditSetttings = new TwoStateButton2(":/ScanPlot/lowVoltageMode.png",":/ScanPlot/lowVoltageMode.png", 40);
    ccdAdditSetttings->setCheckable(true);
    ccdAdditSetttings->setIconSize(QSize(40, 40));
    connect(ccdAdditSetttings, SIGNAL(state_changed(bool)), this, SLOT(showAdditCCDSettings(bool)));

    m_BStart = new TwoStateButton(":/CCD/ccdstart.png",":stop.png");
    m_BStart->setToolTip(tr("Start/stop Acquisition"));
    m_BStart->setFlat(true);
    MyPl->TwoDPlot->overlayButtons->setButtonStyle(m_BStart);
    connect(m_BStart, SIGNAL(state_changed(bool)), this, SLOT(startAcquisition(bool)));

    m_3DBStart = new TwoStateButton(":start.png",":stop.png");
    m_3DBStart->setToolTip(tr("Start/stop Acquisition"));
    m_3DBStart->setFlat(true);
    MyPl->ThreeDPlot->overlayButtons->setButtonStyle(m_3DBStart);
    connect(m_BStart, SIGNAL(state_changed(bool)), m_3DBStart, SLOT(change_state_man(bool)));
    connect(m_3DBStart, SIGNAL(state_changed(bool)), m_BStart, SLOT(change_state_man_with_sig(bool)));

    One_Cont= new TwoStateButton(":/CCD/oneshoot.png",":/CCD/continum.png");
    One_Cont->setParent(m_BStart);
    One_Cont->setFlat(true);
    One_Cont->setToolTip(tr("single/non stop Acquisition"));

    B3DOne_Cont= new TwoStateButton(":/CCD/oneshoot.png",":/CCD/continum.png");
    B3DOne_Cont->setFlat(true);
    MyPl->ThreeDPlot->overlayButtons->setButtonStyle(B3DOne_Cont);
    B3DOne_Cont->setToolTip(tr("single/non stop Acquisition"));
    connect(One_Cont, SIGNAL(state_changed_man(bool)), this, SLOT(oneContChanged()));
    connect(One_Cont, SIGNAL(state_changed(bool)), B3DOne_Cont, SLOT(change_state_man(bool)));
    connect(B3DOne_Cont, SIGNAL(state_changed(bool)), One_Cont, SLOT(change_state_man_with_sig(bool)));

    Monochr = new QPushButton;
    Monochr->setIcon(makeHugeIcon(":/CCD/monochromator.png"));
    Monochr->setToolTip(tr("Run Centaur instrument control"));
    connect(Monochr, SIGNAL(clicked()), this, SLOT(sl_show_Monochromator()));

    B3DMonochr = new QPushButton;
    B3DMonochr->setIcon(makeHugeIcon(":/CCD/monochromator.png"));
    B3DMonochr->setToolTip(tr("Run Centaur instrument control"));
    B3DMonochr->setFlat(true);
    MyPl->ThreeDPlot->overlayButtons->setButtonStyle(B3DMonochr);
    connect(B3DMonochr, SIGNAL(clicked()), this, SLOT(sl_show_Monochromator()));

    BPanoram = new TwoStateButton(":/CCD/Pspecoff.png",":/CCD/PspecoN.png");
    BPanoram->setToolTip(tr("Toggle panoram mode"));
    connect(BPanoram, SIGNAL(state_changed(bool)), this, SLOT(onPanoram(bool)));

    Cut_Pikes = new TwoStateButton(":/CCD/cut_pikes_off.png",":/CCD/cut_pikes_on.png");
    Cut_Pikes->setFlat(true);
    MyPl->TwoDPlot->overlayButtons->setButtonStyle(Cut_Pikes);
    Cut_Pikes->setToolTip(tr("Cut spikes on/off"));

    B3DCut_Pikes = new TwoStateButton(":/CCD/cut_pikes_off.png",":/CCD/cut_pikes_on.png");
    B3DCut_Pikes->setFlat(true);
    MyPl->ThreeDPlot->overlayButtons->setButtonStyle(B3DCut_Pikes);
    B3DCut_Pikes->setToolTip(tr("Cut spikes on/off"));
    connect(Cut_Pikes, SIGNAL(state_changed(bool)), B3DCut_Pikes, SLOT(change_state_man(bool)));
    connect(B3DCut_Pikes, SIGNAL(state_changed(bool)), Cut_Pikes, SLOT(change_state_man_with_sig(bool)));

    Base_line = new TwoStateButton(":/CCD/base_line_off.png",":/CCD/base_line_on.png");
    Base_line->setFlat(true);
    MyPl->TwoDPlot->overlayButtons->setButtonStyle(Base_line);
    Base_line->setToolTip(tr("Base line substract on/off"));

    base_line_save = new QPushButton;
    base_line_save->setFlat(true);
    MyPl->TwoDPlot->overlayButtons->setButtonStyle(base_line_save);
    base_line_save->setIcon(makeHugeIcon(":/CCD/base_line_save.png"));
    base_line_save->setToolTip(tr("Store current line as baseline"));

    base_line_manual = new QPushButton();
    base_line_manual->setFlat(true);
    MyPl->TwoDPlot->overlayButtons->setButtonStyle(base_line_manual);
    base_line_manual->setIcon(makeHugeIcon(":/CCD/base_manual.png"));
    base_line_manual->setToolTip(tr("Create manual baseline"));

    BPMTScan = new QPushButton;
    BPMTScan->setIcon(makeHugeIcon(":/CCD/Sp_part.png"));
    BPMTScan->setToolTip(tr("Scan PMT spectra"));
    connect(BPMTScan, SIGNAL(clicked(bool)), this, SLOT(PMTScan()));

    plotFocus = new QPushButton;
    plotFocus->setIcon(makeHugeIcon(":/plot/focusBtn.png"));
    plotFocus->setToolTip(tr("Show focus settings"));
    MyPl->TwoDPlot->overlayButtons->setButtonStyle(plotFocus);
    connect(plotFocus, SIGNAL(clicked(bool)), this, SLOT(showPlotFocusClicked()));
    
    plotAutoScale = new QPushButton;
    plotAutoScale->setCheckable(true);
    plotAutoScale->setIcon(makeHugeIcon(":/plot/allow_deny_autoscale.png"));
    plotAutoScale->setToolTip(tr("Set autoscale"));
    plotAutoScale->setChecked(MyPl->TwoDPlot->Pure2dPlot->IsAutoScaleEnabled());
    MyPl->TwoDPlot->overlayButtons->setButtonActive(plotAutoScale, MyPl->TwoDPlot->Pure2dPlot->IsAutoScaleEnabled());
    connect(plotAutoScale, SIGNAL(clicked(bool)), this, SLOT(autoScaleClicked()));

    BRelativeScales = new QPushButton();
    BRelativeScales->hide();
    BRelativeScales->setFlat(true);
    MyPl->TwoDPlot->overlayButtons->setButtonStyle(BRelativeScales);
    BRelativeScales->setIcon(makeHugeIcon(":/CCD/ccdRelativeScale.png"));
    BRelativeScales->setToolTip(QObject::tr("Relative scales"));
    MenuRelativeScale = new QMenu;
    BRelativeScales->setMenu(MenuRelativeScale);
    AGScales = new QActionGroup(this);
    QString Scales[3] = {tr("Pixels"), tr("Wavelength"), tr("Raman shift")};
    for (int i = 0; i < 3; i++)
    {
        AGScales->addAction(MenuRelativeScale->addAction(Scales[i]));
        AGScales->actions()[i]->setCheckable(true);
    }
    AGScales->actions()[0]->setChecked(true);
    connect(AGScales, SIGNAL(triggered(QAction*)), this, SLOT(sl_set_X_scale()));

    QSettings settings("Param/ccd.conf", QSettings::IniFormat);
    if (!settings.value("use_pmt", false).toBool())
        BPMTScan->hide();
}
void CCDPlot::createCCDAtt()
{
    Termometer = new QwtThermo();
    Termometer->setOrientation(Qt::Horizontal, QwtThermo::BottomScale);
    Termometer->setFillBrush(Qt::blue);
    Termometer->setRange(-100, 30, 0);

    setAcqMode = new QComboBox;
    ccdControlWidgets << setAcqMode;
    setAcqMode->addItem(tr("Single Scan"));
    setAcqMode->addItem(tr("Accumulate"));
    setAcqMode->addItem(tr("Kinetic Series"));
    connect(setAcqMode, SIGNAL(currentIndexChanged(int)), SLOT(enable_options_acm(int)));

    setReadMode = new QComboBox;
    ccdControlWidgets << setReadMode;
    setReadMode->addItem(tr("Full Vertical Binning"));
    setReadMode->addItem(tr("Single-Track"));
    setReadMode->addItem(tr("Image"));
    connect(setReadMode, SIGNAL(currentIndexChanged(int)), SLOT(enable_options_rm(int)));

    labExpTime = new QLabel(tr("Exposition time") + ", " + tr("s"));

    lineExpTime = new Edit(0.001, 1000);
    lineExpTime->setDecimals(3);
    lineExpTime->setSingleStep(pow(10., -2));
    lineExpTime->setValue(0.1);

    lineSetTemp = new Edit(-100, 100, 0);
    lineSetTemp->setValue(20);

    lb_temp = new QLabel("0");
    lb_temp->setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
    lb_temp->setFrameStyle(QFrame::Panel);

    temp_pic = new QLabel;
    temp_pic->setPixmap(QPixmap(":/CCD/plus.png"));
    temp_pic->setMaximumSize(40,40);

    lineCenterRowTrack = new Edit(0, 10000);
    lineCenterRowTrack->setValue(128);
    ccdControlWidgets << lineCenterRowTrack;

    lineHeighTrack = new Edit(0, 1000);
    lineHeighTrack->setValue(10);
    ccdControlWidgets << lineHeighTrack;

    lineNumAcc = new Edit(1, 1000);
    lineNumAcc->setImmediateMode();
    ccdControlWidgets << lineNumAcc;

    lineAccCycleTime = new Edit(0, 10000, 2);

    //--------------Kinetic series-----------------------------------------------
    Num_Kin_ser = new Edit(1, 1000);
    Num_Kin_ser->setImmediateMode();
    ccdControlWidgets << Num_Kin_ser;

    Time_Kin_ser = new Edit(0, 1000, 2);

    kinSerDACscan = new QComboBox;
    ccdControlWidgets << kinSerDACscan;
    kinSerDACscan->addItem(tr("None"), -1);
    kinSerDACstep = new Edit(0, 10000, 2);
    kinSerDACstep->setFixedWidth(47);


    lb_auto = new QLabel(tr("Status: manual"));

    setTriggerMode = new QComboBox;
    ccdControlWidgets << setTriggerMode;
    setTriggerMode->addItem(tr("Internal"));
    setTriggerMode->addItem(tr("External"));
    connect(setTriggerMode, SIGNAL(currentIndexChanged(int)), SLOT(s_triggerMode(int)));

    AutoCalibration = new TwoStateButton(":/off.png",":/onn.png");
    connect(AutoCalibration, SIGNAL(state_changed(bool)), this, SLOT(sl_calib_change(bool)));
    AutoCalibration->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);

    CBgrating_select = new QComboBox;
    CBgrating_select->addItem(tr("Default grating"));
    connect(CBgrating_select, SIGNAL(currentIndexChanged(int)), SLOT(monoch_grating()));

    CBPreMonochWl = new QComboBox;

    CBexitport = new QComboBox;
    CBexitport->addItem(tr("Axial"));
    CBexitport->addItem(tr("Lateral"));
    connect(CBexitport, SIGNAL(currentIndexChanged(int)), SLOT(monoch_mirror()));

    lb_centralWL = new QLabel(tr("Central WL (nm)"));
    lineCentrWL = new Edit(0, 10000, 2);
    lineCentrWL->setValue(488);
    connect(lineCentrWL, SIGNAL(newValue()), this, SLOT(s_apply_calib_Settings()));

    lb_dispersion = new QLabel(tr("Dispersion (nm/mm)"));
    lineDispersion = new Edit(0, 10000, 3);
    lineDispersion->setValue(10);
    connect(lineDispersion, SIGNAL(newValue()), this, SLOT(s_apply_calib_Settings()));

    lineCentralPx = new Edit(0, 4000);
    lineCentralPx->setValue(512);
    connect(lineCentralPx, SIGNAL(newValue()), this, SLOT(s_apply_calib_Settings()));

    labCentralWlMonoch = new QLabel(tr("Wavelength (nm)"));
    labGrating_select = new QLabel(tr("Grating"));
    labSlitInMonoch = new QLabel(tr("Entrance slit (nm)"));
    labSlitOutMonoch = new QLabel(tr("Exit slit (nm)"));
    labExitport = new QLabel(tr("Exit mirror"));

    lb_laser = new QLabel(tr("Laser") + ", " + tr("nm"));
    lb_laser->hide();

    lineLaserWl = new Edit(0, 10000, 2);
    lineLaserWl->setValue(488);
    lineLaserWl->hide();
    connect(lineLaserWl, SIGNAL(newValue()), this, SLOT(s_apply_calib_Settings()));

    lineCentralWlMonoch = new Edit(0, 10000, 2);
    lineCentralWlMonoch->setValue(488);
    connect(lineCentralWlMonoch, SIGNAL(newValue()), this, SLOT(monoch_wl()));

    NDEdit = new Edit(0, 4, 1);
    connect (NDEdit, SIGNAL(newValue(double)), this, SLOT(nd_filter(double)));

    linePreCentralWlMonoch = new Edit(0, 10000, 2);
    linePreCentralWlMonoch->setValue(532);
    connect(linePreCentralWlMonoch, SIGNAL(newValue()), this, SLOT(monoch_pre_wl()));

    lineSlitInMonoch = new Edit(0, 10000, 1);
    lineSlitInMonoch->setValue(100);
    connect(lineSlitInMonoch, SIGNAL(newValue()), this, SLOT(monoch_slit_in()));

    lineSlitOutMonoch = new Edit(0, 10000, 1);
    lineSlitOutMonoch->setValue(100);
    connect(lineSlitOutMonoch, SIGNAL(newValue()), this, SLOT(monoch_slit_out()));

    central_wl_reset = new QCheckBox(tr("Central WL reset"));

    zPos = new DoubleChanger(0, 10, 4);

    calibLbl = new QLabel("-");
    pixelCalibLbl = new QLabel("-");
    calibLbl->setWordWrap(true);
    pixelCalibLbl->setWordWrap(true);

    calibOpn = new QPushButton(tr("Open"));
    pixelCalibOpn = new QPushButton(tr("Open"));
    connect(calibOpn, SIGNAL(clicked()), SLOT(intensity_calib_open_wl()));
    connect(pixelCalibOpn, SIGNAL(clicked()), SLOT(intensity_calib_open_px()));

    calibEn = new QCheckBox(tr("Enable WL Intensity calib"));
    pixelCalibEn = new QCheckBox(tr("Enable pixel Intensity calib"));
    connect(calibEn, SIGNAL(toggled(bool)), SLOT(intensity_calib_enabled()));
    connect(pixelCalibEn, SIGNAL(toggled(bool)), SLOT(intensity_calib_enabled()));

    box_Zpos = new QGroupBox(tr("Objective Z, ")+tr("mkm"));
    QHBoxLayout *layout_Zpos = new QHBoxLayout(box_Zpos);
    layout_Zpos->addWidget(zPos, 1);
    box_Zpos->setParent(MyPl->TwoDPlot);
    box_Zpos->setVisible(false);

    auto color = QColor(Qt::blue);
    hbMotorZFocusUp = new HoverButton(HoverButton::TopArrow, color);
    hbMotorZFocusDown = new HoverButton(HoverButton::BottomArrow, color);
    hbMotorZFocusEllipse = new HoverButton(HoverButton::Circle, color);
    hbMotorZFocusEllipse->text = "F";
    hbMotorZFocusEllipse->fontColor = color;

    hbMotorZFocusUp->setParent(MyPl->TwoDPlot);
    hbMotorZFocusDown->setParent(MyPl->TwoDPlot);
    hbMotorZFocusEllipse->setParent(MyPl->TwoDPlot);
    hbMotorZFocusUp->setVisible(false);
    hbMotorZFocusDown->setVisible(false);
    hbMotorZFocusEllipse->setVisible(false);

    Cut_Pikes->setParent(MyPl->TwoDPlot);
    Base_line->setParent(MyPl->TwoDPlot);
    base_line_save->setParent(MyPl->TwoDPlot);
    Cut_Pikes->setVisible(false);
    Base_line->setVisible(false);
    base_line_save->setVisible(false);

    monochromatorWidgets << central_wl_reset
                         << labExitport << CBexitport
                         << labGrating_select << CBgrating_select
                         << labCentralWlMonoch << lineCentralWlMonoch
                         << labSlitInMonoch << lineSlitInMonoch
                         << labSlitOutMonoch << lineSlitOutMonoch;
}

void CCDPlot::createLayout_ccd(QVBoxLayout *layCCDMono)
{
    QGroupBox *ccdsett = new QGroupBox(tr("CCD"));
    layCCDMono->addWidget(ccdsett, 0, Qt::AlignTop);
    layoutccd = new QVBoxLayout(ccdsett);
    //layoutccd->setContentsMargins(0, 0, 0, 0);
    //layoutccd->setSpacing(3);
    //layoutccd->setAlignment(Qt::AlignTop);

    temperature_widget = new QWidget();
    QHBoxLayout *temperature_layout = new QHBoxLayout(temperature_widget);
    temperature_layout->addWidget(ccdAdditSetttings);
    temperature_layout->addWidget(Termometer, 1);
    temperature_layout->addWidget(temp_pic);
    temperature_layout->addWidget(lb_temp);
    layoutccd->addWidget(temperature_widget);

    ccdAdditSetttings->setFixedSize(temp_pic->height(), temp_pic->height());

    createLayout_ccd_otherTable();
    createLayout_ccd_accumulation(ccdsett);
    createLayout_ccd_kinetic(ccdsett);
    createLayout_ccd_singleTrack(ccdsett);

    QHBoxLayout *accumKinLay = new QHBoxLayout();
    layoutccd->addLayout(accumKinLay);
    accumKinLay->addWidget(box_accumulation);
    accumKinLay->addWidget(Kin_series);

    layoutccd->addWidget(box_singleTrack);
}

void CCDPlot::createLayout_ccd_otherTable()
{
    auto box = new QGroupBox();
    auto lay = new QHBoxLayout(box);
    QTableWidget *TableMotors = new QTableWidget(4, 2);
    TableMotors->verticalHeader()->setVisible(false);
    TableMotors->horizontalHeader()->setVisible(false);
    lay->addWidget(TableMotors);
    layoutccd->addWidget(box);

    auto labAcqu = new QLabel(tr("Acquisition mode"));
    auto labReadMode = new QLabel(tr("Read mode"));

    TableMotors->setDragEnabled(false);
    TableMotors->setFocusPolicy(Qt::NoFocus);
    TableMotors->horizontalHeader()->setClickable(false);
    TableMotors->horizontalHeader()->setMovable(false);
    TableMotors->horizontalHeader()->setResizeMode(QHeaderView::Stretch);
    TableMotors->verticalHeader()->setResizeMode(QHeaderView::ResizeToContents);
    TableMotors->setSortingEnabled(false);
    TableMotors->setAlternatingRowColors(true);
    TableMotors->verticalScrollBar()->setVisible(false);
    TableMotors->verticalScrollBar()->setEnabled(false);
    TableMotors->verticalScrollBar()->setDisabled(true);
    TableMotors->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

    TableMotors->setCellWidget(0, 0, sendTemp);
    TableMotors->setCellWidget(0, 1, lineSetTemp);
    TableMotors->setCellWidget(1, 0, labExpTime);
    TableMotors->setCellWidget(1, 1, lineExpTime);
    TableMotors->setCellWidget(2, 0, labAcqu);
    TableMotors->setCellWidget(2, 1, setAcqMode);
    TableMotors->setCellWidget(3, 0, labReadMode);
    TableMotors->setCellWidget(3, 1, setReadMode);

    QString styleBold = "font: bold";
    sendTemp->setStyleSheet(styleBold);
    labExpTime->setStyleSheet(styleBold);
    labAcqu->setStyleSheet(styleBold);
    labReadMode->setStyleSheet(styleBold);

    sendTemp->setText(" " + sendTemp->text());
    labExpTime->setText(" " + labExpTime->text());
    labAcqu->setText(" " + labAcqu->text());
    labReadMode->setText(" " + labReadMode->text());

    int minHeight = 0;
    for (int i=0; i<TableMotors->rowCount(); i++)
        minHeight += TableMotors->cellWidget(i, 1)->minimumSizeHint().height();
    minHeight += 5;
    TableMotors->setFixedHeight(minHeight);
}

void CCDPlot::createLayout_ccd_accumulation(QGroupBox *ccdsett)
{
    box_accumulation = new QGroupBox(tr("Accumulation options"), ccdsett);
    box_accumulation->hide();
    QHBoxLayout *accumHLay = new QHBoxLayout(box_accumulation);
    QVBoxLayout **accumVLays = new QVBoxLayout*[2];
    for (int i=0; i<2; i++)
    {
        accumVLays[i] = new QVBoxLayout();
        accumHLay->addLayout(accumVLays[i]);
    }
    accumVLays[0]->addWidget(new QLabel(tr("Count")));
    accumVLays[1]->addWidget(lineNumAcc);

    accumVLays[0]->addWidget(new QLabel(tr("Delay") + ", " + tr("s")));
    accumVLays[1]->addWidget(lineAccCycleTime);

    accumVLays[0]->addWidget(new QLabel(tr("Trigger mode")));
    accumVLays[1]->addWidget(setTriggerMode);
}

void CCDPlot::createLayout_ccd_kinetic(QGroupBox *ccdsett)
{
    Kin_series = new QGroupBox(tr("Kinetic Series options"), ccdsett);
    Kin_series->hide();
    QHBoxLayout *kineticHLay = new QHBoxLayout(Kin_series);
    QVBoxLayout **kineticVLays = new QVBoxLayout*[2];
    for (int i=0; i<2; i++)
    {
        kineticVLays[i] = new QVBoxLayout();
        kineticHLay->addLayout(kineticVLays[i]);
    }
    kineticVLays[0]->addWidget(new QLabel(tr("Count")));
    kineticVLays[1]->addWidget(Num_Kin_ser);

    kineticVLays[0]->addWidget(new QLabel(tr("Delay") + ", " + tr("s")));
    kineticVLays[1]->addWidget(Time_Kin_ser);

    QHBoxLayout *kinserdacl = new QHBoxLayout;
    kinserdacl->addWidget(kinSerDACscan);
    kinserdacl->addWidget(kinSerDACstep);
    kineticVLays[0]->addWidget(new QLabel(tr("DAC step")));
    kineticVLays[1]->addLayout(kinserdacl);
}

void CCDPlot::createLayout_ccd_singleTrack(QGroupBox *ccdsett)
{
    box_singleTrack = new QGroupBox(tr("Single-track options"), ccdsett);
    box_singleTrack->hide();
    QHBoxLayout *singleTrackHLay = new QHBoxLayout(box_singleTrack);
    QVBoxLayout **singleTrackVLays = new QVBoxLayout*[4];
    for (int i=0; i<4; i++)
    {
        singleTrackVLays[i] = new QVBoxLayout();
        singleTrackHLay->addLayout(singleTrackVLays[i]);
    }

    singleTrackVLays[0]->addWidget(new QLabel(tr("Center row")));
    singleTrackVLays[1]->addWidget(lineCenterRowTrack);

    singleTrackVLays[2]->addWidget(new QLabel(tr("Height")));
    singleTrackVLays[3]->addWidget(lineHeighTrack);
}

void CCDPlot::createLayout_monochromator(QVBoxLayout *layCCDMono)
{
    box_monochromator = new QGroupBox(tr("Monochromator"));
    layCCDMono->addWidget(box_monochromator, 0, Qt::AlignTop);
    auto layMonochromator = new QHBoxLayout(box_monochromator);


    QVBoxLayout *leftMonochrLay = new QVBoxLayout();
    layMonochromator->addLayout(leftMonochrLay);

    leftMonochrLay->addWidget(central_wl_reset);

    QHBoxLayout *leftHMonochr = new QHBoxLayout();
    leftMonochrLay->addLayout(leftHMonochr);
    QVBoxLayout **leftVMonochr = new QVBoxLayout*[2];

    for (int i=0; i<2; i++)
    {
        leftVMonochr[i] = new QVBoxLayout();
        leftHMonochr->addLayout(leftVMonochr[i]);
    }

    leftVMonochr[0]->addWidget(labCentralWlMonoch);
    leftVMonochr[1]->addWidget(lineCentralWlMonoch);

    leftVMonochr[0]->addWidget(labGrating_select);
    leftVMonochr[1]->addWidget(CBgrating_select);

    leftVMonochr[0]->addWidget(labSlitInMonoch);
    leftVMonochr[1]->addWidget(lineSlitInMonoch);

    leftVMonochr[0]->addWidget(labSlitOutMonoch);
    leftVMonochr[1]->addWidget(lineSlitOutMonoch);

    leftVMonochr[0]->addWidget(labExitport);
    leftVMonochr[1]->addWidget(CBexitport);

    QHBoxLayout *leftMonochrBtnsLay = new QHBoxLayout();
    leftMonochrLay->addLayout(leftMonochrBtnsLay, 0);
    leftMonochrBtnsLay->setAlignment(Qt::AlignLeft);

    leftMonochrBtnsLay->addWidget(BPanoram, 0, Qt::AlignLeft);
    leftMonochrBtnsLay->addWidget(Monochr, 0, Qt::AlignLeft);
    BPanoram->setFixedSize(40, 40);
    Monochr->setFixedSize(40, 40);

    createLayout_monochromator_panoram(leftMonochrLay);

    TTabWidget *calib_tabs = new TTabWidget();

    layMonochromator->addWidget(calib_tabs);

    createLayout_monochromator_tab_calibration(calib_tabs);
    createLayout_monochromator_tab_intensity(calib_tabs);
    createLayout_monochromator_tab_nonlinear(calib_tabs);
}

void CCDPlot::createLayout_monochromator_tab_calibration(TTabWidget *calib_tabs)
{
    QWidget *calibrationsWid = new QWidget(calib_tabs);
    QHBoxLayout *calibrationLay = new QHBoxLayout(calibrationsWid);
    calib_tabs->addTab(calibrationsWid, tr("Calibrations"));

    QVBoxLayout **calibrationLays = new QVBoxLayout*[2];
    for (int i=0; i<2; i++)
    {
        calibrationLays[i] = new QVBoxLayout();
        calibrationLay->addLayout(calibrationLays[i]);
    }

    calibrationLays[0]->addWidget(lb_auto, 0, Qt::AlignVCenter);
    calibrationLays[1]->addWidget(AutoCalibration, 0, Qt::AlignVCenter);

    calibrationLays[0]->addWidget(new QLabel(tr("Central Pixel")), 0, Qt::AlignVCenter);
    calibrationLays[1]->addWidget(lineCentralPx, 0, Qt::AlignVCenter);

    calibrationLays[0]->addWidget(lb_laser, 0, Qt::AlignVCenter);
    calibrationLays[1]->addWidget(lineLaserWl, 0, Qt::AlignVCenter);

    calibrationLays[0]->addWidget(lb_centralWL, 0, Qt::AlignVCenter);
    calibrationLays[1]->addWidget(lineCentrWL, 0, Qt::AlignVCenter);

    calibrationLays[0]->addWidget(lb_dispersion, 0, Qt::AlignVCenter);
    calibrationLays[1]->addWidget(lineDispersion, 0, Qt::AlignVCenter);
}

void CCDPlot::createLayout_monochromator_tab_intensity(TTabWidget *calib_tabs)
{
    QWidget *intensityWid = new QWidget(calib_tabs);
    QFormLayout *layout_calibration2 = makeFormLayout(intensityWid);
    calib_tabs->addTab(intensityWid, tr("Intensity"));

    layout_calibration2->addRow(calibEn);
    layout_calibration2->addRow(calibOpn, calibLbl);
    layout_calibration2->addRow(pixelCalibEn);
    layout_calibration2->addRow(pixelCalibOpn, pixelCalibLbl);
}

void CCDPlot::createLayout_monochromator_tab_nonlinear(TTabWidget *calib_tabs)
{
    QWidget *nonlinearWid = new QWidget(calib_tabs);
    QGridLayout *layout_calibration3 = new QGridLayout(nonlinearWid);
    calib_tabs->addTab(nonlinearWid, tr("Nonlinear"));

    layout_calibration3->setAlignment(Qt::AlignTop);
    layout_calibration3->addWidget(BNLCNone = new QRadioButton(tr("Disabled")), 0, 0, 1, 2);
    layout_calibration3->addWidget(BNLCAuto = new QRadioButton(tr("Auto - device")), 1, 0, 1, 2);
    layout_calibration3->addWidget(BNLCFormula = new QRadioButton(tr("Formula")), 2, 0, 1, 2);
    layout_calibration3->addWidget(BNLCFile = new QRadioButton(tr("Manual list")), 3, 0, 1, 2);
    BNLCAuto->setChecked(true);

    QLabel *NLCStatus;
    layout_calibration3->addWidget(BNLCCfg = new QPushButton(tr("Edit")), 5, 0);
    layout_calibration3->addWidget(NLCStatus = new QLabel("-"), 5, 1);
    BNLCCfg->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
    BNLCCfg->setEnabled(false);
    connect(NLCEdit, SIGNAL(newStatus(QString)), NLCStatus, SLOT(setText(QString)));
    connect(BNLCCfg, SIGNAL(clicked()), NLCEdit, SLOT(show()));
    connect(BNLCCfg, SIGNAL(clicked()), NLCEdit, SLOT(raise()));

    connect(BNLCNone, SIGNAL(toggled(bool)), NLCEdit, SLOT(hide()));
    connect(BNLCAuto, SIGNAL(toggled(bool)), NLCEdit, SLOT(hide()));

    connect(BNLCNone, SIGNAL(toggled(bool)), NLCEdit, SLOT(setMode()));
    connect(BNLCAuto, SIGNAL(toggled(bool)), NLCEdit, SLOT(setMode()));
    connect(BNLCFormula, SIGNAL(toggled(bool)), NLCEdit, SLOT(setMode()));
    connect(BNLCFile, SIGNAL(toggled(bool)), NLCEdit, SLOT(setMode()));

    connect(BNLCFormula, SIGNAL(toggled(bool)), NLCEdit, SLOT(show()));
    connect(BNLCFile, SIGNAL(toggled(bool)), NLCEdit, SLOT(show()));
    connect(BNLCFormula, SIGNAL(toggled(bool)), NLCEdit, SLOT(raise()));
    connect(BNLCFile, SIGNAL(toggled(bool)), NLCEdit, SLOT(raise()));
}

void CCDPlot::createLayout_monochromator_panoram(QVBoxLayout *leftMonochrLay)
{
    PanoramMin = new Edit(200, 800, 1);
    PanoramMin->setValue(200);
    connect(PanoramMin, SIGNAL(newValue(double)), this, SLOT(onPanoramMin(double)));

    PanoramMax = new Edit(200, 800, 1);
    PanoramMax->setValue(800);
    connect(PanoramMax, SIGNAL(newValue(double)), this, SLOT(onPanoramMax(double)));

    panoramControl = new QGroupBox(tr("Panoram Control"));
    panoramControl->hide();
    QFormLayout *layoutPanoram = makeFormLayout(panoramControl);

    leftMonochrLay->addWidget(panoramControl);
    layoutPanoram->addRow(tr("From, nm"), PanoramMin);
    layoutPanoram->addRow(tr("To, nm"), PanoramMax);
}

void CCDPlot::createLayout_laser(QVBoxLayout *leftLay)
{
    QGroupBox *box_laser = new QGroupBox(tr("Laser control"));
    leftLay->addWidget(box_laser, 0, Qt::AlignBottom);
    QVBoxLayout *mainVLayLaser = new QVBoxLayout(box_laser);

    if (cobolt_gui_list.size() != 0) {
        QTabWidget *laserTabWidget = new QTabWidget(box_laser);
        mainVLayLaser->addWidget(laserTabWidget);

        foreach (CoboltGui *it, cobolt_gui_list)
            laserTabWidget->addTab(it, it->windowTitle());
    }
    QHBoxLayout *mainHLayLaser = new QHBoxLayout();
    QVBoxLayout **VLayLaser = new QVBoxLayout*[4];
    mainVLayLaser->addLayout(mainHLayLaser);
    for (int i=0; i<4; i++)
    {
        VLayLaser[i] = new QVBoxLayout();
        mainHLayLaser->addLayout(VLayLaser[i]);
    }
    {
        QSettings settings("Param/ccd.conf", QSettings::IniFormat);

        VLayLaser[0]->addWidget(new QLabel(tr("ND Filter")));
        VLayLaser[1]->addWidget(NDEdit);
//        if (!settings.value("use_ars", true).toBool()) {
//            laser_l->addRow(tr("Pre-monoch wl"), linePreCentralWlMonoch);
//        }
    }
    VLayLaser[2]->addWidget(new QLabel(tr("Optics configuration")));
    VLayLaser[3]->addWidget(CBPreMonochWl);
}

void CCDPlot::createLayout()
{
    QWidget *w = this;

    QHBoxLayout *MainLayout = new QHBoxLayout(this);
    MainLayout->setContentsMargins(9, 9, 3, 9);

    LeftScroll = new TScrollArea(w);
    //LeftScroll->setMinimumWidth(500);

    MyPl->TwoDPlot->layout()->setContentsMargins(0, 0, 6, 0);
    MainLayout->addWidget(LeftScroll);
    MainLayout->addWidget(MyPl, 1);

    QWidget *LeftWid = new QWidget(LeftScroll);
    LeftScroll->setWidget(LeftWid);
    QVBoxLayout *leftLay = new QVBoxLayout(LeftWid);
    //leftLay->setSpacing(3);
    //leftLay->setContentsMargins(0, 6, 6, 6);

    QVBoxLayout *layCCDMono = new QVBoxLayout(LeftWid);
    leftLay->addLayout(layCCDMono);

    NLCEdit = new CCDNLCEditor(this);

    createLayout_ccd(layCCDMono);
    createLayout_monochromator(layCCDMono);
    createLayout_laser(leftLay);

    //----------Laser----------------------------------



    //-------------------------------------------------


    MyPl->ThreeDPlot->overlayButtons->addUpButton(m_3DBStart);
    MyPl->ThreeDPlot->overlayButtons->addUpButton(B3DOne_Cont);
    MyPl->ThreeDPlot->overlayButtons->addUpButton(B3DCut_Pikes);
    MyPl->ThreeDPlot->overlayButtons->addUpButton(B3DMonochr);

    MyPl->TwoDPlot->overlayButtons->addLeftButton(m_BStart);
    MyPl->TwoDPlot->overlayButtons->addRightButton(plotFocus, 0);
    MyPl->TwoDPlot->overlayButtons->addRightButton(plotAutoScale, 0);
    MyPl->TwoDPlot->overlayButtons->addRightButton(BRelativeScales);

}

QList<QWidget *> CCDPlot::getDeviceStatusList()
{
    QList<QWidget *> list;

    list << (QWidget *)2; //group id
    list << deviceStatus;
    list << deviceStatus2;

    if (cobolt_gui_list.size()) {
        list << (QWidget *)3; //group id
        list << new QLabel(QMainWindow::tr("Lasers:"));
    }
    for (int i = 0; i < cobolt_gui_list.size(); i++)
        list << cobolt_gui_list.at(i)->deviceStatus;

    return list;
}


void CCDPlot::setDeviceStatus2(int idx)
{
    if (!deviceStatus2)
        return;

    auto status = deviceStatus2->status;
    switch (idx) {
    case EDno:
        status->set(StatusWidget::Cross, Qt::red);
        status->setToolTip(tr("No device"));
        break;
    case EDok:
        status->set(StatusWidget::Circle, Qt::green);
        status->setToolTip(tr("Connected"));
        break;
    case EDplay:
        status->set(StatusWidget::Play, Qt::green);
        status->setToolTip(tr("Busy"));
        break;
    }
}

void CCDPlot::setDeviceStatus_handler(int idx)
{
    syncDevStatus = idx;
    if (!devStatusSync->isActive())
        devStatusSync->start(100);
}
void CCDPlot::setDeviceStatus()
{
    if (!deviceStatus || prevDevStatus == syncDevStatus)
        return;

    prevDevStatus = syncDevStatus;
    bool st = (syncDevStatus >= EDok);

    auto status = deviceStatus->status;
    switch (syncDevStatus) {
    case EDno:
        status->set(StatusWidget::Cross, Qt::red);
        status->setToolTip(tr("No device"));
        break;
    case EDok:
        status->set(StatusWidget::Circle, Qt::green);
        status->setToolTip(tr("Connected"));
        break;
    case EDplay:
        status->set(StatusWidget::Play, Qt::green);
        status->setToolTip(tr("Acquiring"));
        break;
    }

    //ccdsett->setEnabled(st);
    //m_BStart->setEnabled(st);

    bool st1 = st && ccdMatrix->hasTemperature;

    bool temp_upd = temperature_widget->isVisibleTo(this) != st1;
    temperature_widget->setVisible(st1);
    sendTemp->setVisible(st1);
    lineSetTemp->setVisible(st1);

    if (!st1) {
        temperatureUpd->stop();
    } else {
        if (!temperatureUpd->isActive())
            temperatureUpd->start(1000);
    }

    void *prev = ccdWidget;
    if (!st && ccdWidget) {
        delete ccdWidget;
        ccdWidget = nullptr;
    }

    if (st && !ccdWidget) {
        ccdWidget = ccdMatrix->getCCDInterfaceWidget();
        if (ccdWidget)
        {
            ccdWidget->setVisible(false);
            layoutccd->insertWidget(1, ccdWidget);
        }
    }

    if (prev != ccdWidget || temp_upd) {
        LeftScroll->setWidget(LeftScroll->takeWidget()); //reset sizeHint
        LeftScroll->updateGeometry(); //notify layout
    }
}

void CCDPlot::monoch_status_update(bool st)
{
    setDeviceStatus2(st ? EDok : EDplay);

    for (auto wid : monochromatorWidgets)
        wid->setEnabled(st);
    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
        mob->setPropIdVar(DataID::setMonochrEnabled, st);
    NDEdit->setEnabled(st && Nspec_Initalized);
    linePreCentralWlMonoch->setEnabled(st && Nspec_Initalized);
    CBPreMonochWl->setEnabled(st && Nspec_Initalized);

    if (!st)
        return;

    if (Nspec_Initalized && nspectr_mutex) {
        QMutexLocker lck(nspectr_mutex);
        NDEdit->setValue(NProxy_GetExitNDFilter());
        linePreCentralWlMonoch->setValue(NProxy_GetWL());
    }

    if (CBgrating_select->isEnabled()) {
        CBgrating_select->blockSignals(true);
        int index = ccdMatrix->m_spectrInterface->getActiveGrating();
        CBgrating_select->setCurrentIndex(index);
        if (mob && mob->need_send())
            mob->setPropIdVar(DataID::setMonochrGrating, index);
        CBgrating_select->blockSignals(false);
    }

    lineSlitInMonoch->setValue(ccdMatrix->m_spectrInterface->getSlitWidth(SPECTR_PORT_IN));
    lineSlitOutMonoch->setValue(ccdMatrix->m_spectrInterface->getSlitWidth(SPECTR_PORT_OUT));
    lineCentralWlMonoch->setValue(ccdMatrix->m_spectrInterface->getWl());
    CBexitport->blockSignals(true);
    int index = ccdMatrix->m_spectrInterface->getPortPosition(SPECTR_PORT_OUT);
    CBexitport->setCurrentIndex(index);
    if (mob && mob->need_send())
        mob->setPropIdVar(DataID::setMonochrExitMirror, index);
    CBexitport->blockSignals(false);
}
void CCDPlot::monoch_laser_wl()
{
    int idx = CBPreMonochWl->currentIndex();
    if (idx == -1)
        return;

    QSettings settings("Param/laser.conf", QSettings::IniFormat);
    settings.setValue("index", idx);

    if (settings.value("set_wl", false).toBool()) {
        if (idx < 0 || idx >= laser_list_wavelength.size()) {
            qDebug() << "wl list fail";
        } else {
            lineLaserWl->setValue(laser_list_wavelength.at(idx));
            lineLaserWl->setNewValue(true);
            linePreCentralWlMonoch->setValue(laser_list_wavelength.at(idx));
            linePreCentralWlMonoch->setNewValue(true);
        }
    }

    if (nspectr_mutex) {
        QMutexLocker lck(nspectr_mutex);

        if (settings.value("set_ef", false).toBool() &&
                idx >= 0 && idx < laser_list_edgefilter_idx.size())
            NProxy_SetEdgeFilterPos(laser_list_edgefilter_idx.at(idx));

        if (settings.value("set_be", false).toBool() &&
                idx >= 0 && idx < laser_list_beamexpander_wl_idx.size() && idx < laser_list_beamexpander_diam_idx.size())
            NProxy_SetBeamExpanderPos(laser_list_beamexpander_wl_idx.at(idx),
                                      laser_list_beamexpander_diam_idx.at(idx));

        if (settings.value("set_mxyz", false).toBool() &&
                idx >= 0 && idx < laser_list_monoch_x.size() && idx < laser_list_monoch_y.size() && idx < laser_list_monoch_z.size())
            NProxy_SetMonochromatorLensPos(laser_list_monoch_z.at(idx),
                                           laser_list_monoch_x.at(idx),
                                           laser_list_monoch_y.at(idx), true);

        if (settings.value("set_cxyz", false).toBool()  &&
                idx >= 0 && idx < laser_list_confocal_pmt_x.size() && idx < laser_list_confocal_pmt_y.size() && idx < laser_list_confocal_pmt_z.size())
            NProxy_SetConfocalLensPos(laser_list_confocal_pmt_z.at(idx),
                                      laser_list_confocal_pmt_x.at(idx),
                                      laser_list_confocal_pmt_y.at(idx), true);
    }

    int laser_idx = CBPreMonochWl->itemData(CBPreMonochWl->currentIndex()).toInt();
    for (int i = 0; i < cobolt_gui_list.size(); i++) {
        auto coboltGui = cobolt_gui_list.at(i);
        if ((!(i == laser_idx) != coboltGui->Laser_onoff->isOn) &&
                coboltGui->isEnabled())
            coboltGui->Laser_onoff->change_state_man_with_sig(!(i == laser_idx));
    }
}
void CCDPlot::nd_filter(double idx)
{
    QSettings settings("Param/ccd.conf", QSettings::IniFormat);
    if (settings.value("use_ars", true).toBool()) {
        ARS3000::setSlitWidth(1, NDEdit->value());
    } else if (nspectr_mutex) {
        QMutexLocker lck(nspectr_mutex);
        NProxy_SetExitNDFilter(idx, FALSE);
    }
}
void CCDPlot::monoch_pre_wl()
{
    QSettings settings("Param/ccd.conf", QSettings::IniFormat);
    if (settings.value("use_ars", true).toBool()) {
        ARS3000::setLaser(CBPreMonochWl->currentIndex());
    } else if (nspectr_mutex) {
        QMutexLocker lck(nspectr_mutex);
        NProxy_SetWL(linePreCentralWlMonoch->value(), true);
    }
}
void CCDPlot::monoch_grating()
{
    if (!ccdMatrix->m_spectrInterface)
        return;

    QSettings settings("Param/ccd.conf", QSettings::IniFormat);
    settings.setValue("central_px_gr_"+QString::number(ccdMatrix->m_spectrInterface->getActiveGrating()), lineCentralPx->text());

    ccdMatrix->m_spectrInterface->setActiveGrating(CBgrating_select->currentIndex());

    lineCentralPx->setValue(settings.value("central_px_gr_"+QString::number(CBgrating_select->currentIndex()), lineCentralPx->value()).toDouble());
    lineCentralPx->setNewValue(true);
    
    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
        mob->setPropIdVar(DataID::setMonochrGrating, CBgrating_select->currentIndex());
}
void CCDPlot::monoch_wl()
{
    ccdMatrix->m_spectrInterface->setWl(lineCentralWlMonoch->value(), central_wl_reset->isChecked());
    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
        mob->setPropIdVar(DataID::setMonochrWaveLength, lineCentralWlMonoch->value());
}
void CCDPlot::monoch_slit_in()
{
    QSettings settings("Param/ccd.conf", QSettings::IniFormat);
    if (settings.value("use_ars", true).toBool()) {
        ARS3000::setSlitWidth(0, lineSlitInMonoch->value() * 1e-3);
    }
    ccdMatrix->m_spectrInterface->setSlitWidth(SPECTR_PORT_IN, lineSlitInMonoch->value());
    
    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
        mob->setPropIdVar(DataID::setMonochrEntrance, lineSlitInMonoch->value());
}
void CCDPlot::monoch_slit_out()
{
    ccdMatrix->m_spectrInterface->setSlitWidth(SPECTR_PORT_OUT, lineSlitOutMonoch->value());
    
    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
        mob->setPropIdVar(DataID::setMonochrExisSlit, lineSlitOutMonoch->value());
}
void CCDPlot::monoch_mirror()
{
    ccdMatrix->m_spectrInterface->setPortPosition(SPECTR_PORT_OUT, CBexitport->currentIndex());
    
    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
        mob->setPropIdVar(DataID::setMonochrExitMirror, CBexitport->currentIndex());
}

void CCDPlot::gui2DPlotSetVisible(bool visible)
{
    if (showPlotFocus && !MyPl->TwoDPlot->overlayButtons->showBack)
        showPlotFocus = false;
    if (showPlotFocus)
    {
        visible = MyPl->TwoDPlot->overlayButtons->B2DBack->isVisible();
        Cut_Pikes->setVisible(false);
        Base_line->setVisible(false);
        base_line_save->setVisible(false);

        box_Zpos->setVisible(visible);
        hbMotorZFocusUp->setVisible(visible);
        hbMotorZFocusDown->setVisible(visible);
        hbMotorZFocusEllipse->setVisible(visible);
    }
    else
    {
        visible = visible && !MyPl->TwoDPlot->overlayButtons->showBack;
        box_Zpos->setVisible(false);
        hbMotorZFocusUp->setVisible(false);
        hbMotorZFocusDown->setVisible(false);
        hbMotorZFocusEllipse->setVisible(false);
        Cut_Pikes->setVisible(visible);
        Base_line->setVisible(visible);
        base_line_save->setVisible(visible);
    }
}

void CCDPlot::gui2DPlotResized(QSize size)
{
    int downY = size.height() - MyPl->TwoDPlot->Pure2dPlot->QPlot->plotLayout()->scaleRect(QwtPlot::xBottom).height();
    int upY = MyPl->TwoDPlot->Pure2dPlot->QPlot->plotLayout()->titleRect().height();
    int upYPerc = upY*0.25;
    upY += upYPerc;
    int cbSize = (size.height() - upY)*0.05*1.14;
    cbSize = 60;

    One_Cont->setParent(m_BStart);
    int miniBtnSize = m_BStart->width()*0.43;
    One_Cont->setFixedSize(miniBtnSize, miniBtnSize);
    One_Cont->move(m_BStart->width()-miniBtnSize, m_BStart->width()-miniBtnSize);

    box_Zpos->setFixedWidth(size.width()*0.4);
    box_Zpos->move(size.width() - 14 - box_Zpos->width(), upY + 10);
    auto fontSmall = QFont(zPos->font().family(), cbSize*0.15);
    zPos->setFont(fontSmall);
    zPos->edit->setFont(fontSmall);
    zPos->slider->setFont(fontSmall);
    box_Zpos->setFixedHeight(90);

    auto fontBig = QFont(zPos->font().family(), cbSize*0.45);
    hbMotorZFocusEllipse->font = fontBig;

    int motorBtnsSize = cbSize*1.5;
    hbMotorZFocusUp->setFixedSize(motorBtnsSize, motorBtnsSize);
    hbMotorZFocusDown->setFixedSize(motorBtnsSize, motorBtnsSize);
    hbMotorZFocusEllipse->setFixedSize(motorBtnsSize, motorBtnsSize);

    int btnX = size.width() - motorBtnsSize*1.5;
    int btnY = size.height()*0.5 - (motorBtnsSize*3 + 5*2)*0.5;
    hbMotorZFocusUp->move(btnX, btnY);
    hbMotorZFocusEllipse->move(btnX, hbMotorZFocusUp->y() + hbMotorZFocusUp->height() + 5);
    hbMotorZFocusDown->move(btnX, hbMotorZFocusEllipse->y() + hbMotorZFocusEllipse->height() + 5);

    int UpButtonSize = MyPl->TwoDPlot->overlayButtons->lastUpButtonSize;
    Cut_Pikes->setFixedSize(UpButtonSize, UpButtonSize);
    Cut_Pikes->setIconSize(QSize(UpButtonSize, UpButtonSize));
    Base_line->setFixedSize(UpButtonSize, UpButtonSize);
    Base_line->setIconSize(QSize(UpButtonSize, UpButtonSize));
    base_line_save->setFixedSize(UpButtonSize, UpButtonSize);
    base_line_save->setIconSize(QSize(UpButtonSize, UpButtonSize));
    int distBetween = UpButtonSize*0.05;
    int y = m_BStart->y() + UpButtonSize*2 + distBetween;
    Cut_Pikes->move(m_BStart->x(), y);
    y += UpButtonSize + distBetween;
    Base_line->move(Cut_Pikes->x(), y);
    y += UpButtonSize + distBetween;
    base_line_save->move(Base_line->x(), y);
}

void CCDPlot::showPlotFocusClicked()
{
    MyPl->TwoDPlot->overlayButtons->showBack = true;
    showPlotFocus = true;
    MyPl->TwoDPlot->overlayButtons->setVisibleGUI2D(true);
}

void CCDPlot::autoScaleClicked()
{
    MyPl->TwoDPlot->Pure2dPlot->SetAutoScaleEnabled(plotAutoScale->isChecked(), true);
    MyPl->TwoDPlot->overlayButtons->setButtonActive(plotAutoScale, plotAutoScale->isChecked());
}

void CCDPlot::focusChanged(double value)
{
    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
        mob->setPropIdVar(DataID::spectrSetFocus, value);
}

void CCDPlot::showAdditCCDSettings(bool show)
{
    ccdWidget->setVisible(show);
    ccdAdditSetttings->setChecked(show);
}

void CCDPlot::Send2dPlotData()
{
    send2dPlotSpectr(MyPl->TwoDPlot);
}

void CCDPlot::send2dPlotSpectr(const plot::T2dPlot *plot2d)
{
    auto mob = MobileServer::GetIt();
    if (!mob)
        return;
    if (!mob->need_send())
        return;

    if (mob->any_need_data("need_2dPlotSpectr"))
    {
        mob->setPropIdVar(DataID::PlotData2dInfo_Spectr, QVariant::fromValue(*((plot::TInfoFor2dPlot *)plot2d->Pure2dPlot)));

        plot::TCurvesDataOwner data = plot::getLock(plot2d->Pure2dPlot->SourceData);
        for (int i = 0; i < data.size(); i++)
            if (data.at(i) && data.at(i)->size())
            {
                auto dt = *data.at(i).data();
                mob->setPropIdVar(DataID::PlotData2d_Spectr, QVariant::fromValue(dt));
            }
    }
}

void CCDPlot::oneContChanged()
{
    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
        mob->setPropIdVar(DataID::spectrSetInfMode, One_Cont->isOn);
}
