#pragma once

#include "ccd.h"
#include "controls.h"
#include "edit.hpp"
#include <QElapsedTimer>
#include <QPointer>
#include <QLayout>
#include <QActionGroup>
#include "ttabwidget.hpp"
#include "hoverbutton.h"

class IExternalScan;
class CCDPlot;
IExternalScan *getNewCCDIExtScan(CCDPlot *ccd);

class QwtThermo;
class CoboltGui;
class StatusWidget1;
class CCDMatrix;
class StatusBar;
class QScrollArea;
class QComboBox;
class QCheckBox;
class QGroupBox;
class QLabel;
class QTabWidget;
class QTextEdit;
class QRadioButton;
namespace plot { class TPlot; }
class CCDPlot : public QWidget
{
    Q_OBJECT

private slots:
    void onPanoram(bool);
    void onPanoramMin(double);
    void onPanoramMax(double);
    void PMTScan();

public slots:
    void showAdditCCDSettings(bool);

    void gui2DPlotSetVisible(bool);
    void gui2DPlotResized(QSize);

    void setDeviceStatus();
    void setDeviceStatus_handler(int);
    void setDeviceStatus2(int);
    void updateSpecBar();

    void monoch_laser_wl();
    void monoch_slit_in();
    void monoch_slit_out();
    void monoch_pre_wl();
    void monoch_wl();
    void monoch_mirror();
    void monoch_grating();
    void monoch_status_update(bool);
    void nd_filter(double);

    void sl_set_X_scale();
    void sl_set_X_scale(int);
    void sl_calib_change(bool);
    void s_apply_calib_Settings();
    void load_data();
    void save_data();

    void s_triggerMode(int mode);
    void slot_updateTemperature();
    void enable_options_rm(int mode);
    void enable_options_acm(int mode);
    void sl_show_Monochromator();

    void startAcquisition(bool);
    void sl_aq_but_off();
    void sl_Plot_Refresh();
    void new2dData(QVector<double> ydata, QVector<double> xdata);

    void sl_set_current_acc_num(int);
    void sl_set_current_kin_num(int);

    void UpdateNxNy();
    void sl_set_new_3D_Limits(const Range &XLimits, const Range &YLimits);
    void SetManualBaseLine();

    void intensity_calib_enabled();
    void intensity_calib_open_wl(QString name = QString());
    void intensity_calib_open_px(QString name = QString());

    void initOptics();
    void opticsReset();
    void cancelOperation();

    void showPlotFocusClicked();
    void focusChanged(double value);
    void autoScaleClicked();

    void AddDataToContainer_slot(plot::TCurvesDataOwner, plot::TMetaStorage);
    void AddDataToContainer_slotPMT(plot::TCurvesDataOwner, plot::TMetaStorage);
    void AddDataToContainer_slot(plot::T3dDataOwner, plot::TMetaStorage);

    void Send2dPlotData();
    void oneContChanged();
    void filter2DChanged();

signals:
    void AddDataToContainer(plot::T3dDataOwner, plot::TMetaStorage);
    void AddDataToContainer(plot::TCurvesDataOwner, plot::TMetaStorage);

public:
    void send2dPlotSpectr(const plot::T2dPlot *plot2d);

    QList<QWidget *> getDeviceStatusList();
    QVector<QWidget *> ccdControlWidgets; //disabled on acq
    QPointer<StatusBar> spec_bar;
    

    

    

    CCDPlot(QWidget *parent = nullptr);
    void fillLaserConf();
    ~CCDPlot() override;
    static CCDPlot* GetIt() noexcept;

    CCDMatrix *ccdMatrix;
    plot::TPlot *MyPl;
    
    bool ScanCCDMode;

    
    plot::TCurvesDataOwner curveData;
    
    plot::TMetaStorage curveCalib;

    QComboBox *CBexitport;
    QComboBox *CBgrating_select;
 
    TwoStateButton *BPanoram;

    HoverButton *hbMotorZFocusUp;
    HoverButton *hbMotorZFocusDown;
    HoverButton *hbMotorZFocusEllipse;

    Edit *lineExpTime;

    Edit *lineSetTemp;
    Edit *lineCentralWlMonoch;
    Edit *lineSlitInMonoch;
    Edit *lineSlitOutMonoch;

    QList<QWidget*> monochromatorWidgets;

    DoubleChanger *zPos;

    TwoStateButton *m_BStart;
    TwoStateButton *One_Cont;
    
    TwoStateButton *Cut_Pikes;
    TwoStateButton *Base_line;
    
    QPushButton *base_line_save;

    QPushButton *BRelativeScales;
    
    QActionGroup *AGScales;
    int currentRelativeScale = 0;

    QRadioButton *BNLCNone, *BNLCAuto, *BNLCFormula, *BNLCFile;
    
    bool AutoSend;
    
    int twodPlotFilter = 0;
    QPushButton *BNLCCfg;
    
    QComboBox *setAcqMode;
    QComboBox *setReadMode;
    QComboBox *setTriggerMode;
    
private:
    QElapsedTimer refresh_limit_timer;
    bool isInterrupted;
    QTimer *acqUpd;
    QTimer *temperatureUpd;
    QTimer *devStatusSync;
    int syncDevStatus, prevDevStatus;
    QPointer<StatusWidget1> deviceStatus; //ccd
    QPointer<StatusWidget1> deviceStatus2; //monoch
    bool showRightMenu;
    plot::T2dPlot *PMTPlot;
    plot::T3dDataOwner imgData;
    plot::TCurvesDataOwner PMTcurveData;
    QVector<CoboltGui *> cobolt_gui_list;
    QList<double> laser_list_wavelength;
    QList<int> laser_list_edgefilter_idx;
    QList<int> laser_list_beamexpander_wl_idx;
    QList<int> laser_list_beamexpander_diam_idx;
    QList<double> laser_list_monoch_x;
    QList<double> laser_list_monoch_y;
    QList<double> laser_list_monoch_z;
    QList<double> laser_list_confocal_pmt_x;
    QList<double> laser_list_confocal_pmt_y;
    QList<double> laser_list_confocal_pmt_z;
    QScrollArea *LeftScroll;
    
    QLabel *labExitport;
    QLabel *labGrating_select;
    QComboBox *CBPreMonochWl;
    QCheckBox *central_wl_reset;
    QGroupBox *box_singleTrack;
    QGroupBox *box_accumulation;
    QGroupBox *box_monochromator;
    Edit *NDEdit;
    
    QVBoxLayout *layoutccd;
    
    Edit *PanoramMin;
    Edit *PanoramMax;
    QGroupBox *panoramControl;
    QLabel *lb_temp;
    QLabel *lb_auto;
    QGroupBox *box_Zpos;
    bool showPlotFocus = false;
    Edit *lineLaserWl;
    Edit *lineCentrWL;
    Edit *lineDispersion;
    Edit *lineCentralPx;
    QLabel *labExpTime;
    Edit *lineNumAcc;
    Edit *lineAccCycleTime;
    Edit *lineCenterRowTrack;
    Edit *lineHeighTrack;
    QLabel *labCentralWlMonoch;
    QLabel *labSlitInMonoch;
    QLabel *labSlitOutMonoch;
    Edit *linePreCentralWlMonoch;
    QCheckBox *calibEn, *pixelCalibEn;
    QLabel *calibLbl, *pixelCalibLbl;
    QPushButton *calibOpn, *pixelCalibOpn;
    QWidget *NLCEdit;
    
    
    Edit *Num_Kin_ser;
    Edit *Time_Kin_ser;
    QGroupBox *Kin_series;
    QComboBox *kinSerDACscan;
    Edit *kinSerDACstep;
    double kinSerDACinitialValue;
    QWidget *ccdWidget;
    QWidget *temperature_widget;
    QwtThermo *Termometer;
    TwoStateButton2 *ccdAdditSetttings;
    TwoStateButton *m_3DBStart;
    TwoStateButton *B3DOne_Cont;
    TwoStateButton *B3DCut_Pikes;
    QPushButton *B3DMonochr;
    TwoStateButton *AutoCalibration;
    QPushButton *Monochr;
    QPushButton *base_line_manual;
    QPushButton *BPMTScan;
    QPushButton *plotAutoScale;
    QPushButton *plotFocus;
    QMenu *MenuRelativeScale;
    bool Nspec_Initalized;
    QMutex *nspectr_mutex;
    int Nx;
    int Ny;
    
    QLabel *lb_centralWL;
    QLabel *lb_dispersion;
    QLabel *lb_laser;
    QLabel *sendTemp;
    QLabel *temp_pic;

protected:
    void createPlot();
    void createButtons();
    void createCCDAtt();
    void createLayout();
    void createLayout_ccd(QVBoxLayout *layCCDMono);
    void createLayout_ccd_otherTable();
    void createLayout_ccd_accumulation(QGroupBox *ccdsett);
    void createLayout_ccd_kinetic(QGroupBox *ccdsett);
    void createLayout_ccd_singleTrack(QGroupBox *ccdsett);
    void createLayout_monochromator(QVBoxLayout *layCCDMono);
    void createLayout_monochromator_panoram(QVBoxLayout *leftMonochrLay);
    void createLayout_monochromator_tab_calibration(TTabWidget *calib_tabs);
    void createLayout_monochromator_tab_intensity(TTabWidget *calib_tabs);
    void createLayout_monochromator_tab_nonlinear(TTabWidget *calib_tabs);
    void createLayout_laser(QVBoxLayout *leftLay);
    bool eventFilter(QObject *Obj, QEvent *Event) override;
    void closeEvent(QCloseEvent *event) override;

};

class ARS3000
{
public:
    static
    bool init();
    static
    bool setSlitWidth(int idx, double width); //mm
    static
    bool setLaser(int idx);
    static
    bool isReady();
};


class QScriptEngine;
class CCDNLCEditor : public QWidget
{
    Q_OBJECT
public:
    CCDNLCEditor(CCDPlot *_plot);

signals:
    void newStatus(QString);

public slots:
    void open();
    void save();
    void setMode();
    void testFormula();
    void setStatus(int st = 1);
    void abort();

public:
    CCDPlot *ccdplot;

    QScriptEngine *eng;
    QTimer *timer;
    QTextEdit *edit;
    QLabel *LFormulaStatus;
    plot::T2dPlot *plot2d;
    plot::TCurvesDataOwner curveData;
    int mode;
    QString dataFormula, dataFile;
    QCheckBox *BSpline;

};
