#pragma once

#include <plot.h>
#include <QWidget>
#include "gui/flowlayout.hpp"
#include "options/options.h"
#include "widgets/threedoverlaybuttons.h"
class QTreeWidget;
class QTreeWidgetItem;
class QProgressBar;

class GwyddionImport;
class TQTableWidget;
class TListWidget;
class TFileProccessing;
class TAbstractInfo;
typedef QVector<TAbstractInfo> TAbstractInfos;

#ifndef _ADROID
class T3dViewer;
#endif

class T3dEnchancement;
class DataListOverlayButtons;
namespace ccd { class TSetOfSpectrum; }

class TContainer: public QWidget
{
    Q_OBJECT

signals:
    void restoreAttributes(plot::TMetaStorage);
    void on3DPlotShowed(int is4d);

public slots:
    void OpenFile(QString name, bool createNewTab);

    void Send2dPlotData();
    void Send3dPlotData();
    void SendContainerData();

    void thread_close();
    void btnOpen_clicked();
    void btnSave_clicked();
    void btnSaveAs_clicked();
    void SaveAll();
    void RemoveCurrentItem();
    void RemoveAll();

    void btnCopy_clicked();
    void CopyGroup();
    void btnPaste_clicked();
    void btnMerge_clicked();

    void showAttributes_toggled(bool checked);
    void updateAttributes();
    void addAttrib_clicked();
    void SaveAttrib();
    void SaveFilters();
    void SavePreview();

    void btnToDevice_clicked();
    void btnToOldNspec_clicked();
    void btnToAscii_clicked();
    void btnToGwyddion_clicked();
    void gwyddion_error();

    void on3dUpdate();
    void btn3d_toggled(bool checked);
    QWidget *create3dPlot(QWidget *parent = nullptr);
    static QImage get3dTexture(plot::T3dDataOwner Info, double zmin, double zmax, QwtLinearColorMap *colorMap);

    void TabChanged(int index);
    void TabClose(int index);
    TListWidget* addTab(QString name = QString());
    void AddBlanks(int num);

    void onFuncInterval_changed(bool pressed);
    void onBackButton_clicked();

    void updateOverlayVisibility();

    void on_FileProccessing_started();
    void on_FileProccessing_finished();
    void on_FileProccessing_UpdatePreview();
    void on_FileProccessing_FileProccessed(double TotalNumber, double Current);
    void on_FileProccessing_AllDataLoaded(const TAbstractInfos *DataList);
    void AutoSave();

    static QString getDateHash();
    static void addDateIdsToAttributes(plot::TMetaStorage &attributes);
    void AddDataToContainer(plot::TCurvesDataOwner, plot::TMetaStorage);
    void AddDataToContainer(plot::T3dDataOwner, plot::TMetaStorage);
    void AddDataToContainer(ccd::TSetOfSpectrum *, plot::TMetaStorage);
    void AddDataToContainer(QImage, plot::TMetaStorage);

    //for in data section&copy
    void AddDataToContainer_slot(plot::TCurvesDataOwner, plot::TMetaStorage);
    void AddDataToContainer_slot(plot::T3dDataOwner, plot::TMetaStorage);

    void UpdateHierarchy();
    void UpdatePreview();

    void attribEditorResize();
    void attribConfigEdited(QTreeWidgetItem *item, int column);

    void on3DImageCrop();
    void on3DCancel_clicked(plot::ThreeDOverlayButtons::SpecialVisibilityMode visibilityMode);
    void on3DApply_clicked(plot::ThreeDOverlayButtons::SpecialVisibilityMode visibilityMode);
    void on3DCancelApply(plot::ThreeDOverlayButtons::SpecialVisibilityMode visibilityMode);
    void on_KeyEnterPressed3D();

    void onItemClicked(const plot::TCurvesDataOwner *);
    void onItemClicked(const plot::T3dDataOwner *);
    void onItemClicked(const ccd::TSetOfSpectrum *);
    void onItemClicked(const QImage);
    void show3d(const plot::T3dData data, bool is4d = false);

    void sl_Func_range_select(bool);
    void onAG_ConvFunc_trigered();
    void onAG_Scale_select_trigered();
    void onAGScaleSelect_trigered();
    void onAGConvolFuncSelect_trigered();

    void DelayedPreviewCreation();
    void CrossEvent(QPointF pt);
    bool SelectEvent(QRectF SelectedRect, int State, int Origin);
    void Error(QString,QString);

    void updateDataListWidth();

public:
    TContainer(plot::TPlot *PlotFromDataPlot, QSplitter *parent = nullptr);
    ~TContainer() override;
    void initPlots();
    void GUICreate();
    bool eventFilter(QObject *obj, QEvent *event) override;
    void dropEvent(QDropEvent *event) override;
    void dragEnterEvent(QDragEnterEvent *) override;

    void DoDrag(TListWidget *from, TListWidget *to);
    bool DataListShortCuts(QKeyEvent *keyEvent);
    void btnSave_clicked(QWidget *parentDialog);

    TFileProccessing *FileProccessing;
    TAbstractInfos *GenerateDataForSave();
    void Load(const QString &FileName);
    void Save(QString &FileName, bool oldFormat = false);
    void SaveAs(bool oldFormat, QWidget *parentDialog = nullptr);
static QString getDataDir();
    void SetDownButtonsEnabled(bool State);
    void addAttribute(QString groupname, QString name, QString val);

    HFlowLayout *prev2DLayoutScaleSelect = nullptr;

    plot::TPlot *Plot;
    plot::ImageItem *imageitem;

    #ifndef _ADROID
    T3dViewer *pl3;
    GwyddionImport *gwi;
#endif

    T3dEnchancement *t3dench;

    TListWidget *CurrentDataList;

    QMap<QString, QTreeWidgetItem *> attribItemsRoots;
    bool isAutosave; int autosave_idx;
    QString FileDir;
    QProgressBar *PBBar;
    QTreeWidget *table;

    #ifndef _ADROID
    QTabWidget *CBDataLists;
    DataListOverlayButtons *overlayButtons = nullptr;
    QSize prevSizes[2];
    int prevSizeIndex = 1;
#endif

    QGroupBox *GTable;
    QSplitter *GTableSplitter;
    QSplitter *ParentSplitter;

    QTimer *updateVisibilityOverlayTimer;

    QVector<void *> info_copypaste;
    QVector<int> copypaste_type;
    QVector<plot::TMetaStorage> copypaste_attrib;
    bool copypaste;
    bool preview_update;



    QVector<void *> info_dragdrop;
    QVector<int> dragdrop_type;
    QVector<plot::TMetaStorage> dragdrop_attrib;

    bool ActiveHBL;
    double func_min, func_max;
    int lastXind, lastYind;

    //-----
    QPushButton *BScaleSelect;
    QActionGroup *AGScaleSelect;
    QPushButton *BConvolFuncSelect;
    QActionGroup *AGConvolFuncSelect;
    TwoStateButton *BFuncInterval;
    QPushButton *B3DImageCrop;
    QPushButton *BShow3DView;
    //-----
    bool new3d;

    QGroupBox *gb4DButtons;
    QVBoxLayout *lay4DButtons;

    QToolButton *Convol_func;
    QActionGroup *AG_Convol_func;

    TwoStateButton *Func_interval;

    QToolButton *Scale_select;
    QActionGroup *AG_Scale_select;

    QTimer *AutosaveTimer;
    int AutoSaveTime;
    void SetAutoSaveTime(int);

    bool showRightMenu = true;
    bool canPaste = false;
    bool canRemove = false;
};

class T3dEnchancement : public QObject
{
    Q_OBJECT
public slots:
    void updateTexture();
    void queueUpdate();

public:
    #ifndef _ADROID
    T3dViewer *pl3;
#endif

    plot::T3dPlot *sourceplot;

    T3dEnchancement(QObject *parent = nullptr);
    QTimer *upd;

    //D&D
    bool eventFilter(QObject *, QEvent *) override;
    //do installEventFilter and setAcceptDrops on 3dviewer
};
