#include "datalist.hpp"
#include "container.h"
#include "ccd/ccd.h"
#include <QsKineticScroller.h>
#include <QScrollBar>
#include <QDragEnterEvent>
#include <QDebug>
#include <qapplication.h>
#include "datalistoverlaybuttons.h"


TListItem::TListItem(plot::TMetaStorage _attributes, void *data, int type, TListWidget *Parent, bool UpdPr)
{
    UpdatePreview = UpdPr;
    Info = data;
    attributes = _attributes;
    InfoType = type;

    QPixmap img = QPixmap::fromImage(attributes.value("Preview").value<QImage>());
    if (img.isNull())
        img = QPixmap(":/container/question.png");
    if (img.isNull()) {
        img = QPixmap(300, 300);
        img.fill(Qt::darkGray);
    }

    Item = Parent->FindFirstBlank();
    Parent->blockSignals(true);
    if (!Item)
        Item = new QListWidgetItem(Parent);

    Item->setIcon(img);
    QString text = attributes.value("Name", TContainer::tr("%1d data").arg(type+1)).toString();
    Item->setText(text);
    Item->setData(Qt::UserRole, InfoType);
    #ifndef _ADROID
    Item->setFlags(Item->flags() | Qt::ItemIsEditable | Qt::ItemIsDragEnabled );
    #endif
    Item->setFlags(Item->flags() | Qt::ItemIsEnabled | Qt::ItemIsSelectable);
    Parent->fitTextInItem(Item);
    Parent->blockSignals(false);
}
TListItem::~TListItem()
{
    if (InfoType == TAbstractInfo::_2d)
        delete (plot::TCurvesDataOwner *)Info;
    else if (InfoType == TAbstractInfo::_3d)
        delete (plot::T3dDataOwner *)Info;
    else if (InfoType == TAbstractInfo::_4d)
        delete (ccd::TSetOfSpectrum *)Info;

    delete Item;
}
//----------------------------------------------------------------------------

TListWidget::TListWidget(TContainer *_cont)
{
    setFont(qApp->font());
    mouseRectSelecting = false;
    rightButtonPressed = false;
    rubberBand = new QRubberBand(QRubberBand::Rectangle, this);
    cont = _cont;
    ClickedItem = nullptr;
    isUpdated = false;

    setResizeMode(QListWidget::Adjust);
    setViewMode(QListWidget::IconMode);
    setMovement(QListWidget::Snap);
    setIconSize(QSize(310, 310));

    setDragEnabled(true);
    viewport()->setAcceptDrops(true);
    setDropIndicatorShown(false);
    //setSortingEnabled(true);

    scroll = addScroller(this);

    connect(scroll, SIGNAL(eventFilterSignal(QObject*, QEvent*)), this, SLOT(eventFilterScroll(QObject*, QEvent*)));

    connect(this, SIGNAL(currentItemChanged(QListWidgetItem*, QListWidgetItem*)), this, SLOT(on_itemPressed(QListWidgetItem*)));
    connect(this, SIGNAL(itemChanged(QListWidgetItem*)), this, SLOT(on_nameChanged(QListWidgetItem*)));

    setSelectionMode(QAbstractItemView::MultiSelection);
    setSelectionRectVisible(true);
    setSelectionBehavior(QAbstractItemView::SelectItems);

    setContextMenuPolicy(Qt::CustomContextMenu);
    connect(this, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(slotCustomMenuRequested(QPoint)));
}
bool TListWidget::event(QEvent *e)
{
    if (!cont) return QListWidget::event(e);

    #ifndef _ADROID
    if (!cont->overlayButtons) return QListWidget::event(e);

    if (x() == 0 && cont->overlayButtons->allBtnsList.count() > 0)
    {
        int x = 0;
        x = cont->overlayButtons->allBtnsList.first()->width();
        move(x, 0);
    }
#endif

    #ifdef _WIN32
    int w = 0;
    for (int i=0; i<cont->CBDataLists->tabBar()->count(); i++)
    {
        w += cont->CBDataLists->tabBar()->tabRect(i).width();
    }
    w = w - x() + 4;
    if (needToResize && width() != w)
        resize(w, height());
#endif

    return QListWidget::event(e);
}
TListWidget::~TListWidget()
{
    disconnect();
    ClearAll();
}
#ifndef _ADROID
void TListWidget::resizeEvent(QResizeEvent *e)
{
    int size = 150;
    int icon = size*0.7;
    QSize sz(size, size);
    setGridSize(sz);
    QString listView = "QListWidget { "
                           "background-color: white; "
                           "background: white; "
                           "padding: 10px; "
                           "show-decoration-selected: 1; " // make the selection span the entire width of the view
                           "alternate-background-color: blue; "
                       "} ";
    QString item = "QListWidget::item { "
                       "height: " + QString::number(size) + "px; "
                       "width: " + QString::number(size) + "px; "
                       "border: 1px solid #7F7F7F; "
                       "background-color: #F7F7F7; "
                       "background: #F7F7F7; "
                       "margin: 10px; "
                   "} ";
    QString itemSelected = "QListWidget::item:selected { "
                               "color: black; "
                           "} ";
    QString alternate = "QListWidget::item:alternate {"
                            "background: white;"
                        "} ";
    setStyleSheet(listView + item + itemSelected + alternate);

    QSize iconSz(icon, icon);
    setIconSize(iconSz);

    setWordWrap(true);
    setTextElideMode(Qt::ElideMiddle);

    QWidget::resizeEvent(e);
}
#endif
void TListWidget::ClearAll()
{
    isUpdated = false;
    blockSignals(true);
    for (auto data : DataList)
        delete data;
    DataList.clear();
    blockSignals(false);
}
void TListWidget::slotCustomMenuRequested(QPoint pos)
{
    QMenu *menu = new QMenu(this);
    QAction *copy = new QAction(tr("Copy"), this);
    QAction *paste = new QAction(tr("Past"), this);
    QAction *Delete = new QAction(tr("Delete"), this);

    if (selectedItems().empty())
    {
        copy->setEnabled(false);
        Delete->setEnabled(false);
    }
    paste->setEnabled(cont->info_copypaste.count());

    connect(copy, SIGNAL(triggered()), cont, SLOT(btnCopy_clicked()));
    connect(paste, SIGNAL(triggered()), cont, SLOT(btnPaste_clicked()));
    connect(Delete, SIGNAL(triggered()), this, SLOT(DeleteSelectedItems()));

    menu->addAction(copy);
    menu->addAction(paste);
    menu->addAction(Delete);

    menu->popup(viewport()->mapToGlobal(pos));
}
void TListWidget::DeleteSelectedItems()
{
    QList<QListWidgetItem*> selected;

    for (int i=0; i<selectedItems().count(); i++)
    {
        selected.append(selectedItems()[i]);
    }
    for (int i=0; i<selected.count(); i++)
    {
        Delete(DataList.indexOf(FindItem(selected[i])));
    }

    ClickedItem = nullptr;
}
void TListWidget::Delete(int N)
{
    if (N < 0 || N >= DataList.size()) {
        N = DataList.indexOf(FindItem(currentItem()));
        if (N == -1)
            return;
    }

    blockSignals(true);
    delete DataList[N];
    blockSignals(false);

    //isUpdated = false;
    DataList.removeAt(N);
}

void TListWidget::AddBlanks(int N)
{
    isUpdated = true;
    blockSignals(true);
    for (int i = 0; i < N; i++)
        (new QListWidgetItem(QIcon(":/container/question.png"), "Proccessing", this))->setData(Qt::UserRole, TAbstractInfo::Blank);
    blockSignals(false);
}
QListWidgetItem *TListWidget::FindFirstBlank()
{
    for (int i = 0; i < count(); i++)
        if (item(i)->data(Qt::UserRole) == TAbstractInfo::Blank)
            return item(i);

    return nullptr;
}
TListItem *TListWidget::FindItem(QListWidgetItem *Item) const
{
    for (auto dl : DataList)
        if (dl->Item == Item)
            return dl;

    return nullptr;
}

void TListWidget::Add(plot::TMetaStorage attributes, plot::TCurvesDataOwner *data, bool UpdatePreview)
{
    isUpdated = true;
    auto item = new TListItem(attributes, data, TAbstractInfo::_2d, this, UpdatePreview);
    DataList.append(item);

    #ifndef _ADROID
    if (cont && cont->overlayButtons && cont->overlayButtons->allBtnsList.count() > 0)
    {
        int x = 0;
        x = cont->overlayButtons->allBtnsList.first()->width();
        move(x, 0);
    }
#endif
}
void TListWidget::Add(plot::TMetaStorage attributes, plot::T3dDataOwner *data, bool UpdatePreview)
{
    isUpdated = true;
    auto item = new TListItem(attributes, data, TAbstractInfo::_3d, this, UpdatePreview);
    DataList.append(item);

    #ifndef _ADROID
    if (cont && cont->overlayButtons && cont->overlayButtons->allBtnsList.count() > 0)
    {
        int x = 0;
        x = cont->overlayButtons->allBtnsList.first()->width();
        move(x, 0);
    }
#endif
}
void TListWidget::Add(plot::TMetaStorage attributes, ccd::TSetOfSpectrum *Info, bool UpdatePreview)
{
    isUpdated = true;
    if (attributes.value("Preview").value<QImage>().size() == QSize(50, 50))
        attributes.remove("Preview");
    auto item = new TListItem(attributes, Info, TAbstractInfo::_4d, this, UpdatePreview);
    DataList.append(item);

    #ifndef _ADROID
    if (cont && cont->overlayButtons && cont->overlayButtons->allBtnsList.count() > 0)
    {
        int x = 0;
        x = cont->overlayButtons->allBtnsList.first()->width();
        move(x, 0);
    }
#endif
}
void TListWidget::Add(plot::TMetaStorage attributes, QImage Info, bool /*UpdatePreview*/)
{
    isUpdated = true;
    attributes.insert("Preview", QVariant::fromValue(Info));
    auto item = new TListItem(attributes, nullptr, TAbstractInfo::_Image, this, false);
    DataList.append(item);

    #ifndef _ADROID
    if (cont && cont->overlayButtons && cont->overlayButtons->allBtnsList.count() > 0)
    {
        int x = 0;
        x = cont->overlayButtons->allBtnsList.first()->width();
        move(x, 0);
    }
#endif
}

void TListWidget::on_nameChanged(QListWidgetItem *Item)
{
    TListItem *it = FindItem(Item);
    if (!it)
        return;

    this->setCurrentItem(Item);

    QString txt = Item->text();
    if (it->attributes.value("Name").toString() == txt) {
        return;
    }

    it->attributes.insert("Name", txt);

    //cont->preview_update = true;
    on_itemPressed(Item);
    //cont->preview_update = false;
}
void TListWidget::fitTextInItem(QListWidgetItem *Item)
{
    int itemSize = this->gridSize().width() - 10;
    auto itemFont = Item->font();
    QFontMetrics metrics(itemFont);

    int lineHeight = this->fontMetrics().height();
    auto text = Item->text();

    int offset = itemSize*0.19;
    QRect itemRect(offset, 0, itemSize - offset, itemSize);
    auto rect = metrics.boundingRect(itemRect, Qt::AlignCenter | Qt::TextWordWrap, text);

    int numLines = rect.height()/lineHeight;
    if (numLines <= 1)
        return;

    auto icon = Item->icon();
    auto avSizes = icon.availableSizes();
    if (avSizes.empty())
        return;
    auto iconSize = icon.availableSizes().first();
    int iconHeight = iconSize.height();
    int reservedPixels = 16;
    int freeTextSpace = itemSize - iconHeight - reservedPixels;

    if (freeTextSpace > rect.height())
        return;

    QPixmap pix = icon.pixmap(iconSize);
    int newSize = itemSize - rect.height() - reservedPixels;
    if (newSize <= 0)
        newSize = 10;
    QSize newIconSize(newSize, newSize);
    pix = pix.scaled(newIconSize, Qt::KeepAspectRatio);
    QIcon newIcon(pix);
    
    clearSelection();
    Item->setIcon(newIcon);
    clearSelection();
}
void TListWidget::on_itemPressed(QListWidgetItem *Item)
{
    if (rightButtonPressed && selectedItems().count())
        return;

    if (!cont->preview_update) {
        cont->SaveFilters();
        cont->SavePreview();
    }
    if (ClickedItem)
        fitTextInItem(ClickedItem->Item);

    if (!(ClickedItem = FindItem(Item)))
        return;
    
    if (!(qApp->queryKeyboardModifiers() & Qt::ControlModifier))
    {
        clearSelection();
    }

#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
    verticalScrollBar()->setSingleStep(qMax(verticalScrollBar()->maximum() / (count() + 1), 2));
#endif

    switch (ClickedItem->InfoType)
    {
        case TAbstractInfo::_2d:
            emit ItemClicked((const plot::TCurvesDataOwner *)ClickedItem->Info);
        break;

        case TAbstractInfo::_3d:
            emit ItemClicked((const plot::T3dDataOwner *)ClickedItem->Info);
        break;

        case TAbstractInfo::_4d:
            emit ItemClicked((const ccd::TSetOfSpectrum *)ClickedItem->Info);
        break;

        default:
            emit ItemClicked(ClickedItem->attributes.value("Preview").value<QImage>());//(const QImage *)ClickedItem->Info);
        break;
    }

    fitTextInItem(Item);
}


void TListWidget::dragEnterEvent(QDragEnterEvent *event)
{
    QListWidget::dragEnterEvent(event);
}
void TListWidget::dragMoveEvent(QDragMoveEvent *event)
{
    if (event->source() != this)
        QListWidget::dragMoveEvent(event);
}
void TListWidget::dropEvent(QDropEvent *event)
{
    if (event->source() == this)
    {
        QList<TListItem*> items;
        int destPos = DataList.indexOf(FindItem(itemAt(event->pos())));
        //qDebug()<<destPos;
        blockSignals(true);
//        for (auto dl:DataList)
//        {
//            for (auto item:selectedItems())
//            {
//                if (dl->Item == item)
//                {
//                    items.append(dl);
//                    break;
//                }
//            }
//        }

        for (int i=0; i<selectedItems().count(); i++)
        {
            auto item = FindItem(selectedItems()[i]);
            //if (DataList.indexOf(item) < destPos)
            //    destPos--;
            items.append(item);
            DataList.removeOne(item);
        }
        for (int i=0; i<items.count(); i++)
        {
            DataList.insert(destPos, items[i]);
        }
        blockSignals(false);
        cont->UpdateHierarchy();
    }
    else
    {
        cont->DoDrag(dynamic_cast<TListWidget*>(event->source()), this);
        dynamic_cast<TListWidget*>(event->source())->DeleteSelectedItems();
    }
}

bool TListWidget::eventFilterScroll(QObject */*Object*/, QEvent *e)
{
    if (blockAllMouseEvents &&
       (e->type() == QEvent::MouseButtonPress ||
        e->type() == QEvent::MouseMove ||
        e->type() == QEvent::MouseButtonRelease ||
        e->type() == QEvent::Gesture))
    {
        e->ignore();
        return true;
    }

    auto me = dynamic_cast<QMouseEvent *>(e);

    if (e->type() == QEvent::MouseButtonPress &&
       me->button() == Qt::LeftButton &&
       itemAt(me->pos()) == nullptr)
    {
        mouseRectSelecting = true;
        originMouseSelectionPos = me->pos();
        rubberBand->setGeometry(QRect(originMouseSelectionPos, QSize()));
        if (!(qApp->queryKeyboardModifiers() & Qt::ControlModifier))
            clearSelection();
    }

    if (e->type() == QEvent::MouseMove &&
        mouseRectSelecting)
    {
        if (rubberBand->isHidden())
            rubberBand->show();
        rubberBand->setGeometry(QRect(originMouseSelectionPos, me->pos()).normalized());
    }

    if (e->type() == QEvent::MouseButtonRelease &&
        me->button() == Qt::LeftButton &&
        mouseRectSelecting)
    {
        mouseRectSelecting = false;
        rubberBand->hide();

        for (auto &dl : DataList)
        {
            if (rubberBand->geometry().intersects(visualItemRect(dl->Item)))
            {
                dl->Item->setSelected(true);
            }
        }

        return true;
    }

    if (e->type() == QEvent::MouseButtonPress ||
        e->type() == QEvent::MouseButtonRelease)
    {
        rightButtonPressed = me->button() == Qt::RightButton;
        auto item = itemAt(me->pos());
        if (rightButtonPressed && item)
        {
            if (selectedItems().empty())
                setCurrentItem(item);
            else if (!selectedItems().contains(item))
            {
                clearSelection();
                setCurrentItem(item);
            }
        }

        if (rightButtonPressed)
            return true;
    }

    if (e->type() == QEvent::MouseButtonRelease)
    {
        auto pos = me->pos();
        for (int i = 0; i < DataList.size(); i++)
            if (itemAt(pos) == nullptr)
                clearSelection();

        //making selection of the last selected item clear all selected items except that item
        if (selectedItems().count() > 1 && itemAt(pos) == selectedItems().last())
        {
            if (!(qApp->queryKeyboardModifiers() & Qt::ControlModifier))
            {
                auto item = itemAt(pos);
                clearSelection();
                item->setSelected(true);
            }
        }
    }

    if (e->type() == QEvent::KeyRelease)
    {
        if (dynamic_cast<QKeyEvent *>(e)->key() == Qt::Key_Delete && selectedItems().count())
            DeleteSelectedItems();
    }

    return true;
}

QStringList TListWidget::mimeTypes() const { return QStringList() << "nstdatamime" << "image/jpeg" << "image/png"; }
QMimeData *TListWidget::mimeData(const QList<QListWidgetItem *> items) const
{
    for (auto item : items) {
        TListItem *it = FindItem(item);
        if (!it) continue;

        if (it->InfoType == TAbstractInfo::_2d) {
            plot::T2dMime *mimeData = new plot::T2dMime;
            mimeData->setProperty("nstmetadata", it->attributes);
            mimeData->setData("nstdatamime", QByteArray());
            mimeData->data = plot::getUnLock(*(plot::TCurvesDataOwner *)it->Info);
            return mimeData;
        }
        if (it->InfoType == TAbstractInfo::_3d) {
            plot::T3dMime *mimeData = new plot::T3dMime;
            mimeData->setProperty("nstmetadata", it->attributes);
            mimeData->setData("nstdatamime", QByteArray());
            mimeData->data = plot::getUnLock(*(plot::T3dDataOwner *)it->Info);
            return mimeData;
        }
        if (it->InfoType == TAbstractInfo::_Image) {
            QMimeData *mimeData = new QMimeData;
            mimeData->setProperty("nstmetadata", it->attributes);
            mimeData->setImageData(it->attributes.value("Preview"));
            return mimeData;
        }
        if (it->InfoType == TAbstractInfo::_4d) {
            plot::T4dMime *mimeData = new plot::T4dMime;
            mimeData->setProperty("nstmetadata", it->attributes);
            mimeData->setData("nstdatamime", QByteArray());
            auto spec = (ccd::TSetOfSpectrum *)it->Info;
            mimeData->data2D = plot::getUnLock(spec->pointCurve);
            mimeData->data3D = plot::getUnLock(spec->Results);
            return mimeData;
        }
    }

    return nullptr;
}
bool TListWidget::dropMimeData(int /*index*/, const QMimeData *data, Qt::DropAction action)
{
    if (action == Qt::IgnoreAction)
        return true;

    auto *m2d = qobject_cast<const plot::T2dMime *>(data);
    auto *m3d = qobject_cast<const plot::T3dMime *>(data);
    bool image = data->hasImage();
    if (!m2d && !m3d && !image)
        return false;

    cont->copypaste = true;
    if (m2d) {
        plot::TCurvesDataOwner old = plot::getLock(m2d->data);
        plot::TCurvesDataOwner newOne;
        plot::ResizeOwner(newOne, old.size());
        for (int i = 0; i < newOne.size(); i++)
            *newOne.at(i) = *old.at(i);

        cont->AddDataToContainer(newOne, m2d->property("nstmetadata").value<plot::TMetaStorage>());
    } else
    if (m3d) {
        plot::T3dDataOwner old = plot::getLock(m3d->data);
        plot::T3dDataOwner newOne;
        plot::ResizeOwner(newOne, old.size());
        for (int i = 0; i < newOne.size(); i++)
            *newOne.at(i) = *old.at(i);

        cont->AddDataToContainer(newOne, m3d->property("nstmetadata").value<plot::TMetaStorage>());
    } else
    if (image) {
        QImage img = data->imageData().value<QImage>();

        plot::TMetaStorage attributes = data->property("nstmetadata").value<plot::TMetaStorage>();
        if (attributes.isEmpty()) {
            attributes.insert("Name", "Image");
            attributes.insert("Source", "External");

            TContainer::addDateIdsToAttributes(attributes);

            attributes.insert("XMin", 0);
            attributes.insert("XMax", 100);
            attributes.insert("YMin", 0);
            attributes.insert("YMax", 100);

            attributes.insert("XYLabel", "%");
            attributes.insert("XYUnit", QString());
        }

        cont->AddDataToContainer(img, attributes);
    }
    //todo oplkill add 4d
    cont->copypaste = false;

    cont->UpdateHierarchy();

    return true;
}
