#include "spcfile.h"
#include <string.h>
#include <stdio.h>
#include <QDebug>
#include <QFile>
#include <QDataStream>

#if _MSC_VER
#define snprintf _snprintf
#pragma warning( disable : 4996 )
#endif

spcfile::spcfile(QString fname)
{
    // init default values and clean mem

    //clean mem
    memset(&hdr, 0, sizeof(SPCHDR));
    memset(&shdr, 0, sizeof(SUBHDR));
    ext_time = nullptr;

    npts = 0;
    filename = fname;

    data_x = nullptr;
    data_y = nullptr;

    // init header with default values
    hdr.ftflgs = TXVALS; // will use unevenly spaced data by default
    hdr.fversn = 0x4b; // new format
    hdr.fexper = SPCRMN; // Raman spectra
    hdr.fexp = 0x80; // Use unpacked float data

    hdr.fnsub = 1; // onle ONE data set per file
    hdr.fxtype = XRAMANS; // cm-1
    hdr.fytype = XARB; // no predifined value for CCD counts
    hdr.fztype = XARB; // no Z value.
    hdr.fpost = 0; // unused

    //default instrument
    snprintf(hdr.fsource, 9,"Andor CCD");
    //default comment
    snprintf(hdr.fcmnt, 131,"Spectrum from Nspec.");

    // init subheader
    shdr.subexp = 0x80;
}

void spcfile::set_time(time_t * t)
{
    ext_time = t;
}

void spcfile::set_instrument_descr(char * s)
{
    snprintf(hdr.fsource, 9,s);
}

void spcfile::set_comment(char * s)
{
    snprintf(hdr.fcmnt, 131,s);
}

void spcfile::set_x_type(char t)
{
    hdr.fxtype = t;
}

void spcfile::set_y_type(char t)
{
    hdr.fytype = t;
}

void spcfile::set_exper(char t)
{
    hdr.fexper = t;
}

void spcfile::set_npts(int n)
{
    npts = n;
}

void spcfile::set_x_data(const float *x)
{
    data_x = x;
}

void spcfile::set_y_data(const float *y)
{
    data_y = y;
}

int spcfile::write_file()
{
    // check if we got all data
    if ((npts<0)||(data_x == nullptr)||(data_y == nullptr))
    {
        qDebug() << "SPC error";
        return -1;
    }

    // finish header
    cur_time = time(ext_time);
    splited_time = localtime(&cur_time);
    hdr.fdate = splited_time->tm_min + (splited_time->tm_hour << 6) + (splited_time->tm_mday << 11) + ((1 + splited_time->tm_mon) << 16) + ((1900 + splited_time->tm_year) << 20); //magic
    hdr.fnpts = npts; // number of data points
    hdr.ffirst = data_x[0]; // first X value. SHOULD BE ALWAYS SET!!!
    hdr.flast = data_x[npts-1]; // last X value. SHOULD BE ALWAYS SET!!!

    QFile dataFile(filename);
    dataFile.open(QIODevice::WriteOnly);
    QDataStream outData(&dataFile);

    outData.writeRawData ( (char*)&hdr, sizeof(SPCHDR));
    outData.writeRawData ( (char*)data_x, sizeof(float)*npts);
    outData.writeRawData ( (char*)&shdr, sizeof(SUBHDR));
    outData.writeRawData ( (char*)data_y, sizeof(float)*npts);

    dataFile.close();

    return 1;
}
