#include "dataplot.h"
#include "container/datalist.hpp"
#include "ccd/ccd.h"
#include "gui/status.hpp"
#include "tools.hpp"
#include <QCloseEvent>
#include <QMessageBox>
#include <QSplitter>
#include <QHBoxLayout>
#include <QDebug>
#include "widgets/plot_widgets.h"
#include "modules.hpp"
#include "gui/flowlayout.hpp"
#include "gui/datalistoverlaybuttons.h"
#include "widgets/threedoverlaybuttons.h"

void DataPlot::closeEvent(QCloseEvent *event)
{
    event->ignore();
}

bool DataPlot::eventFilter(QObject *Obj, QEvent *Event)
{
    if (Obj == Splitter)
    {
        if (Event->type() == QEvent::LayoutRequest)
        {
            Container->CurrentDataList->needToResize = false;
            Container->resize(Splitter->sizes().first() - Container->overlayButtons->UpButtonSize, Container->height());
        }
        return false;
    }
    if (Event->type() == plot::TSelectEvent::id)
        if ( !Container->SelectEvent(((plot::TSelectEvent *)Event)->SelectedRect,
                                     ((plot::TSelectEvent *)Event)->State,
                                     ((plot::TSelectEvent *)Event)->Origin) ) {

            if (((plot::TSelectEvent *)Event)->State == plot::TSelectEvent::St_SelectionApplied)
                on_BCrop_clicked();
        }

    if (Event->type() == plot::TCrossEvent::id)
        Container->CrossEvent(((plot::TCrossEvent *)Event)->Coord);

    if(((QKeyEvent *)Event)->key() == Qt::Key_Escape)
    {
        if (((plot::TCommonPlotControl*)MyPl->ThreeDPlot->ControlLayout->itemAt(1)->widget())->BCancel->isVisible())
            ((plot::TCommonPlotControl*)MyPl->ThreeDPlot->ControlLayout->itemAt(1)->widget())->BCancel->animateClick();
        if (MyPl->TwoDPlot->Pure2dPlot->FitOrder == -2)
            MyPl->TwoDPlot->PlotControl->BCancel->click();
        if (MyPl->ThreeDPlot->TwoDPlot->Pure2dPlot->FitOrder == -2)
            MyPl->ThreeDPlot->TwoDPlot->PlotControl->BCancel->click();
    }

    if (Event->type() == QEvent::KeyPress)
    {
        if (((QKeyEvent *)Event)->key() == Qt::Key_Return || ((QKeyEvent *)Event)->key() == Qt::Key_Enter)
        {
            if (((plot::TCommonPlotControl*)MyPl->ThreeDPlot->ControlLayout->itemAt(1)->widget())->BApply->isVisible())
                ((plot::TCommonPlotControl*)MyPl->ThreeDPlot->ControlLayout->itemAt(1)->widget())->BApply->animateClick();
            if (MyPl->TwoDPlot->Pure2dPlot->FitOrder == -2)
                MyPl->TwoDPlot->PlotControl->BApply->click();
            if (MyPl->ThreeDPlot->TwoDPlot->Pure2dPlot->FitOrder == -2)
                MyPl->ThreeDPlot->TwoDPlot->PlotControl->BApply->click();
        }
        if (MyPl->ThreeDPlot->overlayButtons)
            MyPl->ThreeDPlot->overlayButtons->event(Event);
    }

    return QWidget::eventFilter(Obj, Event);
}

void DataPlot::on_BCrop_clicked()
{
    void *nd = nullptr;

    if (Container->CurrentDataList->ClickedItem == nullptr)
        return;

    Container->copypaste = true;
    if (Container->CurrentDataList->ClickedItem->InfoType == TAbstractInfo::_4d) {
        ccd::TSetOfSpectrum *info = (ccd::TSetOfSpectrum *)Container->CurrentDataList->ClickedItem->Info;
        QRectF rect = MyPl->ThreeDPlot->ThreeDPlots.at(0)->GetSelectedRect();
        int x0 = qRound(plot::DoubleToArrInd(rect.left(), info->NXElems, info->XLimits));
        int x1 = qRound(plot::DoubleToArrInd(rect.right(), info->NXElems, info->XLimits));
        int y0 = qRound(plot::DoubleToArrInd(rect.top(), info->NYElems, info->YLimits));
        int y1 = qRound(plot::DoubleToArrInd(rect.bottom(), info->NYElems, info->YLimits));

        x0 = qBound<int>(0, x0, info->NXElems-1);
        x1 = qBound<int>(0, x1, info->NXElems-1);
        y0 = qBound<int>(0, y0, info->NYElems-1);
        y1 = qBound<int>(0, y1, info->NYElems-1);

        int nx = x1 - x0 + 1;
        int ny = y1 - y0 + 1;

        Range dx, dy;
        dx[0] = info->XLimits[0] + x0*info->dx;
        dx[1] = info->XLimits[0] + x1*info->dx;
        dy[0] = info->YLimits[0] + y0*info->dy;
        dy[1] = info->YLimits[0] + y1*info->dy;

        if (nx < 2 || ny < 2) {
            new Notification(tr("Data") + ": " + tr("Selected region contains not enough data."), Notification::Warn, 10);
            doBeep();
            return;
        }
        if ((dx[1] - dx[0]) / (dy[1] - dy[0]) > 20 || (dy[1] - dy[0]) / (dx[1] - dx[0]) > 20) {
            if (QMessageBox::Yes != QMessageBox::warning(this, tr("Data"), tr("Selected region has high aspect ratio. Do you want to continue?"), QMessageBox::Yes, QMessageBox::No))
                return;
        }

        ccd::TSetOfSpectrum *n4 = new ccd::TSetOfSpectrum;
        ccd::TSetOfSpectrumConstructor I4dAdd;
        I4dAdd.data = n4;
        n4->YscaleUnit = info->YscaleUnit;
        n4->CalibInfo = info->CalibInfo;
        n4->CurrentFunc = info->CurrentFunc;
        I4dAdd.PrepareForNewScan(nx, ny, dx, dy, info->MinForRecord, info->MaxForRecord, info->MinForFuncApply, info->MaxForFuncApply);
        n4->SetDirStPoint(info->Direction, info->StartPoint);

        switch (info->StartPoint) {
        case ccd::TInfoForSetOfSpectrums::TopRight:
            x0 = info->NXElems - 1 - x1;
            break;
        case ccd::TInfoForSetOfSpectrums::TopLeft:
            y0 = info->NYElems - 1 - y1;
            break;
        case ccd::TInfoForSetOfSpectrums::BottomRight:
            x0 = info->NXElems - 1 - x1;
            break;
        }

        if (info->Direction == ccd::TInfoForSetOfSpectrums::Vert) {
            qSwap(nx, ny);
            qSwap(x0, y0);
        }

        for (int i = 0; i < ny; i++) {
            for (int j = 0; j < nx; j++) {
                int y = qBound(0, y0 + i, info->Array.size()-1);
                int x = qBound(0, x0 + j, info->Array[y].size()-1);
                if (info->Direction == ccd::TInfoForSetOfSpectrums::Vert && info->NXElems != info->NYElems) {
                    int n = (y0 + i) * info->NYElems + (x0 + j);
                    y = qBound<int>(0, (n / info->NXElems), info->Array.size()-1);
                    x = qBound<int>(0, (n % info->NXElems), info->Array[y].size()-1);
                }
                I4dAdd.AddNextSpectrum(info->Array[y][x]);
            }
        }

        nd = n4;
        n4->CalcResults();

        plot::TMetaStorage attributes = Container->CurrentDataList->ClickedItem->attributes;
        attributes.insert("XMin", n4->XLimits[0]);
        attributes.insert("XMax", n4->XLimits[1]);
        attributes.insert("YMin", n4->YLimits[0]);
        attributes.insert("YMax", n4->YLimits[1]);

        Container->AddDataToContainer(n4, attributes);
    } else {
        plot::T3dDataOwner dm;
		auto plot = MyPl->ThreeDPlot->ThreeDPlots.at(0);
        dm.append(QSharedPointer<plot::TMin3dData>(plot->SourceData.at(0).data()->Crop(plot->GetSelectedRect(),
			plot->PlotsXLimits,
			plot->PlotsYLimits)));
        auto dt = dm.at(0);

        if (dt->NYElems < 2 || dt->NXElems < 2) {
            new Notification(tr("Data") + ": " + tr("Selected region contains not enough data."), Notification::Warn, 10);
            doBeep();
            return;
        }
        if ((dt->XLimits[1] - dt->XLimits[0]) / (dt->YLimits[1] - dt->YLimits[0]) > 20 ||
                (dt->YLimits[1] - dt->YLimits[0]) / (dt->XLimits[1] - dt->XLimits[0]) > 20) {
            if (QMessageBox::Yes != QMessageBox::warning(this, tr("Data"), tr("Selected region has high aspect ratio. Do you want to continue?"), QMessageBox::Yes, QMessageBox::No))
                return;
        }

        Container->AddDataToContainer_slot(dm, Container->CurrentDataList->ClickedItem->attributes);
        nd = Container->CurrentDataList->DataList.back()->Info;
    }
    Container->UpdateHierarchy();
    Container->copypaste = false;

    Container->CurrentDataList->DataList.back()->UpdatePreview = true;

    QListWidgetItem *cropped = Container->CurrentDataList->item(Container->CurrentDataList->count() - 1);
    for (int i = 0; i < Container->CurrentDataList->DataList.size(); i++) {
        if (Container->CurrentDataList->DataList[i]->Info == nd) {
            cropped = Container->CurrentDataList->DataList[i]->Item;
            break;
        }
    }
    Container->CurrentDataList->setCurrentItem(cropped);
}


DataPlot::DataPlot(QWidget *parent) : QWidget(parent)
{
    Splitter = new QSplitter(this);
    Splitter->installEventFilter(this);

    QHBoxLayout *layout_main = new QHBoxLayout(this);
    layout_main->setContentsMargins(0, 0, 0, 0);
    layout_main->addWidget(Splitter);

    QWidget *Widmypl= new QWidget(Splitter);
    QHBoxLayout *mypl_l = new QHBoxLayout(Widmypl);
    mypl_l->setSpacing(3);
    mypl_l->setContentsMargins(6, 6, 6, 9);

    Splitter->addWidget(Widmypl);

    MyPl = new plot::TPlot(this, Widmypl);

    //MyPl->TwoDPlotVertical = this->

    MyPl->ThreeDPlot->SetZoomBySelection(false);
    MyPl->ThreeDPlot->TwoDPlot->Pure2dPlot->SetZoomBySelection(false);
    MyPl->ThreeDPlot->TwoDPlot->PlotControl->BSelectByRect->hide();
    MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->UseMathOnly = true;
    MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->SetZoomBySelection(false);
    MyPl->ThreeDPlot->TwoDPlotVertical->PlotControl->BSelectByRect->hide();
    MyPl->TwoDPlot->Parent = MyPl->TwoDPlot->Pure2dPlot->Parent = nullptr; //for area select to not crash in crop code
    //MyPl->ThreeDPlot->TwoDPlotVertical->Parent = MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->Parent = 0; //for area select to not crash in crop code
    MyPl->TwoDPlot->Pure2dPlot->SetDefaultXLimits(0, 100);
    MyPl->TwoDPlot->Pure2dPlot->SetDefaultYLimits(0, 100);
    MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->SetDefaultXLimits(0, 100);
    MyPl->ThreeDPlot->TwoDPlotVertical->Pure2dPlot->SetDefaultYLimits(0, 100);
    MyPl->ThreeDPlot->TwoDPlot->SetTitle(QObject::tr("section"));
    MyPl->ThreeDPlot->TwoDPlotVertical->SetTitle(QObject::tr("section"));
    MyPl->ThreeDPlot->PlotControl->BSelectByRect->setToolTip(tr("Image Crop"));
    MyPl->ThreeDPlot->PlotControl->BSelectByRect->setIcon(makeHugeIcon(":/plot/Cuting.png"));
    MyPl->TwoDPlot->PlotControl->BSave->setShortcut(QKeySequence());
    MyPl->ThreeDPlot->PlotControl->BSave->setShortcut(QKeySequence());
    MyPl->ThreeDPlot->PlotControl->setVisible(false);

    MyPl->ThreeDPlot->PlotControl->BCross->setIcon(makeHugeIcon(":/CCD/point_spect_off.png"));
    MyPl->ThreeDPlot->PlotControl->BCross->offstring = ":/CCD/point_spect_off.png";

    mypl_l->addWidget(MyPl, 1);
    MyPl->ThreeDPlot->Slider->hide();

    Container = new TContainer(MyPl, Splitter);
    MyPl->ThreeDPlot->ControlLayout->addWidget(Container->gb4DButtons);
    MyPl->TwoDPlot->PlotControl->ControlMainLayout->addWidget(Container->Scale_select);
    Container->prev2DLayoutScaleSelect = MyPl->TwoDPlot->PlotControl->ControlMainLayout;

    connect(MyPl, SIGNAL(AddDataToContainer(plot::TCurvesDataOwner,plot::TMetaStorage)), Container, SLOT(AddDataToContainer_slot(plot::TCurvesDataOwner,plot::TMetaStorage)));
    connect(MyPl, SIGNAL(AddDataToContainer(plot::T3dDataOwner,plot::TMetaStorage)), Container, SLOT(AddDataToContainer_slot(plot::T3dDataOwner,plot::TMetaStorage)));

    Splitter->insertWidget(0, Container);
    Splitter->setStretchFactor(1, 1); //default is 0 = size, fix left part
    Splitter->setSizes(QList<int>() << MyPl->height()*0.75 << MyPl->height()*0.25);

    installEventFilter(this);

}

DataPlot* DataPlot::GetIt() noexcept
{
    auto data = ModuleManager::instance->find1VoidPtr("data");
    return static_cast<DataPlot*>(data);
}
