//http://blog.codeimproved.net/posts/kinetic-scrolling.html
//Written by raz
//BSD license

#include "QsKineticScroller.h"

#include <QAbstractScrollArea>
#include <QAbstractItemView>
#include <QElapsedTimer>
#include <QTimer>
#include <QtEvents>
#include <QScrollBar>
#include <QApplication>
#include <math.h>

QsKineticScroller* addScroller(QAbstractScrollArea *target) {
    QsKineticScroller *scroll = new QsKineticScroller(target);
    scroll->enableKineticScrollFor(target);
    return scroll;
}
QsKineticScroller* addScroller(QAbstractItemView *target) {
    QsKineticScroller *scroll = addScroller((QAbstractScrollArea *)target);
    target->setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
    return scroll;
}

static const int gTimerInterval = 50;

class QsKineticScrollerImpl
{
public:
    QsKineticScrollerImpl() {}

    QAbstractScrollArea* scrollArea = nullptr;
    bool isPressed = false;
    bool isScroll = false;
    QPoint firstPressPoint;
    QPoint lastPressPoint;
    double velocity = 0.;
    QTimer kineticTimer;
    QElapsedTimer tm;

    QList<QEvent*> ignoreList;
};

QsKineticScroller::QsKineticScroller(QObject *parent)
    : QObject(parent)
    , d(new QsKineticScrollerImpl)
{
    connect(&d->kineticTimer, SIGNAL(timeout()), SLOT(onKineticTimerElapsed()));
}

QsKineticScroller::~QsKineticScroller()
{
    //fuck gcc
}

void QsKineticScroller::enableKineticScrollFor(QAbstractScrollArea* scrollArea)
{
    // remove existing association
    if (d->scrollArea) {
        QList<QWidget *> widget = scrollArea->viewport()->findChildren<QWidget *>();
        Q_FOREACH(QWidget *wid, widget)
            wid->removeEventFilter(this);

        d->scrollArea->viewport()->removeEventFilter(this);
        d->scrollArea->removeEventFilter(this);
    }

    // associate
    d->scrollArea = scrollArea;

    if (d->scrollArea) {
        scrollArea->installEventFilter(this);
        scrollArea->viewport()->installEventFilter(this);

        QList<QWidget *> widget = scrollArea->viewport()->findChildren<QWidget *>();
        Q_FOREACH(QWidget *wid, widget)
            wid->installEventFilter(this);
    }
}

//! intercepts mouse events to make the scrolling work
bool QsKineticScroller::eventFilter(QObject* object, QEvent* event)
{
    const QEvent::Type eventType = event->type();
    const bool isMouseAction = QEvent::MouseButtonPress == eventType
            || QEvent::MouseButtonRelease == eventType;
    const bool isMouseEvent = isMouseAction || QEvent::MouseMove == eventType;

    emit eventFilterSignal(object, event);

    if( !isMouseEvent || !d->scrollArea )
        return false;

    if( isMouseAction && d->ignoreList.removeAll(event) > 0 ) // This event is ignored
        return false;

    QMouseEvent* const mouseEvent = static_cast<QMouseEvent*>(event);
    switch( eventType )
    {
    case QEvent::MouseButtonPress:
    {
        if (!(mouseEvent->buttons() & Qt::LeftButton))
            return false;

        d->isPressed = true;
        d->isScroll = d->kineticTimer.isActive();
        d->lastPressPoint = d->firstPressPoint = mouseEvent->globalPos();
        d->velocity = 0;
        d->kineticTimer.stop();
        d->tm.start();
    }
        break;
    case QEvent::MouseMove:
    {
        if (!d->isPressed)
            return false;

        const int dx = 50;
        if ( qAbs((d->firstPressPoint - mouseEvent->globalPos()).y()) > dx )
            d->isScroll = true;

        // manual scroll
        const int dragDistance = mouseEvent->globalPos().y() - d->lastPressPoint.y();
        const int finalScrollPos = d->scrollArea->verticalScrollBar()->value() - dragDistance;
        d->scrollArea->verticalScrollBar()->setValue(finalScrollPos);

        //int time = qMax<int>(1, d->tm.elapsed());
        d->velocity = -double(d->lastPressPoint.y() - mouseEvent->globalPos().y());// * gTimerInterval / time;
        d->lastPressPoint = mouseEvent->globalPos();
        d->tm.start();
    }
        break;
    case QEvent::MouseButtonRelease:
    {
        if (!d->isPressed)
            return false;

        const QPoint& releasePos = mouseEvent->pos();
        d->isPressed = false;

        const int dx = 50;
        if ( qAbs((d->firstPressPoint - mouseEvent->globalPos()).y()) > dx )
            d->isScroll = true;

        if (!d->isScroll)
        {
            QMouseEvent* mousePress = new QMouseEvent(QEvent::MouseButtonPress,
                                                      releasePos, Qt::LeftButton, Qt::LeftButton, Qt::NoModifier);
            QMouseEvent* mouseRelease = new QMouseEvent(QEvent::MouseButtonRelease,
                                                        releasePos, Qt::LeftButton, Qt::LeftButton, Qt::NoModifier);

            // Ignore these two
            d->ignoreList << mousePress;
            d->ignoreList << mouseRelease;

            QApplication::postEvent(object, mousePress);
            QApplication::postEvent(object, mouseRelease);
        } else {
            if (d->lastPressPoint.y() != mouseEvent->globalPos().y()) {
                //int time = qMax<int>(1, d->tm.elapsed());
                d->velocity = -double(d->lastPressPoint.y() - mouseEvent->globalPos().y());// * gTimerInterval / time;
            }
            d->kineticTimer.start(gTimerInterval);
        }
    }
        break;
    default:
        break;
    }

    return true; // filter event
}

static const double gMaxDecelerationSpeed = 30.;
static const double gFriction = 1.0;

void QsKineticScroller::onKineticTimerElapsed()
{
    if (!d->isPressed) {
        d->velocity = qBound(-gMaxDecelerationSpeed, d->velocity, gMaxDecelerationSpeed);
        if( d->velocity > 0 )
            d->velocity -= gFriction;
        else if( d->velocity < 0 )
            d->velocity += gFriction;
        if( fabs(d->velocity) < gFriction )
            d->kineticTimer.stop();

        const int scrollBarYPos = d->scrollArea->verticalScrollBar()->value();
        const int finalScrollPos = scrollBarYPos - d->velocity;
        d->scrollArea->verticalScrollBar()->setValue(finalScrollPos);
    }
    else
        d->kineticTimer.stop();
}
