#pragma once

#include <QIcon>
#include <QLineEdit>
#include <QFrame>
#include <QPushButton>
class QFormLayout;

QIcon rotareIcon(const QIcon icon, QSize size, int gradus);
void rotareButtonIcon(QPushButton *btn, int gradus);
QIcon makeHugeIcon(QString path);
QIcon makeHugeIconBounded(QString path, QColor color, qreal bWidth);
QString *makeVerticalText(const QString &text);
QFormLayout *makeFormLayout(QWidget *parent = nullptr);

class QDoubleValidator;
class DoubleEdit : public QLineEdit
{
    Q_OBJECT

public slots:
    void stepUp();
    void stepDown();
    void setMin(double value_min);
    void setMax(double value_max);
    void setValue(double value);
    void setNewValue();
    void resetValue();

signals:
    void editingStarted();
    void newValue();
    void newValue(int);
    void newValue(double);

public:
    DoubleEdit(double min, double max, int acc);
    double value() const;

protected:
    void focusInEvent(QFocusEvent *event) override;
    void focusOutEvent(QFocusEvent *event) override;

private:
    double prev_value;
    QDoubleValidator *range_validator;
};

class NSTSlider;
class Edit;
class DoubleChanger : public QFrame
{
    Q_OBJECT

public slots:
    void setDecimals(int decimals);
    void setMinimum(double value_min);
    void setMaximum(double value_max);
    void setValue(double value);
    void setNewValue(bool emitNewValue = true);

signals:
    void editingStarted();
    void editingFinished();
    void newValue();
    void newValue(int);
    void newValue(double);
    void newMinimum(double);
    void newMaximum(double);
    void newDecimals(int);

public:
    DoubleChanger(double min, double max, int acc = 3, bool vertical = false);
    double value() const;
    double maximum() const;
    double minimum() const;

    Edit *edit;
    NSTSlider *slider;
    bool vertical = false;
    
    QSize minimumSizeHint() const override;
};
void linkDoubleChangers(DoubleChanger *master, DoubleChanger *slave, bool linkRange = true);

class TwoStateButton : public QPushButton
{
    Q_OBJECT

public slots:
    void change_state();
    void change_state_noEmit();
    void change_state_man(bool);
    void change_state_man_with_sig(bool st);

signals:
    void state_changed(bool);
    void state_changed_man(bool);

public:
    TwoStateButton(QString off, QString on);
    TwoStateButton(const QString & text,QString off, QString on);
    void rotare(int gradus);

    QString offstring;
    QString onstring;
    bool isOn;
    int rotareGradus = 0;
};


class TwoStateButton2 : public QPushButton
{
    Q_OBJECT

public slots:
    void change_state();
    void change_state_noEmit();
    void change_state_man(bool);
    void change_state_man_with_sig(bool st);

signals:
    void state_changed(bool);

public:
    TwoStateButton2(QString off, QString on, int iconSize);

    QString offstring;
    QString onstring;
    bool isOn;
    int iconWidth;

private:
    bool event(QEvent *e) override;
};
