#include "datalistoverlaybuttons.h"
#include "container/container.h"

DataListOverlayButtons::DataListOverlayButtons(TContainer *container, QTabWidget *tabWid) : QWidget(nullptr)
{
    this->container = container;
    DataLists = tabWid;

    background = new colorRect(QColor(255, 255, 255));

    BToDevice = new QPushButton;
    BToDevice->setObjectName(QString::fromUtf8("BToDevice"));
    BToDevice->setToolTip(tr("Send info to device"));
    BToDevice->setIcon(makeHugeIcon(":/container/todev.png"));
    setButtonStyle(BToDevice);
    BToDevice->hide();
    connect(BToDevice, SIGNAL(clicked()), container, SLOT(btnToDevice_clicked()));

    BToGwyddion = new QPushButton;
    BToGwyddion->setObjectName(QString::fromUtf8("BToGwyddion"));
    BToGwyddion->setToolTip(tr("Send selected info to Gwyddion") + " (Ctrl+G)");
    BToGwyddion->setIcon(makeHugeIcon(":/container/GwyExp.png"));
    BToGwyddion->setShortcut(QKeySequence(Qt::CTRL | Qt::Key_G));
    setButtonStyle(BToGwyddion);
    BToGwyddion->hide();
    connect(BToGwyddion, SIGNAL(clicked()), container, SLOT(btnToGwyddion_clicked()));

    BSave = new QPushButton;
    BSave->setObjectName(QString::fromUtf8("BSave"));
    BSave->setToolTip(tr("Save") + " (Ctrl+S)");
    BSave->setIcon(makeHugeIcon(":/container/save.png"));
    BSave->setShortcut(QKeySequence(Qt::CTRL | Qt::Key_S));
    setButtonStyle(BSave);
    connect(BSave, SIGNAL(clicked()), container, SLOT(btnSave_clicked()));

    BSaveAs = new QPushButton;
    BSaveAs->setObjectName(QString::fromUtf8("BSaveAs"));
    BSaveAs->setToolTip(tr("Save As...") + " (Ctrl+Shift+S)");
    BSaveAs->setIcon(makeHugeIcon(":/container/saveas.png"));
    BSaveAs->setShortcut(QKeySequence(Qt::CTRL | Qt::SHIFT | Qt::Key_S));
    setButtonStyle(BSaveAs);
    connect(BSaveAs, SIGNAL(clicked()), container, SLOT(btnSaveAs_clicked()));

    BToAscii = new QPushButton;
    BToAscii->setObjectName(QString::fromUtf8("BToAscii"));
    BToAscii->setToolTip(tr("Export to CSV"));
    BToAscii->setIcon(makeHugeIcon(":/container/toTXT.png"));
    setButtonStyle(BToAscii);
    connect(BToAscii, SIGNAL(clicked()), container, SLOT(btnToAscii_clicked()));

    BToOldNSpec = new QPushButton;
    BToOldNSpec->setObjectName(QString::fromUtf8("BToOldFormat"));
    BToOldNSpec->setToolTip(tr("Export to Gwyddion"));
    BToOldNSpec->setIcon(makeHugeIcon(":/container/GwyExp.png"));
    setButtonStyle(BToOldNSpec);
    connect(BToOldNSpec, SIGNAL(clicked()), container, SLOT(btnToOldNspec_clicked()));

    BOpen = new QPushButton;
    BOpen->setObjectName(QString::fromUtf8("BOpen"));
    BOpen->setToolTip(tr("Open") + " (Ctrl+O)");
    BOpen->setIcon(makeHugeIcon(":/container/open.png"));
    BOpen->setShortcut(QKeySequence(Qt::CTRL | Qt::Key_O));
    setButtonStyle(BOpen);
    connect(BOpen, SIGNAL(clicked()), container, SLOT(btnOpen_clicked()));

    BMerge = new QPushButton;
    BMerge->setObjectName(QString::fromUtf8("BMerge"));
    BMerge->setToolTip(tr("Merge all tabs"));
    BMerge->setIcon(makeHugeIcon(":/container/merge.png"));
    setButtonStyle(BMerge);
    connect(BMerge, SIGNAL(clicked()), container, SLOT(btnMerge_clicked()));

    BAddTab = new QPushButton;
    BAddTab->setObjectName(QString::fromUtf8("BAddTab"));
    BAddTab->setToolTip(tr("Add new tab") + " (Ctrl+T)");
    BAddTab->setIcon(makeHugeIcon(":/container/newdata.png"));
    BAddTab->setShortcut(QKeySequence(Qt::CTRL | Qt::Key_T));
    setButtonStyle(BAddTab);
    connect(BAddTab, SIGNAL(clicked()), container, SLOT(addTab()));

    allBtnsList << BOpen << BSave << BSaveAs << BAddTab
                << BMerge << BToAscii << BToOldNSpec/* << BToGwyddion << BToDevice*/;

    for (int i=0; i<allBtnsList.size(); i++)
    {
        allBtnsList.at(i)->setFixedSize(UpButtonSize, UpButtonSize);
        allBtnsList.at(i)->setIconSize(QSize(UpButtonSize, UpButtonSize));
    }

    setVisibleButtons(true);
}

void DataListOverlayButtons::setVisibleButtons(bool visible)
{
    background->setParent(DataLists);

    for (int i=0; i<allBtnsList.size(); i++)
        allBtnsList.at(i)->setParent(DataLists);

    resized(DataLists->size());

    background->setVisible(true);
    for (int i=0; i<allBtnsList.size(); i++)
        allBtnsList.at(i)->setVisible(visible);

    int x = 0;
    x = allBtnsList.first()->width();
    container->CurrentDataList->move(x, 0);
}

void DataListOverlayButtons::resized(QSize newSize)
{
    int upY = 0;
#ifdef _WIN32
    if (DataLists && DataLists->tabBar())
        upY = DataLists->tabBar()->sizeHint().height();
#endif
    if (upY == 0)
        upY = 40;

    double mulBtn = 0.15;

    background->setFixedSize(UpButtonSize + 2, container->CurrentDataList->height());

    int x = 0;
    int y = upY;
    background->move(x, y);
    for (int i=0; i<allBtnsList.size(); i++)
    {
        allBtnsList.at(i)->setIconSize(QSize(UpButtonSize, UpButtonSize));
        allBtnsList[i]->move(x, y);
        y += UpButtonSize + UpButtonSize*mulBtn;
    }
}

void DataListOverlayButtons::setButtonStyle(QPushButton *btn, bool active)
{
    QString style = "";

    style += "QPushButton {background-color: rgba(255,255,255,10);";
    if (active)
        style += "border: 2px solid red;}";
    else
        style += "border: 2px solid #9F9F9F;}";
    style += "QToolTip { color: black; background-color: white; border: 1px solid #7F7F7F; }";
    style += "QPushButton:pressed{ background-color: lightgray; }";

    btn->setStyleSheet( style );
}
