#include "devicescheme.h"

deviceScheme::deviceScheme(QWidget *parent) : QWidget(parent)
{
    psd = new schPSD(this);
    inputGain = new schInputGains(this);
    DFLSwitch = new schSwitch(2, this);
    LFLSwitch = new schSwitch(2, this);
    DFLtext = new schQTextEdit(this);
    LFLtext = new schQTextEdit(this);
    INTtext = new schQTextEdit(this);
    LockIn = new schLockIn(this);
    diodeSignals = new schDiodeSignals(this);
    MagText = new schQTextEdit(this);
    MagSinText = new schQTextEdit(this);
    MagCosText = new schQTextEdit(this);
    PhaseText = new schQTextEdit(this);
    PIDSwitch = new schSwitch(6, this);
    setPointEdit = new schEdit(-10, 10, 2, this);
    pid = new schPID(this);
    HVZTria = new schTriangle(schTriangle::triDirection::Down, tr("HVZ"), this);
    feedBack = new schBreaker(this);
    oscilator = new schOscilator(this);
    plus = new schPlus(this);
    plusSwitch = new schSwitch(2, this);
    bias = new schBias(this);
    ZPiezo = new schZPiezo(this);
    ZPiezoArrGd = new schArrowGrounding(this);
    ExtOsc = new schExtOsc(this);
    PiesoArrGd = new schArrowGrounding(this);
    Piezo = new schpiezo(this);
    DFLPoint = new schPoint(0, 1, this);
    LFLPoint = new schPoint(0, 1, this);
    ZPiezoPosEdit = new schEdit*[3]();
    ZPiezoPosEdit[0] = new schEdit(0, 150, 4, this);
    ZPiezoPosEdit[1] = new schEdit(0, 150, 4, this);
    ZPiezoPosEdit[2] = new schEdit(0, 50, 4, this);

    setPointLabel = new QLabel(tr("Set point"), this);
    MagLabel = new QLabel(tr("Mag"), this);
    MagSinLabel = new QLabel(tr("Mag sin"), this);
    MagCosLabel = new QLabel(tr("Mag cos"), this);
    PhaseLabel = new QLabel(tr("Phase"), this);
    ZPiezoPosLabel = new QLabel*[3]();
    ZPiezoPosLabel[0] = new QLabel(tr("X pos"), this);
    ZPiezoPosLabel[1] = new QLabel(tr("Y pos"), this);
    ZPiezoPosLabel[2] = new QLabel(tr("Z pos"), this);
    FeedBackLabel = new QLabel(tr("Feedback"), this);
    FBInputLabel = new QLabel(tr("FB INPUT"), this);

    connect(DFLSwitch, SIGNAL(recalc()), this, SLOT(recalcSwitch_DFL()));
    connect(LFLSwitch, SIGNAL(recalc()), this, SLOT(recalcSwitch_LFL()));
    connect(PIDSwitch, SIGNAL(recalc()), this, SLOT(recalcSwitch_PID()));
    connect(plusSwitch, SIGNAL(recalc()), this, SLOT(recalcSwitch_Plus()));

    connect(diodeSignals, SIGNAL(recalc()), this, SLOT(recalcDiodeSignals()));
    connect(diodeSignals, SIGNAL(initPos(int, int)), this, SLOT(initPosDiodeSignals(int, int)));
    connect(LockIn, SIGNAL(recalc()), this, SLOT(recalcLockIn()));

    inputGain->triangleW = 70*0.85;
    inputGain->triangleH = inputGain->triangleW = (240-240*0.15)*0.4*scaleWidth;
    DFLtext->setReadOnly(true);
    LFLtext->setReadOnly(true);
    INTtext->setReadOnly(true);
    MagText->setReadOnly(true);
    MagSinText->setReadOnly(true);
    MagCosText->setReadOnly(true);
    PhaseText->setReadOnly(true);
    DFLtext->setButtonSymbols(QAbstractSpinBox::NoButtons);
    LFLtext->setButtonSymbols(QAbstractSpinBox::NoButtons);
    INTtext->setButtonSymbols(QAbstractSpinBox::NoButtons);
    MagText->setButtonSymbols(QAbstractSpinBox::NoButtons);
    MagSinText->setButtonSymbols(QAbstractSpinBox::NoButtons);
    MagCosText->setButtonSymbols(QAbstractSpinBox::NoButtons);
    PhaseText->setButtonSymbols(QAbstractSpinBox::NoButtons);
    HVZTria->backgroundColor = Qt::white;
    Piezo->backgroundColor = Qt::white;
    ZPiezo->backgroundColor = Qt::white;
    PiesoArrGd->backgroundColor = QColor(88, 198, 199, 60);//Qt::white;
    ZPiezoArrGd->backgroundColor = QColor(88, 198, 199, 60);//Qt::white;
    plus->backgroundColor = Qt::white;
    QColor outerColor = QColor(0, 171, 255);
    diodeSignals->outterLineColor = outerColor;
    inputGain->outterLineColor = outerColor;
    LockIn->outterLineColor = outerColor;
    pid->outterLineColor = outerColor;
    oscilator->outterLineColor = outerColor;
    ExtOsc->outterLineColor = outerColor;
    bias->outterLineColor = outerColor;
    psd->outterLineColor = outerColor;
    DFLSwitch->outterLineColor = outerColor;
    LFLSwitch->outterLineColor = outerColor;
    PIDSwitch->outterLineColor = outerColor;
    plusSwitch->outterLineColor = outerColor;
    feedBack->outterLineColor = outerColor;
    psd->outterLineWidth = 2;
    diodeSignals->outterLineWidth = inputGain->outterLineWidth = LockIn->outterLineWidth =
            pid->outterLineWidth = oscilator->outterLineWidth = ExtOsc->outterLineWidth =
            bias->outterLineWidth = 4;
    DFLSwitch->outterLineWidth = LFLSwitch->outterLineWidth = PIDSwitch->outterLineWidth =
            plusSwitch->outterLineWidth = feedBack->outterLineWidth = 2;

    elements<<psd<<inputGain<<diodeSignals<<DFLSwitch<<LFLSwitch<<DFLtext
            <<LFLtext<<INTtext<<LockIn<<MagText<<MagCosText<<MagSinText<<PhaseText
            <<PIDSwitch<<setPointEdit<<pid<<HVZTria<<feedBack<<oscilator<<plus<<plusSwitch
            <<bias<<ZPiezo<<ZPiezoArrGd<<ExtOsc<<PiesoArrGd<<Piezo<<DFLPoint<<LFLPoint
            <<ZPiezoPosEdit[0]<<ZPiezoPosEdit[1]<<ZPiezoPosEdit[2];

    labels<<setPointLabel<<MagLabel<<MagSinLabel<<MagCosLabel<<PhaseLabel
          <<ZPiezoPosLabel[0]<<ZPiezoPosLabel[1]<<ZPiezoPosLabel[2]
          <<FeedBackLabel<<FBInputLabel;

    scaleWidth = 1.0;
    scaleHeight = 1.0;

    connectElements();
    rescale();

    diodeSignals->setShadow(diodeSignals, 3, Qt::gray);
    inputGain->setShadow(inputGain, 3, Qt::gray);
    LockIn->setShadow(LockIn, 3, Qt::gray);
    pid->setShadow(pid, 3, Qt::gray);
    oscilator->setShadow(oscilator, 3, Qt::gray);
    ExtOsc->setShadow(ExtOsc, 3, Qt::gray);
    bias->setShadow(bias, 3, Qt::gray);
    psd->setShadow(bias, 3, Qt::gray);
    plusSwitch->currentConnectedIn = 1;

}

void deviceScheme::rescale()
{
    const double recW = 930;
    const double recH = 530;

    scaleWidth = width()/recW;
    scaleHeight = height()/recH;

    if ((240-240*0.15)*0.33*scaleWidth < (240-240*0.15)*0.33)
        inputGain->triangleH = inputGain->triangleW = (240-240*0.15)*0.33;
    else
        inputGain->triangleH = inputGain->triangleW = (240-240*0.15)*0.33*scaleWidth;

    psd->setFixedSize(          150*scaleWidth, 120*scaleHeight);
    diodeSignals->setFixedSize( 70*scaleWidth,  240*scaleHeight);
    DFLSwitch->setFixedSize(    50*scaleWidth,  40*scaleHeight);
    LFLSwitch->setFixedSize(    50*scaleWidth,  40*scaleHeight);
    inputGain->setFixedSize(    100*scaleWidth, 240*scaleHeight);
    DFLtext->setFixedSize(      40*scaleWidth,  25*scaleHeight);
    LFLtext->setFixedSize(      40*scaleWidth,  25*scaleHeight);
    INTtext->setFixedSize(      40*scaleWidth,  25*scaleHeight);
    DFLPoint->setFixedSize(     4*scaleWidth,   4*scaleHeight);
    LFLPoint->setFixedSize(     4*scaleWidth,   4*scaleHeight);
    LockIn->setFixedSize(       100*scaleWidth, 200*scaleHeight);
    MagText->setFixedSize(      55*scaleWidth,  25*scaleHeight);
    MagSinText->setFixedSize(   55*scaleWidth,  25*scaleHeight);
    MagCosText->setFixedSize(   55*scaleWidth,  25*scaleHeight);
    PhaseText->setFixedSize(    55*scaleWidth,  25*scaleHeight);
    PIDSwitch->setFixedSize(    160*scaleWidth, 50*scaleHeight);
    setPointEdit->setFixedSize( 55*scaleWidth,  25*scaleHeight);
    pid->setFixedSize(          100*scaleWidth, 100*scaleHeight);
    HVZTria->setFixedSize(      50*scaleWidth,  50*scaleWidth);
    feedBack->setFixedSize(     50*scaleWidth,  50*scaleWidth);
    oscilator->setFixedSize(    300*scaleWidth, 200*scaleHeight);
    plus->setFixedSize(         30*scaleWidth,  30*scaleWidth);
    plusSwitch->setFixedSize(   60*scaleWidth,  50*scaleHeight);
    bias->setFixedSize(         70*scaleWidth,  70*scaleHeight);
    ZPiezo->setFixedSize(       100*scaleWidth, 150*scaleHeight);
    ZPiezoArrGd->setFixedSize(  75*scaleWidth,  40*scaleHeight);
    ExtOsc->setFixedSize(       70*scaleWidth,  70*scaleHeight);
    PiesoArrGd->setFixedSize(   75*scaleWidth,  40*scaleHeight);
    Piezo->setFixedSize(        120*scaleWidth, 50*scaleHeight);
    ZPiezoPosEdit[0]->setFixedSize(65*scaleWidth, 25*scaleHeight);
    ZPiezoPosEdit[1]->setFixedSize(65*scaleWidth, 25*scaleHeight);
    ZPiezoPosEdit[2]->setFixedSize(65*scaleWidth, 25*scaleHeight);

    for (int i=0; i<labels.count(); i++)
    {
        labels[i]->setScaledContents(true);
        labels[i]->setFont(QFont(labels[i]->font().family(), 10*scaleWidth));
        labels[i]->adjustSize();
    }

    psd->setPosXY(0, 80*scaleHeight);
    inputGain->recalcPoints();
    diodeSignals->setPosXY(psd->width() + 20*scaleWidth, 0);
    diodeSignals->recalcPoints();
    inputGain->recalcPoints();
    inputGain->setPosXY(diodeSignals->absPos->x() + diodeSignals->width() + 150*scaleWidth,
                        0);
                        //diodeSignals->absPos->y() + diodeSignals->ptsOut[1]->y() - inputGain->ptsIn[1]->y());
    diodeSignals->recalcPoints();
    DFLSwitch->recalcPoints();
    DFLSwitch->setPosXY(diodeSignals->absPos->x() + diodeSignals->width() + 80*scaleWidth,
                        inputGain->absPos->y() + inputGain->ptsIn[0]->y() - DFLSwitch->ptsOut[0]->y());
    LFLSwitch->recalcPoints();
    LFLSwitch->setPosXY(diodeSignals->absPos->x() + diodeSignals->width() + 20*scaleWidth,
                        inputGain->absPos->y() + inputGain->ptsIn[1]->y() - LFLSwitch->ptsOut[0]->y());
    DFLtext->recalcPoints();
    DFLtext->setPosXY(inputGain->absPos->x() + inputGain->width() + 20*scaleWidth,
                      inputGain->absPos->y() + inputGain->ptsOut[0]->y() - DFLtext->ptsIn[0]->y());
    LFLtext->recalcPoints();
    LFLtext->setPosXY(inputGain->absPos->x() + inputGain->width() + 20*scaleWidth,
                      inputGain->absPos->y() + inputGain->ptsOut[1]->y() - LFLtext->ptsIn[0]->y());
    INTtext->recalcPoints();
    INTtext->setPosXY(inputGain->absPos->x() + inputGain->width() + 20*scaleWidth,
                      inputGain->absPos->y() + inputGain->ptsOut[2]->y() - INTtext->ptsIn[0]->y());
    DFLPoint->recalcPoints();
    DFLPoint->setPosXY(DFLtext->absPos->x() + DFLtext->ptsOut[0]->x() + 15*scaleWidth,
                       DFLtext->absPos->y() + DFLtext->ptsOut[0]->y() - DFLPoint->ptsOut[0]->y());
    LFLPoint->recalcPoints();
    LFLPoint->setPosXY(LFLtext->absPos->x() + LFLtext->ptsOut[0]->x() + 5*scaleWidth,
                       LFLtext->absPos->y() + LFLtext->ptsOut[0]->y() - LFLPoint->ptsOut[0]->y());
    LockIn->setPosXY(DFLtext->absPos->x() + DFLtext->width() + 30*scaleWidth,
                     30*scaleHeight);
    LockIn->recalcPoints();
    MagText->recalcPoints();
    MagText->setPosXY(LockIn->absPos->x() + LockIn->width() + 20*scaleWidth,
                      LockIn->absPos->y() + LockIn->ptsOut[0]->y() - MagText->ptsIn[0]->y());
    MagSinText->recalcPoints();
    MagSinText->setPosXY(MagText->absPos->x(),
                         LockIn->absPos->y() + LockIn->ptsOut[1]->y() - MagSinText->ptsIn[0]->y());
    MagCosText->recalcPoints();
    MagCosText->setPosXY(MagText->absPos->x(),
                         LockIn->absPos->y() + LockIn->ptsOut[2]->y() - MagCosText->ptsIn[0]->y());
    PhaseText->recalcPoints();
    PhaseText->setPosXY(MagText->absPos->x(),
                        LockIn->absPos->y() + LockIn->ptsOut[3]->y() - PhaseText->ptsIn[0]->y());
    PIDSwitch->recalcPoints();
    PIDSwitch->setPosXY(PhaseText->absPos->x() + PhaseText->width() + 10*scaleWidth,
                        PhaseText->absPos->y() + PhaseText->ptsOut[0]->y() - PIDSwitch->ptsIn[0]->y());
    pid->recalcPoints(); recalcSwitch_PID();
    pid->setPosXY(PIDSwitch->absPos->x() + PIDSwitch->ptsOut[0]->x() - pid->ptsIn[1]->x(),
                  PIDSwitch->absPos->y() + PIDSwitch->height() + 25*scaleHeight);
    HVZTria->recalcPoints();
    HVZTria->setPosXY(pid->absPos->x() + pid->ptsOut[0]->x() - HVZTria->ptsIn[0]->x(),
                      pid->absPos->y() + pid->height() + 30*scaleHeight);
    setPointEdit->recalcPoints();
    setPointEdit->setPosXY(MagText->absPos->x(),
                           pid->absPos->y() - 20*scaleHeight);
    oscilator->setPosXY(LockIn->absPos->x() - 100*scaleWidth,
                        pid->absPos->y());
    plus->recalcPoints();
    plus->setPosXY(oscilator->absPos->x() - 150*scaleWidth,
                   oscilator->absPos->y() + oscilator->ptsIn[0]->y() - plus->ptsOut[0]->y());
    plusSwitch->recalcPoints();
    plusSwitch->setPosXY(plus->absPos->x() - plusSwitch->width() - 20*scaleWidth,
                         plus->absPos->y() + plus->ptsIn[0]->y() - plusSwitch->ptsOut[0]->y());
    bias->recalcPoints();
    bias->setPosXY(plus->absPos->x() + plus->ptsIn[1]->x() - bias->ptsOut[0]->x(),
                   plus->absPos->y() + plus->height() + 26*scaleHeight);
    ZPiezo->setPosXY(plusSwitch->absPos->x() - ZPiezo->width() - 30*scaleWidth,
                     bias->absPos->y() + 26*scaleHeight);
    feedBack->recalcPoints();
    feedBack->setPosXY(bias->absPos->x(),
                       ZPiezo->absPos->y() + ZPiezo->ptsIn[0]->y() - feedBack->ptsOut[0]->y());
    ZPiezoArrGd->recalcPoints();
    ZPiezoArrGd->setPosXY(ZPiezo->absPos->x() - ZPiezoArrGd->width() - 10*scaleWidth,
                          ZPiezo->absPos->y() + ZPiezo->ptsOut[0]->y() - ZPiezoArrGd->ptsIn[0]->y());
    ExtOsc->setPosXY(bias->absPos->x() + bias->width() + 20*scaleWidth,
                     bias->absPos->y());
    Piezo->setPosXY(ZPiezo->absPos->x(),
                    ZPiezo->absPos->y() - Piezo->height() - 10*scaleHeight);
    PiesoArrGd->recalcPoints();
    PiesoArrGd->setPosXY(ZPiezoArrGd->absPos->x(),
                         Piezo->absPos->y() + Piezo->ptsOut[0]->y() - PiesoArrGd->ptsIn[0]->y());
    ZPiezoPosEdit[0]->recalcPoints();
    ZPiezoPosEdit[0]->setPosXY(ZPiezo->absPos->x() - ZPiezoPosEdit[0]->width() - 5*scaleWidth,
                               ZPiezo->absPos->y() + ZPiezo->ptsOut[1]->y() - ZPiezoPosEdit[0]->ptsIn[0]->y());
    ZPiezoPosEdit[1]->recalcPoints();
    ZPiezoPosEdit[1]->setPosXY(ZPiezo->absPos->x() - ZPiezoPosEdit[1]->width() - 5*scaleWidth,
                               ZPiezo->absPos->y() + ZPiezo->ptsOut[2]->y() - ZPiezoPosEdit[1]->ptsIn[0]->y());
    ZPiezoPosEdit[2]->recalcPoints();
    ZPiezoPosEdit[2]->setPosXY(ZPiezo->absPos->x() - ZPiezoPosEdit[2]->width() - 5*scaleWidth,
                               ZPiezo->absPos->y() + ZPiezo->ptsOut[3]->y() - ZPiezoPosEdit[2]->ptsIn[0]->y());

    psd->move(*psd->absPos);
    diodeSignals->move(*diodeSignals->absPos);
    DFLSwitch->move(*DFLSwitch->absPos);
    LFLSwitch->move(*LFLSwitch->absPos);
    inputGain->move(*inputGain->absPos);
    DFLtext->move(*DFLtext->absPos);
    LFLtext->move(*LFLtext->absPos);
    INTtext->move(*INTtext->absPos);
    LockIn->move(*LockIn->absPos);
    MagText->move(*MagText->absPos);
    MagSinText->move(*MagSinText->absPos);
    MagCosText->move(*MagCosText->absPos);
    PhaseText->move(*PhaseText->absPos);
    PIDSwitch->move(*PIDSwitch->absPos);
    setPointEdit->move(*setPointEdit->absPos);
    pid->move(*pid->absPos);
    HVZTria->move(*HVZTria->absPos);
    feedBack->move(*feedBack->absPos);
    oscilator->move(*oscilator->absPos);
    plus->move(*plus->absPos);
    plusSwitch->move(*plusSwitch->absPos);
    bias->move(*bias->absPos);
    ZPiezo->move(*ZPiezo->absPos);
    ZPiezoArrGd->move(*ZPiezoArrGd->absPos);
    ExtOsc->move(*ExtOsc->absPos);
    PiesoArrGd->move(*PiesoArrGd->absPos);
    Piezo->move(*Piezo->absPos);
    DFLPoint->move(*DFLPoint->absPos);
    LFLPoint->move(*LFLPoint->absPos);
    ZPiezoPosEdit[0]->move(*ZPiezoPosEdit[0]->absPos);
    ZPiezoPosEdit[1]->move(*ZPiezoPosEdit[1]->absPos);
    ZPiezoPosEdit[2]->move(*ZPiezoPosEdit[2]->absPos);

    setPointLabel->move(setPointEdit->absPos->x(),
                        setPointEdit->absPos->y() - setPointEdit->height()*0.5);
    MagLabel->move(MagText->absPos->x(),
                   MagText->absPos->y() - MagText->height()*0.5);
    MagSinLabel->move(MagSinText->absPos->x(),
                      MagSinText->absPos->y() - MagSinText->height()*0.5);
    MagCosLabel->move(MagCosText->absPos->x(),
                      MagCosText->absPos->y() - MagCosText->height()*0.5);
    PhaseLabel->move(PhaseText->absPos->x(),
                     PhaseText->absPos->y() - PhaseText->height()*0.5);
    int xPosZPiezo = ZPiezoPosEdit[0]->absPos->x() - ZPiezoPosLabel[0]->width() - 10*scaleWidth;
    if (xPosZPiezo < 0)
        xPosZPiezo = 0;
    ZPiezoPosLabel[0]->move(xPosZPiezo,
                            ZPiezoPosEdit[0]->absPos->y() + ZPiezoPosLabel[0]->height()*0.5);
    ZPiezoPosLabel[1]->move(xPosZPiezo,
                            ZPiezoPosEdit[1]->absPos->y() + ZPiezoPosLabel[1]->height()*0.5);
    ZPiezoPosLabel[2]->move(xPosZPiezo,
                            ZPiezoPosEdit[2]->absPos->y() + ZPiezoPosLabel[2]->height()*0.5);
    FeedBackLabel->move(feedBack->absPos->x(), feedBack->absPos->y() - FeedBackLabel->height());
    FBInputLabel->move(PIDSwitch->absPos->x() + PIDSwitch->ptsOut[0]->x() + 6*scaleWidth,
                       PIDSwitch->absPos->y() + PIDSwitch->height());

    for (auto &el : elements)
        el->recalcPoints();
    setCorners();
}

void deviceScheme::setCorners()
{
    for (auto &el : elements)
        for (uint i=0; i<el->countOut; i++)
            el->cornersOut[i]->clear();

    psd->addCorner(0,
        psd->absPos->x() + psd->ptsOut[0]->x(),
        psd->absPos->y() + psd->ptsOut[0]->y() - 10*scaleHeight);
    psd->addCorner(0,
        diodeSignals->absPos->x(),
        psd->absPos->y() + psd->ptsOut[0]->y() - 10*scaleHeight);
    psd->addCorner(1,
        diodeSignals->absPos->x(),
        psd->absPos->y() + psd->ptsOut[1]->y());
    psd->addCorner(2,
        diodeSignals->absPos->x(),
        psd->absPos->y() + psd->ptsOut[2]->y());
    psd->addCorner(3,
        psd->absPos->x() + psd->ptsOut[3]->x(),
        psd->absPos->y() + psd->ptsOut[3]->y() + 10*scaleHeight);
    psd->addCorner(3,
        diodeSignals->absPos->x(),
        psd->absPos->y() + psd->ptsOut[3]->y() + 10*scaleHeight);

    MagText->addCorner(0,
        PIDSwitch->absPos->x() + PIDSwitch->ptsIn[3]->x(),
        MagText->absPos->y() + MagText->ptsOut[0]->y());

    MagSinText->addCorner(0,
        PIDSwitch->absPos->x() + PIDSwitch->ptsIn[2]->x(),
        MagSinText->absPos->y() + MagSinText->ptsOut[0]->y());

    MagCosText->addCorner(0,
        PIDSwitch->absPos->x() + PIDSwitch->ptsIn[1]->x(),
        MagCosText->absPos->y() + MagCosText->ptsOut[0]->y());

    DFLPoint->addCorner(0,
        DFLPoint->absPos->x() + DFLPoint->ptsOut[0]->x(),
        DFLPoint->absPos->y() + DFLPoint->ptsOut[0]->y() - 40*scaleHeight);
    DFLPoint->addCorner(0,
        PIDSwitch->absPos->x() + PIDSwitch->ptsIn[4]->x(),
        DFLPoint->absPos->y() + DFLPoint->ptsOut[0]->y() - 40*scaleHeight);

    LFLPoint->addCorner(0,
        LFLPoint->absPos->x() + LFLPoint->ptsOut[0]->x(),
        LFLPoint->absPos->y() + LFLPoint->ptsOut[0]->y() - 120*scaleHeight);
    LFLPoint->addCorner(0,
        PIDSwitch->absPos->x() + PIDSwitch->ptsIn[5]->x(),
        LFLPoint->absPos->y() + LFLPoint->ptsOut[0]->y() - 120*scaleHeight);

    ZPiezoArrGd->addCorner(0,
        ZPiezoArrGd->absPos->x() + ZPiezoArrGd->ptsOut[0]->x() - 10*scaleWidth,
        ZPiezoArrGd->absPos->y() + ZPiezoArrGd->ptsOut[0]->y());
    ZPiezoArrGd->addCorner(0,
        ZPiezoArrGd->absPos->x() + ZPiezoArrGd->ptsOut[0]->x() - 10*scaleWidth,
        diodeSignals->absPos->y() + diodeSignals->height() + 10*scaleHeight);
    ZPiezoArrGd->addCorner(0,
        LFLSwitch->absPos->x() + LFLSwitch->ptsIn[1]->x(),
        diodeSignals->absPos->y() + diodeSignals->height() + 10*scaleHeight);

    PiesoArrGd->addCorner(0,
        PiesoArrGd->absPos->x() + PiesoArrGd->ptsOut[0]->x(),
        diodeSignals->absPos->y() + diodeSignals->height() + 30*scaleHeight);
    PiesoArrGd->addCorner(0,
        DFLSwitch->absPos->x() + DFLSwitch->ptsIn[1]->x(),
        diodeSignals->absPos->y() + diodeSignals->height() + 30*scaleHeight);

    setPointEdit->addCorner(0,
        pid->absPos->x() + pid->ptsIn[0]->x(),
        setPointEdit->absPos->y() + setPointEdit->ptsOut[0]->y());

    HVZTria->addCorner(0,
        HVZTria->absPos->x() + HVZTria->ptsOut[0]->x(),
        ZPiezo->absPos->y() + ZPiezo->ptsIn[0]->y());

    Piezo->addCorner(1,
        Piezo->absPos->x() + Piezo->ptsOut[1]->x(),
        plusSwitch->absPos->y() + plusSwitch->ptsIn[0]->y());
    Piezo->addCorner(2,
        Piezo->absPos->x() + Piezo->ptsOut[2]->x(),
        plusSwitch->absPos->y() + plusSwitch->ptsIn[1]->y());
}

void deviceScheme::connectElements()
{
    psd->connectAbs(0, diodeSignals, 0);
    psd->connectAbs(1, diodeSignals, 1);
    psd->connectAbs(2, diodeSignals, 2);
    psd->connectAbs(3, diodeSignals, 3);

    diodeSignals->connectAbs(0, DFLSwitch, 0);
    diodeSignals->connectAbs(1, LFLSwitch, 0);
    diodeSignals->connectAbs(2, inputGain, 2);

    DFLSwitch->connectAbs(0, inputGain, 0);
    LFLSwitch->connectAbs(0, inputGain, 1);

    inputGain->connectAbs(0, DFLtext, 0);
    inputGain->connectAbs(1, LFLtext, 0);
    inputGain->connectAbs(2, INTtext, 0);

    DFLtext->connectAbs(0, LockIn, 0);
    LFLtext->connectAbs(0, LockIn, 1);
    INTtext->connectAbs(0, LockIn, 2);

    LockIn->connectAbs(0, MagText,    0);
    LockIn->connectAbs(1, MagSinText, 0);
    LockIn->connectAbs(2, MagCosText, 0);
    LockIn->connectAbs(3, PhaseText,  0);

    PhaseText->connectAbs(0,  PIDSwitch, 0);
    MagCosText->connectAbs(0, PIDSwitch, 1);
    MagSinText->connectAbs(0, PIDSwitch, 2);
    MagText->connectAbs(0,    PIDSwitch, 3);
    DFLPoint->connectAbs(0,   PIDSwitch, 4);
    LFLPoint->connectAbs(0,   PIDSwitch, 5);

    PIDSwitch->connectAbs(0, pid, 1);

    setPointEdit->connectAbs(0, pid, 0);
    pid->connectAbs(0, HVZTria, 0);
    HVZTria->connectAbs(0, feedBack, 0);
    feedBack->connectAbs(0, ZPiezo, 0);

    ZPiezo->connectAbs(0, ZPiezoArrGd, 0);
    ZPiezo->connectAbs(1, ZPiezoPosEdit[0], 0);
    ZPiezo->connectAbs(2, ZPiezoPosEdit[1], 0);
    ZPiezo->connectAbs(3, ZPiezoPosEdit[2], 0);
    plusSwitch->connectAbs(0, plus, 0);
    bias->connectAbs(0, plus, 1);
    plus->connectAbs(0, oscilator, 0);
    oscilator->connectAbs(0, LockIn, 3);
    ZPiezoArrGd->connectAbs(0, LFLSwitch, 1);
    PiesoArrGd->connectAbs(0, DFLSwitch, 1);
    Piezo->connectAbs(0, PiesoArrGd, 0);
    Piezo->connectAbs(1, plusSwitch, 0);
    Piezo->connectAbs(2, plusSwitch, 1);
}

void deviceScheme::paintEvent(QPaintEvent *e)
{
    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing, true);
    painter.setPen(QColor(Qt::black));

    DFLtext->updatePointsPosition();
    LFLtext->updatePointsPosition();
    INTtext->updatePointsPosition();

    for (auto &el : elements)
    {
        QPoint pos = *el->absPos;
        for (uint b=0; b<el->countOut; b++)
        {
            if (el->absOut[b] == nullptr)
                continue;

            QPoint prevPoint = *el->ptsOut[b] + pos;
            for (auto point : *el->cornersOut[b])
            {
                painter.drawLine(prevPoint, *point);
                prevPoint = *point;
            }

            auto pos2 = *el->absOut[b]->absPos;
            auto point = *el->absOut[b]->ptsIn[el->connectOutId[b]] + pos2;
            painter.drawLine(prevPoint, point);
        }
    }

    QWidget::paintEvent(e);
}

void deviceScheme::recalcSwitch_DFL()
{
    DFLSwitch->setPtsInXY(0, DFLSwitch->rectCenter + DFLSwitch->offset,
                             DFLSwitch->rectCenter + DFLSwitch->offset);
    DFLSwitch->setPtsInXY(1, DFLSwitch->rectCenter + DFLSwitch->offset,
                             DFLSwitch->height() - DFLSwitch->rectCenter - DFLSwitch->offset);
    DFLSwitch->setPtsOutXY(0, DFLSwitch->width() - DFLSwitch->rectCenter - DFLSwitch->offset,
                              DFLSwitch->rectCenter + DFLSwitch->offset);
    DFLSwitch->clearArcsPolygons();

    QPolygon *pol1 = new QPolygon();
    pol1->append(QPoint(0, 0));
    pol1->append(QPoint(DFLSwitch->rectCenter, 0));
    pol1->append(*DFLSwitch->ptsOut[0]);
    pol1->append(QPoint(0, DFLSwitch->height()*0.5));
    DFLSwitch->vecPolygons->push_back(pol1);

    QPolygon *pol2 = new QPolygon();
    pol2->append(QPoint(0, DFLSwitch->height()));
    pol2->append(QPoint(DFLSwitch->rectCenter*5, DFLSwitch->height()));
    pol2->append(*DFLSwitch->ptsOut[0]);
    pol2->append(QPoint(0, DFLSwitch->height()*0.5));
    DFLSwitch->vecPolygons->push_back(pol2);
}

void deviceScheme::recalcSwitch_LFL()
{
    LFLSwitch->setPtsInXY(0, LFLSwitch->rectCenter + LFLSwitch->offset,
                             LFLSwitch->rectCenter + LFLSwitch->offset);
    LFLSwitch->setPtsInXY(1, LFLSwitch->rectCenter + LFLSwitch->offset,
                             LFLSwitch->height() - LFLSwitch->rectCenter - LFLSwitch->offset);
    LFLSwitch->setPtsOutXY(0, LFLSwitch->width() - LFLSwitch->rectCenter - LFLSwitch->offset,
                              LFLSwitch->rectCenter + DFLSwitch->offset);
    LFLSwitch->clearArcsPolygons();

    QPolygon *pol1 = new QPolygon();
    pol1->append(QPoint(0, 0));
    pol1->append(QPoint(LFLSwitch->rectCenter, 0));
    pol1->append(*LFLSwitch->ptsOut[0]);
    pol1->append(QPoint(0, LFLSwitch->height()*0.5));
    LFLSwitch->vecPolygons->push_back(pol1);

    QPolygon *pol2 = new QPolygon();
    pol2->append(QPoint(0, LFLSwitch->height()));
    pol2->append(QPoint(LFLSwitch->rectCenter*5, LFLSwitch->height()));
    pol2->append(*LFLSwitch->ptsOut[0]);
    pol2->append(QPoint(0, LFLSwitch->height()*0.5));
    LFLSwitch->vecPolygons->push_back(pol2);
}

void deviceScheme::recalcSwitch_PID()
{
    double pidDist = (PIDSwitch->width() - (PIDSwitch->offset + PIDSwitch->rectCenter)*2)/(PIDSwitch->countIn-1); //fixme losing fractional part
    double pidDistSum = PIDSwitch->offset + PIDSwitch->rectCenter;
    for (uint i=0; i<PIDSwitch->countIn; i++)
    {
        PIDSwitch->setPtsInXY(i, pidDistSum,
                                 PIDSwitch->rectCenter + PIDSwitch->offset);
        pidDistSum += pidDist;
    }
    PIDSwitch->setPtsOutXY(0, PIDSwitch->width()*0.5,
                              PIDSwitch->height() - PIDSwitch->rectCenter - PIDSwitch->offset);
    PIDSwitch->clearArcsPolygons();

    pidDistSum = 0.;
    QPolygon *pol;
    {//first
        pol = new QPolygon();
        pol->append(QPoint(pidDistSum, 15));
        pol->append(QPoint(pidDistSum, 0));
        pol->append(QPoint(pidDistSum + PIDSwitch->ptsIn[0]->x() + pidDist*0.25, 0));
        pol->append(*PIDSwitch->ptsOut[0]);
        pidDistSum += PIDSwitch->ptsIn[0]->x() + pidDist*0.25;
        PIDSwitch->vecPolygons->push_back(pol);
    }
    {//second
        pol = new QPolygon();
        pol->append(QPoint(pidDistSum, 0));
        pol->append(QPoint(pidDistSum + pidDist*1.25, 0));
        pol->append(*PIDSwitch->ptsOut[0]);
        pidDistSum += pidDist*1.25;
        PIDSwitch->vecPolygons->push_back(pol);
    }
    for (uint i=2; i<PIDSwitch->countIn-2; i++)
    {
        pol = new QPolygon();
        pol->append(QPoint(pidDistSum, 0));
        pol->append(QPoint(pidDistSum + pidDist, 0));
        pol->append(*PIDSwitch->ptsOut[0]);
        pidDistSum += pidDist;
        PIDSwitch->vecPolygons->push_back(pol);
    }
    {//Prev last
        pol = new QPolygon();
        pol->append(QPoint(pidDistSum, 0));
        pol->append(QPoint(pidDistSum + pidDist*1.25, 0));
        pol->append(*PIDSwitch->ptsOut[0]);

        pidDistSum += pidDist*1.25;
        PIDSwitch->vecPolygons->push_back(pol);
    }
    {//last
        pol = new QPolygon();
        pol->append(QPoint(pidDistSum, 0));
        pol->append(QPoint(pidDistSum + pidDist, 0));
        pol->append(QPoint(pidDistSum + pidDist, 15));
        pol->append(*PIDSwitch->ptsOut[0]);

        pidDistSum += pidDist;
        PIDSwitch->vecPolygons->push_back(pol);
    }
}

void deviceScheme::recalcSwitch_Plus()
{
    plusSwitch->setPtsInXY(0, plusSwitch->rectCenter + plusSwitch->offset,
                              plusSwitch->rectCenter + plusSwitch->offset);
    plusSwitch->setPtsInXY(1, plusSwitch->rectCenter + plusSwitch->offset,
                              plusSwitch->height() - plusSwitch->rectCenter - plusSwitch->offset);
    plusSwitch->setPtsOutXY(0, plusSwitch->width() - plusSwitch->rectCenter - plusSwitch->offset,
                               plusSwitch->height()*0.5);
    plusSwitch->clearArcsPolygons();

    QPolygon *pol1 = new QPolygon();
    pol1->append(QPoint(0, 0));
    pol1->append(QPoint(plusSwitch->rectCenter*5, 0));
    pol1->append(*plusSwitch->ptsOut[0]);
    pol1->append(QPoint(0, plusSwitch->height()*0.5));
    plusSwitch->vecPolygons->push_back(pol1);

    QPolygon *pol2 = new QPolygon();
    pol2->append(QPoint(0, plusSwitch->height()));
    pol2->append(QPoint(plusSwitch->rectCenter*5, plusSwitch->height()));
    pol2->append(*plusSwitch->ptsOut[0]);
    pol2->append(QPoint(0, plusSwitch->height()*0.5));
    plusSwitch->vecPolygons->push_back(pol2);
}

void deviceScheme::recalcDiodeSignals()
{
    for (uint i=0; i<4; i++)
    {
        if (!psd->cornersOut[i]->empty())
            diodeSignals->setPtsInXY(i, 2,
                psd->cornersOut[i]->last()->y());
    }

    diodeSignals->update();
}

void deviceScheme::initPosDiodeSignals(int , int )
{
    diodeSignals->miniBoxY[0] = inputGain->ptsIn[0]->y() - diodeSignals->miniBoxH*0.5;
    diodeSignals->miniBoxY[1] = inputGain->ptsIn[1]->y() - diodeSignals->miniBoxH*0.5;
    diodeSignals->miniBoxY[2] = inputGain->ptsIn[2]->y() - diodeSignals->miniBoxH*0.5;
}

void deviceScheme::recalcLockIn()
{
    LockIn->setPtsInXY(0, 0,
        DFLtext->absPos->y() + DFLtext->ptsOut[0]->y() - LockIn->absPos->y());
    LockIn->setPtsInXY(1, 0,
        LFLtext->absPos->y() + LFLtext->ptsOut[0]->y() - LockIn->absPos->y());
    LockIn->setPtsInXY(2, 0,
        INTtext->absPos->y() + INTtext->ptsOut[0]->y() - LockIn->absPos->y());
}

bool deviceScheme::event(QEvent *e)
{
    if (QEventResizedMoved(e))
        rescale();

    return QWidget::event(e);
}

deviceScheme::~deviceScheme()
{
    elements.clear();
}
