#ifndef DEVICESCHEME_H
#define DEVICESCHEME_H

#include <QObject>
#include <QWidget>
#include "scheme/abstractinoutlines.h"
#include "scheme/schinputgains.h"
#include "scheme/schpsd.h"
#include "scheme/schswitch.h"
#include "scheme/schqtextedit.h"
#include "scheme/schlockin.h"
#include "scheme/schdiodesignals.h"
#include "scheme/schpid.h"
#include "scheme/schoscilator.h"
#include "scheme/schPlus.h"
#include "scheme/schbias.h"
#include "scheme/schzpiezo.h"
#include "scheme/scharrowgrounding.h"
#include "scheme/schextosc.h"
#include "scheme/schpiezo.h"
#include "scheme/schpoint.h"
#include "scheme/schedit.h"
#include "scheme/schtriangle.h"
#include "scheme/schbreaker.h"
#include <QPainter>
#include <QDebug>
#include <QtGlobal>
#include <QTimer>
#include <QPolygon>
#include <QPaintEngine>

class deviceScheme : public QWidget
{
    Q_OBJECT
public:
    explicit deviceScheme(QWidget *parent = nullptr);
    ~deviceScheme() override;
    void rescale();

    schPSD          *psd;
    schDiodeSignals *diodeSignals;
    schSwitch       *DFLSwitch;
    schSwitch       *LFLSwitch;
    schInputGains   *inputGain;
    schQTextEdit    *DFLtext;
    schQTextEdit    *LFLtext;
    schQTextEdit    *INTtext;
    schPoint        *DFLPoint;
    schPoint        *LFLPoint;
    schLockIn       *LockIn;
    schQTextEdit    *MagText;
    schQTextEdit    *MagSinText;
    schQTextEdit    *MagCosText;
    schQTextEdit    *PhaseText;
    schSwitch       *PIDSwitch;
    schEdit         *setPointEdit;
    schPID          *pid;
    schTriangle     *HVZTria;
    schBreaker      *feedBack;
    schOscilator    *oscilator;
    schPlus         *plus;
    schSwitch       *plusSwitch;
    schBias         *bias;
    schZPiezo       *ZPiezo;
    schArrowGrounding *ZPiezoArrGd;
    schArrowGrounding *PiesoArrGd;
    schExtOsc       *ExtOsc;
    schpiezo        *Piezo;
    schEdit         **ZPiezoPosEdit;

    QLabel *setPointLabel;
    QLabel *MagLabel;
    QLabel *MagSinLabel;
    QLabel *MagCosLabel;
    QLabel *PhaseLabel;
    QLabel **ZPiezoPosLabel;
    QLabel *FeedBackLabel;
    QLabel *FBInputLabel;

    double scaleWidth;
    double scaleHeight;

public slots:
    void recalcSwitch_DFL();
    void recalcSwitch_LFL();
    void recalcSwitch_PID();
    void recalcSwitch_Plus();

    void recalcDiodeSignals();
    void recalcLockIn();
    void initPosDiodeSignals(int w, int h);

protected:
    void paintEvent(QPaintEvent *event) override;
    bool event(QEvent *) override;

private:
    QVector<AbstractInOutLines *> elements;
    QVector<QLabel *> labels;

    void connectElements();
    void setCorners();



signals:

public slots:
};

#endif // DEVICESCHEME_H
