#include "mslider.hpp"

#include <QApplication>
#include <QDesktopWidget>
#include <QtEvents>
#include <QPainter>
#include <QPainterPath>

MotorSlider::MotorSlider(bool horizontal)
{
    auto_disable = true;
    pressed = false;
    active = 0;
    this->horizontal = horizontal;

#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)) && !defined(_ADROID)
    double scale = horizontal ? qApp->desktop()->logicalDpiY() : qApp->desktop()->logicalDpiX();
#else
    double scale = horizontal ? qApp->desktop()->physicalDpiY() : qApp->desktop()->physicalDpiX();
#endif
    setAttribute(Qt::WA_AcceptTouchEvents);
    if (horizontal){
        setMinimumSize(300, 100);
        setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    }
    else {
        setFixedWidth(0.7*scale);
        setBaseSize(100, 100);
        setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Expanding);
    }
}
void MotorSlider::setPos(int pos)
{
    active = pos;
    update();
}
void MotorSlider::paintEvent(QPaintEvent *)
{
    QPainter painter(this);

    int x0 = 0;
    int p = 11;
    do {
        int x1 = x0;

        if (horizontal)
            x1 += (width()-1 - x0) / p;
        else
            x1 += (height()-1 - x0) / p;

        p--;

        QRect r(
            horizontal ? x0:0,
            horizontal ? 0:x0,
            horizontal ? (x1 - x0):(width()-1),
            horizontal ? (height()-1):(x1 - x0));

        painter.setBrush((p - 5 == active) ? QColor(200, 200, 255) : QColor(255 - qMax(0, p-5)*8, 255 - qMax(0, -(p-5))*8, 255 - qAbs(p-5)*8));
        painter.setPen(QPen(QBrush(isEnabled() ? Qt::darkGray : Qt::lightGray), 1));
        painter.drawRect(r);

        painter.setPen(QPen(QBrush(isEnabled() ? Qt::black : Qt::gray), 1));
        painter.drawText(r, Qt::AlignCenter, QString::number(p - 5));

        x0 = x1;
    } while (p != 0);
}
void MotorSlider::mousePressEvent(QMouseEvent *e)
{
    pressed = true;
    mouseMoveEvent(e);

    if (active == 0)
        emit newPos(active);
}
void MotorSlider::mouseReleaseEvent(QMouseEvent *)
{
    pressed = false;
    if (auto_disable && active != 0) {
        active = 0;
        update();
        emit newPos(active);
    }
}
void MotorSlider::mouseMoveEvent(QMouseEvent *e)
{
    if (!pressed)
        return;

    int new_active = 5;
    if (horizontal)
        new_active -= int(11. * e->x() / width());
    else
        new_active -= int(11. * e->y() / height());

    if (!rect().contains(e->pos())) {
        if (auto_disable)
            new_active = 0;
        else
            return;
    }

    if (new_active != active) {
        active = new_active;
        update();
        emit newPos(active);
    }
}
