#include "scharrowgrounding.h"

schArrowGrounding::schArrowGrounding(QWidget *parent) : QWidget(parent), AbstractInOutLines(1, 1)
{
    textLabel = new QLabel(tr("I/V"), this);
    textLabel->setScaledContents(true);
    initPositions(0, 0);
}

void schArrowGrounding::initPositions(int w, int )
{
    triaLX = 0;
    triaW = w*0.4;
    triaH = w*0.4;
    triaLY = triaH*0.5;
    triaRX = triaW;
    triaRY = 0;
    double scale = w/75.;
    if (scale > maxFontScale)
        scale = maxFontScale;
    textLabel->setFont(QFont(textLabel->font().family(), 10*scale));
    textLabel->adjustSize();
}

void schArrowGrounding::recalcPoints()
{
    initPositions(width(), height());
    setPtsInXY(0, triaRX, triaLY);
    setPtsOutXY(0, triaLX,  triaLY);
}

bool schArrowGrounding::event(QEvent *e)
{
    if (QEventResizedMoved(e))
        recalcPoints();

    return QWidget::event(e);
}

void schArrowGrounding::paintEvent(QPaintEvent *)
{
    QPainter painter(this);

    int w = width();
    int h = height();

    initPositions(w, h);
    painter.setRenderHint(QPainter::Antialiasing, true);

    //Triangle
    painter.setPen(QColor(Qt::black));
    painter.setBrush(QBrush(backgroundColor));
    auto ptsTria = new QPoint[3];
    ptsTria[0].setX(triaLX);
    ptsTria[0].setY(triaLY);
    ptsTria[1].setX(triaRX);
    ptsTria[1].setY(triaRY);
    ptsTria[2].setX(triaRX);
    ptsTria[2].setY(triaRY + triaH);
    painter.drawPolygon(ptsTria, 3);
    painter.setBrush(QBrush());

    //Grounding
    int tx = triaRX + w*0.25;
    int ty = triaRY + triaH*0.8;
    painter.drawLine(triaRX, ty, tx, ty);
    painter.drawLine(tx, ty, tx, h*0.95);
    painter.drawLine(tx - 10, h*0.95, tx + 10, h*0.95);

    //text
    int widthFreeSpaceForText = triaRX - (triaLX + triaW*0.4);
    int xText;
    if (textLabel->width() > widthFreeSpaceForText)
        xText = triaRX - textLabel->width();
    else
        xText = triaRX - widthFreeSpaceForText*0.5 - textLabel->width()*0.5;
    textLabel->move(xText, triaLY - textLabel->height()*0.5);
}
