#include "schbias.h"

schBias::schBias(QWidget *parent) : QWidget(parent), AbstractInOutLines(1, 0)
{
    bias = new Edit(-10, 10, 0);
    bias->setParent(this);
    biasLabel = new QLabel(tr("Bias v"), this);
    biasLabel->setScaledContents(true);

    initPositions(100, 100);
}

void schBias::initPositions(int w, int h)
{
    boxX = 0;
    boxY = h*0.15;
    boxW = w;
    boxH = h - boxY;

    double scale = w/60.;
    if (scale > maxFontScale)
        scale = maxFontScale;
    QFont font = QFont(biasLabel->font().family(), 10*scale);
    biasLabel->setFont(font);
    biasLabel->adjustSize();
    int newSize = boxW - 5*2;
    if (newSize <= 0) 
        newSize = 1;
    bias->setFixedWidth(newSize);
}

void schBias::recalcPoints()
{
    initPositions(width(), height());
    setPtsOutXY(0, boxW*0.5, boxY);
}

bool schBias::event(QEvent *e)
{
    if (QEventResizedMoved(e))
        recalcPoints();

    return QWidget::event(e);
}

void schBias::paintEvent(QPaintEvent *)
{
    QPainter painter(this);

    int w = width();
    int h = height();

    initPositions(w, h);
    painter.setRenderHint(QPainter::Antialiasing, true);

    //Box
    painter.setPen(QPen(QBrush(outterLineColor), outterLineWidth));
    painter.setBrush(QBrush(backgroundColor));
    painter.drawRect(boxX, boxY, boxW, boxH);
    painter.setBrush(QBrush());

    painter.setPen(QColor(Qt::black));

    float y = boxY + boxH*0.4;
    biasLabel->move(5, y - bias->height());
    bias->move(5, y);
}
