#include "schbreaker.h"
#include <QPainterPath>

schBreaker::schBreaker(QWidget *parent) :
    QWidget(parent), AbstractInOutLines(1, 1)
{
    offset = 10;
    offPos = new QPoint(0, 0);
    polyHover = new QPolygon*[2];
    polyHover[0] = new QPolygon();
    polyHover[1] = new QPolygon();
    turnOn = false;
    hovered[0] = hovered[1] = false;
    initPositions(0, 0);

    setMouseTracking(true);
}

void schBreaker::recalcPoints()
{
    initPositions(width(), height());

    offPos->setX(offset + rectCenter);
    offPos->setY(offset + rectCenter);
    setPtsInXY(0, width() - rectCenter - offset, height() - rectCenter - offset);
    setPtsOutXY(0, offset + rectCenter, height() - rectCenter - offset);
    polyHover[0]->clear();
    polyHover[0]->append(QPoint(0, 0));
    polyHover[0]->append(QPoint(rectCenter*5, 0));
    polyHover[0]->append(*ptsIn[0]);
    polyHover[0]->append(QPoint(0, height()*0.5));

    polyHover[1]->clear();
    polyHover[1]->append(QPoint(0, height()));
    polyHover[1]->append(QPoint(rectCenter*5, height()));
    polyHover[1]->append(*ptsIn[0]);
    polyHover[1]->append(QPoint(0, height()*0.5));
}

void schBreaker::initPositions(int w, int )
{

}

void schBreaker::setState(bool on)
{
    setState_man(on);
    emit changedState(turnOn);
}

void schBreaker::setState_man(bool on)
{
    turnOn = on;
    update();
}

bool schBreaker::event(QEvent *e)
{
    if (QEventResizedMoved(e))
        recalcPoints();

    if (e->type() == QEvent::Leave)
    {
        hovered[0] = hovered[1] = false;
        update();
    }
    else if (e->type() == QEvent::MouseMove)
    {
        auto me = dynamic_cast<QMouseEvent *>(e);
        hovered[0] = polyHover[0]->containsPoint(me->pos(), Qt::OddEvenFill);
        hovered[1] = polyHover[1]->containsPoint(me->pos(), Qt::OddEvenFill);

        update();
    }
    if (e->type() == QEvent::MouseButtonRelease && (hovered[0] || hovered[1]))
    {
        if ((hovered[0] && turnOn) || (hovered[1] && !turnOn))
            setState(!turnOn);
    }

    return QWidget::event(e);
}

void schBreaker::paintEvent(QPaintEvent *)
{
    QPainter painter(this);

    initPositions(width(), height());
    painter.setRenderHint(QPainter::Antialiasing, true);
    painter.setPen(QColor(Qt::black));
    painter.setBrush(QBrush(Qt::black));

    int addit = 1;
    int center = rectCenter + addit;

    //boundrect
    painter.setPen(QPen(QBrush(outterLineColor), outterLineWidth));
    painter.setBrush(QBrush(backgroundColor));
    QPainterPath path;
    QRectF outRect(outterLineWidth/2, outterLineWidth/2, width() - outterLineWidth, height() - outterLineWidth);
    path.addRoundedRect(outRect, 15, 15);
    painter.fillPath(path, backgroundColor);
    painter.drawPath(path);

    painter.setBrush(QBrush(Qt::black));
    painter.setPen(QColor(Qt::black));

    //In point
    painter.drawRect((ptsIn[0]->x() - rectCenter) + addit, (ptsIn[0]->y() - rectCenter) + addit, center, center);

    //Out point
    painter.drawRect((ptsOut[0]->x() - rectCenter) + addit, (ptsOut[0]->y() - rectCenter) + addit, center, center);

    //Off point
    painter.drawRect((offPos->x() - rectCenter) + addit, (offPos->y() - rectCenter) + addit, center, center);

    //Connecting line
    if (turnOn)
        painter.drawLine(*ptsIn[0], *ptsOut[0]);
    else
        painter.drawLine(*ptsIn[0], *offPos);

    //Hovered
    painter.setOpacity(0.5);
    painter.setBrush(QBrush(Qt::blue));
    if (hovered[0] && turnOn)
        painter.drawPolygon(*polyHover[0]);
    else if (hovered[1] && !turnOn)
        painter.drawPolygon(*polyHover[1]);


}
