#include "schdiodesignals.h"
#include <QPicture>

schDiodeSignals::schDiodeSignals(QWidget *parent) : QWidget(parent), AbstractInOutLines (3, 4)
{
    textDFL = new QLabel(tr("DFL"), this);
    textLFL = new QLabel(tr("LFL"), this);
    textINT = new QLabel(tr("INT"), this);
    textDFL->setScaledContents(true);
    textLFL->setScaledContents(true);
    textINT->setScaledContents(true);
    miniBoxY[0] = 0;
    miniBoxY[1] = 0;
    miniBoxY[2] = 0;
    initPositions(0, 0);
}

void schDiodeSignals::recalcPoints()
{
    initPositions(width(), height());

    setPtsOutXY(0, boxX + boxW*0.5 + maxWidthMinBox*0.5, miniBoxY[0] + maxWidthMinBox*0.5);
    setPtsOutXY(1, boxX + boxW*0.5 + maxWidthMinBox*0.5, miniBoxY[1] + maxWidthMinBox*0.5);
    setPtsOutXY(2, boxX + boxW*0.5 + maxWidthMinBox*0.5, miniBoxY[2] + maxWidthMinBox*0.5);

    emit recalc();
}

void schDiodeSignals::initPositions(int w, int h)
{
    boxX = 0;
    boxY = h*0.15;
    boxW = w;
    boxH = h - boxY;


    maxWidthMinBox = textDFL->width() + textDFL->width()*0.2;
    miniBoxH = maxWidthMinBox;

    double scale = width()/70.;
    if (scale > maxFontScale)
        scale = maxFontScale;
    QFont font = QFont(textDFL->font().family(), 10*scale);
    textDFL->setFont(font);
    textDFL->adjustSize();
    textLFL->setFont(font);
    textLFL->adjustSize();
    textINT->setFont(font);
    textINT->adjustSize();

    emit initPos(w, h);
}

bool schDiodeSignals::event(QEvent *e)
{
    if (QEventResizedMoved(e))
        recalcPoints();

    return QWidget::event(e);
}

void schDiodeSignals::paintEvent(QPaintEvent *)
{
    QPainter painter(this);

    int w = width();
    int h = height();

    initPositions(w, h);
    painter.setRenderHint(QPainter::Antialiasing, true);

    //box
    painter.setPen(QPen(QBrush(outterLineColor), outterLineWidth));
    painter.setBrush(QBrush(backgroundColor));
    painter.drawRect(boxX, boxY, boxW, boxH);
    painter.setBrush(QBrush());

    painter.setPen(QColor(Qt::black));

    //DFL, LFL, INT

    painter.drawRect(boxX + boxW*0.5 - maxWidthMinBox*0.5, miniBoxY[0], maxWidthMinBox, maxWidthMinBox);
    painter.drawRect(boxX + boxW*0.5 - maxWidthMinBox*0.5, miniBoxY[1], maxWidthMinBox, maxWidthMinBox);
    painter.drawRect(boxX + boxW*0.5 - maxWidthMinBox*0.5, miniBoxY[2], maxWidthMinBox, maxWidthMinBox);
    textDFL->move(w*0.5 - textDFL->width()*0.5, miniBoxY[0] + maxWidthMinBox*0.5 - textDFL->height()*0.5);
    textLFL->move(w*0.5 - textLFL->width()*0.5, miniBoxY[1] + maxWidthMinBox*0.5 - textLFL->height()*0.5);
    textINT->move(w*0.5 - textINT->width()*0.5, miniBoxY[2] + maxWidthMinBox*0.5 - textINT->height()*0.5);

    //out lines under widget
    painter.drawLine(ptsOut[0]->x(), ptsOut[0]->y(), boxX + boxW, ptsOut[0]->y());
    painter.drawLine(ptsOut[1]->x(), ptsOut[1]->y(), boxX + boxW, ptsOut[1]->y());
    painter.drawLine(ptsOut[2]->x(), ptsOut[2]->y(), boxX + boxW, ptsOut[2]->y());
}
