#include "schlockin.h"

schLockIn::schLockIn(QWidget *parent) : QWidget(parent), AbstractInOutLines (4, 4)
{
    LPText = new QLabel(tr("Low Pass Filter"), this);
    LPText->setScaledContents(true);
    PShText = new QLabel(tr("Phase Shift"), this);
    PShText->setScaledContents(true);
    edLP = new Edit(0, 20, 1);
    edLP->setSpecialValueText(tr("min"));
    edPSh = new Edit(-180, 180, 1);
    edLP->setParent(this);
    edPSh->setParent(this);
    headText = new QLabel(tr("Lock in"), this);
    headText->setScaledContents(true);
    edLP->show();
    edPSh->show();
    edLP->setFixedSize(edPSh->size());
    initPositions(100, 100);
}

void schLockIn::initPositions(int w, int h)
{
    boxX = 0;
    boxY = h*0.15;
    boxW = w;
    boxH = h - boxY;

    double scale = width()/100.;
    if (scale > maxFontScale)
        scale = maxFontScale;
    QFont font = QFont(headText->font().family(), 10*scale);
    headText->setFont(font);
    headText->adjustSize();
    LPText->setFont(font);
    LPText->adjustSize();
    PShText->setFont(font);
    PShText->adjustSize();

    edLP->setFont(font);
    edLP->setFixedWidth(boxW - 5*2);
    edLP->adjustSize();
    edPSh->setFont(font);
    edPSh->setFixedWidth(boxW - 5*2);
    edLP->adjustSize();
}

bool schLockIn::event(QEvent *e)
{
    if (QEventResizedMoved(e))
        recalcPoints();

    return QWidget::event(e);
}

void schLockIn::paintEvent(QPaintEvent *)
{
    QPainter painter(this);

    int w = width();
    int h = height();

    initPositions(w, h);
    painter.setRenderHint(QPainter::Antialiasing, true);

    //Box
    painter.setPen(QPen(QBrush(outterLineColor), outterLineWidth));
    painter.setBrush(QBrush(backgroundColor));
    painter.drawRect(boxX, boxY, boxW, boxH);
    painter.setBrush(QBrush());

    painter.setPen(QColor(Qt::black));

    //LP, FSh
    float LPy = boxY + boxH*0.3;
    float FShy = boxY + boxH*0.6;
    edLP->move(5, LPy);
    edPSh->move(5, FShy);
    LPText->move(5, LPy - LPText->height());
    PShText->move(5, FShy - PShText->height());

    //'Lock in' text
    headText->move(boxX, boxY - headText->height());
}

void schLockIn::recalcPoints()
{
    int w = width();
    int h = height();
    initPositions(w, h);

    setPtsInXY(0, boxX, boxY + boxH*0.25);
    setPtsInXY(1, boxX, boxY + boxH*0.50);
    setPtsInXY(2, boxX, boxY + boxH*0.75);
    setPtsInXY(3, boxW*0.5, boxY + boxH);

    setPtsOutXY(0, boxX + boxW, boxY + boxH*0.13);
    setPtsOutXY(1, boxX + boxW, boxY + boxH*0.37);
    setPtsOutXY(2, boxX + boxW, boxY + boxH*0.6);
    setPtsOutXY(3, boxX + boxW, boxY + boxH*0.84);

    emit recalc();
}
