#include "schpid.h"

schPID::schPID(QWidget *parent) : QWidget(parent), AbstractInOutLines(1, 2)
{
    mainVLay = new QVBoxLayout(this);
    auto hLays = new QHBoxLayout*[3];
    for(int i=0; i<3; i++)
    {
        hLays[i] = new QHBoxLayout();
        mainVLay->addLayout(hLays[i]);
    }
    edPr = new Edit(0, 10, 1);
    edPr->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
    edInt = new Edit(0, 1, 1);
    edInt->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
    headText = new QLabel(tr("PID"), this);
    headText->setScaledContents(true);
    prText = new QLabel(tr("Pr"));
    prText->setScaledContents(true);
    hLays[0]->addWidget(prText);
    hLays[0]->addWidget(edPr);
    intText = new QLabel(tr("Int"));
    intText->setScaledContents(true);
    hLays[1]->addWidget(intText);
    hLays[1]->addWidget(edInt);
    tsbFBSign = new TwoStateButton(":/SPMTuning_window/plus.png",":/SPMTuning_window/minus.png");
    tsbFBSign->setToolTip(tr("Feed Back sign"));
    FBSignText = new QLabel(tr("FB sign"));
    hLays[2]->addWidget(FBSignText, Qt::AlignCenter);
    hLays[2]->addWidget(tsbFBSign);
    edPr->show();
    edInt->show();
    initPositions(0, 0);
}

void schPID::recalcPoints()
{
    initPositions(width(), height());

    setPtsInXY(0, headText->width() + 7, boxY);
    setPtsInXY(1, boxW*0.5, boxY);
    setPtsOutXY(0, boxW*0.5, boxY + boxH);
    edInt->setFixedSize(edPr->size());
}

void schPID::initPositions(int w, int h)
{
    boxX = 0;
    boxY = h*0.15;
    boxW = w;
    boxH = h - boxY;
    mainVLay->setGeometry(QRect(0, boxY, boxW, boxH));

    double scale = w/100.;
    if (scale > maxFontScale)
        scale = maxFontScale;
    QFont font = QFont(headText->font().family(), 10*scale);
    headText->setFont(font);
    headText->adjustSize();
    prText->setFont(font);
    prText->adjustSize();
    intText->setFont(font);
    intText->adjustSize();
    FBSignText->setFont(font);
    FBSignText->adjustSize();
}

bool schPID::event(QEvent *e)
{
    if (QEventResizedMoved(e))
        recalcPoints();

    return QWidget::event(e);
}

void schPID::paintEvent(QPaintEvent *)
{
    QPainter painter(this);

    int w = width();
    int h = height();

    initPositions(w, h);
    painter.setRenderHint(QPainter::Antialiasing, true);

    //Box
    painter.setPen(QPen(QBrush(outterLineColor), outterLineWidth));
    painter.setBrush(QBrush(backgroundColor));
    painter.drawRect(boxX, boxY, boxW, boxH);
    painter.setBrush(QBrush());

    painter.setPen(QColor(Qt::black));

    //PSO text
    headText->move(boxX, boxY - headText->height());
}
