#include "schpiezo.h"

schpiezo::schpiezo(QWidget *parent) : QWidget(parent), AbstractInOutLines (3, 0)
{
    lBoxColor = QColor(253, 240, 1, 75);
    rBoxColor = QColor(242, 218, 177);
    piezoText = new QLabel(tr("Piezo"), this);
    initPositions(0, 0);
}

void schpiezo::initPositions(int w, int h)
{
    box1X = 0;
    box1Y = h*0.3;
    box1W = w*0.6;
    box1H = h*0.3;
    box2X = box1X + box1W;
    box2Y = 0;
    box2W = w - box1W;
    box2H = box1Y + box1H;

    double scale = w/120.;
    if (scale > maxFontScale)
        scale = maxFontScale;
    piezoText->setFont(QFont(piezoText->font().family(), 10*scale));
    piezoText->setScaledContents(true);
    piezoText->adjustSize();
}

void schpiezo::recalcPoints()
{
    initPositions(width(), height());
    setPtsOutXY(0, 0, height()*0.5);
    setPtsOutXY(1, box1X + box1W*0.5, box1Y);
    setPtsOutXY(2, box2X + box2W - 2, box2Y + box2H*0.5);
}

bool schpiezo::event(QEvent *e)
{
    if (QEventResizedMoved(e))
        recalcPoints();

    return QWidget::event(e);
}

void schpiezo::paintEvent(QPaintEvent *)
{
    QPainter painter(this);

    int w = width();
    int h = height();

    initPositions(w, h);
    painter.setRenderHint(QPainter::Antialiasing, true);

    //Box left
    painter.setPen(QColor(Qt::black));
    painter.setBrush(QBrush(lBoxColor));
    auto ptsBox = new QPoint[7];
    ptsBox[0].setX(box1X);
    ptsBox[0].setY(box1Y);
    ptsBox[1].setX(ptsBox[0].x() + box1W);
    ptsBox[1].setY(ptsBox[0].y());
    ptsBox[2].setX(ptsBox[1].x());
    ptsBox[2].setY(box1Y + box1H);
    ptsBox[3].setX(ptsBox[1].x() - box1W*0.3);
    ptsBox[3].setY(ptsBox[2].y());
    ptsBox[4].setX(ptsBox[3].x() - box1W*0.15);
    ptsBox[4].setY(h);
    ptsBox[5].setX(ptsBox[4].x() - box1W*0.15);
    ptsBox[5].setY(ptsBox[3].y());
    ptsBox[6].setX(ptsBox[0].x());
    ptsBox[6].setY(ptsBox[2].y());
    painter.drawPolygon(ptsBox, 7);

    //box right
    painter.setPen(QColor(Qt::black));
    painter.setBrush(QBrush(rBoxColor));
    painter.drawRect(box2X, box2Y, box2W, box2H);
    painter.setBrush(QBrush());

    //Piezo text
    piezoText->move(box2X + 8, box2Y + box2H*0.5 - piezoText->height()*0.5);
}
