#include "schswitch.h"
#include <QPainterPath>

schSwitch::schSwitch(uint numIn, QWidget *parent) :
    QWidget(parent), AbstractInOutLines(1, numIn)
{
    offset = 10;
    initPositions(0, 0);
    recalcingPoints = false;
    currentHoveredId = -1;
    currentConnectedIn = 0;
    vecPolygons = new QVector<QPolygon*>();
    setMouseTracking(true);
}

void schSwitch::recalcPoints()
{
    recalcingPoints = true;
    initPositions(width(), height());
    emit recalc();
    recalcingPoints = false;
}

void schSwitch::initPositions(int w, int )
{

}

bool schSwitch::event(QEvent *e)
{
    if (QEventResizedMoved(e))
        recalcPoints();

    if (e->type() == QEvent::Leave)
    {
        currentHoveredId = -1;
        update();
    }
    else if (e->type() == QEvent::MouseMove && !recalcingPoints)
    {
        auto me = dynamic_cast<QMouseEvent *>(e);
        int hoveredId = -1;
        for (int i=0; i<vecPolygons->count(); i++)
        {
            if (recalcingPoints)
                break;
            if (vecPolygons->at(i) == nullptr)
                continue;
            if (vecPolygons->at(i)->containsPoint(me->pos(), Qt::OddEvenFill))
            {
                if (i == currentConnectedIn)
                    hoveredId = -1;
                else
                    hoveredId = i;
                break;
            }
        }
        currentHoveredId = hoveredId;
        update();
    }
    if (e->type() == QEvent::MouseButtonRelease && currentHoveredId != -1)
    {
        currentConnectedIn = currentHoveredId;
        currentHoveredId = -1;
        update();
        emit indexChanged(currentConnectedIn);
    }

    return QWidget::event(e);
}

void schSwitch::paintEvent(QPaintEvent *)
{
    QPainter painter(this);

    initPositions(width(), height());
    painter.setRenderHint(QPainter::Antialiasing, true);
    painter.setPen(QColor(Qt::black));

    //boundrect
    painter.setPen(QPen(QBrush(outterLineColor), outterLineWidth));
    painter.setBrush(QBrush(backgroundColor));
    QPainterPath path;
    QRectF outRect(outterLineWidth/2, outterLineWidth/2, width() - outterLineWidth, height() - outterLineWidth);
    path.addRoundedRect(outRect, 15, 15);
    painter.fillPath(path, backgroundColor);
    painter.drawPath(path);

    painter.setPen(QColor(Qt::black));
    painter.setBrush(QBrush(Qt::black));

    int addit = 1;
    int center = rectCenter + addit;

    //Out point
    painter.drawRect((ptsOut[0]->x() - rectCenter) + addit,
                     (ptsOut[0]->y() - rectCenter) + addit, center, center);

    //In points
    for (uint i=0; i<countIn; i++)
    {
        painter.drawRect((ptsIn[i]->x() - rectCenter) + addit,
                         (ptsIn[i]->y() - rectCenter) + addit, center, center);
    }

    //Connecting line
    painter.drawLine(*ptsOut[0], *ptsIn[currentConnectedIn]);

    //Hovered point
    if (currentHoveredId != -1)
    {
        painter.setOpacity(0.5);
        painter.setBrush(QBrush(Qt::blue));
        painter.drawPolygon(*vecPolygons->at(currentHoveredId));
    }
}

void schSwitch::clearArcsPolygons()
{
    for (int i=0; i<vecPolygons->count(); i++)
    {
        delete vecPolygons->at(i);
    }
    vecPolygons->clear();
}
