#include "splashaboutwindow.h"

QSplashScreen *makeSplash()
{
    QPixmap spl(":/Splash_window/spr_sh.png");

    {//update splash with text
        QPainter pp;
        if (!pp.begin(&spl))
            qDebug() << "no splash img";

        pp.setPen(Qt::white);
        pp.drawText(spl.rect().bottomLeft() + QPointF(5, -7), TAboutWindow::tr("__SOFTWARE_VERSION_STRING"));
        pp.setPen(Qt::gray);
        pp.drawText(spl.rect().bottomRight() + QPointF(-400, -70), TAboutWindow::tr("Developers :"));
        pp.drawText(spl.rect().bottomRight() + QPointF(-280, -70), TAboutWindow::tr("Anton Ikonnikov"));
        pp.drawText(spl.rect().bottomRight() + QPointF(-280, -50), TAboutWindow::tr("Igor Dushkin"));
        pp.drawText(spl.rect().bottomLeft() + QPointF(200, -300), TAboutWindow::tr("Nano Scan Technologies Ltd."));
        pp.drawText(spl.rect().bottomLeft() + QPointF(200, -270), TAboutWindow::tr("Powered by QT and QWT"));
        pp.end();
    }

    QSplashScreen *splash = new QSplashScreen;
    splash->setWindowFlags(splash->windowFlags() | Qt::WindowStaysOnTopHint);
    splash->setPixmap(spl);
    spl = QPixmap(); //clear memory
    splash->setEnabled(false);
    splash->show();

    int sp_align = Qt::AlignBottom | Qt::AlignRight;
    splash->showMessage(QObject::tr("Creating application interface..."), sp_align, Qt::white);

    return splash;
}

void TAboutModule::on_PBAbout_clicked()
{
    auto mainWindow = MainWindow::GetIt();
    if (!wid)
        wid = new TAboutWindow(mainWindow); //TODO auto-delete on close
    wid->layout()->activate();
    wid->move(mainWindow->mapToGlobal(mainWindow->rect().center() - wid->rect().center()));
    wid->show();
}
TAboutModule::TAboutModule()
{
    wid = nullptr;
}
void TAboutModule::connected(const QLatin1String name, I_MODULE *module)
{
    if (name == QLatin1String("mainwindow")) {
        MainWindow *m = (MainWindow *)module->basecast();

        pb = new QPushButton(makeHugeIcon(":/MainToolbar/About_nst.png"), MainWindow::tr("About"));
        pb->setFlat(true);
        connect(pb, SIGNAL(clicked()), this, SLOT(on_PBAbout_clicked()));

        QToolBar *TBMain = m->findChild<QToolBar *>("MainTB");
        TBMain->addWidget(pb);
    }
}

TAboutWindow::TAboutWindow(QWidget *parent) : QWidget(parent, Qt::Dialog)
{
    setWindowFlags((Qt::WindowFlags)(windowFlags() & (~Qt::WindowContextHelpButtonHint)));
    setWindowModality(Qt::WindowModal);
    setWindowTitle(tr("About - NSpec"));

    QString build;
    QFile buildf(qApp->applicationDirPath() + "/nst_build.txt");
    if (buildf.exists() && buildf.open(QIODevice::ReadOnly)) {
        buildf.readLine(); //"do not remove.." warning
        build = ' ' + QString(buildf.readAll());
        buildf.close();
    } else {
        QFile buildf(":/nst_build.txt");
        buildf.open(QIODevice::ReadOnly);
        buildf.readLine(); //"do not remove.." warning
        build = ' ' + QString(buildf.readAll());
        buildf.close();
    }

    //"\n" for plainText, "<br>" for richText!
    QLabel *Label = new QLabel(
                tr("Nano Scan Technologies Ltd.")+"<br>"+
                tr("SPM & Spectroscopy Software")+"<br>"+
                tr("__SOFTWARE_VERSION_STRING")+build+"<br><br>"+
                "<a href='"+tr("http://www.nanoscantech.ru/en/")+"'>"+tr("http://www.nanoscantech.ru/en/")+"</a><br>"+
                tr("e-mail: ")+"<a href='mailto:info@nanoscantech.ru'>info@nanoscantech.ru</a><br>", this);
    //tr("http://www.nanoscantech.ru/en/")+"\n"+
    //tr("e-mail: ")+"info@nanoscantech.ru");
    Label->setTextFormat(Qt::RichText);
    Label->setTextInteractionFlags(Qt::TextSelectableByMouse | Qt::LinksAccessibleByMouse);
    Label->setOpenExternalLinks(true);

    QLabel *LPixmap = new QLabel(this);
    LPixmap->setPixmap(QPixmap(":/Splash_window/spr_sh.png").scaled(200, 200, Qt::KeepAspectRatio, Qt::SmoothTransformation));

    QHBoxLayout *MainLayout = new QHBoxLayout(this);
    QVBoxLayout *RL = new QVBoxLayout;

    MainLayout->addWidget(LPixmap);
    MainLayout->addLayout(RL);

    RL->addWidget(Label);

    QPushButton *PBOk = new QPushButton(tr("Ok"), this);
    PBOk->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
    PBOk->setShortcut(QKeySequence(Qt::Key_Escape));
    QShortcut *QSaccept = new QShortcut(QKeySequence(Qt::Key_Return), this);
    QShortcut *QSaccept2 = new QShortcut(QKeySequence(Qt::Key_Enter), this);
    connect(QSaccept, SIGNAL(activated()), PBOk, SLOT(animateClick()));
    connect(QSaccept2, SIGNAL(activated()), PBOk, SLOT(animateClick()));

    RL->addWidget(PBOk, Qt::AlignBottom);

    connect(PBOk, SIGNAL(clicked()), this, SLOT(hide()));

    MainLayout->setSizeConstraint(QLayout::SetFixedSize);
}
