#include "t3dchooser.h"
#include "gui/qdoubleclickbutton.h"
#include <QDebug>

t3dChooser::t3dChooser(plot::T3dPlot *t3d, QWidget *parent) : QWidget(parent)
{
    t3dPlot = t3d;
    hMainLay = new QVBoxLayout(this);
    pButtons = new QVector<QDoubleClickButton*>();

    hMainLay->addWidget(new QLabel(tr("Channels")), 0, Qt::AlignHCenter);
    char word = 'A';

    checkedBtnStyle = "QPushButton:checked { \
            border: 1px solid red; \
            border-radius: 1px; \
            padding: 1px; \
        }";

    for (uint i = 0; i < t3dPlot->numPlotByY; i++)
    {
        QHBoxLayout *vLay = new QHBoxLayout();
        hMainLay->addLayout(vLay);
        for (uint j = i * t3dPlot->numPlotByX; j < (i + 1) * t3dPlot->numPlotByX && j < t3dPlot->ThreeDPlots.size(); j++)
        {
            auto btn = new QDoubleClickButton(QString(word + t3dPlot->orderChannel->at(j)));
            vLay->addWidget(btn);
            btn->setMaximumSize(30,30);
            btn->setCheckable(true);
            btn->setStyleSheet(checkedBtnStyle);
            connect(btn, SIGNAL(toggled(bool)), this, SLOT(buttonChecked(bool)));
            pButtons->append(btn);
        }
    }
}

void t3dChooser::buttonChecked(bool checked)
{
    if (!checked)
        return;
    int num = 0;
    int btnsChecked[2];
    int plotChecked[2];

    for(int i=0; i<pButtons->size(); i++)
    {
        if (pButtons->at(i)->isChecked())
        {
            btnsChecked[num] = i;
            plotChecked[num++] = t3dPlot->positionChannel->at(i);
        }
    }

    if (num != 2)
        return;

    pButtons->at(btnsChecked[0])->setChecked(false);
    pButtons->at(btnsChecked[1])->setChecked(false);

    t3dPlot->SwapPlots(plotChecked[0], plotChecked[1]);

    QString str = pButtons->at(btnsChecked[0])->text();
    pButtons->at(btnsChecked[0])->setText(pButtons->at(btnsChecked[1])->text());
    pButtons->at(btnsChecked[1])->setText(str);

    emit plotsChanged();
}

void t3dChooser::setPositions(QList<QVariant> order)
{
    for (int i=0; i<order.size(); i++)
    {
        if (t3dPlot->orderChannel->at(i) != order[i].toInt())
        {
            for (int j=i+1; j<order.size(); j++)
            {
                if (t3dPlot->orderChannel->at(j) == order[i].toInt())
                {
                    t3dPlot->SwapPlots(j, i);
                    break;
                }
            }
        }

    }

    for (int i=0;  i<order.size(); i++)
        pButtons->at(i)->setText(QString('A' + t3dPlot->positionChannel->at(i)));
}

void t3dChooser::plotDoubleClicked()
{
    for (int i=0; i<pButtons->size(); i++)
    {
        pButtons->at(i)->setEnabled(currFullWindPlot != -1);
    }
    currFullWindPlot = (currFullWindPlot == -1) ? 0 : -1;
}
