#include "t3dcontrol.hpp"
#include "viewer3d.hpp"
#include "flowlayout.hpp"
#include "controls.h"

#include <QPushButton>
#include <QFileDialog>
#include <QTimer>
#include <QGroupBox>
#include <QEvent>

void T3DControl::top_view()
{
    pl3->x_rot = 90;
    pl3->y_rot = 0;
    pl3->z_rot = 0;
    pl3->updateGL();
}

void T3DControl::save_click()
{
    QString SelectedFilter;
    QString FileName;


    FileName = QFileDialog::getSaveFileName(nullptr,
                                            tr("Save as..."),
                                            QString(),
                                            tr("*.png;;*.jpg;;*.tiff;;*.bmp"),
                                            &SelectedFilter,
                                            QFileDialog::DontUseNativeDialog);

    if (FileName.isEmpty())
        return;

    if (!FileName.endsWith(SelectedFilter.remove(0, 1)))
        FileName += SelectedFilter;

    blockVisible = true;
    setVisible(false);

    pl3->grabFrameBuffer().save(FileName, SelectedFilter.remove(0, 1).toAscii().data());

    blockVisible = false;
    mouseEvent(new QEvent(QEvent::Leave));
}

void T3DControl::move_toggle(bool checked)
{
    pl3->mousemode = checked ? 1 : 0;
    if (!checked) {
        BMoveFlag->setIcon(makeHugeIconBounded(":/3D/move.png", boundColor, boundWidth));
        return;
    }

    BMoveFlag->setIcon(makeHugeIconBounded(":/3D/move_on.png", boundColor, boundWidth));
}

void T3DControl::scale_click()
{
    pl3->z_scale = 0.2;
    pl3->scale = 1;
    pl3->x_pos = pl3->y_pos = pl3->z_pos = 0;
    pl3->resizeGL(pl3->width(), pl3->height());
    pl3->updateGL();
}

void T3DControl::background_toggle(bool checked)
{
    float bg = palette().color(QPalette::Background).value();
    if (checked) bg = 256 - bg;
    pl3->bgColor = QColor(bg, bg, bg);

    float fg = (bg > 127) ? 0 : 255;
    pl3->fgColor = QColor(fg, fg, fg);

    pl3->updateGL();
}
void T3DControl::box_toggle(bool checked)
{
    if (!checked) {
        BBox->setIcon(makeHugeIconBounded(":/3D/wireOn.png", boundColor, boundWidth));
        pl3->showAxes = true;
        pl3->updateGL();
        return;
    }

    BABox->setChecked(false);
    BBox->setIcon(makeHugeIconBounded(":/3D/wireOF.png", boundColor, boundWidth));
    pl3->showAxes = false;
    pl3->updateGL();
}
void T3DControl::allbox_toggle(bool checked)
{
    if (!checked) {
        BABox->setIcon(makeHugeIconBounded(":/3D/scaleOn.png", boundColor, boundWidth));
        BBox->setIcon(makeHugeIconBounded(":/3D/wireOn.png", boundColor, boundWidth));
        pl3->showAxes = true;
        pl3->updateGL();
        return;
    }

    BBox->setChecked(false);
    BABox->setIcon(makeHugeIconBounded(":/3D/scaleOf.png", boundColor, boundWidth));
    BBox->setIcon(makeHugeIconBounded(":/3D/wireOF.png", boundColor, boundWidth));
    pl3->showAxes = false;
    pl3->updateGL();
}

void T3DControl::rotate_click()
{
    if (BRotate->isChecked()) {
        rot_timer->start(50);
        BRotate->setIcon(makeHugeIconBounded(":/3D/rotateOn.png", boundColor, boundWidth));
    } else {
        rot_timer->stop();
        BRotate->setIcon(makeHugeIconBounded(":/3D/rotate.png", boundColor, boundWidth));
    }
}

void T3DControl::rotate()
{
    if (!BRotate->isChecked())
        return;

    pl3->z_rot += 1;
    pl3->updateGL();
}

void T3DControl::onBack()
{
    if (bBackExit)
    {
        emit exit(false);
        return;
    }
    bBackExit = false;
    setVisible(true);
}

void T3DControl::setVisible(bool visible)
{
    if (visible && blockVisible)
        return;

    BBack->setParent(pl3);

    for (int i=0; i<allBtnsList.size(); i++)
        allBtnsList.at(i)->setParent(pl3);

    resized(pl3->size());

    BBack->setVisible(visible);
    for (int i=0; i<allBtnsList.size(); i++)
    {
        allBtnsList.at(i)->setVisible(visible);
        //allBtnsList.at(i)->repaint();
    }
}

void T3DControl::setButtonStyle(QPushButton *btn)
{
    QString style = "";

    style += "QPushButton {";
//    style += "background-color: rgba(255,255,255,10);";
    //style += "border: 4px solid #000000;";
    style += "}";
    style += "QToolTip { color: black; background-color: #FFFFE1; border: 1px solid black; }";
    style += "QPushButton:pressed{ background-color: lightgray; }";

    btn->setStyleSheet( style );
}

void T3DControl::resized(QSize newSize)
{
    int leftX = 0;
    int rightX = leftX*0.2;
    int upY = 0;
    if (upY == 0)
        upY = 13;
    int downY = newSize.height();
    int lessSz = newSize.width() < newSize.height() ? newSize.width():newSize.height();
    int BtnsSize = (lessSz - leftX)*0.2;
    int UpButtonSize = BtnsSize*0.8;
    if (UpButtonSize < 55) UpButtonSize = 55;
    if (UpButtonSize > 70) UpButtonSize = 70;

    for (int i=0; i<allBtnsList.size(); i++)
    {
        allBtnsList.at(i)->setFixedSize(UpButtonSize, UpButtonSize);
        allBtnsList.at(i)->setIconSize(QSize(UpButtonSize, UpButtonSize));
    }

    double mulBtn = 0.15;

    int x = pl3->width() - UpButtonSize*allBtnsList.size()/2.5;
    int y = upY;
    for (int i=0; i<allBtnsList.size(); i++)
    {
        allBtnsList[i]->move(x, y);
        x -= UpButtonSize + UpButtonSize*mulBtn;
    }

    BBack->setFixedSize(UpButtonSize, UpButtonSize);
    BBack->setIconSize(QSize(UpButtonSize, UpButtonSize));
    BBack->move(15, upY);
}

void T3DControl::mouseEvent(QEvent *event)
{
    if (event->type() == QEvent::Enter || event->type() == QEvent::MouseMove)
    {
        setVisible(true);
    }
    else if (event->type() == QEvent::Leave)
    {
		auto rect = pl3->rect();
        int boundH = rect.height()*0.01;
        int boundW = rect.width()*0.01;
        QRect smallerRect(rect.left()+boundW, rect.top()+boundH,
			rect.width()-boundW*2, rect.height()-boundH*2);
        auto pos = pl3->mapFromGlobal(QCursor::pos());
        setVisible(smallerRect.contains(pos));
    }
}

T3DControl::T3DControl(T3dViewer *pl, QWidget *parent) : QWidget(parent)
{
    pl3 = pl;
    background_toggle(false);

    QPushButton *BTop = new QPushButton;
    BTop->setIcon(makeHugeIconBounded(":/container/TopView.png", boundColor, boundWidth));
    BTop->setToolTip(QObject::tr("Top view"));
    setButtonStyle(BTop);

    BMoveFlag = new QPushButton;
    BMoveFlag->setIcon(makeHugeIconBounded(":/3D/move.png", boundColor, boundWidth));
    BMoveFlag->setToolTip(QObject::tr("Move mode"));
    BMoveFlag->setCheckable(true);
    setButtonStyle(BMoveFlag);

    QPushButton *BBackground = new QPushButton;
    BBackground->setIcon(makeHugeIconBounded(":/3D/blackwhite.png", boundColor, boundWidth));
    BBackground->setToolTip(QObject::tr("Toggle black\\white background"));
    BBackground->setCheckable(true);
    setButtonStyle(BBackground);

    BBox = new QPushButton;
    BBox->setIcon(makeHugeIconBounded(":/3D/wireOn.png", boundColor, boundWidth));
    BBox->setToolTip(QObject::tr("Show\\hide graph box"));
    BBox->setCheckable(true);
    setButtonStyle(BBox);

    BABox = new QPushButton;
    BABox->setIcon(makeHugeIconBounded(":/3D/scaleOn.png", boundColor, boundWidth));
    BABox->setToolTip(QObject::tr("Show\\hide graph scale"));
    BABox->setCheckable(true);
    BABox->hide();
    setButtonStyle(BABox);

    QPushButton *BScale = new QPushButton;
    BScale->setIcon(makeHugeIconBounded(":/3D/restore_scale.png", boundColor, boundWidth));
    BScale->setToolTip(QObject::tr("Restore scale"));
    setButtonStyle(BScale);

    QPushButton *BSave = new QPushButton;
    BSave->setIcon(makeHugeIconBounded(":/snap_shot.png", boundColor, boundWidth));
    BSave->setToolTip(QObject::tr("Save image"));
    setButtonStyle(BSave);

    BRotate = new QPushButton;
    BRotate->setIcon(makeHugeIconBounded(":/3D/rotate.png", boundColor, boundWidth));
    BRotate->setToolTip(QObject::tr("Auto rotation"));
    BRotate->setCheckable(true);
    setButtonStyle(BRotate);

    BBack = new QPushButton();
    BBack->hide();
    BBack->setIcon(makeHugeIconBounded(":/ScanPlot/backPlot3D.png", boundColor, boundWidth));
    BBack->setToolTip(QObject::tr("Back"));
    BBack->setShortcut(QKeySequence(Qt::Key_Escape));
    setButtonStyle(BBack);
    connect(BBack, SIGNAL(released()), this, SLOT(onBack()));

    rot_timer = new QTimer(this);
    connect(rot_timer, SIGNAL(timeout()), SLOT(rotate()));

    connect(BMoveFlag, SIGNAL(toggled(bool)), SLOT(move_toggle(bool)));
    connect(BBackground, SIGNAL(toggled(bool)), SLOT(background_toggle(bool)));
    connect(BBox, SIGNAL(toggled(bool)), SLOT(box_toggle(bool)));
    connect(BABox, SIGNAL(toggled(bool)), SLOT(allbox_toggle(bool)));
    connect(BScale, SIGNAL(clicked()), SLOT(scale_click()));
    connect(BSave, SIGNAL(clicked()), SLOT(save_click()));
    connect(BRotate, SIGNAL(clicked()), SLOT(rotate_click()));
    connect(BTop, SIGNAL(clicked()), SLOT(top_view()));

    connect(pl3, SIGNAL(resized(QSize)), this, SLOT(resized(QSize)));
    connect(pl3, SIGNAL(eventSig(QEvent*)), this, SLOT(mouseEvent(QEvent*)));

    allBtnsList << /*BMoveFlag << BBackground << BBox <<*/ BABox
                << BScale << BSave << BRotate/* << BTop*/;
}

bool T3DControl::event(QEvent *event)
{
    mouseEvent(event);
    if (event->type() == QEvent::Resize)
        resized(pl3->size());

    return QWidget::event(event);
}
