#pragma once

#include "defs.h"
#include "tools.hpp"
#include <QThread>
#include <QPointer>
#include <QMutex>

double analog_calib(double value, const char *name, double a0 = -10., double b0 = 10.);

double input_calib(BYTE ch, double val);
double input_gain(BYTE ch);
double input_shift(BYTE ch);
void input_gain_set(BYTE ch, double gain);
void input_gain_clear(BYTE ch);
void input_shift_set(BYTE ch, double shift);
void input_shift_clear(BYTE ch);

QPair<double, double> GetPairAnalogMapCache(int key);

QString input_units(BYTE ch);
void input_unit_set(BYTE ch, QString unit);
void input_unit_clear(BYTE ch);


// r/w threads

class USB_WRITE_THREAD: public QThread, private IWriter::IEG3KWrite
{
public:
    USB_WRITE_THREAD();
    void run() override;

    void start_stop(bool);
    volatile bool want_running;

    QMutex write_lock;
    void *devHandle; //external setup before use
    TNodeContainer2<IWriter> writers;

private:
    //IWriter::IEG3KWrite
    BYTE raw_send(const void *buffer, WORD data_size) override;
    BYTE send_packet(BYTE Address, BYTE SubAddress, const void *buffer, WORD data_size) override;
};

struct eg3000_read_handle;
class USB_READ_THREAD: public QThread
{
    Q_OBJECT
signals:
    void capBits(int n, long bits);
    void status(int);

public:
    USB_READ_THREAD();
    void run() override;

    void start_stop(bool);
    volatile bool want_running;

    QMutex read_lock;
    void *devHandle; //external setup before use
    struct eg3000_read_handle *read_handle; //external setup before use
    TNodeContainer2<IProcessPacket> readers;
};


//main controller

class StatusWidget1;
class EG3000_CONTROL: public QThread
{
    Q_OBJECT

public:
    EG3000_CONTROL();
    void run() override;

signals:
    void restart_boot_timer();
    void device_status(int code);
    void usbStatus(int);

public slots:
    void thread_close();
    void drop_connect();
    void s_usbStat(int);

    void program(QString);
    void s_boot_timeout();

public:
    volatile bool m_closeThread;
    volatile bool m_waitMainInit;
    volatile bool need_reconnect;
    volatile bool boot_timeout;

    int prev_usb_stat;
    QPointer<StatusWidget1> SWusb1;

    volatile bool need_program;
    QString path;

    USB_WRITE_THREAD wThread;
    USB_READ_THREAD rThread;
};
