#include "moticx.hpp"
#include <QNetworkReply>
#include <QTcpSocket>
#include <QElapsedTimer>
#include <QCheckBox>
#include <QFormLayout>
#include <QSettings>
#include <QTimer>
#include <QApplication>
#include <QDialog>
#include <QProgressBar>
#include "tools.hpp"

MoticX::MoticX()
{
	// http request to moticX for make resolution 640x480
    /*requestSetResolution640 = QNetworkRequest(QUrl("http://" + ip + ":8080/?action=command&dest=0&plugin=0&group=2&id=123&value=1"));
    requestSetResolution640.setHeader(QNetworkRequest::ContentTypeHeader,
                                             "application/x-www-form-urlencoded");*/
    pos_ba00 = -1;
    init = false;
    reply = nullptr;

    manager.moveToThread(this);
    moveToThread(this);
}

MoticX::~MoticX()
{
    disconnect();
    terminate();
    if (reply) {
        reply->abort();
    }

    closeCamera();
}

void MoticX::Start()
{
    if (!init)
        return;

    start();
}

void MoticX::Stop()
{
    if (!init)
        return;

    quit();
}

void MoticX::newData()
{
    const unsigned char ba00[4] = {0xFF, 0xd8, 0xFF, 0}; //jpg start
    const unsigned char ba01[3] = {0xFF, 0xd9, 0}; //jpg end
    //"--boundary" - separator

    if (!reply || reply->bytesAvailable() <= 0)
        return;

    int prev_size = ba.size();
    ba.append(reply->readAll());

    if (pos_ba00 == -1)
        pos_ba00 = ba.indexOf((const char *)ba00, qMax(0, prev_size - 2));
    if (pos_ba00 == -1)
        return;

    int pos_ba01 = ba.indexOf((const char *)ba01, pos_ba00 + 3);
    if (pos_ba01 == -1)
        return;
    pos_ba01 += 2;

    QImage img;
    if (img.loadFromData((const uchar *)ba.constData() + pos_ba00, ba.size() - pos_ba00))
    {
		//FPS
        /*static int cadr = 0;
        static QTime first = QTime::currentTime();
        cadr++;
        int secs = -QTime::currentTime().secsTo(first);
        if (secs >= 1)
        {
            //qDebug()<<"fps"<<cadr<<img.width();
            first = QTime::currentTime();
            cadr = 0;
        }*/

        emit ImgReceived(img);

#if (QT_VERSION >= QT_VERSION_CHECK(5, 9, 0))
        //HACK
        //by standart @downloadProgress@ signal emits 1 times in 100ms
        //there we emitting it faster for increasing FPS
        SleepExport::msleep2(15);
        emit reply->downloadProgress(0, 0);
        SleepExport::msleep2(15);
        emit reply->downloadProgress(0, 0);
        SleepExport::msleep2(15);
        emit reply->downloadProgress(0, 0);
        SleepExport::msleep2(15);
        emit reply->downloadProgress(0, 0);
        SleepExport::msleep2(15);
        emit reply->downloadProgress(0, 0);
        SleepExport::msleep2(15);
        emit reply->downloadProgress(0, 0);
#endif
    }

    ba.remove(0, pos_ba01);
    pos_ba00 = -1;
}

void MoticX::run()
{
    reply = manager.get(QNetworkRequest(QUrl("http://" + ip + ":8080/?action=stream")));
    connect(reply, SIGNAL(downloadProgress(qint64,qint64)), this, SLOT(newData()));

    exec();

    reply->abort();
    delete reply;
    reply = nullptr;

    ba.clear();
    pos_ba00 = -1;
}

void MoticX::closeCamera()
{
    init = false;
}

bool MoticX::openCamera()
{
    closeCamera();

    /*
    QNetworkAccessManager manager;
    QNetworkReply *reply = manager.get(QNetworkRequest(QUrl("http://192.168.1.151:8080/"))); //fucking qt is freezing for 1 sec somewhere!
    QElapsedTimer t; t.start();
    while (!reply->isFinished() && t.elapsed() < 60) {
        qApp->processEvents();
        SleepExport::msleep2(2);
    }

    if (reply->error() != QNetworkReply::NoError || t.elapsed() >= 60) {
        reply->abort();
        reply->deleteLater();
        return false;
    }
    reply->abort();
    reply->deleteLater();
    //qDebug() << t.elapsed();
    */
    QTcpSocket sock;
    sock.connectToHost(ip = "192.168.1.151", 8080);
    if (sock.waitForConnected(100))
        return init = true;
#ifndef _WIN32
    sock.connectToHost(ip = "192.168.10.1", 8080);
    if (sock.waitForConnected(100))
        return init = true;
#endif

    return false;
}
