#pragma once

#include "modules.hpp"
#include "dataplot.h"
#include "oscilloscope.h"
#include "mobile/remote_mobile.hpp"

#include <QMainWindow>
#include <QScrollBar>
#include <QFuture>

#include "remote_nst_dataid.hpp"
void send2dPlot(int id, const plot::T2dPlot *plot, bool can_drop = true);
void send3dPlot(int id, const plot::T3dPlot *plot, int plotid = 0, bool can_drop = true, bool forceSend = false, bool isCCD = false);

//-----

class TScrollArea : public QScrollArea
{ //vertical, reserves space for scroll bar
public:
    TScrollArea(QWidget *parent = nullptr) : QScrollArea(parent) {
        setWidgetResizable(true);
        setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        setFrameStyle(QFrame::NoFrame);
        setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Minimum);
    }
    QSize sizeHint() const override { return QScrollArea::sizeHint() + QSize(verticalScrollBar()->sizeHint().width(), 0); }
};

//-----


class MainWindow: public QMainWindow, public I_MODULE, I_NET_MODULE_BASE
{
    Q_OBJECT

public slots:
    void s_update_singleapp();

    void AttachDetachModuleWindow();
    void moduleButtonClicked();

    void cancelOperation(); //eg3k disconnected

public:
    MainWindow();
    static MainWindow* GetIt() noexcept;

    QStackedWidget *ContentStackedWidget;
    QMap<int, QAction *> statusGroups;

    void ShowModuleWidget(I_MODULE *m);
    QHash<I_MODULE *, QPushButton *> guiModuleButtons;
    QPushButton *prev_button;

    //I_MODULE
    static const QLatin1String name() { return QLatin1String("mainwindow"); }
    void *basecast0(const QLatin1String name) { return name.latin1() != nullptr ? nullptr : static_cast<MainWindow *>(this); } //fuck msvc optimizer
    void *basecast(const QLatin1String name) override { return name == QLatin1String("net") ? static_cast<I_NET_MODULE_BASE *>(this) : basecast0(name); }
    //void *basecast(const QLatin1String name) { return name == QLatin1String("net") ? static_cast<I_NET_MODULE_BASE *>(this) : name.latin1() != 0 ? 0 : static_cast<MainWindow *>(this); }
    void connected(const QLatin1String name, I_MODULE *module) override;
    void disconnected(const QLatin1String name, I_MODULE *module) override;
    void setRunning(bool on) override;

    //I_NET_MODULE_BASE
    void initialSend(MobileInstance *client) override;
    void setValue(int id, QVariant val, QObject *sender) override;
    QMultiMap<int, QObject *> netidmap; //dataid->connected objects map
    template <class src> void add(QWidget *wid, const char *signal, const char *slot)
    {
        bool ok;
        foreach (src *ch, wid->findChildren<src *>()) {
            int id = ch->property("id").toInt(&ok);
            if (ok) {
                connect(ch, signal, slot);
                netidmap.insert(id, ch);
            }
        }
    }

protected:
    bool eventFilter(QObject *obj, QEvent *event) override;
    void resizeEvent(QResizeEvent *) override;

public slots: //net
    void setPropBool(bool val);
    void setPropInt(int val);
    void setPropDbl(double val);
    void setPropVar(QVariant val = QVariant());

};

class ManagedThreadedCleaner : public I_MODULE
{
public:
    //TODO runManagedFunc(T func); //in new thread, thread managed by ManagedThreadedCleaner, not qthreadpool
    static void registerManagedObject(QObject *obj);
    static void registerManagedFuture(QFuture<void> obj);
    static void unregisterManagedObject(QObject *obj);
    static void unregisterManagedFuture(QFuture<void> obj);

    static ManagedThreadedCleaner* GetIt() noexcept;

    //I_MODULE
    static const QLatin1String name() { return QLatin1String("managedthreadedcleaner"); }
    void *basecast(const QLatin1String name) override { return name.latin1() != nullptr ? nullptr : static_cast<ManagedThreadedCleaner *>(this); }
    void setRunning(bool on) override;

protected:
    QMutex shutdownDataMutex;
    QVector<QFuture<void> > shutdownActions;
    QVector<QObject *> shutdownObjects;
};
