#pragma once

#include <QVector>
#include <QTcpServer>

class INetworkClient : public QObject
{
    Q_OBJECT
public:
    INetworkClient(QTcpSocket *socket);
    virtual ~INetworkClient() override;

signals:
    void closed();

public slots:
    virtual void setSocket(QTcpSocket *socket);
    virtual void newRead();
    virtual void newError();

public:
    QTcpSocket *socket;
    QString lastError;

};


typedef INetworkClient *(*pINC)(QTcpSocket *);
extern INetworkClient *baseClientCtor(QTcpSocket *);


class QTcpSocket;
class NetworkServer : public QTcpServer
{
    Q_OBJECT
public:
    NetworkServer(int port, pINC client = baseClientCtor);
    ~NetworkServer() override;

signals:
    void newClient(INetworkClient *);

public slots:
    void newConnected();
    void newError();

public:
    pINC clientCtor;
    QVector<INetworkClient *> clients;

};

INetworkClient *baseClientCtor(QTcpSocket *);
class BaseNetworkClient : public INetworkClient
{
    Q_OBJECT
public:
    BaseNetworkClient(QTcpSocket *socket);

signals:
    void newData();

public slots:
    virtual void setSocket(QTcpSocket *socket) override;
    virtual void newRead() override;
    void sendData(quint32 addr, quint8 *data, quint32 size);

public:
    int version;
    bool clientsPacketStarted;
    int clientsPacketLength;
    QList<quint32> address;
    QList<QByteArray> data;

};
