#pragma once

#include <QString>
#include <QList>

class I_MODULE
{
public:
    I_MODULE() { }
    virtual ~I_MODULE();

    static const QLatin1String name() { return QLatin1String("noname"); }
    static const QList<QLatin1String> requires() { return QList<QLatin1String>(); }
    void *basecast(const char *basename = nullptr) { return basecast(QLatin1String(basename)); }
    virtual void *basecast(const QLatin1String /*basename*/) { return nullptr; }
    virtual void connected(const QLatin1String /*name*/, I_MODULE * /*module*/) { } //callback after new module loaded
    virtual void disconnected(const QLatin1String /*name*/, I_MODULE * /*module*/) { } //callback before old module unloaded
    virtual void setRunning(bool /*on*/) { } //callback on start/stop
};

class QWidget;
class QPushButton;
class I_GUI_MODULE_BASE : public I_MODULE
{
public:
    virtual QPushButton *detachButton() { return nullptr; }
    virtual QPushButton *createPanelButton() { return nullptr; } //new QPB(...
    virtual QList<QWidget *> getStatusWidgets() { return QList<QWidget *>(); } //<100 = wanted group index
};

#define GUI_MODULE_CTOR(classname) classname(QWidget *parent = nullptr) : I_GUI_MODULE(parent)
template <class T>
class I_GUI_MODULE : public T, public I_GUI_MODULE_BASE
{
public:
    I_GUI_MODULE(QWidget *parent = nullptr) : T(parent) { }
    void *basecast(const QLatin1String name) override {
        if (name.latin1() == nullptr)
            return static_cast<T *>(this);
        if (name == QLatin1String("gui"))
            return static_cast<I_GUI_MODULE_BASE *>(this);
        return nullptr;
    }
};


#include <QMultiMap>
#include <QHash>
#include <QSharedPointer>

class ModuleManager
{
public:
    ModuleManager();
    ~ModuleManager();
    static ModuleManager *instance;

    template <class Impl>
    I_MODULE *load(QWidget *parent = nullptr) { return load(Impl::name(), parent); }
    inline I_MODULE *load(const char *name, QWidget *parent = nullptr) { return load(QLatin1String(name), parent); }
    I_MODULE *load(const QLatin1String name, QWidget *parent = nullptr);
    template <class Impl>
    void add(Impl *module) { add(Impl::name(), module); }
    void add(const QLatin1String name, I_MODULE *module);
    void *find1VoidPtr(const QLatin1String name) const noexcept;
    inline void *find1VoidPtr(const char *name) const { return find1VoidPtr(QLatin1String(name)); }
    void *find1Safety(const QLatin1String name, bool throwExeption = true) const;
    inline void *find1Safety(const char *name, bool throwExeption = true) const { return find1Safety(QLatin1String(name), throwExeption); }
    inline I_MODULE *find1(const char *name) const { return find1(QLatin1String(name)); }
    I_MODULE *find1(const QLatin1String name) const;
    inline const QList<I_MODULE *> find(const char *name) const { return find(QLatin1String(name)); }
    const QList<I_MODULE *> find(const QLatin1String name) const;
    const QLatin1String find(I_MODULE *module) const;
    inline void unload(const char *name) { unload(QLatin1String(name)); } //destroy
    void unload(const QLatin1String name); //destroy
    void unload(I_MODULE *module); //disown

    QMultiMap<QLatin1String, I_MODULE *> modules;

    void setRunning(bool running = true);
    bool isRunning;

    template <class Impl>
    void addToFactory() { factory[Impl::name()] = FactoryRec(&TFNcreate<Impl>, nullptr, &TFNrequires<Impl>); }
    template <class Impl>
    void addToFactoryGui() { factory[Impl::name()] = FactoryRec(&TFNcreate<Impl>, &TFNcreategui<Impl>, &TFNrequires<Impl>); }

private:
    typedef const QList<QLatin1String> (*FNreq)();
    typedef I_MODULE * (*FNcr)();
    typedef I_MODULE * (*FNcrgui)(QWidget *parent);
    struct FactoryRec {
        FactoryRec(FNcr a1 = nullptr, FNcrgui a2 = nullptr, FNreq a3 = nullptr) : create(a1), creategui(a2), requires(a3) { }
        FNcr create;
        FNcrgui creategui;
        FNreq requires;
    };

    template <class T>
    static const QList<QLatin1String> TFNrequires() { return T::requires(); }
    template <class T>
    static I_MODULE *TFNcreate() { return new T; }
    template <class T>
    static I_MODULE *TFNcreategui(QWidget *parent = nullptr) { return new T(parent); }

protected:
    QHash<QLatin1String, FactoryRec> factory;
};


/* //example modules
   //addToFactory<Module>();
   //addToFactoryGui<GuiModule>();
#include <QObject>
#include <QWidget>
class Module : public QObject, public I_MODULE
{
public:
    static const QLatin1String name() { return QLatin1String("module"); }
    //static const QList<QLatin1String> requires() { return QList<QLatin1String>(); }
    void *basecast(const QLatin1String name) { return name.latin1() != 0 ? 0 : static_cast<QObject *>(this); }
    //void connected(const QLatin1String name, I_MODULE *module);
};
class GuiModule : public I_GUI_MODULE<QWidget>
{
public:
    GUI_MODULE_CTOR(GuiModule) { }
    static const QLatin1String name() { return QLatin1String("guimodule"); }
    //static const QList<QLatin1String> requires() { return QList<QLatin1String>(); }
    //void connected(const QLatin1String name, I_MODULE *module);
};
*/
