#pragma once

#include <defs.h>
#include <QWidget>
#include <QPointer>
#include "controls.h"
#include "edit.hpp"
#include "tools.hpp"
#include <QTableWidget>
#include <QHeaderView>
#include <QHBoxLayout>
#include "controls.h"
#include "edit.hpp"
#include "tools.hpp"
#include "spm_motors.hpp"
#include <QTime>
#include <QLabel>

class StatusWidget1;
class MotorSlider;
class QVBoxLayout;
class QGroupBox;
class QCheckBox;
class QComboBox;
class QRadioButton;
class QTabWidget;
class Motors : public QWidget, public IProcessPacket, public IWriter
{
    Q_OBJECT

signals:
    void mland_stopped();
    void motorStatusChanged(int motorVirt, bool atEnd, bool active);
    void motorStatusChangedRealId(int i, bool atEnd, bool active);

public slots:
    //num 1-based, virtual#
    void go(int num, int dir, double speed, double steps); //mkm
    void goUpE(int num = -1, double speed = 0);
    void goUp(int num = -1, double speed = 0, double dist = 0); //mkm
    void goDownE(int num = -1, double speed = 0);
    void goDown(int num = -1, double speed = 0, double dist = 0); //mkm
    void goStop(int num = -1);
    void goShuttleInt(int speedidx, int num = -1); //-5..5, with extrapolation
    void goShuttle(double speedidx, int num = -1); //-5..5, with extrapolation
    double getShuttleSpeed(int speed, int num);

    void goRoleUp(int role = RoleNone, double speed = 0, double dist = 0); //mkm
    void goRoleDown(int role = RoleNone, double speed = 0, double dist = 0); //mkm

    void init();
    void setBLspeed();
    void Slot_Set_Motor_Status_handler();
    void Slot_Set_Motor_Status();
    void updateMotorStatusWidget(unsigned count, bool* atEnd, bool* active);
    void capBits(int, long);

    void set_raw_motors_control(unsigned char, unsigned short);
    void set_step_motors_leveling(int,unsigned char, double, double);

    void save_params();
    void save_device(int count_to_save = -1); //-1: use active count
    void clear_device() { save_device(0); }
    void save_current();
    void save_shuttleType();
    void save_minSpeed();
    void SetCurrent(int num = 0); //num 0-based
    void FillMUI(int num = 0); //num 0-based
    void ShowHideRows();

    void addLandingSett(QWidget *w);
    void addShuttleSett(QWidget *w, int stretch);

public:
    Motors(QWidget *parent = nullptr);
    ~Motors() override;
    static Motors* GetIt() noexcept;

    //1-based motors, virtual
    static
    QList<int> getRoleMotors(int role); //see MotorRoles enum
    static
    int getRoleMotor(int role); //see MotorRoles enum
    static
    int mapVirtToPhys(int motor);
    // !! not static
    int mapPhysToVirt(int motor);


    enum MotorRoles {
        RoleNone,
        RoleX1, RoleY1, RoleZ1Land,
        RoleStageX, RoleStageY, RoleZ2Focus,
        RoleX3, RoleY3, RoleZ3,
        RoleUnused
    };


    bool processPacket(const BYTE addr, const BYTE sub_addr, const void *data, const DWORD size) override;
    bool sendData(const void *handle) override;
    void resetWriter() override;

#ifdef _NEW_MOTORS_CHANGES
    BYTE csb_motor_count[4];
    BYTE csb_motor_count_reported[4];
#else
    int csb_motor_count;
#endif

#ifdef _NEW_MOTORS_CHANGES
    QLabel **motorsDebug;
    int motorsDebugNumMot[3];
#endif

    QWidget *mainw;
    QVBoxLayout *ShuttleLay;
    QHBoxLayout *LandingLay;

    Edit *Epitch[16];
    Edit *Estepsrev[16];
    
    bool orient[16];
    QPointer<StatusWidget1> SWmotor1;
    bool Motor_on[16];

#ifdef _NEW_MOTORS_CHANGES
    BYTE reqch;

private:
    void write_motor_flash_cfg(BYTE idx, WORD data);
#endif

    
private:
    Edit *minSpeed;

    long motor_step[16+1];
    long motor_speed[16+1];
    WORD motor_cont_byte[16+1];
    WORD motor_inv_byte[16];
    DWORD motor_bl_steps[16]; //todo make "16" as a constant everywhere
    WORD motor_bl_speed;
    DWORD motor_land_safe_calib;
    BYTE rawmotoraddr;
    WORD rawmotordata;
    
    TMessageFlag sendReset;
    TMessageFlag updMotor[16+1];
    bool sendLMheight;

    BYTE csb_motor_addr;
    TMessageFlag updateCsbMotorAddr;
    
    TMessageFlag updateCsbMotorCount;

    BYTE motorRst;
    TMessageFlag send_control[16+1], update_inv_end[16], send_bl_steps[16], sendBLspeed, sendLandSafeCalib, sendCustom;
    
    Edit *Enum;
    QRadioButton *PBselector[16];
    int active_motor;
    int last_command;
    
    Edit *Espeed;
    Edit *Edist;
    QTableWidget *TableMotors;

    QPushButton *BSave;
    QPushButton* BClearDevice;
    QComboBox *LErole[16];
    QComboBox *ShuttleType;
    Edit *Eidx[16];
    QCheckBox *CBinv[16];
    QCheckBox *CBdir[16];
    Edit *EBLdist[16]; //backlash
    Edit *EMinSpeed[16];
    Edit *EMaxSpeed[16];
    Edit *EBLspeed; //backlash
    
    QTimer *motorSync;
    BYTE Motor[16];
    int motorStatusLimit;

    bool blocksave;
    bool reqsent;
};
