#include "options_window.h"
#include "mainwindow.h"
#include "ccd/ccdplot.h"
#include "scanplot.h"
#include "spmtunning.h"
#include "autocalib.hpp"
#include "gui/status.hpp"
#include "gui/ttabwidget.hpp"
#include "options/options.h"

#include <QStyledItemDelegate>
#include <QVBoxLayout>
#include <QFormLayout>
#include <QCheckBox>
#include <QSettings>
#include <QTreeWidget>
#include <QHeaderView>
#include <QFontDialog>
#include <QFileDialog>
#include <QToolBar>
#include <QApplication>
#include "container/datalist.hpp"
#include "gui/rectslider.h"

class EditorDelegate: public QStyledItemDelegate
{
public:
    EditorDelegate(QObject *parent) : QStyledItemDelegate(parent) {}
    QWidget *createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const override
    {/*
        if (index.data().toString() == "false" || index.data().toString() == "true") {
            QCheckBox *box = new QCheckBox(parent);
            box->setAutoFillBackground(true);
            return box;
        }*/
        return index.column() != 1 ? nullptr : QStyledItemDelegate::createEditor(parent, option, index);
    }/*
    void setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const
    {
        QCheckBox *edit = qobject_cast<QCheckBox *>(editor);
        if (!edit)
            return QStyledItemDelegate::setModelData(editor, model, index);

        model->setData(index, edit->isChecked(), Qt::EditRole);
    }
    void setEditorData(QWidget *editor, const QModelIndex &index) const
    {
        QCheckBox *edit = qobject_cast<QCheckBox *>(editor);
        if (!edit)
            return QStyledItemDelegate::setEditorData(editor, index);

        edit->setChecked(index.model()->data(index, Qt::EditRole).toBool());
    }
    void updateEditorGeometry(QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &index) const
    {
        QCheckBox *edit = qobject_cast<QCheckBox *>(editor);
        if (!edit)
            return QStyledItemDelegate::updateEditorGeometry(editor, option, index);

        edit->setGeometry(option.rect);
    }*/
};

void TOptionsWindow::selectCustomTR()
{
    if (Ctranslation->currentIndex() != 2)
        return;

    QString f = QFileDialog::getOpenFileName(this, tr("Select custom translation"), QString(), "*.qm", nullptr, QFileDialog::DontUseNativeDialog);
    if (f.isEmpty())
        f = qApp->property("translation").toString();

    if (f.endsWith(":/translation_base"))
        Ctranslation->setCurrentIndex(0);
    else if (f.endsWith(":/translation_russian"))
        Ctranslation->setCurrentIndex(1);
    else {
        Ctranslation->setCurrentIndex(2);
        Ctranslation->setItemData(2, f);
    }

    Apply(true);
}

void TOptionsWindow::applyFilterSettings(plot::T3dPlot *t3plot, plot::T3dPlotControl *PlotControl)
{
    auto scan = ScanPlot::GetIt();
    bool xDirScan = false;
    if (scan)
        xDirScan = (scan->m_BStart->dir == swapDir::Direction::dir_right);

    PlotControl->MMathProcessing->actions()[0]->setChecked((xDirScan ? chFitLines->isChecked():false));
    //PlotControl->MMathProcessing->actions()[0]->trigger();
    PlotControl->MMathProcessing->actions()[1]->setChecked((xDirScan ? false:chFitLines->isChecked()));
    //PlotControl->MMathProcessing->actions()[1]->trigger();
    PlotControl->AGSubtr->actions()[cbSubstract->currentIndex()]->trigger();
    PlotControl->AGXFitOrders->actions()[xDirScan ? cbFitting->currentIndex():0]->setChecked(true);
    //PlotControl->AGXFitOrders->actions()[xDirScan ? cbFitting->currentIndex():0]->trigger();
    PlotControl->AGYFitOrders->actions()[xDirScan ? 0:cbFitting->currentIndex()]->setChecked(true);
    //PlotControl->AGYFitOrders->actions()[xDirScan ? 0:cbFitting->currentIndex()]->trigger();
    PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfMeanValueFilter())]->setChecked(chMeanValue->isChecked());
    PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfMedianValueFilter())]->setChecked(chMedianValue->isChecked());
    PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfGaussianFilter())]->setChecked(chGaussian->isChecked());
    PlotControl->MMath2->actions()[PlotControl->GetNumOfAction(PlotControl->NameOfHistogramFilter())]->setChecked(chHistogramEqualization->isChecked());

    for (int i=0; i<t3plot->ThreeDPlots.count(); i++)
    {
        auto plot = t3plot->ThreeDPlots.at(i);
        plot->MakeCorrectLineByMedianaXDir(xDirScan ? chFitLines->isChecked():false);
        plot->MakeCorrectLineByMedianaYDir(xDirScan ? false:chFitLines->isChecked());
        plot->MakeSubtr(cbSubstract->currentIndex() - 1);
        plot->MakeXFit((xDirScan ? cbFitting->currentIndex():0) - 1);
        plot->MakeYFit((xDirScan ? 0:cbFitting->currentIndex()) - 1);
        plot->MeanValueFilterEnabled = chMeanValue->isChecked();
        plot->MedianValueFilterEnabled = chMedianValue->isChecked();
        plot->GaussianValueFilterEnabled = chGaussian->isChecked();
        plot->MeanValueFilterSize = edMeanValue->value();
        plot->MedianValueFilterSize = edMedianValue->value();
        plot->GaussianFilterSize = edGaussian->value();
        plot->HistogramEqualization = chHistogramEqualization->isChecked();
        plot->ShowHistogram(chHistogramEqualization->isChecked());
        plot->Refresh();
    }
}

void TOptionsWindow::applyFilterSettings()
{
    auto scan = ScanPlot::GetIt();

    if (scan)
    {
        auto PlotControl = scan->Plot->PlotControl;
        if (PlotControl)
        {
            applyFilterSettings(scan->Plot, PlotControl);
            scan->Plot->SetAllFilterSizesTextInAction();
        }
    }

    auto data = DataPlot::GetIt();
    if (data)
    {
        auto PlotControl = data->Container->Plot->ThreeDPlot->PlotControl;
        //if (!data->Container->Plot->ThreeDPlot->overlayButtons->hideFilterButton)
        if (PlotControl && data->Container->CurrentDataList && data->Container->CurrentDataList->count())
        {
            applyFilterSettings(data->Container->Plot->ThreeDPlot, PlotControl);
            data->Container->Plot->ThreeDPlot->SetAllFilterSizesTextInAction();
        }
    }
}

void TOptionsWindow::saveScanFilterSettings()
{
    QSettings settings("Param/scan.conf", QSettings::IniFormat);
    settings.beginGroup("3D_plot_filter");
    settings.setValue("fit_lines", chFitLines->isChecked());
    settings.setValue("substract", cbSubstract->currentIndex());
    settings.setValue("fitting", cbFitting->currentIndex());
    settings.setValue("mean_value", chMeanValue->isChecked());
    settings.setValue("median_value", chMedianValue->isChecked());
    settings.setValue("gaussian", chGaussian->isChecked());
    settings.setValue("histogam_equalization", chHistogramEqualization->isChecked());
    settings.setValue("ed_mean_value", edMeanValue->value());
    settings.setValue("ed_median_value", edMedianValue->value());
    settings.setValue("ed_gaussian", edGaussian->value());
    settings.endGroup();

    applyFilterSettings();
}

void TOptionsWindow::saveScanAllFilterSettings()
{
    QSettings settings("Param/scan.conf", QSettings::IniFormat);
    settings.beginGroup("all_plot_filter");
    settings.setValue("enable_by_def", cbByDefEnableFilters->isChecked());
    settings.endGroup();
}

void TOptionsWindow::applyFilter2DSettings()
{
    int fitOrder = cb2DFilter->currentIndex();
    auto scan = ScanPlot::GetIt();
    if (scan)
    {
        auto PlotControl = scan->Plot->TwoDPlot->PlotControl;
        if (PlotControl && scan->twodPlotFilterScanMode)
        {
            PlotControl->AGFitOrders->actions()[fitOrder]->setChecked(true);
            PlotControl->AGFitOrders->actions()[fitOrder]->trigger();

            scan->Plot->TwoDPlot->Refresh();
            scan->Plot->TwoDPlotVertical->Refresh();
        }
    }

    auto data = DataPlot::GetIt();
    if (data)
    {
        auto PlotControl = data->Container->Plot->ThreeDPlot->TwoDPlot->PlotControl;

        if (PlotControl)
        {
            PlotControl->AGFitOrders->actions()[fitOrder]->setChecked(true);
            PlotControl->AGFitOrders->actions()[fitOrder]->trigger();

            data->Container->Plot->ThreeDPlot->TwoDPlot->Refresh();
        }
    }

    auto mob = MobileServer::GetIt();
    if (mob)
        mob->setPropIdVar(DataID::plot2DDefaultFilterSettings, fitOrder);
}

void TOptionsWindow::saveRectSliderMouseWheelPower(int val)
{
    QSettings settings("Param/scan.conf", QSettings::IniFormat);
    settings.setValue("RectSliderMouseWheelPower", val);

    RectSlider::setWheelPower(val/100.0);
}

void TOptionsWindow::saveScan2DFilterSettings()
{
    QSettings settings("Param/scan.conf", QSettings::IniFormat);
    settings.beginGroup("2D_plot_filter");
    settings.setValue("filter", cb2DFilter->currentIndex());
    settings.endGroup();

    applyFilter2DSettings();
}

TOptionsWindow::TOptionsWindow(QWidget *parent) : QWidget(parent, Qt::Dialog)
{
    optionsParent = (options *)parent;
    QSettings settings2("Param/gui.conf", QSettings::IniFormat);
    bool ratisMode = settings2.value("gui/Ratis_xyz_stage_mode", false).toBool();
    initGeneralGUI();

    //config editor
    editor = new QTreeWidget();
    editor->setItemDelegate(new EditorDelegate(this));

    QWidget *configWidget = new QWidget;
    QVBoxLayout *cl1 = new QVBoxLayout(configWidget);
    if (!ratisMode)
        optionsParent->addNewTab(configWidget, tr("Hardware settings"), 1);

    cl1->addWidget(editor);

    QHBoxLayout *cwL = new QHBoxLayout;
    cwL->addWidget(new QLabel(tr("* - requires restart of program/device to apply")));
    cl1->addLayout(cwL);


    QStringList header;
    header.append(tr("File")+"/"+tr("Variable"));
    header.append(tr("Value"));
    header.append(tr("Comment"));
    editor->setColumnCount(3);
    editor->setHeaderLabels(header);
    int size = qApp->font().pixelSize();
    if (size < 1) size = 1;
    editor->header()->setMinimumHeight(size);


    //TODO per-module-enabled configs
#ifdef USE_EG3K_EMULATOR
    addConfig("Emulator.conf", "usb", false, "* " + tr("Use EG3000 emulator"));
#endif
#ifdef USE_CCD_EMULATOR
    addConfig("Emulator.conf", "ccd", false, "* " + tr("Use CCD emulator"));
#endif

    { QSettings settings("Param/ccd.conf", QSettings::IniFormat);
        addConfig(settings, "ccd.conf", "/", tr("Device selection, parameters"));
    addConfig(settings, "ccd.conf", "scale_reverse", true, tr("CCD x axis mirror"));
    addConfig(settings, "ccd.conf", "use_pmt", false, "* " + tr("Use internal PMTs"));
    addConfig(settings, "ccd.conf", "use_ormins", false, "* " + tr("Use Ormins CCD"));
    addConfig(settings, "ccd.conf", "use_sol_ccd", false, "* " + tr("Use Sol CCD"));
    addConfig(settings, "ccd.conf", "use_ars", true, "* " + tr("Use ARS3000 confocal module"));
    addConfig(settings, "ccd.conf", "use_m266", false, "* " + tr("Use M266 instead of M833 spectrograph"));
    addConfig(settings, "ccd.conf", "use_shamrock", false, "* " + tr("Use Shamrock spectrograph"));
    addConfig(settings, "ccd.conf", "use_solart2", false, "* " + tr("Use SolarT2 spectrograph"));
    addConfig(settings, "ccd.conf", "baseline_before_intensity", false, "* ");
    addConfig(settings, "ccd.conf", "andor_fast", false, tr("Enable fast Andor mode"));
    addConfig(settings, "ccd.conf", "shutter_open", 30, tr("opening time, ms"));
    addConfig(settings, "ccd.conf", "shutter_close", 30, tr("closing time, ms"));

    if (settings.value("use_ars", true).toBool()) {
        QSettings settings("Param/ars.conf", QSettings::IniFormat);
        addConfig(settings, "ars.conf", "motor_k/", tr("Distance correction factor from phys to virtual(global), mm/mm"));
        const char *mnames[6] = { QT_TR_NOOP("laser"),
                            QT_TR_NOOP("slit0/in"),
                            QT_TR_NOOP("slit1/nd_filter") };
        for (int i = 0; i < 6; i++)
            addConfig(settings, "ars.conf", "motor_k/"+QString::number(i), 1., tr(mnames[i]));
        addConfig(settings, "ars.conf", "laser_select_pos/", tr("Motor pos for each laser, mm"));
        for (int i = 0; i < 3; i++)
            addConfig(settings, "ars.conf", "laser_select_pos/"+QString::number(i), i*10);
        addConfig(settings, "ars.conf", "slit_k/", tr("Slit correction factor from width to motor dist, mm/mm"));
        for (int i = 0; i < 2; i++)
            addConfig(settings, "ars.conf", "slit_k/"+QString::number(i), 1);
        addConfig(settings, "ars.conf", "slit_zeropos/", tr("Slit dist from limit switch to zero pos, mm"));
        for (int i = 0; i < 2; i++)
            addConfig(settings, "ars.conf", "slit_zeropos/"+QString::number(i), 0.); //fuck msvc
    }
    }

    { QSettings settings("Param/controller.conf", QSettings::IniFormat);
        addConfig(settings, "controller.conf", "/", tr("Device selection, limits, autocalib parameters, etc."));
    addConfig(settings, "controller.conf", "use_eg4000", true, tr("Enable libusb - EG4000 and later"));
    addConfig(settings, "controller.conf", "csb_speed_settings_idx", 0, "* " + tr("sensor rate index in rate table"));
    addConfig(settings, "controller.conf", "double_c_output", 0, "* " + tr("C output with two sensors on channel. 0=C1-C2, 1=C1, 2=C2"));
    addConfig(settings, "controller.conf", "limit_scan_rate", 10, tr("Maximum scan rate, Hz"));
    addConfig(settings, "controller.conf", "sensors_autocalib_gain", 100, tr("Gain to use in calibration process"));
    addConfig(settings, "controller.conf", "sensors_autocalib_reserve", 0.025, tr("% of overcalibration, 1.0 = 100%"));
    QString ctrps = tr("Center position with account for nonlinearity");
    addConfig(settings, "controller.conf", "sensors_autocalib_middle_x_h", 0.55, ctrps);
    addConfig(settings, "controller.conf", "sensors_autocalib_middle_y_h", 0.48, ctrps);
    addConfig(settings, "controller.conf", "sensors_autocalib_middle_z_h", 0.55, ctrps);
    addConfig(settings, "controller.conf", "sensors_autocalib_middle_x_s", 0.55, ctrps);
    addConfig(settings, "controller.conf", "sensors_autocalib_middle_y_s", 0.55, ctrps);
    addConfig(settings, "controller.conf", "sensors_autocalib_middle_z_s", 0.55, ctrps);
    addConfig(settings, "controller.conf", "sensors_autocalib_optical_range_correction", 1.1, tr("divider for optical sensor ranges"));
    addConfig(settings, "controller.conf", "auto_i_divisor", 500, tr("I = P/divisor - when I is auto-adjusted"));
    addConfig(settings, "controller.conf", "pi_autocalib_p_scale", 0.45, tr("Multiplier for P"));
    addConfig(settings, "controller.conf", "pi_autocalib_i_scale", 10, tr("Divisor for I"));
    addConfig(settings, "controller.conf", "pi_autocalib_spm_p_scale", 0.44, tr("Multiplier for P"));
    addConfig(settings, "controller.conf", "pi_autocalib_spm_i_scale", 2, tr("Divisor for I"));
    addConfig(settings, "controller.conf", "hvsplit/z2_k", 0, tr("scale from slow to fast. 0 means disabled"));
    addConfig(settings, "controller.conf", "hvsplit/z2_freq", 1, tr("lpf freq khz"));
    addConfig(settings, "controller.conf", "mult_spm_pid_head", 1, tr("Multiply factor for head SPM P/I/D"));
    addConfig(settings, "controller.conf", "mult_spm_pid_stage", 1, tr("Multiply factor for stage SPM P/I/D"));
    addConfig(settings, "controller.conf", "autocalib_pid", false, tr("Enable PID autotuning on start"));
    addConfig(settings, "controller.conf", "autocalib_sensors", false, tr("Enable Sensors autocalibration on start"));
    }

    { QSettings settings("Param/motor.conf", QSettings::IniFormat);
    addConfig(settings, "motor.conf", "csb_motors_addr", -1, tr("-1, 0 - leave default, 1-head, 2-stage, 3-z3, 5-motors"));

#ifdef _NEW_MOTORS_CHANGES
    addConfig(settings, "motor.conf", "csb_motors_count", 255, "* " + tr("255 - default/auto, 0.. - count for 1st channel"));
    addConfig(settings, "motor.conf", "csb_motors_count2", 0, "* " + tr("count for 2nd channel when not default/auto"));
    addConfig(settings, "motor.conf", "csb_motors_count3", 0, "* " + tr("count for 3rd channel when not default/auto"));
#else
    addConfig(settings, "motor.conf", "csb_motors_count", -1, "* " + tr("-1 - default/auto, 0.. - count"));
#endif
    }

    addConfig("shuttle.conf", "shuttle_xpress", false, "* " + tr("Enable small model buttons"));
    addConfig("shuttle.conf", "piezo/delay", 50, "* " + tr("Delay between steps, ms"));

    {
        QSettings settings("Param/laser.conf", QSettings::IniFormat);
        if (settings.allKeys().size())
            addConfig(settings, "laser.conf", "/", tr("See readme.txt"));

        foreach (QString key, settings.allKeys())
            addConfig(settings, "laser.conf", key);
    }

    { QSettings settings("Param/analog.conf", QSettings::IniFormat);
        addConfig(settings, "analog.conf", "/", tr("Analog inputs/outputs calibrations"));
    QString smin = tr("Min/max voltage for ");
    QString sadc = tr("ADC");
    QString sdac = tr("DAC");
    QStringList l, c;
    l << "1" << "1mag" << "2" << "2mag" << "3" << "4" << "5" << "1hr" << "2hr";
    c << "1/DFL" << "1-MAG" << "2/LFL/I_TIP" << "2-MAG" << "3/INT" << "4" << "5" << "AUX 1" << "AUX 2";
    addConfig(settings, "analog.conf", "adc/", smin + sadc);
    for (int i = 0; i < l.size() && i < c.size(); i++) {
        addConfig(settings, "analog.conf", "adc/v" + l.at(i) + "_min", -10, c.at(i));
        addConfig(settings, "analog.conf", "adc/v" + l.at(i) + "_max", 10, c.at(i));
    }

    addConfig(settings, "analog.conf", "dac/", smin + sdac);
    addConfig(settings, "analog.conf", "dac/vdrive_min", 0, "* " + tr("Drive"));
    addConfig(settings, "analog.conf", "dac/vdrive_max", 10, "* " + tr("Drive"));

    l.clear(); c.clear();
    l << "tip" << "1" << "2" << "3hr" << "4hr";
    c << tr("Tip") << "1" << "2" << "AUX 1" << "AUX 2";
    for (int i = 0; i < l.size() && i < c.size(); i++) {
        addConfig(settings, "analog.conf", "dac/v" + l.at(i) + "_min", -10, "* " + c.at(i));
        addConfig(settings, "analog.conf", "dac/v" + l.at(i) + "_max", 10, "* " + c.at(i));
    }
    }

    { QSettings settings("Param/tipMonitor.conf", QSettings::IniFormat);
        addConfig(settings, "tipMonitor.conf", "/", tr("Tip monitor settings"));
        addConfig(settings, "tipMonitor.conf", "max_tip_drive_amplitude", 3.0, tr("Maximum tip drive amplitude"));
        addConfig(settings, "tipMonitor.conf", "auto_land_free_amplitude", 2.0, tr("Auto landing free amplitude"));
        addConfig(settings, "tipMonitor.conf", "auto_land_step_size", 0.1, tr("Auto landing step size"));
        addConfig(settings, "tipMonitor.conf", "auto_land_step_wait_ms", 50, tr("Auto landing wait step ms"));
    }

    //editor->header()->resizeSections(QHeaderView::ResizeToContents);

    connect(editor, SIGNAL(itemChanged(QTreeWidgetItem*,int)), this, SLOT(configEdited(QTreeWidgetItem*,int)));
    connect(editor, SIGNAL(collapsed(QModelIndex)), this, SLOT(editorResize()));
    connect(editor, SIGNAL(expanded(QModelIndex)), this, SLOT(editorResize()));

    //connect(Bedit, SIGNAL(clicked()), configWidget, SLOT(show()));
    //connect(Bedit, SIGNAL(clicked()), Bok, SLOT(click()));



    Show();

    connect(Ctranslation, SIGNAL(currentIndexChanged(int)), this, SLOT(Apply()));
    connect(szbox, SIGNAL(newValue()), this, SLOT(Apply()));
    connect(as_time, SIGNAL(newValue()), this, SLOT(Apply()));
    connect(chUseSound, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chAutoSendToData, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(CBgwy_imp, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(CBremote_net, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(CBremote_log, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(CBremote_guest, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(CBremote_pwd, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(remote_port, SIGNAL(newValue()), this, SLOT(Apply()));
    connect(remote_pwd, SIGNAL(textChanged(const QString &)), this, SLOT(Apply()));
    connect(chDataModule, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chsSpectroscopyModule, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chScanModule, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chSpmModule, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chOscilloscopeModule, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chVideoModule1, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chVideoModule2, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chLithographyModule, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chAboutModule, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chShuttleStatusModule, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chHeadScannerModule, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chStageScannerModule, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chZ3ScannerModule, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chRatisXYZStageMode, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chShowBwChan, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chShowFz, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chShow4D, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chShowLiftMod, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chShowContScan, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chShowDataRightMenu, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chUnlockToolbars, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chOpticInitOnStart, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chHideDidntFindedSensorsErrorMessages, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chIgnoreScanSetPointError, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chShowSecondChannelTab, SIGNAL(toggled(bool)), this, SLOT(Apply()));
#ifdef _OLD_EXTRA_CONTROL
    connect(chShowExtraControl, SIGNAL(toggled(bool)), this, SLOT(Apply()));
#endif
    connect(chShowDeviceControl, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(edRectSliderMouseWheelPower, SIGNAL(newValue()), this, SLOT(Apply()));
    #ifdef _SCAN_SLICE
    connect(edScanZoomSlice, SIGNAL(newValue()), this, SLOT(Apply()));
    connect(chEnableScanSlice, SIGNAL(toggled(bool)), this, SLOT(Apply()));
#endif
    connect(chShowScanSlider, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chShowScanRightMenu, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chShowSpectrRightMenu, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(chMoveXYZToZeroOff, SIGNAL(toggled(bool)), this, SLOT(Apply()));
    connect(edZeroWaitPerChannel, SIGNAL(newValue()), this, SLOT(Apply()));
}

void TOptionsWindow::initGeneralGUI()
{
    QSettings settings2("Param/gui.conf", QSettings::IniFormat);
    bool ratisMode = settings2.value("gui/Ratis_xyz_stage_mode", false).toBool();
    QWidget *optWid1 = new QWidget();
    QVBoxLayout *Mainlayout = new QVBoxLayout(optWid1);
    QHBoxLayout *Mainlayout2 = new QHBoxLayout();
    Mainlayout->addLayout(Mainlayout2, 1);
    Mainlayout->addWidget(new QLabel("* - " + tr("Requires program restart")), 0, Qt::AlignBottom);
    QVBoxLayout *plotsLay = new QVBoxLayout();
    Mainlayout2->addLayout(plotsLay, 1);

    if (!ratisMode)
        optionsParent->addNewTab(optWid1, tr("General"), 0);

//----Environment settings-------

    QPushButton *Bfont = new QPushButton(tr("Select font"));
    QPushButton *Bdata = new QPushButton(tr("Select"));
    connect(Bfont, SIGNAL(clicked()), this, SLOT(BFont()));
    connect(Bdata, SIGNAL(clicked()), this, SLOT(BData()));
    szbox = new Edit(10, 500);
    szbox->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
    as_time = new Edit(0, 500);
    as_time->setSpecialValueText(tr("Disabled"));
    as_time->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
    chUseSound = new QCheckBox();
    chAutoSendToData = new QCheckBox();

    Ctranslation = new QComboBox();
    Ctranslation->addItem(tr("English"), QString(":/translation_base"));
    Ctranslation->addItem(tr("Russian"), QString(":/translation_russian"));
    Ctranslation->addItem(tr("Custom..."), QString("translation_russian.qm"));
    connect(Ctranslation, SIGNAL(activated(int)), this, SLOT(selectCustomTR()));

    auto *environmentSettingsGB = new QGroupBox(tr("Environment settings"));
    auto *layEnvironmentSettings = new QVBoxLayout(environmentSettingsGB);
    plotsLay->addWidget(environmentSettingsGB);
    QFormLayout *BoplayoutFormLay = makeFormLayout();

    layEnvironmentSettings->addLayout(BoplayoutFormLay);
    BoplayoutFormLay->setFieldGrowthPolicy(QFormLayout::ExpandingFieldsGrow);
    BoplayoutFormLay->setVerticalSpacing(10);
    BoplayoutFormLay->addRow(tr("Language *"), Ctranslation);
    BoplayoutFormLay->addRow(tr("Buttons size *"), szbox);
    BoplayoutFormLay->addRow(tr("Font *"), Bfont);
    BoplayoutFormLay->addRow(tr("Data directory"), Bdata);
    BoplayoutFormLay->addRow(tr("Data autosave time: "), as_time);
    BoplayoutFormLay->addRow(tr("Use sound notifications"), chUseSound);
    BoplayoutFormLay->addRow(tr("Enable Auto-send to Data from Scan"), chAutoSendToData);

//-------------------------------------

//--------2D plot

    QGroupBox *twoDPlotSettingsGB = new QGroupBox(tr("2D plot settings"));
    QVBoxLayout *twoDPlotSettingsLay = new QVBoxLayout(twoDPlotSettingsGB);
    QGridLayout *twoDPlotSettingsGL = new QGridLayout();
    twoDPlotSettingsLay->addLayout(twoDPlotSettingsGL);
    plotsLay->addWidget(twoDPlotSettingsGB, 0);

    chShowSpectrRightMenu = new QCheckBox(tr("Show spectroscopy right menu") + " *");
    twoDPlotSettingsLay->addWidget(chShowSpectrRightMenu, 0);

    //----------Filter 2D settings
    QGroupBox *Filter2DGB = new QGroupBox(tr("Filter 2D settings"));
    QGridLayout *gl2DFilters = new QGridLayout(Filter2DGB);
    twoDPlotSettingsLay->addWidget(Filter2DGB);

    const QString FitOrders2D[] = {tr("Fit. None"), tr("Fit. 0 order"), tr("Fit. 1 order"), tr("Fit. 2 order"),
                                   tr("Fit. 3 order"), tr("Fit. 4 order"), tr("Fit. 5 order"), tr("Manual fit")};
    cb2DFilter = new QComboBox();

    for (auto &fitOrder:FitOrders2D)
    {
        cb2DFilter->addItem(fitOrder);
    }
    gl2DFilters->addWidget(new QLabel(tr("Filter")), 0, 0);
    gl2DFilters->addWidget(cb2DFilter, 0, 1);

    //loading filter settings
    {
        QSettings settings("Param/scan.conf", QSettings::IniFormat);
        settings.beginGroup("2D_plot_filter");
        cb2DFilter->setCurrentIndex(settings.value("filter", 0).toInt());
        settings.endGroup();
    }
    connect(cb2DFilter, SIGNAL(currentIndexChanged(int)), this, SLOT(saveScan2DFilterSettings()));
    //---------

//----------------------------------------

//----3D Plot settings
    QGroupBox *plotSettingsGB = new QGroupBox(tr("3D plot settings"));
    QVBoxLayout *plotSettingsLay = new QVBoxLayout(plotSettingsGB);
    plotsLay->addWidget(plotSettingsGB);

    chShowScanRightMenu = new QCheckBox(tr("Show scan right menu") + " *");
    chShowScanSlider = new QCheckBox(tr("Show scan slider") + " *");

    #ifdef _SCAN_SLICE
    edScanZoomSlice = new Edit(1, 99);
    edScanZoomSlice->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
    chEnableScanSlice = new QCheckBox(tr("Enable scan bounds slice"));

    QHBoxLayout *scanSliceLay = new QHBoxLayout();
    scanSliceLay->addWidget(new QLabel(tr("Scan bounds slice %")));
    scanSliceLay->addWidget(edScanZoomSlice);
#endif

    plotSettingsLay->addWidget(chShowScanRightMenu, 1);
    plotSettingsLay->addWidget(chShowScanSlider, 1);
    #ifdef _SCAN_SLICE
    plotSettingsLay->addWidget(chEnableScanSlice, 1);
    plotSettingsLay->addLayout(scanSliceLay, 1);
#endif

    //--------- 3D filters
    QGroupBox *FilterGB = new QGroupBox(tr("Filter 3D settings"));
    QGridLayout *glFilters = new QGridLayout(FilterGB);
    plotSettingsLay->addWidget(FilterGB, 1);

    cbByDefEnableFilters = new QCheckBox();
    chFitLines = new QCheckBox();
    cbSubstract = new QComboBox();
    cbFitting = new QComboBox();
    chMeanValue = new QCheckBox();
    chMedianValue = new QCheckBox();
    chGaussian = new QCheckBox();
    chHistogramEqualization = new QCheckBox();
    edMeanValue = new Edit(2, 30);
    edMedianValue = new Edit(2, 30);
    edGaussian = new Edit(2, 30);
    
    edMeanValue->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
    edMedianValue->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
    edGaussian->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);

    {
        QSettings settings("Param/scan.conf", QSettings::IniFormat);
        settings.beginGroup("all_plot_filter");
        cbByDefEnableFilters->setChecked(settings.value("enable_by_def", false).toBool());
        settings.endGroup();
    }
    connect(cbByDefEnableFilters, SIGNAL(toggled(bool)), this, SLOT(saveScanAllFilterSettings()));

    const QString FitOrders[] = {tr("None"), tr("0 order"), tr("1st order"), tr("2nd order"), tr("3rd order"), tr("4th order"), tr("5th order")};
    for (auto &fitOrder:FitOrders)
    {
        cbSubstract->addItem(fitOrder);
        cbFitting->addItem(fitOrder);
    }

    QHBoxLayout *meanLay = new QHBoxLayout();
    QHBoxLayout *medianLay = new QHBoxLayout();
    QHBoxLayout *gaussianLay = new QHBoxLayout();

    meanLay->addWidget(chMeanValue);
    meanLay->addWidget(edMeanValue);

    medianLay->addWidget(chMedianValue);
    medianLay->addWidget(edMedianValue);

    gaussianLay->addWidget(chGaussian);
    gaussianLay->addWidget(edGaussian);

    glFilters->addWidget(new QLabel(tr("Turn on by default scan filters")), 0, 0);
    glFilters->addWidget(new QLabel(tr("Fit lines")), 1, 0);
    glFilters->addWidget(new QLabel(tr("Substract")), 2, 0);
    glFilters->addWidget(new QLabel(tr("Fitting")), 3, 0);
    glFilters->addWidget(new QLabel(tr("Mean value")), 4, 0);
    glFilters->addWidget(new QLabel(tr("Median value")), 5, 0);
    glFilters->addWidget(new QLabel(tr("Gaussian")), 6, 0);
    glFilters->addWidget(new QLabel(tr("Histogram equalization")), 7, 0);
    glFilters->addWidget(cbByDefEnableFilters, 0, 1);
    glFilters->addWidget(chFitLines, 1, 1);
    glFilters->addWidget(cbSubstract, 2, 1);
    glFilters->addWidget(cbFitting, 3, 1);
    glFilters->addLayout(meanLay, 4, 1);
    glFilters->addLayout(medianLay, 5, 1);
    glFilters->addLayout(gaussianLay, 6, 1);
    glFilters->addWidget(chHistogramEqualization, 7, 1);

    //loading filter settings
    {
        QSettings settings("Param/scan.conf", QSettings::IniFormat);
        settings.beginGroup("3D_plot_filter");
        chFitLines->setChecked(settings.value("fit_lines", false).toBool());
        cbSubstract->setCurrentIndex(settings.value("substract", 0).toInt());
        cbFitting->setCurrentIndex(settings.value("fitting", 0).toInt());
        chMeanValue->setChecked(settings.value("mean_value", false).toBool());
        chMedianValue->setChecked(settings.value("median_value", false).toBool());
        chGaussian->setChecked(settings.value("gaussian", false).toBool());
        chHistogramEqualization->setChecked(settings.value("histogam_equalization", false).toBool());
        edMeanValue->setValue(settings.value("ed_mean_value", 2).toInt());
        edMedianValue->setValue(settings.value("ed_median_value", 2).toInt());
        edGaussian->setValue(settings.value("ed_gaussian", 2).toInt());
        settings.endGroup();
    }

    connect(chFitLines, SIGNAL(toggled(bool)), this, SLOT(saveScanFilterSettings()));
    connect(cbSubstract, SIGNAL(currentIndexChanged(int)), this, SLOT(saveScanFilterSettings()));
    connect(cbFitting, SIGNAL(currentIndexChanged(int)), this, SLOT(saveScanFilterSettings()));
    connect(chMeanValue, SIGNAL(toggled(bool)), this, SLOT(saveScanFilterSettings()));
    connect(chMedianValue, SIGNAL(toggled(bool)), this, SLOT(saveScanFilterSettings()));
    connect(chGaussian, SIGNAL(toggled(bool)), this, SLOT(saveScanFilterSettings()));
    connect(chHistogramEqualization, SIGNAL(toggled(bool)), this, SLOT(saveScanFilterSettings()));
    connect(edMeanValue, SIGNAL(newValue()), this, SLOT(saveScanFilterSettings()));
    connect(edMedianValue, SIGNAL(newValue()), this, SLOT(saveScanFilterSettings()));
    connect(edGaussian, SIGNAL(newValue()), this, SLOT(saveScanFilterSettings()));
    //---------
//----------------------------------------

    QVBoxLayout *secondLay = new QVBoxLayout();
    Mainlayout2->addLayout(secondLay, 1);

//------FUnctionality

    chOpticInitOnStart = new QCheckBox(tr("Enable Optics initialization on start"));

    chUnlockToolbars = new QCheckBox(tr("Unlock toolbars"));
    chUnlockToolbars->hide();
    {
        QSettings settings("Param/gui.conf", QSettings::IniFormat);
        if (!settings.value("display/spectroscopy", false).toBool())
            chOpticInitOnStart->hide();
    }
    chShowBwChan = new QCheckBox(tr("Show Bw channel") + " *");
    chShowFz = new QCheckBox(tr("Show F[z]") + " *");
    chShow4D = new QCheckBox(tr("Show 4D") + " *");
    chShowLiftMod = new QCheckBox(tr("Show lift mod") + " *");
    chShowContScan = new QCheckBox(tr("Show continues scan") + " *");
    chShowDataRightMenu = new QCheckBox(tr("Show data right menu") + " *");
    chHideDidntFindedSensorsErrorMessages = new QCheckBox(tr("Hide didn't finded sensors error messages"));
    chIgnoreScanSetPointError = new QCheckBox(tr("Ignore \'Didn't finded setpoint\' errors"));

    edRectSliderMouseWheelPower = new Edit(1, 1000);
    edRectSliderMouseWheelPower->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
    QGridLayout *glOtherScan = new QGridLayout();
    glOtherScan->addWidget(new QLabel(tr("rect slider mouse wheel power(%)")), 0, 0);
    glOtherScan->addWidget(edRectSliderMouseWheelPower, 0, 1);
    {
        QSettings settings("Param/scan.conf", QSettings::IniFormat);
        edRectSliderMouseWheelPower->setValue(settings.value("RectSliderMouseWheelPower", 35).toInt());

        RectSlider::setWheelPower(edRectSliderMouseWheelPower->value()/100.0);
    }
    connect(edRectSliderMouseWheelPower, SIGNAL(newValue(int)), this, SLOT(saveRectSliderMouseWheelPower(int)));

    chShowSecondChannelTab = new QCheckBox(tr("Show second channel tab") + " *");
    #ifdef _OLD_EXTRA_CONTROL
    chShowExtraControl = new QCheckBox(tr("Show extra control") + " *");
#endif
    chShowDeviceControl = new QCheckBox(tr("Show device control") + " *");

    QGroupBox *functionalityGB = new QGroupBox(tr("Functionality"));
    QVBoxLayout *functionalityLay = new QVBoxLayout(functionalityGB);
    secondLay->addWidget(functionalityGB, 1);

    functionalityLay->addWidget(chShowBwChan, 1, Qt::AlignVCenter);
    functionalityLay->addWidget(chShowFz, 1, Qt::AlignVCenter);
    functionalityLay->addWidget(chShow4D, 1, Qt::AlignVCenter);
    functionalityLay->addWidget(chShowLiftMod, 1, Qt::AlignVCenter);
    functionalityLay->addWidget(chShowContScan, 1, Qt::AlignVCenter);
    functionalityLay->addWidget(chShowDataRightMenu, 1, Qt::AlignVCenter);
    functionalityLay->addWidget(chUnlockToolbars, 1, Qt::AlignVCenter);
    functionalityLay->addWidget(chOpticInitOnStart, 1, Qt::AlignVCenter);
    functionalityLay->addWidget(chHideDidntFindedSensorsErrorMessages, 1, Qt::AlignVCenter);
    functionalityLay->addWidget(chIgnoreScanSetPointError, 1, Qt::AlignVCenter);
    functionalityLay->addWidget(chShowSecondChannelTab, 1, Qt::AlignVCenter);
    #ifdef _OLD_EXTRA_CONTROL
    functionalityLay->addWidget(chShowExtraControl, 1, Qt::AlignVCenter);
#endif
    functionalityLay->addWidget(chShowDeviceControl, 1, Qt::AlignVCenter);
    functionalityLay->addLayout(glOtherScan, 1);

//--------------------------------

//------Network

    CBgwy_imp = new QCheckBox(tr("Enable Gwyddion import server") + " *");
    CBremote_net = new QCheckBox(tr("Enable Nspec Light server") + " *");
    CBremote_log = new QCheckBox(tr("Enable Nspec Light access logging"));
    CBremote_guest = new QCheckBox(tr("Nspec Light is read-only"));
    CBremote_pwd = new QCheckBox(tr("Nspec Light needs password"));
    remote_pwd = new QLineEdit;
    remote_pwd->setEchoMode(QLineEdit::PasswordEchoOnEdit);
    remote_pwd->setPlaceholderText(tr("empty"));
    remote_pwd->setFixedWidth(140);
    remote_port = new Edit(1, 65535);
    remote_port->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);

    QGroupBox *NetworkGB = new QGroupBox(tr("Network"));
    QVBoxLayout *networkLay = new QVBoxLayout();
    QFormLayout *networkFormLay = makeFormLayout();
    networkFormLay->addRow(tr("Nspec Light password"), remote_pwd);
    networkFormLay->addRow(tr("Nspec Light port *"), remote_port);
    NetworkGB->setLayout(networkLay);
    secondLay->addWidget(NetworkGB, 1);

    networkLay->addWidget(CBgwy_imp, 1, Qt::AlignVCenter);
    networkLay->addWidget(CBremote_net, 1, Qt::AlignVCenter);
    networkLay->addWidget(CBremote_log, 1, Qt::AlignVCenter);
    networkLay->addWidget(CBremote_guest, 1, Qt::AlignVCenter);
    networkLay->addWidget(CBremote_pwd, 1, Qt::AlignVCenter);
    networkLay->addLayout(networkFormLay, 1);


//-------------------------

    QVBoxLayout *thirdLay = new QVBoxLayout();
    Mainlayout2->addLayout(thirdLay, 1);

//-----Modules------------


    chDataModule = new QCheckBox(tr("Data") + " *");
    chsSpectroscopyModule = new QCheckBox(tr("Spectroscopy") + " *");
    chScanModule = new QCheckBox(tr("Scan") + " *");
    chSpmModule = new QCheckBox(tr("SPM") + " *");
    chOscilloscopeModule = new QCheckBox(tr("Oscilloscope") + " *");
    chLithographyModule = new QCheckBox(tr("Lithography") + " *");
    chVideoModule1 = new QCheckBox(tr("Video") + " *");
    chVideoModule2 = new QCheckBox(tr("Video") + " 2" " *");
    chAboutModule = new QCheckBox(tr("About button") + " *");
    chShuttleStatusModule = new QCheckBox(tr("Shuttle Status") + " *");
    chHeadScannerModule = new QCheckBox(tr("Head scanner") + " *");
    chStageScannerModule = new QCheckBox(tr("Stage scanner") + " *");
    chZ3ScannerModule = new QCheckBox(tr("Z3 scanner") + " *");
    chRatisXYZStageMode = new QCheckBox(tr("Ratis XYZ stage mode") + " *");

    QGroupBox *ModulesGB = new QGroupBox(tr("Modules"));
    QVBoxLayout *Boplayout2 = new QVBoxLayout();
    ModulesGB->setLayout(Boplayout2);
    thirdLay->addWidget(ModulesGB, 1);

    Boplayout2->addWidget(chDataModule, 1, Qt::AlignVCenter);
    Boplayout2->addWidget(chsSpectroscopyModule, 1, Qt::AlignVCenter);
    Boplayout2->addWidget(chScanModule, 1, Qt::AlignVCenter);
    Boplayout2->addWidget(chSpmModule, 1, Qt::AlignVCenter);
    Boplayout2->addWidget(chOscilloscopeModule, 1, Qt::AlignVCenter);
    Boplayout2->addWidget(chVideoModule1, 1, Qt::AlignVCenter);
    Boplayout2->addWidget(chVideoModule2, 1, Qt::AlignVCenter);
    Boplayout2->addWidget(chLithographyModule, 1, Qt::AlignVCenter);
    Boplayout2->addWidget(chAboutModule, 1, Qt::AlignVCenter);
    Boplayout2->addWidget(chShuttleStatusModule, 1, Qt::AlignVCenter);
    Boplayout2->addWidget(chHeadScannerModule, 1, Qt::AlignVCenter);
    Boplayout2->addWidget(chStageScannerModule, 1, Qt::AlignVCenter);
    Boplayout2->addWidget(chZ3ScannerModule, 1, Qt::AlignVCenter);
    Boplayout2->addWidget(chRatisXYZStageMode, 1, Qt::AlignVCenter);

//----------------------------------

//-----sofware-of-functionality-----

    auto lMoveXYZToZeroOff = new QLabel(tr("Move X,Y,Z to 0"));
    chMoveXYZToZeroOff = new QCheckBox();

    auto lZeroWaitPerChannel = new QLabel(tr("Wait ms after change channel to zero"));
    edZeroWaitPerChannel = new Edit(1, 10000);
    edZeroWaitPerChannel->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);

    QGroupBox *softwareFuncGB = new QGroupBox(tr("Software of functionality"));
    QGridLayout *glSoftwareFunc = new QGridLayout(softwareFuncGB);
    thirdLay->addWidget(softwareFuncGB, 0);

    glSoftwareFunc->addWidget(lMoveXYZToZeroOff, 0, 0);
    glSoftwareFunc->addWidget(chMoveXYZToZeroOff, 0, 1);

    glSoftwareFunc->addWidget(lZeroWaitPerChannel, 1, 0);
    glSoftwareFunc->addWidget(edZeroWaitPerChannel, 1, 1);

//----------------------------------
}

void TOptionsWindow::addConfig(const QString &file, const QString &path, const QString &comment)
{
    addConfig(file, path, QVariant(), comment);
}
void TOptionsWindow::addConfig(const QString &file, const QString &path, const QVariant &default_val, const QString &comment)
{
    QSettings settings("Param/" + file, QSettings::IniFormat);
    addConfig(settings, file, path, default_val, comment);
}
void TOptionsWindow::addConfig(QSettings &settings, const QString &file, const QString &path, const QString &comment)
{
    addConfig(settings, file, path, QVariant(), comment);
}
void TOptionsWindow::addConfig(QSettings &settings, const QString &file, const QString &path, const QVariant &default_val, const QString &comment)
{
#ifdef NDEBUG
    //In debug empty key(path) causing an error
    QVariant value = settings.value(path, default_val);
#else
    QVariant value = (path == "/") ? QVariant() : settings.value(path, default_val);
#endif

    QStringList pathparts = path.split("/");
    QString lastpart = pathparts.last();
    pathparts.removeLast();
    pathparts.prepend(file);

    QTreeWidgetItem *root;
    QString acc;
    while (!pathparts.isEmpty()) {
        if (!items.contains(acc + pathparts.first())) {
            if (acc.isEmpty())
                root = new QTreeWidgetItem(editor, QStringList() << file);
            else
                root = new QTreeWidgetItem(root, QStringList() << pathparts.first());

            items.insert(acc + pathparts.first(), root);
        } else
            root = items.value(acc + pathparts.first());

        acc += pathparts.first();
        pathparts.removeFirst();
    }
    if (lastpart.isEmpty()) {
        root->setText(2, comment);
        return;
    }

    editor->setFont(qApp->font());
    QTreeWidgetItem *item = new QTreeWidgetItem(root, QStringList() << lastpart << value.toString() << comment);
    item->setData(0, Qt::UserRole, file);
    item->setData(0, Qt::UserRole+1, path);
    item->setData(0, Qt::UserRole+3, default_val);

    if (value.toString() == "false" || value.toString() == "true") {
        item->setCheckState(1, value.toBool() ? Qt::Checked : Qt::Unchecked);
        item->setData(1, Qt::DisplayRole, QVariant());
        item->setData(0, Qt::UserRole+2, true);
    } else
        item->setFlags(item->flags() | Qt::ItemIsEditable);
}

void TOptionsWindow::editorResize()
{
    editor->header()->resizeSections(QHeaderView::ResizeToContents);
}

void TOptionsWindow::configEdited(QTreeWidgetItem *item, int)
{
    QSettings settings("Param/" + item->data(0, Qt::UserRole).toString(), QSettings::IniFormat);
    QString path = item->data(0, Qt::UserRole+1).toString();
    QVariant default_val = item->data(0, Qt::UserRole+3);
    if (!item->data(0, Qt::UserRole+2).toBool()) { // !bool
        if (item->text(1).isEmpty() || item->text(1) == default_val.toString()) {
            item->setText(1, default_val.toString());
            settings.remove(path);
        } else
            settings.setValue(path, item->text(1));
    } else {
        if ((item->checkState(1) == Qt::Checked) == default_val.toBool())
            settings.remove(path);
        else
            settings.setValue(path, item->checkState(1) == Qt::Checked);
    }
}

void TOptionsWindow::BFont()
{
    bool ok;
    QFont font_tmp = QFontDialog::getFont(&ok, font, this);

    if (ok) {
        font = font_tmp;
        Apply(true);
    }
}
void TOptionsWindow::BData()
{
    QString newdir = QFileDialog::getExistingDirectory(this, tr("Default data directory"), datadir,
                                                       QFileDialog::ShowDirsOnly | QFileDialog::DontUseNativeDialog);

    if (!newdir.isEmpty())
    {
        datadir = newdir;
        Apply();
    }
}

bool checkSetConfig(QSettings &settings, const QString &name, const QVariant &def_v, const QVariant &new_v)
{
    if (settings.value(name, def_v) != new_v) {
        if (new_v == def_v)
            settings.remove(name);
        else
            settings.setValue(name, new_v);
        return true;
    }
    return false;
}

void TOptionsWindow::Apply(bool definitlyRestart)
{
    bool restart = definitlyRestart;

    QSettings settings("Param/gui.conf", QSettings::IniFormat);

    restart = checkSetConfig(settings, "net/gwy_import_srv", true, CBgwy_imp->isChecked()) || restart;
    restart = checkSetConfig(settings, "net/srv_on", false, CBremote_net->isChecked()) || restart;
    settings.setValue("net/srv_guest", CBremote_guest->isChecked());
    settings.setValue("net/srv_log", CBremote_log->isChecked());
    settings.setValue("net/srv_pwd_on", CBremote_pwd->isChecked());

    settings.setValue("net/srv_pwd", remote_pwd->text());
    restart = checkSetConfig(settings, "net/srv_port", 33331, remote_port->value()) || restart;

    restart = checkSetConfig(settings, "gui/Ratis_xyz_stage_mode", false, chRatisXYZStageMode->isChecked()) || restart;
    if (chRatisXYZStageMode->isChecked())
    {
        chDataModule->setChecked(false);
        chsSpectroscopyModule->setChecked(false);
        chSpmModule->setChecked(false);
        chVideoModule1->setChecked(false);
        chVideoModule2->setChecked(false);
        chLithographyModule->setChecked(false);
        chAboutModule->setChecked(false);
        chShuttleStatusModule->setChecked(false);
        chZ3ScannerModule->setChecked(false);
        chHeadScannerModule->setChecked(false);
    }

    //TODO table/loop
    restart = checkSetConfig(settings, "display/data", true, chDataModule->isChecked()) || restart;
    restart = checkSetConfig(settings, "display/spectroscopy", false, chsSpectroscopyModule->isChecked()) || restart;
    restart = checkSetConfig(settings, "display/scan", true, chScanModule->isChecked()) || restart;
    restart = checkSetConfig(settings, "display/spm", true, chSpmModule->isChecked()) || restart;
    restart = checkSetConfig(settings, "display/oscilloscope", true, chOscilloscopeModule->isChecked()) || restart;
    restart = checkSetConfig(settings, "display/video", true, chVideoModule1->isChecked()) || restart;
    restart = checkSetConfig(settings, "display/video2", false, chVideoModule2->isChecked()) || restart;
    restart = checkSetConfig(settings, "display/lithography", false, chLithographyModule->isChecked()) || restart;

    restart = checkSetConfig(settings, "display/oscilloscope/head", true, chHeadScannerModule->isChecked()) || restart;
    restart = checkSetConfig(settings, "display/oscilloscope/stage", true, chStageScannerModule->isChecked()) || restart;
    restart = checkSetConfig(settings, "display/oscilloscope/z3", false, chZ3ScannerModule->isChecked()) || restart;
    restart = checkSetConfig(settings, "display/about", true, chAboutModule->isChecked()) || restart;
    restart = checkSetConfig(settings, "display/shuttle_status", true, chShuttleStatusModule->isChecked()) || restart;

    restart = checkSetConfig(settings, "gui/show_bw_chan", true, chShowBwChan->isChecked()) || restart;
    restart = checkSetConfig(settings, "gui/show_fz", true, chShowFz->isChecked()) || restart;
    restart = checkSetConfig(settings, "gui/show_4d", true, chShow4D->isChecked()) || restart;
    restart = checkSetConfig(settings, "gui/show_lift_mod", true, chShowLiftMod->isChecked()) || restart;
    restart = checkSetConfig(settings, "gui/show_cont_scan", true, chShowContScan->isChecked()) || restart;
    restart = checkSetConfig(settings, "gui/show_second_chan", true, chShowSecondChannelTab->isChecked()) || restart;
    #ifdef _OLD_EXTRA_CONTROL
    restart = checkSetConfig(settings, "gui/show_extra_control", true, chShowExtraControl->isChecked()) || restart;
#endif
    restart = checkSetConfig(settings, "gui/show_device_control", true, chShowDeviceControl->isChecked()) || restart;
    restart = checkSetConfig(settings, "gui/show_scan_slider", true, chShowScanSlider->isChecked()) || restart;
    restart = checkSetConfig(settings, "gui/show_scan_right_menu", true, chShowScanRightMenu->isChecked()) || restart;
    restart = checkSetConfig(settings, "gui/show_data_right_menu", true, chShowDataRightMenu->isChecked()) || restart;
    restart = checkSetConfig(settings, "gui/show_spectr_right_menu", true, chShowSpectrRightMenu->isChecked()) || restart;

    settings.setValue("scan/autosend_data", chAutoSendToData->isChecked());
    settings.setValue("autosave_time", as_time->value());

    settings.setValue("autoinit_ccd", chOpticInitOnStart->isChecked());

    settings.setValue("hide_didnt_finded_sensors_error_messages", chHideDidntFindedSensorsErrorMessages->isChecked());
    settings.setValue("ignore_scan_setpoint_errors", chIgnoreScanSetPointError->isChecked());

    #ifdef _SCAN_SLICE
    settings.setValue("scan_zoom_slice_enabled", chEnableScanSlice->isChecked());
    settings.setValue("scan_zoom_slice", edScanZoomSlice->value());
#endif

    settings.setValue("firmware/setXYZto0", chMoveXYZToZeroOff->isChecked());
    settings.setValue("firmware/waitXYZto0", edZeroWaitPerChannel->value());

    //TODO reloadConfig() signal
    if (auto scan = ScanPlot::GetIt()) {
        scan->Autosend = chAutoSendToData->isChecked();

        //scan->scan_slice();
    }
    if (auto spm = SPMTunning::GetIt()) {
        spm->AutoSend = chAutoSendToData->isChecked();
    }
    if (auto ccd = CCDPlot::GetIt()) {
        ccd->AutoSend = chAutoSendToData->isChecked();
    }
    if (auto data = DataPlot::GetIt()) {
        data->Container->SetAutoSaveTime(as_time->value()*60*1000);
    }

    if (checkSetConfig(settings, "gui_unlock", false, chUnlockToolbars->isChecked()))
    {
        auto mainWindow = MainWindow::GetIt();
        foreach (QToolBar *t, mainWindow->findChildren<QToolBar *>())
            t->setMovable(settings.value("gui_unlock", false).toBool());
    }
    settings.setValue("sound_notification", chUseSound->isChecked());

    if (font != qApp->font()) {
        settings.setValue("font", font.toString());
        restart = true;
    }
    if (datadir != settings.value("data_dir", TContainer::getDataDir()).toString())
        settings.setValue("data_dir", datadir);


    if (Ctranslation->itemData(Ctranslation->currentIndex()) !=
            settings.value("translation", QString(":/translation_base")).toString())
    {
        settings.setValue("translation", Ctranslation->itemData(Ctranslation->currentIndex()));
        restart = true;
    }

    int Bsize = szbox->value();
    if (Bsize != qApp->property("buttonsize").toInt()) {
        settings.setValue("buttonsize", Bsize);
        restart = true;
    }

    if (restart) {
        new Notification(tr("Settings") + ": " + tr("Please restart application to apply changes."), Notification::Good, 10);
        doBeep();
    }

    hide();
}

void TOptionsWindow::Show()
{
    QSettings settings("Param/gui.conf", QSettings::IniFormat);

    CBgwy_imp->setChecked(settings.value("net/gwy_import_srv", true).toBool());
    CBremote_net->setChecked(settings.value("net/srv_on", false).toBool());
    CBremote_guest->setChecked(settings.value("net/srv_guest", true).toBool());
    CBremote_log->setChecked(settings.value("net/srv_log", true).toBool());
    CBremote_pwd->setChecked(settings.value("net/srv_pwd_on", false).toBool());

    remote_pwd->setText(settings.value("net/srv_pwd", "").toString());
    remote_port->setValue(settings.value("net/srv_port", 33331).toInt());

    chDataModule->setChecked(settings.value("display/data", true).toBool());
    chsSpectroscopyModule->setChecked(settings.value("display/spectroscopy", false).toBool());
    chScanModule->setChecked(settings.value("display/scan", true).toBool());
    chSpmModule->setChecked(settings.value("display/spm", true).toBool());
    chOscilloscopeModule->setChecked(settings.value("display/oscilloscope", true).toBool());
    chVideoModule1->setChecked(settings.value("display/video", true).toBool());
    chVideoModule2->setChecked(settings.value("display/video2", false).toBool());
    chLithographyModule->setChecked(settings.value("display/lithography", false).toBool());

    chHeadScannerModule->setChecked(settings.value("display/oscilloscope/head", true).toBool());
    chStageScannerModule->setChecked(settings.value("display/oscilloscope/stage", true).toBool());
    chZ3ScannerModule->setChecked(settings.value("display/oscilloscope/z3", false).toBool());
    chAboutModule->setChecked(settings.value("display/about", true).toBool());
    chShuttleStatusModule->setChecked(settings.value("display/shuttle_status", true).toBool());

    chAutoSendToData->setChecked(settings.value("scan/autosend_data", true).toBool());
    as_time->setValue(settings.value("autosave_time", 30).toInt());

    chOpticInitOnStart->setChecked(settings.value("autoinit_ccd", false).toBool());

    chUseSound->setChecked(settings.value("sound_notification", true).toBool());
    chUnlockToolbars->setChecked(true);

    chShowBwChan->setChecked(settings.value("gui/show_bw_chan", true).toBool());
    chShowFz->setChecked(settings.value("gui/show_fz", true).toBool());
    chShow4D->setChecked(settings.value("gui/show_4d", true).toBool());
    chShowLiftMod->setChecked(settings.value("gui/show_lift_mod", true).toBool());
    chShowContScan->setChecked(settings.value("gui/show_cont_scan", true).toBool());
    chShowSecondChannelTab->setChecked(settings.value("gui/show_second_chan", true).toBool());
    #ifdef _OLD_EXTRA_CONTROL
    chShowExtraControl->setChecked(settings.value("gui/show_extra_control", true).toBool());
#endif
    chShowDeviceControl->setChecked(settings.value("gui/show_device_control", true).toBool());
    chShowScanSlider->setChecked(settings.value("gui/show_scan_slider", true).toBool());
    chShowScanRightMenu->setChecked(settings.value("gui/show_scan_right_menu", true).toBool());
    chShowDataRightMenu->setChecked(settings.value("gui/show_data_right_menu", true).toBool());
    chShowSpectrRightMenu->setChecked(settings.value("gui/show_spectr_right_menu", true).toBool());
    chRatisXYZStageMode->setChecked(settings.value("gui/Ratis_xyz_stage_mode", false).toBool());
    chHideDidntFindedSensorsErrorMessages->setChecked(settings.value("hide_didnt_finded_sensors_error_messages", false).toBool());
    chIgnoreScanSetPointError->setChecked(settings.value("ignore_scan_setpoint_errors", false).toBool());
    chMoveXYZToZeroOff->setChecked(settings.value("firmware/setXYZto0", false).toBool());
    #ifdef _SCAN_SLICE
    edScanZoomSlice->setValue(settings.value("scan_zoom_slice", 10).toDouble());
    chEnableScanSlice->setChecked(settings.value("scan_zoom_slice_enabled", false).toBool());
#endif
    edZeroWaitPerChannel->setValue(settings.value("firmware/waitXYZto0", 200).toInt());

    font = qApp->font();
    datadir = settings.value("data_dir", TContainer::getDataDir()).toString();

    szbox->setValue(qApp->property("buttonsize").toInt());

    QString transl = qApp->property("translation").toString();

    if (transl.endsWith(":/translation_base"))
        Ctranslation->setCurrentIndex(0);
    else if (transl.endsWith(":/translation_russian"))
        Ctranslation->setCurrentIndex(1);
    else {
        Ctranslation->setCurrentIndex(2);
        Ctranslation->setItemData(2, transl);
    }
}
