#include "scanplot.h"

#include "qwt_plot_layout.h"
#include "widgets/threedoverlaybuttons.h"
#include "widgets/twodoverlaybuttons.h"
#include <QApplication>
#include <qwt_thermo.h>
#include <qwt_plot_marker.h>
#include <QLayout>
#include "remote_nst_dataid.hpp"
#include "gui/ttabwidget.hpp"
#include "flowlayout.hpp"
#include "oscilloscope.h"
#include <QScrollArea>
#include "mainwindow.h"
#include "options_window.h"

void ScanPlot::updateScan2DPlotVisible()
{
    bool showScan2DPlot = false;
    for (auto &sec : CBSec)
    {
        if (sec->isChecked())
        {
            showScan2DPlot = true;
            break;
        }
    }
    Plot->setScanSPM2DPlot(showScan2DPlot);
    if (showScan2DPlot)
        Plot->TwoDPlot->Refresh();
    
    update2DPlotScanMode();
}
void ScanPlot::update2DPlotScanMode()
{
    bool scanMode = !(CBChannel[1]->currentData() == srcCCD && 
                      CBSec[1]->isChecked() && 
                      CBExternalMode->currentIndex() == 1);
    if (scanMode != twodPlotFilterScanMode)
    {
        twodPlotFilterScanMode = scanMode;
        if (scanMode)
        {
            auto opt = options::GetIt();
            if (opt)
                opt->optWin->applyFilter2DSettings();
        }
        else
        {
            if (externalScans.count() > 0 && externalScans[0]->name == "ccdscan")
            {
                int filter = externalScans[0]->get2DPlotFilter();
                
                Plot->TwoDPlot->PlotControl->AGFitOrders->actions()[filter]->setChecked(true);
                Plot->TwoDPlot->PlotControl->AGFitOrders->actions()[filter]->trigger();
                
                Plot->TwoDPlotVertical->PlotControl->AGFitOrders->actions()[filter]->setChecked(true);
                Plot->TwoDPlotVertical->PlotControl->AGFitOrders->actions()[filter]->trigger();
    
                Plot->TwoDPlot->Refresh();
                Plot->TwoDPlotVertical->Refresh();
            }
        }
    }
}
void ScanPlot::spectrButtonStateChanged(bool offOn)
{
    if (offOn)
    {
        for (int i = 0; i < numScanChannel; i++)
            CBSec[i]->setChecked(false);
    }
}

void ScanPlot::fontSize_changed()
{
    int i = AGFontSizes->actions().indexOf(AGFontSizes->checkedAction());
    Plot->PlotControl->AGFontSizes->actions()[i]->setChecked(true);
    Plot->on_FontSize_changed();
}

void ScanPlot::plotFontSize_changed()
{
    int i = Plot->PlotControl->AGFontSizes->actions().indexOf(Plot->PlotControl->AGFontSizes->checkedAction());
    AGFontSizes->actions()[i]->setChecked(true);
}

#ifdef _OLD_EXTRA_CONTROL
void ScanPlot::extraControlTabCreated()
{
    QScrollArea *scrollArea = new QScrollArea(this);
    scrollArea->setWidgetResizable(true);

    QWidget *extraControlTab = new QWidget();
    scrollArea->setWidget(extraControlTab);
    QVBoxLayout *extraControlTabLay = new QVBoxLayout(extraControlTab);

    QSettings settingsGui("Param/gui.conf", QSettings::IniFormat);
    if (!settingsGui.value("gui/show_extra_control", true).toBool())
        scrollArea->hide();
    else
        Plot->mainTab->addTab(scrollArea, tr("Extra control"));

    QGroupBox *gbDelays = new QGroupBox(tr("Delays"));
    QGroupBox *gbSync = new QGroupBox(tr("Sync"));

    QHBoxLayout *layDelays = new QHBoxLayout(gbDelays);
    QHBoxLayout *laySync = new QHBoxLayout(gbSync);

    layDelays->addWidget(BoxDelays);
    laySync->addWidget(BoxSync);

    extraControlTabLay->addWidget(gbDelays);
    extraControlTabLay->addWidget(gbSync);
    extraControlTabLay->addWidget(scanTabGB);
}
#endif

void ScanPlot::onSaveImage_clicked()
{
    Plot->overlayButtons->blockShowing3DPlotButtons = true;
    videoChannelSetVisibleGUI(videoChannelId, false);
    Plot->on_BSavePlotPixmap_clicked();
    Plot->overlayButtons->blockShowing3DPlotButtons = false;
    videoChannelSetVisibleGUI(videoChannelId, true);
}

void ScanPlot::onSendToData_clicked()
{
    int n = Plot->ThreeDPlots.indexOf((plot::TPure3dPlot*)Plot->overlayButtons->BBackButton->parent());
    Plot->SetActivePlot(n);
    Plot->on_BSave_clicked();
}
void ScanPlot::SignalsTabChanged(int tabId)
{
    //bool visible = (tabId != pos4DTab && tabId != posFzTab);
    //plotChooser->setVisible(visible);
//    if (tabId == posFzTab && fzWidget->layout() == nullptr)
//    {
//        fzWidget->setLayout(fzHMainLayout);
//    }
    resizeEvent();
}
void ScanPlot::splitterMoved(int /*pos*/, int /*index*/)
{
    if (Plot->mainTab->currentIndex() != 3)
    {
        QSettings settings("Param/gui.conf", QSettings::IniFormat);
        settings.setValue("scan/height_3dplot", Plot->SplitPlotTab->sizes().first());
        settings.setValue("scan/height_2dplot", Plot->SplitPlotTab->sizes().last());
    }
}
void ScanPlot::currentLineFontSettingsChanged(int)
{
    int lineSelectedId = Plot->TwoDPlot->overlayButtons->CBLineWidth->currentIndex();
    int lineWidth = Plot->TwoDPlot->PlotControl->AGCurveThickness->actions().at(lineSelectedId)->text().toInt();
    int fontSelectedId = Plot->TwoDPlot->overlayButtons->CBFont->currentIndex();
    int fontSize = Plot->TwoDPlot->PlotControl->AGFontSizes->actions().at(fontSelectedId)->text().toInt();

    QList<QVariant> list;
    list << QVariant::fromValue(lastSelectedLineColor)
         << lineWidth
         << fontSize;

    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
        mob->setPropIdVar(DataID::plot2DLineFontSettings, list);
}

void ScanPlot::currentLineColorChanged(QColor color, int)
{
    Plot->TwoDPlot->Pure2dPlot->SetAutoScaleEnabled(m_BStart->on_off, false);
    Plot->TwoDPlotVertical->Pure2dPlot->SetAutoScaleEnabled(m_BStart->on_off, false);
    lastSelectedLineColor = color;
    currentLineFontSettingsChanged(0);
}
void ScanPlot::doubleClickedLaserSpotWindow()
{
    if (isLaserSpotWindowIncreased)
    {
        mainLay->removeWidget(LaserSpotWindow);
        spmMonitorGBHlay->addWidget(LaserSpotWindow, 1);
        for (int i = 0; i < mainLay->count(); ++i)
        {
            QWidget *widget = mainLay->itemAt(i)->widget();
            if (widget != nullptr)
            {
                widget->setVisible(true);
            }
        }
    }
    else
    {
        for (int i = 0; i < mainLay->count(); ++i)
        {
            QWidget *widget = mainLay->itemAt(i)->widget();
            if (widget != nullptr)
            {
                widget->setVisible(false);
            }
        }
        spmMonitorGBHlay->removeWidget(LaserSpotWindow);
        mainLay->addWidget(LaserSpotWindow, 1);
    }

    isLaserSpotWindowIncreased = !isLaserSpotWindowIncreased;
}
void ScanPlot::ShowRightMenu(bool show)
{
    if (show)
    {
//        gb4DButtons->setVisible(CBExternalMode->currentIndex() != 0);
//        show4dMenu = true;
//        int active = Plot->ActivePlot;
//        Plot->SetActivePlot(active == 0 ? 1:0);
//        Plot->SetActivePlot(active);
    }
    else
    {
        gb4DButtons->setVisible(false);
        show4dMenu = false;
        Plot->PlotControl->hide();
        Plot->TwoDPlot->PlotControl->hide();
        Plot->TwoDPlotVertical->PlotControl->hide();
    }
}

void ScanPlot::on4DSendToData_clicked()
{
    if (currExtScan == nullptr)
        return;

    currExtScan->save();
}
void ScanPlot::on4DClear_clicked()
{
    if (currExtScan == nullptr)
        return;

    currExtScan->clear();
}
void ScanPlot::on4DSelectCapturedSpectr_clicked(bool pressed)
{
    if (currExtScan == nullptr)
        return;

    currExtScan->sl_Spectrum_range_select(pressed);
}
void ScanPlot::on4DSelectConvolutionFunc_clicked(bool pressed)
{
    if (currExtScan == nullptr)
        return;

    currExtScan->sl_Func_range_select(pressed);
}
void ScanPlot::on4DSwitchSpectra_clicked(bool pressed)
{
    if (currExtScan == nullptr)
        return;

    currExtScan->setPointSpection(pressed);
}

void ScanPlot::ChannelResized(int n, QSize newSize)
{
    int leftX = Plot->ThreeDPlots[n]->QPlot->plotLayout()->scaleRect(QwtPlot::yLeft).width()*1.15;
    int upY = Plot->ThreeDPlots[n]->QPlot->plotLayout()->titleRect().height();
    int upYPerc = upY*0.25;
    upY += upYPerc;
    int videoBtnsSize = (newSize.width() - leftX)*0.2;
    if (videoBtnsSize <= 0) videoBtnsSize = 1;
    if (videoBtnsSize > 9999) videoBtnsSize = 9999;
    int cbSize = (newSize.height() - upY)*0.05*1.14;
    if (cbSize <= 0) cbSize = 1;
    if (cbSize > 9999) cbSize = 9999;
    CBPointsOX->setFixedSize(videoBtnsSize, cbSize);
    CBPointsOY->setFixedSize(videoBtnsSize, cbSize);
    LEdDeltaX->setFixedSize(videoBtnsSize, cbSize);
    LEdDeltaY->setFixedSize(videoBtnsSize, cbSize);

    auto font = QFont(LPointsOX->font().family(), cbSize*0.45);
    font.setBold(true);
    LPointsOX->setScaledContents(true);
    LPointsOX->setFont(font);
    LPointsOX->adjustSize();
    LPointsOY->setScaledContents(true);
    LPointsOY->setFont(font);
    LPointsOY->adjustSize();
    LDeltaX->setScaledContents(true);
    LDeltaX->setFont(font);
    LDeltaX->adjustSize();
    LDeltaY->setScaledContents(true);
    LDeltaY->setFont(font);
    LDeltaY->adjustSize();

    CBPointsOX->setFont(font);
    CBPointsOY->setFont(font);
    LEdDeltaX->setFont(font);
    LEdDeltaY->setFont(font);

    auto view = CBPointsOX->view();
    view->setFont(font);
    view = CBPointsOY->view();
    view->setFont(font);

    int downYH = Plot->ThreeDPlots[n]->QPlot->plotLayout()->scaleRect(QwtPlot::xBottom).height();

    int downY = newSize.height() - upYPerc -
            downYH - CBPointsOX->height();

    int xPos = leftX;
    LPointsOX->move(xPos, downY + LPointsOX->height()*0.3);
    xPos += LPointsOX->width() + 10;
    CBPointsOX->move(xPos, downY);

    xPos = newSize.width() - CBPointsOY->width() - 10;
    CBPointsOY->move(xPos, downY);
    xPos -= LPointsOY->width() + 10;
    LPointsOY->move(xPos, downY + LPointsOY->height()*0.3);

    xPos = leftX;
    //downY -= CBPointsOY->height();
    LDeltaX->move(xPos, downY + LDeltaX->height()*0.3);
    xPos += LDeltaX->width() + 10;
    LEdDeltaX->move(xPos, downY);

    xPos = newSize.width() - LEdDeltaY->width() - 10;
    LEdDeltaY->move(xPos, downY);
    xPos -= LDeltaY->width() + 10;
    LDeltaY->move(xPos, downY + LDeltaY->height()*0.3);

    int leftXSecWid;
    if (CBPointsOX->x() < LEdDeltaX->x())
        leftXSecWid = LEdDeltaX->x();
    else
        leftXSecWid = CBPointsOX->x();
    LEdDeltaX->move(leftXSecWid, LEdDeltaX->y());
    CBPointsOX->move(leftXSecWid, CBPointsOX->y());

    lowVoltageModeButton->move(leftX, BSendToData->pos().y());


    #ifndef _OLD_EXTRA_CONTROL
    const int edge = 20;
    const int distanceBetween = 10;
    int maxRightX = 0;
    int cbsHeight = 0;
    int maxLeftWidth = 0;
    int maxRightWidth = 0;
    for (int i=0; i<extraControlsLabels.size(); i++)
    {
        int widthLab = extraControlsLabels.at(i)->width();
        if (widthLab > maxLeftWidth)
            maxLeftWidth = widthLab;
        int widthRight = extraControlsRightPart.at(i)->minimumSizeHint().width();
        if (widthRight > maxRightWidth)
            maxRightWidth = widthRight;
        cbsHeight += extraControlsRightPart.at(i)->height();
        cbsHeight += distanceBetween;
    }
    maxRightX = maxLeftWidth + edge + maxRightWidth;

    EdExScanFirstPDelay->setFixedWidth(maxRightWidth);
    EdExLineFirstPDelay->setFixedWidth(maxRightWidth);
    pointSync->setFixedWidth(maxRightWidth);
    EdExSyncPointDelay->setFixedWidth(maxRightWidth);
    CBScanner->setFixedWidth(maxRightWidth);

    ExtraBackground->setFixedSize(maxRightX + edge*1.5, cbsHeight + edge*1.5);
    int x = newSize.width()*0.5 - ExtraBackground->width()*0.5;
    int y = newSize.height()*0.5 - ExtraBackground->height()*0.5;
    ExtraBackground->move(x, y);
    x += edge;
    y += edge;
    for (int i=0; i<extraControlsLabels.size(); i++)
    {
        extraControlsLabels.at(i)->move(x, y + extraControlsLabels.at(i)->height()*0.2);
        extraControlsRightPart.at(i)->move(x + maxRightX - extraControlsRightPart.at(i)->width(), y);
        y += (extraControlsLabels.at(i)->height() < extraControlsRightPart.at(i)->height() ?
                  extraControlsRightPart.at(i)->height():extraControlsLabels.at(i)->height());
        y += distanceBetween;
    }
#endif
}
void ScanPlot::ChannelSetVisibleGUI(int n, bool visible, plot::ThreeDOverlayButtons::SpecialVisibilityMode visibilityMode)
{
    if (visible && Plot->overlayButtons->blockShowing3DPlotButtons)
        return;

    auto plotPure = (n == -1) ? nullptr : Plot->ThreeDPlots[n];

    if (plotPure != nullptr)
    {
        LPointsOX->setParent(plotPure);
        CBPointsOX->setParent(plotPure);
        LPointsOY->setParent(plotPure);
        CBPointsOY->setParent(plotPure);
        LDeltaX->setParent(plotPure);
        LEdDeltaX->setParent(plotPure);
        LDeltaY->setParent(plotPure);
        LEdDeltaY->setParent(plotPure);
        lowVoltageModeButton->setParent(plotPure);

        #ifndef _OLD_EXTRA_CONTROL
        ExtraBackground->setParent(plotPure);

        for (int i=0; i<extraControlsLabels.size(); i++)
        {
            extraControlsLabels.at(i)->setParent(plotPure);
            extraControlsRightPart.at(i)->setParent(plotPure);
        }
#endif
    }

    bool visibleAreaSelection = visibilityMode == plot::ThreeDOverlayButtons::svm_area_selection &&
                                Plot->overlayButtons->BBackButton->isVisible();

    LDeltaX->setVisible(visibleAreaSelection);
    LEdDeltaX->setVisible(visibleAreaSelection);
    LDeltaY->setVisible(visibleAreaSelection);
    LEdDeltaY->setVisible(visibleAreaSelection);

    #ifndef _OLD_EXTRA_CONTROL
    bool visibleExtraControlMenu = visible && visibilityMode == plot::ThreeDOverlayButtons::svm_extra_control;

    ExtraBackground->setVisible(visibleExtraControlMenu);
    for (int i=0; i<extraControlsLabels.size(); i++)
    {
        extraControlsLabels.at(i)->setVisible(visibleExtraControlMenu);
        extraControlsRightPart.at(i)->setVisible(visibleExtraControlMenu);
    }
#endif

    bool visibleNormalMode = (visibilityMode == plot::ThreeDOverlayButtons::svm_none && visible);

    LPointsOX->setVisible(visibleNormalMode);
    CBPointsOX->setVisible(visibleNormalMode);
    LPointsOY->setVisible(visibleNormalMode);
    CBPointsOY->setVisible(visibleNormalMode);

    lowVoltageModeButton->setVisible(visibleNormalMode);
    lowVoltageModeButton->setIconSize(QSize(lowVoltageModeButton->width(), lowVoltageModeButton->height()));
}

void ScanPlot::on_BSave_clicked()
{
    if (Plot->ActivePlot == videoChannelId && showingVideoFrame)
    {
        auto video = VideoPlayer::GetIt();
        if (!video)
            return;

        video->on_BToData_clicked();
        return;
    }

    Plot->on_BSave_clicked();
}



void ScanPlot::onFilter_changed(bool pressed)
{
    auto mob = MobileServer::GetIt();
    if (!mob)
        return;
    if (!mob->need_send())
        return;

    QList<QVariant> list;
    list << Plot->ActivePlot
         << pressed;
    mob->setPropIdVar(DataID::scanFilterOnOff, list);
}

void ScanPlot::onFilter2D_changed(bool pressed)
{
    auto mob = MobileServer::GetIt();
    if (!mob)
        return;
    if (!mob->need_send())
        return;

    mob->setPropIdVar(DataID::plot2DFilterOnOff, pressed);
}

void ScanPlot::Set_CBPoints(QComboBox *box)
{
    box->clear();

    box->addItem(QString::number(5));
    box->addItem(QString::number(10));
    box->addItem(QString::number(20));
    box->addItem(QString::number(30));
    box->addItem(QString::number(50));

    for (int i = 1; i * 100 <= 1000; i++)
        box->addItem(QString::number(i * 100));

    box->addItem(QString::number(2000));
    box->addItem(QString::number(3000));
    box->addItem(QString::number(4000));
    box->addItem(QString::number(5000));
    box->addItem(QString::number(6000));
    box->addItem(QString::number(8000));
    box->addItem(QString::number(10000));

    box->setCurrentIndex(7);
}

#ifndef _OLD_EXTRA_CONTROL
void ScanPlot::showExtraControlOnPlot()
{
    int n = Plot->ThreeDPlots.indexOf((plot::TPure3dPlot*)lowVoltageModeButton->parent());
    Plot->SetActivePlot(n);
    Plot->overlayButtons->visibilityMode[n] = Plot->overlayButtons->svm_extra_control;
    Plot->overlayButtons->BBackButton->setVisible(true);
    Plot->overlayButtons->showingBackButtonPlot[n] = true;
    Plot->overlayButtons->setVisibleMenu(n, Plot->ThreeDPlots[n], false);
    Plot->overlayButtons->setVisibleMenu(n, Plot->ThreeDPlots[n], true);
}
#endif

void ScanPlot::GUICreate_Plot()
{
    QSettings settingsGui("Param/gui.conf", QSettings::IniFormat);
    {
        show4dMenu = settingsGui.value("gui/show_scan_right_menu", true).toBool();
        ratisMode = settingsGui.value("gui/Ratis_xyz_stage_mode", false).toBool();
    }

    bool filtetingByDef = false;
    {
        QSettings settings("Param/scan.conf", QSettings::IniFormat);
        settings.beginGroup("all_plot_filter");
        filtetingByDef = settings.value("enable_by_def", false).toBool();
        settings.endGroup();
    }

    // Plot
    Plot = new plot::T3dPlot(numScanChannel, this, this);
    disconnect(Plot->PlotControl->BSave, SIGNAL(clicked()), Plot, SLOT(on_BSave_clicked()));
    for (int i=0; i<Plot->ThreeDPlots.count(); i++)
    {
        auto plot = Plot->ThreeDPlots[i];
        plot->isFiltering = filtetingByDef;
        plot->xScanning = 1;
        plot->blockRefresh = true;
        plot->QPlot->plotLayout()->setLegendPosition(QwtPlot::LeftLegend);
        plotLegends[i] = tr("OFF");
        Plot->SetTitle(getLegendNameChannel(i) + tr("OFF"), i);
    }
    Plot->TwoDPlot->Pure2dPlot->isFiltering = filtetingByDef;
    Plot->TwoDPlotVertical->Pure2dPlot->isFiltering = filtetingByDef;

    {
        QSettings settingsGui("Param/gui.conf", QSettings::IniFormat);
        bool show = settingsGui.value("gui/show_scan_right_menu", true).toBool();
        Plot->TwoDPlot->showPlotControl = show;
        Plot->TwoDPlotVertical->showPlotControl = show;
        Plot->showPlotControl = show;
    }
    Plot->PlotControl->BSelectByRect->setIcon(makeHugeIcon(":/ScanPlot/selectFigure.png"));
    Plot->PlotControl->BRestoreScale->setIcon(makeHugeIcon(":/ScanPlot/restore1x12D.png"));

    connect(Plot->TwoDPlot->overlayButtons->B2DBack, SIGNAL(released()), this, SLOT(on2DBackButton_clicked()));
    connect(Plot->TwoDPlotVertical->overlayButtons->B2DBack, SIGNAL(released()), this, SLOT(on2DBackButton_clicked()));

    connect(Plot->overlayButtons, &plot::ThreeDOverlayButtons::BackCancelClicked,
            this, &ScanPlot::cancel_clicked);
    connect(Plot->overlayButtons, &plot::ThreeDOverlayButtons::BackCancelClicked,
            this, &ScanPlot::cancelApply_clicked);
    connect(Plot->overlayButtons, &plot::ThreeDOverlayButtons::ApplyEnterClicked,
            this, &ScanPlot::cancelApply_clicked);
    connect(Plot->PlotControl->BSelectByRect, SIGNAL(clicked()), this, SLOT(selectByRect_clicked()));
    disconnect(Plot->PlotControl->BSelectByRect, SIGNAL(clicked()), Plot, SLOT(on_BSelectByRect_clicked()));
    Plot->SetDefaultXLimits(0, Xmax * 1e-6);
    Plot->SetDefaultYLimits(0, Ymax * 1e-6);
    Plot->SetXYDescAndSIUnit(QString(), tr("m"), -1);
    Plot->SetPlotsXScales(0, 1e-4);
    Plot->SetPlotsYScales(0, 1e-4);
    Plot->TwoDPlot->Pure2dPlot->SetDefaultXLimits(0, 200);
    Plot->TwoDPlot->Pure2dPlot->SetDefaultYLimits(-50, 50);
    Plot->TwoDPlotVertical->Pure2dPlot->SetDefaultYLimits(0, 200);
    Plot->TwoDPlotVertical->Pure2dPlot->SetDefaultXLimits(-50, 50);
    connect(Plot, SIGNAL(ActivePlotChanged(int)), this, SLOT(sl_section_legend(int)));
    char literChannel = 'A';
    auto title = tr("Active channel:") + " " + literChannel;
    Plot->TwoDPlot->SetTitle(title);
    Plot->TwoDPlotVertical->SetTitle(title);
    Plot->overlayButtons->setActivePlotByMouseMove = false;

    // 3D plot overlay buttons
    BSendToData = new QPushButton();
    BSendToData->hide();
    BSendToData->setFlat(true);
    Plot->overlayButtons->setButtonStyle(BSendToData);
    BSendToData->setIcon(makeHugeIcon(":/plot/attachPlot2D.png"));
    BSendToData->setToolTip(QObject::tr("Send to data"));
    connect(BSendToData, SIGNAL(released()), this, SLOT(onSendToData_clicked()));

    BVideoSaveImage = new QPushButton();
    BVideoSaveImage->hide();
    BVideoSaveImage->setFlat(true);
    Plot->overlayButtons->setButtonStyle(BVideoSaveImage);
    BVideoSaveImage->setIcon(makeHugeIcon(":/plot/picturePlot2D.png"));
    BVideoSaveImage->setToolTip(QObject::tr("Image snapshot"));
    connect(BVideoSaveImage, SIGNAL(released()), this, SLOT(onSaveImage_clicked()));

    BVideoSendToData = new QPushButton();
    BVideoSendToData->hide();
    BVideoSendToData->setFlat(true);
    Plot->overlayButtons->setButtonStyle(BVideoSendToData);
    BVideoSendToData->setIcon(makeHugeIcon(":/plot/attachPlot2D.png"));
    BVideoSendToData->setToolTip(QObject::tr("Send to data"));
    connect(BVideoSendToData, SIGNAL(released()), this, SLOT(onSendToDataVideo_clicked()));


    connect(Plot->overlayButtons->BFilters, SIGNAL(state_changed(bool)), this, SLOT(onFilter_changed(bool)));
    connect(Plot->TwoDPlot->overlayButtons->B2DFilter, SIGNAL(state_changed(bool)), this, SLOT(onFilter2D_changed(bool)));

    connect(Plot->TwoDPlot->overlayButtons->B4DSameXYPoints, SIGNAL(state_changed(bool)), this, SLOT(sl_ext_mode()));
    //connect(Plot->TwoDPlot->overlayButtons, SIGNAL(on_BCancelApply_clickedSignal()), this, SLOT(cancelApply_clicked()));
    connect(Plot->TwoDPlot->overlayButtons->B4DSelectCapturedSpectr, SIGNAL(toggled(bool)), this, SLOT(on4DSelectCapturedSpectr_clicked(bool)));
    connect(Plot->TwoDPlot->overlayButtons->B4DSelectConvolutionFunc, SIGNAL(toggled(bool)), this, SLOT(on4DSelectConvolutionFunc_clicked(bool)));
    connect(Plot->TwoDPlot->overlayButtons->B4DSendToData, SIGNAL(released()), this, SLOT(on4DSendToData_clicked()));
    connect(Plot->TwoDPlot->overlayButtons->B4DClearSpectrum, SIGNAL(released()), this, SLOT(on4DClear_clicked()));
    connect(Plot->TwoDPlot->overlayButtons->B4DSectionPointSpectr, SIGNAL(state_changed(bool)), this, SLOT(on4DSwitchSpectra_clicked(bool)));

    // Video
    videoChannelAutoFocusButtonPrec = new TwoStateButton(":/USBVideo/MAFPrecise.png",":/USBVideo/MAFPreciseActive.png");
    videoChannelAutoFocusButtonFast = new TwoStateButton(":/USBVideo/MAFFast.png",":/USBVideo/MAFFastActive.png");
    videoChannelAutoFocusButtonPAF = new TwoStateButton(":/USBVideo/PAF.png",":/USBVideo/PAFActive.png");
    videoChannelPlayCamera = new TwoStateButton(":/USBVideo/CameraStart.png",":/USBVideo/CameraStop.png");

    videoChannelAutoFocusButtonPrec->setFlat(true);
    Plot->overlayButtons->setButtonStyle(videoChannelAutoFocusButtonPrec);
    videoChannelAutoFocusButtonFast->setFlat(true);
    Plot->overlayButtons->setButtonStyle(videoChannelAutoFocusButtonFast);
    videoChannelAutoFocusButtonPAF->setFlat(true);
    Plot->overlayButtons->setButtonStyle(videoChannelAutoFocusButtonPAF);
    videoChannelPlayCamera->setFlat(true);
    Plot->overlayButtons->setButtonStyle(videoChannelPlayCamera);

    //    QList<QString> imgs;
    //    imgs.append(":/USBVideo/autoFocusOnCantUp.png");
    //    imgs.append(":/USBVideo/autoFocusOnCantUpActive.png");
    //    imgs.append(":/USBVideo/autoFocusOnCantDown.png");
    //    imgs.append(":/USBVideo/autoFocusOnCantDownActive.png");
    //    videoChannelAutoFocusUpDownBtn = new ManyStateButton(imgs);
    //    videoChannelAutoFocusUpDownBtn->setVisible(false);
    //    videoChannelAutoFocusUpDownBtn->setEnabled(false);
    //    videoChannelAutoFocusUpDownBtn->setFlat(true);
    //    Plot->overlayButtons->setButtonStyle(videoChannelAutoFocusUpDownBtn);

    allVideoBtnsList << videoChannelAutoFocusButtonPrec << videoChannelAutoFocusButtonFast
                     << videoChannelAutoFocusButtonPAF << videoChannelPlayCamera// << videoChannelAutoFocusUpDownBtn
                     << BVideoSaveImage << BVideoSendToData;

    BXYLock = new TwoStateButton(":/ScanPlot/LockOff2.png",":/ScanPlot/LockOn2.png");
    BXYLock->setToolTip(tr("Sync XY size & points"));
    BXYLock->change_state_man(true);
    BXYLock->setFlat(true);
    Plot->overlayButtons->setButtonStyle(BXYLock);
    Plot->PlotControl->BSelectByRect->setFlat(true);
    Plot->overlayButtons->setButtonStyle(Plot->PlotControl->BSelectByRect);
    Plot->PlotControl->BRestoreScale->setFlat(true);
    Plot->overlayButtons->setButtonStyle(Plot->PlotControl->BRestoreScale);

    videoChannelMagnificationLabel = new QLabel(tr("Magnification"));
    videoChannelMagnification = new QComboBox();

    QString labelChannelColor = "QLabel { color: rgb(100,100,100)}";
    videoChannelMagnificationLabel->setStyleSheet(labelChannelColor);

    LPointsOX = new QLabel(tr("X points"));
    LPointsOY = new QLabel(tr("Y points"));
    LDeltaX = new QLabel(tr("Width, ")+tr("mkm"));
    LDeltaY = new QLabel(tr("Height, ")+tr("mkm"));
    LPointsOX->setStyleSheet(labelChannelColor);
    LPointsOY->setStyleSheet(labelChannelColor);
    LDeltaX->setStyleSheet(labelChannelColor);
    LDeltaY->setStyleSheet(labelChannelColor);

    m_BStart = new swapTwoStateButton(":/USBVideo/CameraStart.png", ":/USBVideo/CameraStartDown.png",":/stop.png");
    m_BStart->setToolTip(tr("Start scanning"));
    connect(m_BStart, SIGNAL(state_changed(bool)), this, SLOT(scan_start_stop(bool)));
    connect(m_BStart, SIGNAL(swapped(swapDir::Direction)), this, SLOT(scan_direction_changed(swapDir::Direction)));
    m_BStart->setFlat(true);
    Plot->overlayButtons->setButtonStyle(m_BStart);

    lowVoltageModeButton = new TwoStateButton(":/ScanPlot/lowVoltageMode.png", ":/ScanPlot/lowVoltageMode.png");
#ifdef _OLD_EXTRA_CONTROL
    connect(lowVoltageModeButton, SIGNAL(state_changed(bool)), this, SLOT(lowVoltageModeChanged(bool)));
    lowVoltageModeButton->setToolTip(tr("Low voltage mode"));
#else
    connect(lowVoltageModeButton, SIGNAL(state_changed(bool)), this, SLOT(showExtraControlOnPlot()));
    lowVoltageModeButton->setToolTip(tr("Extra control"));
#endif
    lowVoltageModeButton->setFlat(true);
    Plot->overlayButtons->setButtonStyle(lowVoltageModeButton);

    CBStartPos = new QComboBox;
    scanControlWidgets.append(CBStartPos);
    CBStartPos->addItem(makeHugeIcon(":/ScanPlot/Top_left_X.png"), QString());
    CBStartPos->addItem(makeHugeIcon(":/ScanPlot/Top_left_Y.png"), QString());
    CBStartPos->addItem(makeHugeIcon(":/ScanPlot/Top_right_X.png"), QString());
    CBStartPos->addItem(makeHugeIcon(":/ScanPlot/Top_right_Y.png"), QString());
    CBStartPos->addItem(makeHugeIcon(":/ScanPlot/Bottom_left_X.png"), QString());
    CBStartPos->addItem(makeHugeIcon(":/ScanPlot/Bottom_left_Y.png"), QString());
    CBStartPos->addItem(makeHugeIcon(":/ScanPlot/Bottom_right_X.png"), QString());
    CBStartPos->addItem(makeHugeIcon(":/ScanPlot/Bottom_right_Y.png"), QString());
    connect(CBStartPos, SIGNAL(currentIndexChanged(int)), this, SLOT(updateDirection()));

    CBPointsOX = new QComboBox;
    CBPointsOY = new QComboBox;
    scanControlWidgets.append(CBPointsOX);
    scanControlWidgets.append(CBPointsOY);

    Set_CBPoints(CBPointsOX);
    Set_CBPoints(CBPointsOY);
    LEdDeltaX =  new Edit(10e-9 * 1e6, 200, 3);
    LEdDeltaX->setEnabled(false);

    LEdDeltaY = new Edit(10e-9 * 1e6, 200, 3);
    LEdDeltaY->setEnabled(false);
    connect(LEdDeltaX, SIGNAL(newValue()), this, SLOT(onLEdDeltaX_editingFinished()));
    connect(LEdDeltaY, SIGNAL(newValue()), this, SLOT(onLEdDeltaY_editingFinished()));

    connect(CBPointsOX, SIGNAL(currentIndexChanged(int)), this, SLOT(check_nx_ny()));
    connect(CBPointsOY, SIGNAL(currentIndexChanged(int)), this, SLOT(check_nx_ny()));

    Plot->overlayButtons->setComboBoxStyle(CBStartPos);
    Plot->overlayButtons->setComboBoxStyle(CBPointsOX);
    Plot->overlayButtons->setComboBoxStyle(CBPointsOY);
    Plot->overlayButtons->setDoubleSpinBoxStyle(LEdDeltaX);
    Plot->overlayButtons->setDoubleSpinBoxStyle(LEdDeltaY);

    Plot->overlayButtons->scanButtonsSetVisibleGUI(false);

    #ifdef _SCAN_SLICE
    connect(Plot->PlotControl->BRestoreScale, SIGNAL(clicked()), this, SLOT(scan_slice()));
    #endif

    Plot->overlayButtons->upBtnsList.insert(0, m_BStart);
    Plot->overlayButtons->upBtnsList << Plot->PlotControl->BRestoreScale << Plot->PlotControl->BSelectByRect;
    Plot->overlayButtons->allBtnsList << m_BStart << Plot->PlotControl->BRestoreScale << Plot->PlotControl->BSelectByRect
                                      << lowVoltageModeButton;
    if (ratisMode)
    {
        Plot->overlayButtons->setVisibleFilterButton(false);
        Plot->overlayButtons->ActivateRatisMode();
    }
    else
    {
        Plot->overlayButtons->rightBtnsList << BSendToData;
        Plot->overlayButtons->allBtnsList << BSendToData;
    }

    connect(Plot->overlayButtons, SIGNAL(GUIPlotResized(int, QSize)),
            this, SLOT(ChannelResized(int, QSize)));
//    connect(Plot->overlayButtons, SIGNAL(GUIPlotSetVisible(int, bool, plot::ThreeDOverlayButtons::SpecialVisibilityMode)),
//            this, SLOT(ChannelSetVisibleGUI(int, bool, plot::ThreeDOverlayButtons::SpecialVisibilityMode)));
    connect(Plot->overlayButtons, &plot::ThreeDOverlayButtons::GUIPlotSetVisible,
            this, &ScanPlot::ChannelSetVisibleGUI);

    connect(Plot->TwoDPlot->overlayButtons->CBLineColor, SIGNAL(currentIndexChanged(int)),
            this, SLOT(currentLineFontSettingsChanged(int)));
    connect(Plot->TwoDPlot->overlayButtons->CBLineWidth, SIGNAL(currentIndexChanged(int)),
            this, SLOT(currentLineFontSettingsChanged(int)));
    connect(Plot->TwoDPlot->overlayButtons->CBFont, SIGNAL(currentIndexChanged(int)),
            this, SLOT(currentLineFontSettingsChanged(int)));
    connect(Plot->TwoDPlot, SIGNAL(CurveColourChanged(QColor, int)),
            this, SLOT(currentLineColorChanged(QColor, int)));
    connect(Plot->TwoDPlotVertical, SIGNAL(CurveColourChanged(QColor, int)),
            this, SLOT(currentLineColorChanged(QColor, int)));



    BMakeScreen = new TwoStateButton(":/SPMTuning_window/Video.png",":/SPMTuning_window/VideoOff.png");
    BMakeScreen->setObjectName("BMakeScreen");
    BMakeScreen->setToolTip(QObject::tr("Show videocamera in channel D"));
    connect(BMakeScreen, SIGNAL(state_changed(bool)), this, SLOT(make_screen_videocam(bool)));

    BAutoFocus = new TwoStateButton(":/USBVideo/MAF.png",":/USBVideo/MAFActive.png");
    BAutoFocus->setToolTip(QObject::tr("Start autofocus"));
    BAutoFocus2 = new TwoStateButton(":/USBVideo/PAF.png",":/USBVideo/PAFActive.png");
    BAutoFocus2->setToolTip(QObject::tr("Start autofocus2"));

    auto color = QColor(Qt::red);
    hbMotorZFocusUp = new HoverButton(HoverButton::TopArrow, color);
    hbMotorZFocusDown = new HoverButton(HoverButton::BottomArrow, color);
    hbMotorXStageLeft = new HoverButton(HoverButton::LeftArrow, color);
    hbMotorXStageRight = new HoverButton(HoverButton::RightArrow, color);
    hbMotorYStageTop = new HoverButton(HoverButton::TopArrow, color);
    hbMotorYStageBottom = new HoverButton(HoverButton::BottomArrow, color);
    hbMotorZFocusEllipse = new HoverButton(HoverButton::Circle, color);
    hbMotorStageEllipse = new HoverButton(HoverButton::Circle, color);

    hbMotorZFocusEllipse->text = "F";
    hbMotorStageEllipse->text = "S";

    comboMaterialThin = new QComboBox();
    comboMaterialThin->addItem("0.5" + tr("mm"), 0.5);
    for (int i=1; i<21; i++)
    {
        comboMaterialThin->addItem(QString::number(i) + tr("mm"), i);
    }
    {
        QSettings settings("Param/scan.conf", QSettings::IniFormat);
        comboMaterialThin->setCurrentIndex(settings.value("MaterialThinCBIndex", 0).toInt());
    }
    connect(comboMaterialThin, SIGNAL(currentIndexChanged(int)), this, SLOT(materialThinChangedSave(int)));

//    connect(videoChannelAutoFocusUpDownBtn, SIGNAL(state_changed(int, int)), this, SLOT(autoFocusUpDownBtn(int, int)));
//    connect(videoChannelAutoFocusUpDownBtn, SIGNAL(state_changed_man(int, int)), this, SLOT(autoFocusUpDownBtn_man(int, int)));

    connect(hbMotorZFocusUp, SIGNAL(holdingPressed(bool)), this, SLOT(motorZFocusUp(bool)));
    connect(hbMotorZFocusDown, SIGNAL(holdingPressed(bool)), this, SLOT(motorZFocusDown(bool)));
    connect(hbMotorXStageLeft, SIGNAL(holdingPressed(bool)), this, SLOT(motorXStageLeft(bool)));
    connect(hbMotorXStageRight, SIGNAL(holdingPressed(bool)), this, SLOT(motorXStageRight(bool)));
    connect(hbMotorYStageTop, SIGNAL(holdingPressed(bool)), this, SLOT(motorYStageTop(bool)));
    connect(hbMotorYStageBottom, SIGNAL(holdingPressed(bool)), this, SLOT(motorYStageBottom(bool)));

    connect(Plot->ThreeDPlots[videoChannelId], SIGNAL(onRestoredScales()), this, SLOT(updateVideoChannel()));

    videoChannelAutoFocusButtonPrec->setParent(Plot->ThreeDPlots[videoChannelId]);
    videoChannelAutoFocusButtonFast->setParent(Plot->ThreeDPlots[videoChannelId]);
    videoChannelAutoFocusButtonPAF->setParent(Plot->ThreeDPlots[videoChannelId]);
    videoChannelPlayCamera->setParent(Plot->ThreeDPlots[videoChannelId]);
    videoChannelMagnificationLabel->setParent(Plot->ThreeDPlots[videoChannelId]);
    videoChannelMagnification->setParent(Plot->ThreeDPlots[videoChannelId]);
    hbMotorZFocusUp->setParent(Plot->ThreeDPlots[videoChannelId]);
    hbMotorZFocusDown->setParent(Plot->ThreeDPlots[videoChannelId]);
    hbMotorXStageLeft->setParent(Plot->ThreeDPlots[videoChannelId]);
    hbMotorXStageRight->setParent(Plot->ThreeDPlots[videoChannelId]);
    hbMotorYStageTop->setParent(Plot->ThreeDPlots[videoChannelId]);
    hbMotorYStageBottom->setParent(Plot->ThreeDPlots[videoChannelId]);
    hbMotorZFocusEllipse->setParent(Plot->ThreeDPlots[videoChannelId]);
    hbMotorStageEllipse->setParent(Plot->ThreeDPlots[videoChannelId]);
    //videoChannelAutoFocusUpDownBtn->setParent(Plot->ThreeDPlots[videoChannelId]);
    BVideoSaveImage->setParent(Plot->ThreeDPlots[videoChannelId]);
    BVideoSendToData->setParent(Plot->ThreeDPlots[videoChannelId]);

    videoChannelMagnification->setStyleSheet("QComboBox {border-width:0; background-color:rgba(255,255,255,100);}\
                            QComboBox:pressed { background-color:rgba(255,255,255,100); border-width:1;}");

    videoChannelSetVisibleGUI(videoChannelId, false);

    lb_ccd_scan = new QLabel;
    lb_ccd_scan->hide();

    lb_ccd_scan_sm = new QLabel;
    lb_ccd_scan_sm->hide();

    comboVideoProfile = new QComboBox();

    connect(videoChannelAutoFocusButtonPrec, SIGNAL(state_changed(bool)), this, SLOT(autoFocusPrec_ChangedState(bool)));
    connect(videoChannelAutoFocusButtonFast, SIGNAL(state_changed(bool)), this, SLOT(autoFocusFast_ChangedState(bool)));
    connect(videoChannelPlayCamera, SIGNAL(state_changed(bool)), this, SLOT(make_screen_videocam(bool)));

    gb4DButtons = new QGroupBox(tr("4D"));
    lay4DButtons = new QVBoxLayout(gb4DButtons);
    gb4DButtons->hide();
    Plot->ControlLayout->addWidget(gb4DButtons);
    BCCDXYLock = new TwoStateButton(":/ScanPlot/LockOFF.png",":/ScanPlot/LockON.png");
    scanControlWidgets.append(BCCDXYLock);
    BCCDXYLock->setToolTip(tr("Same X/Y points as scan"));
    BCCDXYLock->change_state_man(true);
    connect(BCCDXYLock, SIGNAL(state_changed(bool)), this, SLOT(sl_ext_mode()));

    lay4DButtons->addWidget(BCCDXYLock, 0, Qt::AlignLeft);

    for (int i = 0; i < numScanChannel; i++)
        Plot->ThreeDPlots[i]->blockRefresh = false;
    Plot->Refresh();

    QPushButton *BSendToGraphEditor = new QPushButton;
    BSendToGraphEditor->setObjectName("BSendToGraphEditor");
    BSendToGraphEditor->setIcon(makeHugeIcon(":/ScanPlot/litho.png"));
    BSendToGraphEditor->setToolTip(QObject::tr("use area for lithography"));
    connect(BSendToGraphEditor, SIGNAL(clicked()), this, SLOT(BSendToGraphEditor_clicked()));
    BSendToGraphEditor->hide();

    Plot->PlotControl->ControlMainLayout->addWidget(BSendToGraphEditor);

    scanControlWidgets << Plot->PlotControl->BMarkers <<
                          Plot->PlotControl->BCross <<
                          Plot->PlotControl->BSelectByRect <<
                          BSendToGraphEditor <<
                          Plot->PlotControl->BFixX <<
                          Plot->PlotControl->BFixY <<
                          Plot->PlotControl->BRestoreScale <<
                          Plot->TwoDPlot->PlotControl->CBChooseActiveCurve <<
                          Plot->TwoDPlot->PlotControl->TBCurveColour;

    mainLay->addWidget(Plot, 1);

    connect(Plot->mainTab, SIGNAL(currentChanged(int)), this, SLOT(Plot3DTabChanged(int)));

    ShowRightMenu(show4dMenu);

    for (int i = 0; i < numScanChannel; i++) {
        scanLineInd[i] = new QwtPlotMarker;
        scanLineInd[i]->setXValue(0);
        scanLineInd[i]->setYValue(0);
        scanLineInd[i]->setLinePen(QPen(QColor(255, 0, 0), 0, Qt::SolidLine));
        scanLineInd[i]->setLineStyle(QwtPlotMarker::HLine);
        scanLineInd[i]->attach(Plot->ThreeDPlots.at(i)->QPlot);
    }

    connect(Plot->SplitPlotTab, SIGNAL(splitterMoved(int, int)), this, SLOT(splitterMoved(int, int)));

    {
        QSettings settings("Param/gui.conf", QSettings::IniFormat);
        int first = settings.value("scan/height_3dplot", 663).toInt();
        int second = settings.value("scan/height_2dplot", 250).toInt();
        Plot->SplitPlotTab->setSizes(QList<int>() << first << second);
    }
}

void ScanPlot::GUICreate_TipMonitor()
{
    tipMonitorGB = new QGroupBox(tr("Tip monitor"));
    QHBoxLayout *lay_rectSlidersBox = new QHBoxLayout(tipMonitorGB);
    LeftLay->addWidget(tipMonitorGB);

    GUICreate_TipMonitor_Approach(lay_rectSlidersBox);
    GUICreate_TipMonitor_Speed(lay_rectSlidersBox);
    GUICreate_TipMonitor_Sensitivity(lay_rectSlidersBox);
    GUICreate_TipMonitor_TipPush(lay_rectSlidersBox);
    GUICreate_TipMonitor_TipDrive(lay_rectSlidersBox);


}

void ScanPlot::GUICreate_TipMonitor_Approach(QHBoxLayout *lay_rectSlidersBox)
{
    smartLanding = new ThreeStateButton(":/SPMTuning_window/SmartLandingOff.png", ":/SPMTuning_window/SmartLandingOn.png", 4.0);
    smartLanding->setObjectName("smartLanding");
    smartLanding->setStyleSheet("QPushButton#" + smartLanding->objectName() + " {background-color: rgb(255,255,255); \
        border-style: outset; border-width: 1px; border-color: rgb(45,45,45);}");
    smartLanding->setToolTip(tr("Smart landing start/stop"));
    auto shadow = new QGraphicsDropShadowEffect();
    shadow->setXOffset(1.);
    shadow->setYOffset(1.);
    smartLanding->setGraphicsEffect(shadow);
    QVBoxLayout *lay_smartLand = new QVBoxLayout();
    lay_smartLand->addWidget(new QLabel(tr("Approach")), 0, Qt::AlignCenter);
    lay_smartLand->addWidget(smartLanding);
    lay_rectSlidersBox->addLayout(lay_smartLand);

    connect(smartLanding, SIGNAL(state_changed(int, int)), this, SLOT(spm_SmartLandingButtonChanged(int, int)));
    connect(smartLanding, SIGNAL(state_changed_man(int, int)), this, SLOT(spm_SmartLandingButtonChangedMan(int, int)));

    smartLandingBtn1 = new QPushButton("↑" + tr("S.OFF"));
    smartLandingBtn2 = new QPushButton(tr("S.ON") + "↓");
    smartLandingBtn1->setCheckable(true);
    smartLandingBtn2->setCheckable(true);
    smartLandingBtn1->setStyleSheet("color: green");
    smartLandingBtn2->setStyleSheet("color: red");
    smartLandingBtn1->setParent(smartLanding);
    smartLandingBtn2->setParent(smartLanding);
    int spaceWidth = 10;
    smartLandingBtn1->setFixedWidth(smartLandingBtn1->fontMetrics().horizontalAdvance(smartLandingBtn1->text()) + spaceWidth);
    smartLandingBtn2->setFixedWidth(smartLandingBtn2->fontMetrics().horizontalAdvance(smartLandingBtn2->text()) + spaceWidth);
    smartLandingBtn2->move(smartLandingBtn1->width(), 0);
    connect(smartLandingBtn1, SIGNAL(toggled(bool)), this, SLOT(zLandButtonSysOff(bool)));
    connect(smartLandingBtn2, SIGNAL(toggled(bool)), this, SLOT(zLandButtonSysOn(bool)));

    videoChannelMaterialThinLabel = new QLabel(tr("Sm.Th"));
    videoChannelMaterialThin = new QComboBox();
    videoChannelMaterialThin->setProperty("id", DataID::TipPush_sampleThickness);
    videoChannelMaterialThinLabel->setParent(smartLanding);
    videoChannelMaterialThin->setParent(smartLanding);
    videoChannelMaterialThinLabel->setVisible(true);
    videoChannelMaterialThin->setVisible(true);
    videoChannelMaterialThin->setStyleSheet("QComboBox {border-width:0; background-color:transparent;}\
                            QComboBox:pressed { background-color:RGBT(174,79,173,195); border-width:1;}");

    videoChannelMaterialThin->addItem("0.5" + tr("mm"), 0.5);
    for (int i=1; i<21; i++)
    {
        videoChannelMaterialThin->addItem(QString::number(i) + tr("mm"), i);
    }
    {
        QSettings settings("Param/scan.conf", QSettings::IniFormat);
        videoChannelMaterialThin->setCurrentIndex(settings.value("MaterialThinCBIndex", 0).toInt());
    }
    connect(videoChannelMaterialThin, SIGNAL(currentIndexChanged(int)), this, SLOT(materialThinChangedSave(int)));


    int smartLandBtnSize = smartLandingBtn1->width() + smartLandingBtn2->width();
    minimumRectSliderSize = smartLandBtnSize;
    smartLanding->setFixedSize(smartLandBtnSize, smartLandBtnSize);
//    smartLanding->setFixedSize(smartLandBtnSize, smartLandBtnSize);
//    smartLanding->setIconSize(QSize(smartLandBtnSize, smartLandBtnSize));
    //smartLanding->setProperty("buttonsize", smartLandBtnSize);

    videoChannelMaterialThin->setFixedWidth(smartLanding->width() - videoChannelMaterialThinLabel->width());
    videoChannelMaterialThinLabel->move(0, smartLanding->height() - videoChannelMaterialThinLabel->height() - 2);
    videoChannelMaterialThin->move(videoChannelMaterialThinLabel->width(), smartLanding->height() - videoChannelMaterialThin->height());
}

void ScanPlot::GUICreate_TipMonitor_Speed(QHBoxLayout *lay_rectSlidersBox)
{
    rectSlFbp = new RectSlider();
    rectSlFbp->SetDimension(tr("line/s"), true);
    rectSlFbp->SetNumberAfterPoint(2);
    rectSlFbp->SetHoldIncreaseSpeed(0.01);
    rectSlFbp->reduceWheelPowerBy = 10.;
    rectSlFbp->SetClickIncrease(0.01);
    rectSlFbp->SetHoldIncreaseSpeed(0.01);
    rectSlFbp->SetMinValue(0.01);
    rectSlFbp->SetTickNSeconds(25);
    rectSlFbp->EnableSecondSpeed(true);
    rectSlFbp->setValue(user_scan_speed = requested_scan_speed = 1);
    connect(rectSlFbp, SIGNAL(valueChanged()), this, SLOT(check_speed()));
    {
        QSettings settings("Param/controller.conf", QSettings::IniFormat);
        rectSlFbp->SetMaxValue(settings_double(settings, "limit_scan_rate", 10));
    }
    QVBoxLayout *lay_rectSlFbp = new QVBoxLayout();
    lay_rectSlFbp->addWidget(new QLabel(tr("Speed")), 0, Qt::AlignCenter);
    lay_rectSlFbp->addWidget(rectSlFbp);
    lay_rectSlidersBox->addLayout(lay_rectSlFbp);

    rectSlFbp->setFixedSize(minimumRectSliderSize, minimumRectSliderSize);
}

void ScanPlot::GUICreate_TipMonitor_Sensitivity(QHBoxLayout *lay_rectSlidersBox)
{
    rectSlVelocity = new RectSlider();
    rectSlVelocity->arcLeftAngle = 280;
    rectSlVelocity->arcRightAngle = -310;
    rectSlVelocity->centralWidgetPos = new QPointF(0., 0.);
    auto rectVelocityCBox = new QComboBox(rectSlVelocity);
    rectSlVelocity->centralWidget = rectVelocityCBox;
    rectVelocityCBox->setFont(QFont("Arial", 8));
    rectVelocityCBox->setFrame(false);
    rectVelocityCBox->hide();
    rectVelocityCBox->setStyleSheet("QComboBox {border-width:0; background-color:transparent;}\
                            QComboBox:pressed { background-color:RGBT(174,79,173,195); border-width:1;}");
    rectVelocityCBox->addItem("x0.1");
    rectVelocityCBox->addItem("x1");
    rectVelocityCBox->addItem("x5");
    rectVelocityCBox->addItem("x10");
    rectVelocityCBox->addItem("x50");
    rectVelocityCBox->addItem("x100");
    rectVelocityCBox->show();
    rectVelocityCBox->setProperty("id", DataID::TipPush_sensitivity);
    rectVelocityCBox->setFixedWidth(70);
    {
        QSettings settings("Param/controller.conf", QSettings::IniFormat);
        double scale = settings_double(settings, "mult_spm_pid_stage", 1.0);
        rectVelocityCBox->setCurrentIndex(5);
        for (int i=0; i<rectVelocityCBox->count(); i++)
            if (velocityMulfactorValues[i] >= scale)
            {
                rectVelocityCBox->setCurrentIndex(i);
                break;
            }
    }
    connect(rectVelocityCBox, SIGNAL(currentIndexChanged(int)), this, SLOT(multSpmPidStageChanged(int)));

    //=====todo Remove it with replacing all connects that are connected to that widget
    DCfbPPerc = new DoubleChanger(0, 100, 2); //Sensitivity
    DCfbP = new DoubleChanger(0, 10, 2);
    DCfbP->setProperty("id", DataID::SPM_P);
    connect(DCfbPPerc, SIGNAL(newValue(double)), this, SLOT(DCfbPPercNewValue(double)));
    connect(DCfbP, SIGNAL(newValue(double)), this, SLOT(DCfbPNewValue_emit(double)));
    //============================

    rectSlVelocity->SetTickNSeconds(10);
    rectSlVelocity->setValue(DCfbPPerc->value());
    connect(DCfbPPerc, SIGNAL(newValue(double)), rectSlVelocity, SLOT(setValue_man(double)));
    connect(rectSlVelocity, SIGNAL(valueChanged(double)), DCfbPPerc, SLOT(setValue(double)));
    connect(rectSlVelocity, SIGNAL(valueChanged(double)), this, SLOT(DCfbPPercNewValue(double)));
    QVBoxLayout *lay_rectSlVelocity = new QVBoxLayout();
    lay_rectSlVelocity->addWidget(new QLabel(tr("Sensitivity")), 0, Qt::AlignCenter);
    lay_rectSlVelocity->addWidget(rectSlVelocity);
    lay_rectSlidersBox->addLayout(lay_rectSlVelocity);

    rectSlVelocity->setFixedSize(minimumRectSliderSize, minimumRectSliderSize);
}

void ScanPlot::GUICreate_TipMonitor_TipPush(QHBoxLayout *lay_rectSlidersBox)
{
    rectSlFbi = new RectSlider();
    auto fbiPushBtn = new QPushButton(rectSlFbi);
    rectSlFbi->centralWidget = fbiPushBtn;
    fbiPushBtn->setFont(QFont("Arial", 8));
    fbiPushBtn->setCheckable(true);
    fbiPushBtn->setStyleSheet("QPushButton { color: gray; }\
                            QPushButton:checked { color: red; }");
    rectSlFbi->centralWidgetPos = new QPointF(0.27, 0.);
    rectSlFbi->arcLeftAngle = 260;
    rectSlFbi->arcRightAngle = -320;
    fbiPushBtn->setMaximumWidth(40*1.2);
    fbiPushBtn->setText(tr("FB off"));
    fbiPushBtn->setProperty("id", DataID::TipPush_fbOff);
    connect(fbiPushBtn, SIGNAL(toggled(bool)), this, SLOT(setFBoffon(bool)));
    fbiPushBtn->show();
    rectSlFbi->setValue(50.0);
    connect(rectSlFbi, SIGNAL(valueChanged(double)), this, SLOT(rectSliderSlFbiChanged(double)));
    rectSlFbi->SetTickNSeconds(10);
    QVBoxLayout *lay_rectSlFbi = new QVBoxLayout();
    lay_rectSlFbi->addWidget(new QLabel(tr("Tip push")), 0, Qt::AlignCenter);
    lay_rectSlFbi->addWidget(rectSlFbi);
    lay_rectSlidersBox->addLayout(lay_rectSlFbi);

    rectSlFbi->setFixedSize(minimumRectSliderSize, minimumRectSliderSize);
}

void ScanPlot::GUICreate_TipMonitor_TipDrive(QHBoxLayout *lay_rectSlidersBox)
{
    rectAmplitude = new RectSlider();
    auto ampliturePushBtn = new QPushButton(rectAmplitude);
    rectAmplitude->centralWidget = ampliturePushBtn;
    ampliturePushBtn->setFont(QFont("Arial", 8));
    ampliturePushBtn->setCheckable(true);
    ampliturePushBtn->setStyleSheet("QPushButton { color: gray; }\
                            QPushButton:checked { color: red; }");
    rectAmplitude->centralWidgetPos = new QPointF(0.3, 0.);
    rectAmplitude->arcLeftAngle = 280;
    rectAmplitude->arcRightAngle = -310;
    connect(ampliturePushBtn, SIGNAL(toggled(bool)), this, SLOT(setAutoTipPush(bool)));
    ampliturePushBtn->setMaximumWidth(35*1.2);
    {
        QSettings settings("Param/gui.conf", QSettings::IniFormat);
        bool checked = settings.value("scan/auto_tip_push", false).toBool();
        ampliturePushBtn->setChecked(checked);
        ampliturePushBtn->setText(checked ? tr("auto") : tr("mnl"));
    }
    ampliturePushBtn->show();
    rectAmplitude->SetTickNSeconds(10);

    QVBoxLayout *lay_rectAmplitude = new QVBoxLayout();
    lay_rectAmplitude->addWidget(new QLabel(tr("Tip drive")), 0, Qt::AlignCenter);
    lay_rectAmplitude->addWidget(rectAmplitude);
    lay_rectSlidersBox->addLayout(lay_rectAmplitude);

    rectAmplitude->setFixedSize(minimumRectSliderSize, minimumRectSliderSize);
}

void ScanPlot::GUICreate_Signals()
{
    QSettings settingsGui("Param/gui.conf", QSettings::IniFormat);

    signalsGB = new QGroupBox(tr("Signals"));
    QHBoxLayout *channelTabGBlay = new QHBoxLayout(signalsGB);
    tabSignals = new TTabWidget();
    channelTabGBlay->addWidget(tabSignals, 0, Qt::AlignTop);
    LeftLay->addWidget(signalsGB, 1);

    GUICreate_Signals_Channels();
    GUICreate_Signals_Fz();
    GUICreate_Signals_4D();
    GUICreate_Signals_LiftMode();
    GUICreate_Signals_ContScan();

    minimumSignalsHeight = signalsGB->minimumSizeHint().height();
    qDebug()<<"x7x_"<<"minimum height"<<signalsGB->minimumSizeHint().height();

    {//todo unused GUI
        //---------FBControl--------------------------------------------------------------------------
        spmPrecision = new DoubleChanger(0, 20, 5);
        spmPrecision->setValue(20);
        connect(spmPrecision, SIGNAL(newValue(double)), this, SLOT(spm_pid_precision(double)));

        {
            QSettings settings("Param/analog.conf", QSettings::IniFormat);
            double a1 = settings_double(settings, "dac/vdrive_min", 0.);
            double b1 = settings_double(settings, "dac/vdrive_max", 10.);
            driveInt = new DoubleChanger(a1, b1, 3);
        }

        DCfbI = new DoubleChanger(0, 1, 3);
        DCfbSetPoint = new DoubleChanger(-10, 10, 2);

        DCfbI->setProperty("id", DataID::SPM_I);
        DCfbSetPoint->setProperty("id", DataID::SPM_SetPoint);

        QGroupBox *box_fbp = new QGroupBox(tr("FB P"));
        QHBoxLayout *layout_fbp = new QHBoxLayout(box_fbp);
        layout_fbp->addWidget(DCfbP);

        QGroupBox *box_fbi = new QGroupBox(tr("FB I"));
        QHBoxLayout *layout_fbi = new QHBoxLayout(box_fbi);
        layout_fbi->addWidget(DCfbI);

        QGroupBox *box_setPoint = new QGroupBox(tr("Set point"));
        QHBoxLayout *layout_setPoint = new QHBoxLayout(box_setPoint);
        layout_setPoint->addWidget(DCfbSetPoint);

        QGroupBox *amplitude = new QGroupBox(tr("Amplitude"));
        QHBoxLayout *layout_amplitude = new QHBoxLayout(amplitude);
        layout_amplitude->addWidget(driveInt);

        QWidget* box_FB = new QWidget;
        QVBoxLayout *layout_FB = new QVBoxLayout(box_FB);
        layout_FB->setAlignment(Qt::AlignTop);
        layout_FB->addWidget(box_setPoint);
        layout_FB->addWidget(box_fbp);
        layout_FB->addWidget(box_fbi);
        layout_FB->addWidget(amplitude);

        QGroupBox *box_spm_prec = new QGroupBox(tr("Set point precision"));
        QHBoxLayout *layout_spm_prec = new QHBoxLayout(box_spm_prec);
        layout_spm_prec->addWidget(spmPrecision);
        box_spm_prec->hide();
    }


    GUICreate_Signals_ChannelChooser(channelTabGBlay);

    connect(tabSignals, SIGNAL(currentChanged(int)), this, SLOT(SignalsTabChanged(int)));
}

void ScanPlot::GUICreate_Signals_Channels()
{
    QSettings settingsGui("Param/gui.conf", QSettings::IniFormat);

    QWidget *WScanFW = new QWidget;
    QHBoxLayout *layWScanFW = new QHBoxLayout(WScanFW);
    QVBoxLayout *GLChannelsForGrB = new QVBoxLayout();
    QHBoxLayout *GLChannelsForGrB_1 = new QHBoxLayout();
    QHBoxLayout *GLChannelsForGrB_2 = new QHBoxLayout();
    layWScanFW->addLayout(GLChannelsForGrB);
    GLChannelsForGrB_1->setAlignment(Qt::AlignTop);
    GLChannelsForGrB_2->setAlignment(Qt::AlignTop);
    GLChannelsForGrB->addLayout(GLChannelsForGrB_1);
    GLChannelsForGrB->addLayout(GLChannelsForGrB_2);

    QWidget *WScanBW = new QWidget;
    QHBoxLayout *layWScanBW = new QHBoxLayout(WScanBW);
    QVBoxLayout *GLChannelsForGrB2 = new QVBoxLayout();
    QHBoxLayout *GLChannelsForGrB2_1 = new QHBoxLayout();
    QHBoxLayout *GLChannelsForGrB2_2 = new QHBoxLayout();
    layWScanBW->addLayout(GLChannelsForGrB2);
    GLChannelsForGrB2_1->setAlignment(Qt::AlignTop);
    GLChannelsForGrB2_1->setAlignment(Qt::AlignTop);
    GLChannelsForGrB2->addLayout(GLChannelsForGrB2_1);
    GLChannelsForGrB2->addLayout(GLChannelsForGrB2_2);

    tabSignals->addTab(WScanFW, tr("Fw Chan"));
    if (!settingsGui.value("gui/show_bw_chan", true).toBool())
        WScanBW->hide();
    else
        tabSignals->addTab(WScanBW, tr("Bw Chan"));

    for (int i = 0; i < numScanChannel; i++)
        LmSwitch[i] = new QCheckBox;

    LmSwitch[0]->setText(tr("Lift A"));
    LmSwitch[1]->setText(tr("Lift B"));
    LmSwitch[2]->setText(tr("Lift C"));
    LmSwitch[3]->setText(tr("Lift D"));

    for (int i = 0; i < 4; i++) {
        CBChannel[i] = new QComboBox;
        scanControlWidgets.append(CBChannel[i]);
        fillCBchannels_(CBChannel[i], false);
        CBChannel[i]->setProperty("id", DataID::ScanChA+i);
    }
    CBChannel[1]->addItem(QObject::tr("CCD")+"/EXT", srcCCD);

    for (auto &chan : CBChannel)
        connect(chan, SIGNAL(currentIndexChanged(int)), this, SLOT(sl_CB_update()));

    connect(CBChannel[videoChannelId], SIGNAL(currentIndexChanged(int)), this, SLOT(onVideoCBChannelChanged(int)));

    QPalette plt;
    CBSec[0] = new QCheckBox(tr("Sect") + " A");
    CBSec[0]->setProperty("id", DataID::ScanChSecA);
    plt.setColor(QPalette::WindowText, Qt::black); CBSec[0]->setPalette(plt);
    CBSec[1] = new QCheckBox(tr("Sect") + " B");
    CBSec[1]->setProperty("id", DataID::ScanChSecB);
    plt.setColor(QPalette::WindowText, Qt::red); CBSec[1]->setPalette(plt);
    CBSec[2] = new QCheckBox(tr("Sect") + " C");
    CBSec[2]->setProperty("id", DataID::ScanChSecC);
    plt.setColor(QPalette::WindowText, Qt::green); CBSec[2]->setPalette(plt);
    CBSec[3] = new QCheckBox(tr("Sect") + " D");
    CBSec[3]->setProperty("id", DataID::ScanChSecD);
    plt.setColor(QPalette::WindowText, Qt::blue); CBSec[3]->setPalette(plt);

    for (auto &sec : CBSec)
        connect(sec, SIGNAL(stateChanged(int)), this, SLOT(updateScan2DPlotVisible()));

    GLChannelsForGrB_1->addWidget(new QLabel(tr("Chnl A")), 0, Qt::AlignTop | Qt::AlignLeft);
    GLChannelsForGrB_1->addWidget(CBChannel[0], 1, Qt::AlignTop | Qt::AlignLeft);
    GLChannelsForGrB_1->addWidget(CBSec[0], 0, Qt::AlignTop | Qt::AlignRight);
    GLChannelsForGrB_1->addWidget(LmSwitch[0], 0, Qt::AlignTop | Qt::AlignRight);
    GLChannelsForGrB_2->addWidget(new QLabel(tr("Chnl B")), 0, Qt::AlignTop | Qt::AlignLeft);
    GLChannelsForGrB_2->addWidget(CBChannel[1], 1, Qt::AlignTop | Qt::AlignLeft);
    GLChannelsForGrB_2->addWidget(CBSec[1], 0, Qt::AlignTop | Qt::AlignRight);
    GLChannelsForGrB_2->addWidget(LmSwitch[1], 0, Qt::AlignTop | Qt::AlignRight);


    GLChannelsForGrB2_1->addWidget(new QLabel(tr("Chnl C")), 0, Qt::AlignTop | Qt::AlignLeft);
    GLChannelsForGrB2_1->addWidget(CBChannel[2], 1, Qt::AlignTop | Qt::AlignLeft);
    GLChannelsForGrB2_1->addWidget(CBSec[2], 0, Qt::AlignTop | Qt::AlignRight);
    GLChannelsForGrB2_1->addWidget(LmSwitch[2], 0, Qt::AlignTop | Qt::AlignRight);
    GLChannelsForGrB2_2->addWidget(new QLabel(tr("Chnl D")), 0, Qt::AlignTop | Qt::AlignLeft);
    GLChannelsForGrB2_2->addWidget(CBChannel[3], 1, Qt::AlignTop | Qt::AlignLeft);
    GLChannelsForGrB2_2->addWidget(CBSec[3], 0, Qt::AlignTop | Qt::AlignRight);
    GLChannelsForGrB2_2->addWidget(LmSwitch[3], 0, Qt::AlignTop | Qt::AlignRight);

    for (int i = 0; i < numScanChannel; i++)
        LmSwitch[i]->hide();
}

void ScanPlot::GUICreate_Signals_Fz()
{
    QMargins marginZero(0, 0, 0, 0);
    QSettings settingsGui("Param/gui.conf", QSettings::IniFormat);
    fzWidget = new QWidget;
    fzHMainLayout = new QHBoxLayout();
    fzWidget->setLayout(fzHMainLayout);

    ev_Speed_spectr = new Edit(0.1, 1000, 1);
    ev_Speed_spectr->setImmediateMode();
    ev_Speed_spectr->setValue(10);

    CB_Channel = new QComboBox;
    ev_Z1_spectr = new Edit(0, 500, 4);
    ev_Z2_spectr = new Edit(0, 500, 4);

    CB_Points_spectr = new QComboBox;
    CB_Points_spectr->addItem("50");  CB_Points_spectr->addItem("100");
    CB_Points_spectr->addItem("200"); CB_Points_spectr->addItem("300");
    CB_Points_spectr->addItem("400"); CB_Points_spectr->addItem("500");
    CB_Points_spectr->addItem("600"); CB_Points_spectr->addItem("700");
    CB_Points_spectr->addItem("800"); CB_Points_spectr->addItem("900");
    CB_Points_spectr->addItem("1000");
    CB_Points_spectr->setCurrentIndex(10);

    PBSpectr_start_stop = new TwoStateButton(":start.png",":stop.png");
    connect(PBSpectr_start_stop, SIGNAL(state_changed(bool)), this, SLOT(spectrButtonStateChanged(bool)));


    QVBoxLayout *fzV1Layout = new QVBoxLayout();
    QVBoxLayout *fzV2Layout = new QVBoxLayout();
    QVBoxLayout *fzV3Layout = new QVBoxLayout();
    fzHMainLayout->addLayout(fzV1Layout);
    fzHMainLayout->addLayout(fzV2Layout);
    fzHMainLayout->addLayout(fzV3Layout);
    fzHMainLayout->setAlignment(Qt::AlignLeft);
    fzV1Layout->setAlignment(Qt::AlignLeft);
    fzV2Layout->setAlignment(Qt::AlignLeft);
    fzV3Layout->setAlignment(Qt::AlignLeft);
    QHBoxLayout **fzH1Layout = new QHBoxLayout*[2];
    for (int i=0; i<2; i++)
    {
        fzH1Layout[i] = new QHBoxLayout;
        fzH1Layout[i]->setContentsMargins(marginZero);
        //fzHLayout[i]->setAlignment(Qt::AlignLeft | Qt::AlignTop);
        fzV1Layout->addLayout(fzH1Layout[i], 0);
    }
    QWidget *tFzChannelCmb = new QWidget();
    tFzChannelCmb->setContentsMargins(marginZero);
    QHBoxLayout *tFzChannelCmbLay = new QHBoxLayout(tFzChannelCmb);
    tFzChannelCmbLay->setContentsMargins(marginZero);
    tFzChannelCmbLay->addWidget(new QLabel(tr("Channel")), 1, Qt::AlignLeft);
    tFzChannelCmbLay->addWidget(CB_Channel, 1, Qt::AlignLeft);

    QWidget *tFzSpd = new QWidget();
    tFzSpd->setContentsMargins(marginZero);
    QHBoxLayout *tFzSpdLay = new QHBoxLayout(tFzSpd);
    tFzSpdLay->setContentsMargins(marginZero);
    tFzSpdLay->addWidget(new QLabel(tr("Spd, ") + tr("mkm")+"/"+tr("s")), 0, Qt::AlignLeft);
    tFzSpdLay->addWidget(ev_Speed_spectr, 0, Qt::AlignLeft);

    QWidget *tFzPts = new QWidget();
    tFzPts->setContentsMargins(marginZero);
    QHBoxLayout *tFzPtsLay = new QHBoxLayout(tFzPts);
    tFzPtsLay->setContentsMargins(marginZero);
    tFzPtsLay->addWidget(new QLabel(tr("Pts")), 0, Qt::AlignLeft);
    tFzPtsLay->addWidget(CB_Points_spectr, 0, Qt::AlignLeft);

    fzH1Layout[0]->addWidget(tFzChannelCmb, 0, Qt::AlignLeft);
    fzH1Layout[1]->addWidget(tFzSpd);
    fzH1Layout[1]->addWidget(tFzPts);

    QHBoxLayout **fzH2Layout = new QHBoxLayout*[2];
    for (int i=0; i<2; i++)
    {
        fzH2Layout[i] = new QHBoxLayout;
        fzH2Layout[i]->setContentsMargins(marginZero);
        //fzH2Layout[i]->setAlignment(Qt::AlignLeft | Qt::AlignTop);
        fzV2Layout->addLayout(fzH2Layout[i], 0);
    }
    QWidget *tZ1Wid = new QWidget();
    tZ1Wid->setContentsMargins(marginZero);
    QHBoxLayout *tZ1Lay = new QHBoxLayout(tZ1Wid);
    tZ1Lay->setContentsMargins(marginZero);
    tZ1Lay->setAlignment(Qt::AlignLeft | Qt::AlignTop);
    tZ1Lay->addWidget(new QLabel(QChar(0x0394)+tr("Z1")), 0, Qt::AlignLeft);
    tZ1Lay->addWidget(ev_Z1_spectr, 0, Qt::AlignLeft);

    QWidget *tZ2Wid = new QWidget();
    tZ2Wid->setContentsMargins(marginZero);
    QHBoxLayout *tZ2Lay = new QHBoxLayout(tZ2Wid);
    tZ2Lay->setContentsMargins(marginZero);
    tZ2Lay->setAlignment(Qt::AlignLeft | Qt::AlignTop);
    tZ2Lay->addWidget(new QLabel(QChar(0x0394)+tr("Z2")), 0, Qt::AlignLeft);
    tZ2Lay->addWidget(ev_Z2_spectr, 0, Qt::AlignLeft);
    fzH2Layout[0]->addWidget(tZ1Wid, 0, Qt::AlignLeft);
    fzH2Layout[1]->addWidget(tZ2Wid, 0, Qt::AlignLeft);

    fzV3Layout->addWidget(PBSpectr_start_stop, 0, Qt::AlignLeft);

    if (!settingsGui.value("gui/show_fz", true).toBool())
        fzWidget->hide();
    else
        tabSignals->insertTab(posFzTab, fzWidget, tr("F[z]"));
}

void ScanPlot::GUICreate_Signals_4D()
{
    QSettings settingsGui("Param/gui.conf", QSettings::IniFormat);
    BoxExternal = new QWidget;
    ExtraLay = new QVBoxLayout(BoxExternal);
    QHBoxLayout *fourDLayBoxs = new QHBoxLayout();
    ExtraLay->addLayout(fourDLayBoxs);
    fourDLayBoxs->setAlignment(Qt::AlignLeft | Qt::AlignTop);
    lay4D = new QVBoxLayout*[3];
    for (int i=0; i<3; i++)
    {
        lay4D[i] = new QVBoxLayout();
        fourDLayBoxs->addLayout(lay4D[i]);
        lay4D[i]->setAlignment(Qt::AlignLeft | Qt::AlignTop);
    }

    CBExternalMode = new QComboBox;
    scanControlWidgets.append(CBExternalMode);
    CBExternalMode->addItem(tr("None"));
    connect(CBExternalMode, SIGNAL(currentIndexChanged(int)), this, SLOT(sl_ext_mode()));

    auto laySrc = new QHBoxLayout();
    laySrc->addWidget(new QLabel(tr("Src")), 0, Qt::AlignLeft | Qt::AlignTop);
    laySrc->addWidget(CBExternalMode, 0, Qt::AlignLeft | Qt::AlignTop);
    lay4D[0]->addLayout(laySrc, 0);

    CBPointsOXext = new QComboBox;
    CBPointsOYext = new QComboBox;
    scanControlWidgets.append(CBPointsOXext);
    scanControlWidgets.append(CBPointsOYext);
    Set_CBPoints(CBPointsOXext);
    Set_CBPoints(CBPointsOYext);

    connect(CBPointsOXext, SIGNAL(currentIndexChanged(int)), this, SLOT(sl_ext_mode()));
    connect(CBPointsOYext, SIGNAL(currentIndexChanged(int)), this, SLOT(sl_ext_mode()));

    auto layXp = new QHBoxLayout();
    layXp->addWidget(new QLabel(tr("Xp")), 0, Qt::AlignLeft | Qt::AlignTop);
    layXp->addWidget(CBPointsOXext, 0, Qt::AlignLeft | Qt::AlignTop);
    lay4D[1]->addLayout(layXp);

    auto layYp = new QHBoxLayout();
    layYp->addWidget(new QLabel(tr("Yp")), 0, Qt::AlignLeft | Qt::AlignTop);
    layYp->addWidget(CBPointsOYext, 0, Qt::AlignLeft | Qt::AlignTop);
    lay4D[1]->addLayout(layYp);

    if (!settingsGui.value("gui/show_4d", true).toBool())
        BoxExternal->hide();
    else
        tabSignals->insertTab(pos4DTab, BoxExternal, tr("4D"));
}

void ScanPlot::GUICreate_Signals_LiftMode()
{
    QSettings settingsGui("Param/gui.conf", QSettings::IniFormat);
    LblLiftMode = new QLabel(tr("LM height, nm"));
    LblLiftMode->hide();
    LEd_LFmodeHeight = new Edit(-2000, 20000, -1);
    LEd_LFmodeHeight->setValue(1000);
    connect(LEd_LFmodeHeight, SIGNAL(newValue(double)), this, SLOT(scan_lm_height(double)));
    LEd_LFmodeHeight->hide();

    QWidget *liftModTabWid = new QWidget();
    QVBoxLayout *liftModTabVLay = new QVBoxLayout(liftModTabWid);
    QHBoxLayout **liftModLays = new QHBoxLayout*[2];
    for (int i=0; i<2; i++)
    {
        liftModLays[i] = new QHBoxLayout();
        liftModTabVLay->addLayout(liftModLays[i]);
    }

    liftModLays[0]->addWidget(cbLiftMode = new QCheckBox(tr("Enable")), 0, Qt::AlignTop);

    liftModLays[1]->addWidget(LblLiftMode);
    liftModLays[1]->addWidget(LEd_LFmodeHeight);

    connect(cbLiftMode, SIGNAL(toggled(bool)), this, SLOT(LmOnOff(bool)));
    if (!settingsGui.value("gui/show_lift_mod", true).toBool())
        liftModTabWid->hide();
    else
        tabSignals->addTab(liftModTabWid, tr("Lift mod"));
}

void ScanPlot::GUICreate_Signals_ContScan()
{
    QSettings settingsGui("Param/gui.conf", QSettings::IniFormat);
    QWidget *contScanTabWid = new QWidget();
    QHBoxLayout *contScanTabHLay = new QHBoxLayout(contScanTabWid);
    QVBoxLayout **contScanLays = new QVBoxLayout*[4];
    for (int i=0; i<4; i++)
    {
        contScanLays[i] = new QVBoxLayout();
        contScanTabHLay->addLayout(contScanLays[i]);
    }

    cbContScan = new QCheckBox(tr("Enable"));
    connect(cbContScan, SIGNAL(toggled(bool)), this, SLOT(sl_dacscan(bool)));
    contScanLays[0]->addWidget(cbContScan, 0, Qt::AlignTop);

    CBDACscanCh = new QComboBox;
    CBDACscanCh->addItem(tr("None "), -1);
    EDACstep = new Edit(0, 10000, 2);
    EDACcount = new Edit(2, 100);
    EDACcount->setImmediateMode();

    contScanLays[2]->addWidget(new QLabel(tr("Channel")), 0, Qt::AlignRight);
    contScanLays[3]->addWidget(CBDACscanCh, 1);

    QHBoxLayout *countLay = new QHBoxLayout();
    countLay->addWidget(new QLabel(tr("Count")));
    countLay->addWidget(EDACcount);

    contScanLays[0]->addLayout(countLay);
    contScanLays[2]->addWidget(new QLabel(tr("Step")), 0, Qt::AlignRight);
    contScanLays[3]->addWidget(EDACstep, 1);

    if (!settingsGui.value("gui/show_cont_scan", true).toBool())
        contScanTabWid->hide();
    else
        tabSignals->addTab(contScanTabWid, tr("Cont.Sc."));
}

void ScanPlot::GUICreate_Signals_ChannelChooser(QHBoxLayout *channelTabGBlay)
{
    QSettings settingsGui("Param/gui.conf", QSettings::IniFormat);
    plotChooser = new t3dChooser(Plot);
    for (int i=0; i<Plot->ThreeDPlots.size(); i++)
    {
        connect(Plot->ThreeDPlots[i], SIGNAL(doubleClicked()), plotChooser, SLOT(plotDoubleClicked()));
        int sz = plotChooser->pButtons->at(i)->width()*1.05;
        if (sz <= 0) sz = 1;
        if (sz > 200) sz = 200;
        plotChooser->pButtons->at(i)->setFixedSize(sz, sz);
    }
    channelTabGBlay->addWidget(plotChooser, 0, Qt::AlignRight | Qt::AlignTop);
    {
        QSettings settings("Param/gui.conf", QSettings::IniFormat);
        QList<QVariant> order = QList<QVariant>();
        order.append(0); order.append(1); order.append(2); order.append(3);
        order = settings.value("scan/plot_order", order).toList();
        plotChooser->setPositions(order);
    }
    connect(plotChooser, SIGNAL(plotsChanged()), this, SLOT(plotsChanged()));

    if (!settingsGui.value("gui/show_second_chan", true).toBool())
    {
        Plot->numTabsFor3DPlot = 1;
        auto wid = Plot->mainTab->widget(1);
        auto lay = wid->layout();
        while (lay->count() > 0)
            lay->removeWidget(lay->itemAt(0)->widget());
        wid->hide();
        Plot->mainTab->removeTab(1);
    }
}

void ScanPlot::GUICreate_ExtraControl()
{
    fastReturn = new QCheckBox();
    fastReturn->setToolTip(tr("Good with low scan rate or per-point EXT mode(e.g. CCD)")
                           + "\n" + tr("Restricts data to Channels A and B only"));
    connect(fastReturn, SIGNAL(toggled(bool)), this, SLOT(scan_fast_return(bool)));

    pointSync = new QComboBox;
    pointSync->addItem(tr("Internal"));
    pointSync->addItem(tr("Rising edge"));
    pointSync->addItem(tr("Falling edge"));
    connect(pointSync, SIGNAL(currentIndexChanged(int)), this, SLOT(scan_sync_mode(int)));

    FastScanCorrection = new QToolButton;
    scanControlWidgets.append(FastScanCorrection);
    FastScanCorrection->setPopupMode(QToolButton::InstantPopup);
    FastScanCorrection->setIcon(makeHugeIcon(":/SPMTuning_window/FastScanCorrOn.png"));
    FastScanCorrection->setToolTip(tr("Point scan/Raster scan selection"));
    FastScanCorrection->setMenu(new QMenu);
    QActionGroup *fast_correct_ag = new QActionGroup(FastScanCorrection);
    fast_correct_ag->addAction(FastScanCorrection->menu()->addAction(tr("None")))->setCheckable(true);
    fast_correct_ag->addAction(FastScanCorrection->menu()->addAction(tr("Simple")))->setCheckable(true);
    connect(FastScanCorrection, SIGNAL(triggered(QAction*)), this, SLOT(sl_ScanCorrection()));

    BPreciseScan = new TwoStateButton(":/ScanPlot/PresizeScan.png",":/ScanPlot/PresizeScanOn.png");
    BPreciseScan->setToolTip(tr("Enable precise scanning"));
    BPreciseScan->hide();

    BMotorLevel = new QPushButton();
    BMotorLevel->setIcon(makeHugeIcon(":/SPMTuning_window/Motors_leveling.png"));
    BMotorLevel->setToolTip(tr("Level Head by scan data"));
    BMotorLevel->hide();
    connect(BMotorLevel, SIGNAL(clicked()), this, SLOT(sl_motor_level()));

    CBScanner = new QComboBox;
    scanControlWidgets.append(CBScanner);
    CBScanner->setProperty("id", DataID::ScanScanner);
    CBScanner->addItem(tr("Head"));
    CBScanner->addItem(tr("Stage"));

    {
        QSettings settings("Param/controller.conf", QSettings::IniFormat);
        int i = settings.value("fast_scan_correction", 0).toInt();
        fast_correct_ag->actions()[0]->setChecked(false);
        fast_correct_ag->actions()[1]->setChecked(false);
        fast_correct_ag->actions()[i]->setChecked(true);
        fast_scan_correction = i;
        FastScanCorrection->setIcon(makeHugeIcon(i > 0 ? ":/SPMTuning_window/FastScanCorrOff.png" : ":/SPMTuning_window/FastScanCorrOn.png"));

        i = settings.value("scanner_head_stage", 0).toInt();
        CBScanner->setCurrentIndex(i);
    }
    connect(CBScanner, SIGNAL(currentIndexChanged(int)), this, SLOT(setScanner(int)));


    #ifdef _OLD_EXTRA_CONTROL
    fastReturn->setText((tr("Fast backward line")));

    scanTabGB = new QGroupBox(tr("Scaning"));

    QHBoxLayout *scanControl_mainLayHor = new QHBoxLayout(scanTabGB);
    QVBoxLayout **scanControl_laysVert = new QVBoxLayout*[4];
    for (int i=0; i<4; i++)
    {
        scanControl_laysVert[i] = new QVBoxLayout();
        scanControl_mainLayHor->addLayout(scanControl_laysVert[i]);
    }
    scanControl_laysVert[0]->addWidget(FastScanCorrection, 0, Qt::AlignLeft);
    scanControl_laysVert[0]->addWidget(BPreciseScan, 0, Qt::AlignLeft);
    scanControl_laysVert[0]->addWidget(BMotorLevel, 0, Qt::AlignLeft);
    scanControl_laysVert[1]->addWidget(LScanner = new QLabel(tr("Scanner")));
    scanControl_laysVert[2]->addWidget(CBScanner);

    BoxDelays = new QWidget;
    QVBoxLayout *layout_add = new QVBoxLayout(BoxDelays);
    layout_add->setAlignment(Qt::AlignTop);

    firstDelay = new DoubleChanger(0, 2, 1);
    connect(firstDelay, SIGNAL(newValue(double)), this, SLOT(scan_first_delay(double)));
    QGroupBox *box_firstDelay = new QGroupBox(tr("Scan First point delay, s"));
    QHBoxLayout *layout_firstDelay = new QHBoxLayout(box_firstDelay);
    layout_firstDelay->addWidget(firstDelay);
    box_firstDelay->setMaximumWidth(400);

    lineDelay = new DoubleChanger(0, 0.5, 2);
    connect(lineDelay, SIGNAL(newValue(double)), this, SLOT(scan_line_delay(double)));


    QGroupBox *box_lineDelay = new QGroupBox(tr("Line First point delay, s"));
    QHBoxLayout *layout_lineDelay = new QHBoxLayout(box_lineDelay);
    layout_lineDelay->addWidget(lineDelay);
    box_lineDelay->setMaximumWidth(400);

    layout_add->addWidget(box_firstDelay, 1);
    layout_add->addWidget(box_lineDelay, 1);
    layout_add->addWidget(fastReturn);

    pointDelay = new DoubleChanger(0, 1, 4);
    pointDelay->setProperty("id", DataID::ScanPointDelay);
    connect(pointDelay, SIGNAL(newValue(double)), this, SLOT(scan_point_delay(double)));

    QGroupBox *box_pointDelay = new QGroupBox(tr("Point delay, s"));
    QHBoxLayout *layout_pointDelay = new QHBoxLayout(box_pointDelay);
    layout_pointDelay->addWidget(pointDelay);
    box_pointDelay->setMaximumWidth(400);

    pointSync->setMaximumWidth(400);

    BoxSync = new QWidget();
    QVBoxLayout *layout_sync = new QVBoxLayout(BoxSync);
    layout_sync->setAlignment(Qt::AlignTop);
    layout_sync->addWidget(pointSync);
    layout_sync->addWidget(box_pointDelay, 1);
    QLabel *sync_warn = new QLabel(tr("Sync output is enabled for nonzero delay or with external sync input."));
    sync_warn->setHidden(true);
    sync_warn->setWordWrap(true);
    layout_sync->addWidget(sync_warn);

    extraControlTabCreated();
#else
    LExScanFirstPDelay = new QLabel(tr("First point delay"));
    EdExScanFirstPDelay = new Edit(0, 2, 1);
    LExLineFirstPDelay = new QLabel(tr("Line first P delay"));
    EdExLineFirstPDelay = new Edit(0, 0.5, 2);
    LExFastBackward = new QLabel(tr("Fast backward line"));
    LExSync = new QLabel(tr("Sync"));
    LExSyncPointDelay = new QLabel(tr("Sync point delay"));
    EdExSyncPointDelay = new Edit(0, 1, 4);
    LExScanType = new QLabel(tr("Scan type"));
    LExScanner = new QLabel(tr("Scanner"));
    LExLowVoltage = new QLabel(tr("Low voltage"));
    CBExLowVoltage = new QCheckBox();

    extraControlsLabels << LExScanFirstPDelay << LExLineFirstPDelay << LExFastBackward
                        << LExSync << LExSyncPointDelay << LExScanType << LExScanner << LExLowVoltage;
    extraControlsRightPart << EdExScanFirstPDelay << EdExLineFirstPDelay << fastReturn
                           << pointSync << EdExSyncPointDelay << FastScanCorrection << CBScanner << CBExLowVoltage;

    ExtraBackground = new colorRect2(QColor(240, 240, 240, 126));
    ExtraBackground->setStyleSheet("QSplitter {border: 40px solid black; border-style: outset; border-width: 40px; border-radius: 10px; border-color: #B4B4B4; background-color: #E3E3EC;}");

    QString style = "color: black; font: bold 17px;";
    for (auto lab:extraControlsLabels)
    {
        lab->setStyleSheet(style);
    }

    connect(EdExScanFirstPDelay, SIGNAL(newValue(double)), this, SLOT(scan_first_delay(double)));
    connect(EdExLineFirstPDelay, SIGNAL(newValue(double)), this, SLOT(scan_line_delay(double)));
    connect(EdExSyncPointDelay, SIGNAL(newValue(double)), this, SLOT(scan_point_delay(double)));
    connect(CBExLowVoltage, SIGNAL(toggled(bool)), this, SLOT(lowVoltageModeChanged(bool)));
    #endif
}

void ScanPlot::GUICreate_SPMMonitor()
{
    QVBoxLayout *spmButtsLay = new QVBoxLayout();
    spmMonitorGB = new QGroupBox(tr("SPM monitor"));
    spmMonitorGBHlay = new QHBoxLayout(spmMonitorGB);
    LeftLay->addWidget(spmMonitorGB);

    spmMonitorGBHlay->addLayout(spmButtsLay);

    increaseLaserSpotWindow = new QPushButton(makeHugeIcon(":/SPMTuning_window/IncreaseLaserSpotWindow.png"), QString());
    increaseLaserSpotWindow->setToolTip(tr("SPM monitor full screen"));
    spmMonitorFeedback = new TwoStateButton(":/off.png",":/onn2.png");
    spmMonitorFeedback->setToolTip(tr("Feed Back : on/off button"));

    spmButtsLay->addWidget(spmMonitorFeedback, 0, Qt::AlignTop);
    spmMonitorFeedback->change_state_man_with_sig(false);
    spmButtsLay->addWidget(increaseLaserSpotWindow, 1, Qt::AlignBottom);

    int btnSize = qApp->property("buttonsize").toInt() + 5;
    if (btnSize < 0) btnSize = 0;
    if (btnSize > 9999) btnSize = 9999;
    increaseLaserSpotWindow->setMaximumSize(btnSize, btnSize);
    spmMonitorFeedback->setMaximumSize(btnSize, btnSize);

    connect(increaseLaserSpotWindow, SIGNAL(clicked()), this, SLOT(doubleClickedLaserSpotWindow()));

    increaseLaserSpotWindow->hide();
    spmMonitorFeedback->hide();
}

void ScanPlot::GUICreate_DeviceControl()
{
    QSettings settingsGui("Param/gui.conf", QSettings::IniFormat);
    deviceSchemeWid = new deviceScheme();
    scrollAreaSchemeWid = new QScrollArea(this);
    scrollAreaSchemeWid->setWidgetResizable(true);
    scrollAreaSchemeWid->setWidget(deviceSchemeWid);
    if (!settingsGui.value("gui/show_device_control", true).toBool())
        scrollAreaSchemeWid->hide();
    else
        Plot->mainTab->addTab(scrollAreaSchemeWid, tr("Device control"));
}

void ScanPlot::GUICreate()
{
    mainLay = new QHBoxLayout(this);
    LeftLay = new QVBoxLayout();
    mainLay->addLayout(LeftLay);

    GUICreate_Plot();
    GUICreate_TipMonitor();
    GUICreate_Signals();
    GUICreate_ExtraControl();
    GUICreate_SPMMonitor();
    GUICreate_DeviceControl();

    //minimumLeftPanelWidth = LeftWidget->width();

    minimumTipMonitorHeight = tipMonitorGB->minimumSizeHint().height();

    {//todo unused in GUI
        TBFontSize = new QToolButton;
        TBFontSize->setIcon(makeHugeIcon(":/plot/fontsize.png"));
        TBFontSize->setToolTip(QObject::tr("Font size"));
        TBFontSize->setStyleSheet("QPushButton {border-width:0; background-color:rgba(255,255,255,10); color:rgba(255,255,255,10);}\
                                  QToolButton:pressed { background-color:rgba(255,255,255,100); border-width:1;}");
        TBFontSize->setPopupMode(QToolButton::InstantPopup);
        QMenu *MFontSize = new QMenu;
        TBFontSize->setMenu(MFontSize);

        AGFontSizes = new QActionGroup(this);
        AGFontSizes->addAction(MFontSize->addAction(QString::number(6)));
        AGFontSizes->actions()[0]->setCheckable(true);
        AGFontSizes->addAction(MFontSize->addAction(QString::number(8)));
        AGFontSizes->actions()[1]->setCheckable(true);
        AGFontSizes->addAction(MFontSize->addAction(QString::number(10)));
        AGFontSizes->actions()[2]->setCheckable(true);

        for (int i = 0; i < 11; i++) {
            AGFontSizes->addAction(MFontSize->addAction(QString::number(12 + 4*i)));
            AGFontSizes->actions()[i+3]->setCheckable(true);
        }
        AGFontSizes->actions()[3]->setChecked(true);
        connect(AGFontSizes, SIGNAL(triggered(QAction*)), this, SLOT(fontSize_changed()));
        connect(Plot->PlotControl->AGFontSizes, SIGNAL(triggered(QAction*)), this, SLOT(plotFontSize_changed()));

        //-------- ZPos ---------------------------------------------------------------------
        Zposition = new QwtThermo();
        Zposition->setOrientation(Qt::Horizontal, QwtThermo:: NoScale);
        Zposition->setFillBrush(Qt::green);
        Zposition->setRange(0, 1, 0);
        ZposValue = new QLabel("Z pos = 100%");

        QGroupBox *box_zposition = new QGroupBox(tr("Current Tip Position"));
        QHBoxLayout *layout_zposition = new QHBoxLayout(box_zposition);
        layout_zposition->setContentsMargins(6, 6, 6, 6);
        layout_zposition->addWidget(Zposition, 1);
        layout_zposition->addWidget(ZposValue);
        //------------------------------

        //-------- GrBCurrentPoint ---------------------------------------------------------------------
        LEdX = new Edit(0, 100, 3);
        LEdY = new Edit(0, 100, 3);
        connect(LEdX, SIGNAL(newValue()), this, SLOT(sl_Update_XYZ_pos()));
        connect(LEdY, SIGNAL(newValue()), this, SLOT(sl_Update_XYZ_pos()));

        QGridLayout *MainLayoutForCurPos = new QGridLayout();
        MainLayoutForCurPos->addWidget(new QLabel(tr("Xpos, ")+tr("mkm")), 1, 0);
        MainLayoutForCurPos->addWidget(LEdX, 1, 1);
        MainLayoutForCurPos->addWidget(new QLabel(tr("Ypos, ")+tr("mkm")), 2, 0);
        MainLayoutForCurPos->addWidget(LEdY, 2, 1);
    }

    GUIInited = true;

//    QTimer *tmr = new QTimer();
//    tmr->setInterval(1000);
//    connect(tmr, SIGNAL(timeout()), this, SLOT(resizeEvent()));
//    tmr->start();
}

void ScanPlot::GUISetSizeTipMonitor(int width)
{
    if (width <= 0)
        return;

    int doubleMargin = spmMonitorGB->contentsMargins().left()*2 + spmMonitorGB->layout()->contentsMargins().left()*2;

    int rectWidth = width - doubleMargin*3;
    rectWidth /= 5;

    if (rectWidth <= 0)
        return;

    smartLanding->setFixedSize(rectWidth, rectWidth);
    videoChannelMaterialThinLabel->move(0, rectWidth - videoChannelMaterialThinLabel->height() - 2);
    videoChannelMaterialThin->move(videoChannelMaterialThinLabel->width(), rectWidth - videoChannelMaterialThin->height());
    smartLandingBtn2->setFixedWidth(rectWidth - smartLandingBtn1->width());
    rectSlFbp->setFixedSize(rectWidth, rectWidth);
    rectSlVelocity->setFixedSize(rectWidth, rectWidth);
    rectSlFbi->setFixedSize(rectWidth, rectWidth);
    rectAmplitude->setFixedSize(rectWidth, rectWidth);
}

int ScanPlot::GUIMinimumWidthTipMonitor()
{
    int doubleMargin = spmMonitorGB->contentsMargins().left()*2 + spmMonitorGB->layout()->contentsMargins().left()*2;
    int minWidth = doubleMargin*5;

    int maxWidth = 0;
    maxWidth = qMax(maxWidth, smartLanding->minRectWidth());
    maxWidth = qMax(maxWidth, rectSlFbp->minRectWidth());
    maxWidth = qMax(maxWidth, rectSlVelocity->minRectWidth());
    maxWidth = qMax(maxWidth, rectSlFbi->minRectWidth());
    maxWidth = qMax(maxWidth, rectAmplitude->minRectWidth());
    minWidth += maxWidth*5;

    return minWidth;
}

void ScanPlot::resizeEvent(QResizeEvent *e)
{
    if (!GUIInited)
        return;

    int doubleMargin = spmMonitorGB->contentsMargins().left()*2 + spmMonitorGB->layout()->contentsMargins().left()*2;
    doubleMargin *= 2;

    bool isSignalsWidthIsMinimum = (signalsGB->width() - 10 < signalsGB->minimumSizeHint().width());
    bool isSignalsHeightIsMinimum = (signalsGB->height() - 10 < signalsGB->minimumSizeHint().height());

    GUIResizeSPM();

    int minimumWidthSignals = signalsGB->minimumSizeHint().width();
    minimumWidthSignals = qMax(minimumWidthSignals, tabSignals->tabBar()->sizeHint().width());

    if (!isSignalsHeightIsMinimum)
    {
        int maximumWidthSignals = minimumWidthSignals*1.2;
        int howMuchCanIncreaseSignals = maximumWidthSignals - signalsGB->width();
        int howMuchNeedIncreaseSignals = signalsGB->height() - signalsGB->minimumSizeHint().height();
        int howMuchWillIncreaseSignals = qMin(howMuchCanIncreaseSignals, howMuchNeedIncreaseSignals);
        int newWidthTipMonitor = tipMonitorGB->width() + howMuchWillIncreaseSignals;

        GUISetSizeTipMonitor(newWidthTipMonitor);
    }
    if (!isSignalsWidthIsMinimum)
    {
        int minWidthTipMonitor = GUIMinimumWidthTipMonitor();
        bool isTipMonitorMinimum = (tipMonitorGB->width() - 10 < minWidthTipMonitor);
        if (!isTipMonitorMinimum)
        {
            int howMuchNeedShrinkTipMonitor = spmMonitorGB->width() - spmMonitorGB->height();
            int howMuchCanShrinkTipMonitor = tipMonitorGB->width() - minWidthTipMonitor;
            int howMuchCanShrinkSignals = signalsGB->width() - minimumWidthSignals;
            int howMuchWillShrinkTipMonitor = qMin(howMuchNeedShrinkTipMonitor, howMuchCanShrinkSignals);
            howMuchWillShrinkTipMonitor = qMin(howMuchWillShrinkTipMonitor, howMuchCanShrinkTipMonitor);
            int newWidthTipMonitor = tipMonitorGB->width() - howMuchWillShrinkTipMonitor;

            GUISetSizeTipMonitor(newWidthTipMonitor);
        }
    }

    GUIResizeSPM();
}

void ScanPlot::GUIResizeSPM()
{
    int doubleMargin = spmMonitorGB->contentsMargins().left()*2 + spmMonitorGB->layout()->contentsMargins().left()*2;
    doubleMargin *= 2;

    if (spmMonitorGB->width() <= spmMonitorGB->height())
        return;

    int isSignalsHeightIsMinimum = (signalsGB->height() - 10 < signalsGB->minimumSizeHint().height());
    int size = 0;
    if (isSignalsHeightIsMinimum)
    {
        size = spmMonitorGB->height();
    }
    else
    {
        int freeSpace = signalsGB->height() - signalsGB->minimumSizeHint().height();
        size = spmMonitorGB->height() + freeSpace;
//            if (size > spmMonitorGB->width())
//                size = spmMonitorGB->width();
        //qDebug()<<"x7x_"<<">>"<<signalsGB->height()<<size;
    }

    size -= doubleMargin;
    LaserSpotWindow->setFixedSize(size, size);
}

void ScanPlot::resizeEvent()
{
    resizeEvent(nullptr);
}
