#include "scanplot.h"

#include "mainwindow.h"
#include "spmtunning.h"
#include "oscilloscope.h"
#include "modules.hpp"
#include "qwt_plot_layout.h"
#include "motors.hpp"


void ScanPlot::updateVideoProfiles(VideoPlayer *v, int currProfId)
{
    comboVideoProfile->blockSignals(true);
    videoChannelMagnification->blockSignals(true);
    comboVideoProfile->clear();
    videoChannelMagnification->clear();
    QList<QString> *list = v->getVideoProfiles();
    for (int i = 0; i < list->size(); i++)
    {
        comboVideoProfile->addItem(list->value(i));
        videoChannelMagnification->addItem(list->value(i));
    }
    comboVideoProfile->setCurrentIndex(currProfId);
    videoChannelMagnification->setCurrentIndex(currProfId);
    comboVideoProfile->blockSignals(false);
    videoChannelMagnification->blockSignals(false);

    updateVideoChannel();
}

void ScanPlot::onSendToDataVideo_clicked()
{
    Plot->SetActivePlot(videoChannelId);
    auto v = VideoPlayer::GetIt();
    if (v)
        v->on_BToData_clicked();

}

void ScanPlot::onVideoCBChannelChanged(int index)
{
    bool showVideo = (index == 0);

    Plot->overlayButtons->MouseEvent(new QEvent(QEvent::Leave), Plot->ThreeDPlots[videoChannelId], videoChannelId);
    Plot->overlayButtons->replacedGUIButtons[videoChannelId] = showVideo;
    Plot->ThreeDPlots[videoChannelId]->stopMouseEvent = showVideo;
    Plot->overlayButtons->setVisibleIfUnderMouse(videoChannelId);
}

void ScanPlot::videoChannelSetVisibleGUI(int n, bool visible)
{
//    if (visible && n != -1)
//        Plot->SetActivePlot(n);

    if (visible && Plot->overlayButtons->blockShowing3DPlotButtons)
        return;

    if (visible)
    {
        videoChannelResized(videoChannelId, Plot->ThreeDPlots[videoChannelId]->size());
    }

    auto v = VideoPlayer::GetIt();
    videoChannelAutoFocusButtonPrec->setVisible(v && (visible && v->mafPrcsEnabled->isChecked()));
    videoChannelAutoFocusButtonFast->setVisible(v && (visible && v->mafFastEnabled->isChecked()));
    videoChannelAutoFocusButtonPAF->setVisible(v && (visible && v->pafEnabled->isChecked()));
    videoChannelPlayCamera->setVisible(visible);
    videoChannelMagnificationLabel->setVisible(visible);
    videoChannelMagnification->setVisible(visible);

    hbMotorZFocusUp->setVisible(visible);
    hbMotorZFocusDown->setVisible(visible);
    hbMotorZFocusEllipse->setVisible(visible);
    hbMotorXStageLeft->setVisible(visible);
    hbMotorXStageRight->setVisible(visible);
    hbMotorYStageTop->setVisible(visible);
    hbMotorYStageBottom->setVisible(visible);
    hbMotorStageEllipse->setVisible(visible);

    BVideoSaveImage->setVisible(visible);
    BVideoSendToData->setVisible(visible);

//    if (videoChannelAutoFocusUpDownBtn->isEnabled())
//        videoChannelAutoFocusUpDownBtn->setVisible(visible);
}
void ScanPlot::videoChannelResized(int n, QSize newSize)
{
    if (n != videoChannelId)
        return;

    auto v = VideoPlayer::GetIt();
    if (!v) return;
    int leftX = Plot->ThreeDPlots[videoChannelId]->QPlot->plotLayout()->scaleRect(QwtPlot::yLeft).width()*1.15;
    int width = newSize.width() - leftX;
    if (width <= 0) width = 1;
    int height = newSize.height();
    if (height <= 0) height = 1;
    int minBtnsDistBetw = width*0.007;
    if (minBtnsDistBetw == 0)
        minBtnsDistBetw = 1;
    int upY = Plot->ThreeDPlots[videoChannelId]->QPlot->plotLayout()->titleRect().height();
    int upYPerc = upY*0.25;
    upY += upYPerc;
    int videoBtnsSize = (newSize.width() - leftX)*0.2;
//    int UpButtonSize = videoBtnsSize*0.8;
//    if (UpButtonSize < 0) UpButtonSize = 1;
//    if (UpButtonSize > 300) UpButtonSize = 300;
    int UpButtonSize;
    int btnsDistBetw;
    int upNum = 1;
    if (v->mafPrcsEnabled->isChecked())
        upNum++;
    if (v->mafFastEnabled->isChecked())
        upNum++;
    if (v->pafEnabled->isChecked())
        upNum++;
//    if (videoChannelAutoFocusUpDownBtn->isEnabled())
//        upNum++;
    int origUpNum = upNum;
    upNum = upNum + ((upNum >= 6) ? 0 : 1);
    UpButtonSize = width/upNum - (upNum)*minBtnsDistBetw;
    int minUpButtonSize = width/4 - 4*minBtnsDistBetw;
    int maxUpButtonSize = width/5 - 5*minBtnsDistBetw;
    if (UpButtonSize < minUpButtonSize) UpButtonSize = minUpButtonSize;
    if (UpButtonSize > maxUpButtonSize) UpButtonSize = maxUpButtonSize;

    if (upNum <= 2)
        upNum += 2;
    else if (upNum <= 3)
        upNum++;

    btnsDistBetw = (width - UpButtonSize*upNum)/(upNum-1);
    btnsDistBetw *= 0.95;
    if (btnsDistBetw < 10) btnsDistBetw = 10;
    for (int i = 0; i<allVideoBtnsList.size(); i++)
    {
        allVideoBtnsList.at(i)->setFixedSize(UpButtonSize, UpButtonSize);
        allVideoBtnsList.at(i)->setIconSize(QSize(UpButtonSize, UpButtonSize));
    }

    int motorBtnsSize = videoBtnsSize*0.5;
    hbMotorZFocusUp->setFixedSize(motorBtnsSize, motorBtnsSize);
    hbMotorZFocusDown->setFixedSize(motorBtnsSize, motorBtnsSize);
    hbMotorXStageLeft->setFixedSize(motorBtnsSize, motorBtnsSize);
    hbMotorXStageRight->setFixedSize(motorBtnsSize, motorBtnsSize);
    hbMotorYStageTop->setFixedSize(motorBtnsSize, motorBtnsSize);
    hbMotorYStageBottom->setFixedSize(motorBtnsSize, motorBtnsSize);
    hbMotorZFocusEllipse->setFixedSize(motorBtnsSize, motorBtnsSize);
    hbMotorStageEllipse->setFixedSize(motorBtnsSize, motorBtnsSize);

    int cbSize = (newSize.height() - upY)*0.05*1.14;
    if (cbSize < 0) cbSize = 0;
    if (cbSize > 9999) cbSize = 9999;
    videoChannelMagnification->setFixedSize(videoBtnsSize, cbSize);

    auto font = QFont(videoChannelMagnificationLabel->font().family(), cbSize*0.45);
    font.setBold(true);
    videoChannelMagnificationLabel->setScaledContents(true);
    videoChannelMagnificationLabel->setFont(font);
    videoChannelMagnificationLabel->adjustSize();

    videoChannelMagnification->setFont(font);
    auto view = videoChannelMagnification->view();
    view->setFont(font);

    auto fontBig = QFont(videoChannelMagnificationLabel->font().family(), motorBtnsSize*0.45);
    hbMotorZFocusEllipse->font = fontBig;
    hbMotorStageEllipse->font = fontBig;

    int downY = newSize.height() - upYPerc -
            Plot->ThreeDPlots[videoChannelId]->QPlot->plotLayout()->scaleRect(QwtPlot::xBottom).height() -
            videoChannelMagnification->height();

    int btnX = 0;
    int btnY = upY;
    videoChannelPlayCamera->move(btnX += leftX, btnY);
    if (v->mafPrcsEnabled->isChecked())
    {
        btnX += UpButtonSize + btnsDistBetw;
        videoChannelAutoFocusButtonPrec->move(btnX, btnY);
    }
    if (v->mafFastEnabled->isChecked())
    {
        btnX += UpButtonSize + btnsDistBetw;
        videoChannelAutoFocusButtonFast->move(btnX, btnY);
    }
    if (v->pafEnabled->isChecked())
    {
        btnX += UpButtonSize + btnsDistBetw;
        videoChannelAutoFocusButtonPAF->move(btnX, btnY);
    }
//    if (videoChannelAutoFocusUpDownBtn->isEnabled())
//    {
//        btnX += UpButtonSize + btnsDistBetw;
//        videoChannelAutoFocusUpDownBtn->move(btnX, btnY);
//    }

    if (origUpNum <= 4)
        btnX += UpButtonSize + btnsDistBetw;
    if (origUpNum == 5)
        btnY += UpButtonSize + btnsDistBetw;
    BVideoSaveImage->move(btnX, btnY);
    if (origUpNum >= 3)
        btnY += UpButtonSize + btnsDistBetw;
    else
        btnX += UpButtonSize + btnsDistBetw;
    BVideoSendToData->move(btnX, btnY);

    btnX = newSize.width() - motorBtnsSize*1.2;
    btnY = BVideoSendToData->y() + UpButtonSize + 20;
    if (btnY < height*0.4)
        btnY = height*0.4;
    hbMotorZFocusUp->move(btnX, btnY);
    hbMotorZFocusEllipse->move(btnX, hbMotorZFocusUp->y() + hbMotorZFocusUp->height() + 5);
    hbMotorZFocusDown->move(btnX, hbMotorZFocusEllipse->y() + hbMotorZFocusEllipse->height() + 5);

    QPoint motorXYCenter(leftX + motorBtnsSize*0.5 + newSize.width()*0.1,
                         hbMotorZFocusEllipse->y() + hbMotorZFocusEllipse->height()*0.5);
    hbMotorXStageLeft->move(motorXYCenter.x() - motorBtnsSize*0.5 - motorBtnsSize,
                            motorXYCenter.y() - motorBtnsSize*0.5);
    hbMotorXStageRight->move(motorXYCenter.x() + motorBtnsSize*0.5,
                         motorXYCenter.y() - motorBtnsSize*0.5);
    hbMotorYStageTop->move(motorXYCenter.x() - motorBtnsSize*0.5,
                           motorXYCenter.y() - motorBtnsSize*0.5 - motorBtnsSize);
    hbMotorYStageBottom->move(motorXYCenter.x() - motorBtnsSize*0.5,
                              motorXYCenter.y() + motorBtnsSize*0.5);
    hbMotorStageEllipse->move(motorXYCenter.x() - motorBtnsSize*0.5, motorXYCenter.y() - motorBtnsSize*0.5);

    int xPos = leftX;
    videoChannelMagnificationLabel->move(xPos, downY + videoChannelMagnificationLabel->height()*0.3);
    xPos += videoChannelMagnificationLabel->width() + 10;
    videoChannelMagnification->move(xPos, downY);
}

void ScanPlot::autoFocusEnded(bool sucsess)
{
    qDebug()<<"AutoFocus ended"<<sucsess;
    auto v = VideoPlayer::GetIt();
    disconnect(v, SIGNAL(AutoFocusEnded(bool)), this, SLOT(autoFocusEnded(bool)));
    BAutoFocus->change_state_man(false);
    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
    {
        mob->setPropIdVar(DataID::videoMafPrecise, false);
        mob->setPropIdVar(DataID::videoMafFast, false);
    }
    videoChannelAutoFocusButtonPrec->change_state_man(false);
    videoChannelAutoFocusButtonFast->change_state_man(false);
    videoChannelAutoFocusButtonFast->setEnabled(true);
    videoChannelAutoFocusButtonPrec->setEnabled(true);
    videoChannelAutoFocusButtonPAF->setEnabled(true);
    BAutoFocus->setEnabled(true);
    BAutoFocus2->setEnabled(true);
    if (zLandingWaitsMAF)
    {
        zLandingOff_stage = 1;
        auto m = Motors::GetIt();
        auto mZLanding = m->getRoleMotor(Motors::MotorRoles::RoleZ1Land);
        connect(m, SIGNAL(motorStatusChanged(int, bool, bool)), this, SLOT(zLandButtonSysOn_motorEvent(int, bool, bool)));
        zLandButtonSysOn_motorEvent(mZLanding, true, false);
    }
    if (sucsess)
    {
        {
            if (v->mafPaf->isChecked())
                BAutoFocus2->change_state_man_with_sig(true);
        }
    }
    if (videoChannelAutoFocusPrevMode != -1)
        v->autoFocusMode->setCurrentIndex(videoChannelAutoFocusPrevMode);
    videoChannelAutoFocusPrevMode = -1;
    auto spm = SPMTunning::GetIt();
    if (spm->OptionsForMotor->sysOnMAF)
        spm->BLaserOnOff->change_state_man_with_sig(false);
}
void ScanPlot::autoFocus2Ended(bool sucsess)
{
    qDebug()<<"AutoFocus2 ended"<<sucsess;
    BAutoFocus2->change_state_man(false);
    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
        mob->setPropIdVar(DataID::videoPaf, false);
    videoChannelAutoFocusButtonPAF->change_state_man(false);
    videoChannelAutoFocusButtonFast->setEnabled(true);
    videoChannelAutoFocusButtonPAF->setEnabled(true);
    videoChannelAutoFocusButtonPrec->setEnabled(true);
    BAutoFocus->setEnabled(true);
    BAutoFocus2->setEnabled(true);
    BMakeScreen->change_state_man_with_sig(videoChannelAutoFocusPrevCameraState);
    auto v = VideoPlayer::GetIt();
    if (v->mafPaf->isChecked())
    {
//        videoChannelAutoFocusUpDownBtn->setEnabled(true);
//        videoChannelAutoFocusUpDownBtn->setVisible(videoChannelAutoFocusButtonPrec->isVisible());
//        videoChannelAutoFocusUpDownBtn->change_state_man(1);
    }
}
void ScanPlot::autoFocus_ChangedState(bool on)
{
    auto video = VideoPlayer::GetIt();

    if (video->autoFocusScanVariant == 0)
        videoChannelAutoFocusButtonPrec->setEnabled(!on);
    else
        videoChannelAutoFocusButtonFast->setEnabled(!on);

    if (on)
    {
        videoChannelAutoFocusPrevCameraState = BMakeScreen->isOn;
        video->BAutoFocus_clicked(on);
        connect(video, SIGNAL(AutoFocusEnded(bool)), this, SLOT(autoFocusEnded(bool)));
    }
    else {
        video->BAutoFocus_clicked(on);
        BMakeScreen->change_state_man_with_sig(videoChannelAutoFocusPrevCameraState);
    }
}
void ScanPlot::autoFocusUpDownBtn_man(int oldSt, int newSt)
{
    if ((oldSt == 2 && newSt == 3) || (oldSt == 4 && newSt == 1))
    {
        videoChannelAutoFocusButtonPrec->setEnabled(true);
        videoChannelAutoFocusButtonFast->setEnabled(true);
        videoChannelAutoFocusButtonPAF->setEnabled(true);
        BAutoFocus2->setEnabled(true);
        BAutoFocus->setEnabled(true);

        auto mob = MobileServer::GetIt();
        if (mob && mob->need_send())
        {
            mob->setPropIdVar(DataID::videoFocusUpDown, newSt);
        }
    }
}
void ScanPlot::autoFocusUpDownBtnWaiting()
{
    SleepExport::msleep_gui(1000);
    auto motors = Motors::GetIt();
    auto v = VideoPlayer::GetIt();
    auto m = Motors::GetIt();
    auto mZfocus = motors->getRoleMotor(Motors::MotorRoles::RoleZ2Focus);
    connect(m, SIGNAL(motorStatusChanged(int, bool, bool)), v, SLOT(autoFocusStages(int, bool, bool)));
    v->autoFocusStages(mZfocus, false, false);
}
void ScanPlot::autoFocusUpDownBtn(int oldSt, int newSt)
{
    auto v = VideoPlayer::GetIt();
    auto m = Motors::GetIt();
    auto mZfocus = m->getRoleMotor(Motors::MotorRoles::RoleZ2Focus);
    auto spm = SPMTunning::GetIt();

    if (oldSt == 1 && newSt == 2)
    {
        spm->BLaserOnOff->change_state_man_with_sig(true);
        videoChannelAutoFocusButtonPrec->setEnabled(false);
        videoChannelAutoFocusButtonFast->setEnabled(false);
        videoChannelAutoFocusButtonPAF->setEnabled(false);
        BAutoFocus2->setEnabled(false);
        BAutoFocus->setEnabled(false);

        videoChannelAutoFocusUpDownPrevMode = v->autoFocusMode->currentIndex();
        v->autoFocusMode->setCurrentIndex(0);
        disconnect(m, SIGNAL(motorStatusChanged(int, bool, bool)), v, SLOT(autoFocusStages(int, bool, bool)));
        v->autoFocusStage = 1;
        v->autoFocusMagicMultiplyCurrentId = 1;
        v->autoFocusDirection = 1;
        v->autoFocusStopped = false;
        v->isInterrupted = false;
        //m->goStop(mZfocus);
        connect(v, SIGNAL(AutoFocusEnded(bool)), this, SLOT(autoFocusUpDownEnded(bool)));
        //connect(m, SIGNAL(motorStatusChanged(int, bool, bool)), v, SLOT(autoFocusStages(int, bool, bool)));
        m->goDown(mZfocus, 9999, v->autoFocusExitDistance);
        autoFocusUpDownBtnWaiting();
    }
    else if ((oldSt == 2 && newSt == 3) || (oldSt == 4 && newSt == 1))
    { // stopping
        spm->BLaserOnOff->change_state_man_with_sig(false);
        //videoChannelAutoFocusUpDownBtn->setEnabled(false);
        //videoChannelAutoFocusUpDownBtn->setVisible(false);
        //videoChannelAutoFocusUpDownBtn->change_state_man(1);
        v->isInterrupted = true;
        v->autoFocusStage = -10;
        v->BAutoFocus_clicked(false);
    }
    else if (oldSt == 3 && newSt == 4)
    {
        spm->BLaserOnOff->change_state_man_with_sig(true);
        videoChannelAutoFocusButtonPrec->setEnabled(false);
        videoChannelAutoFocusButtonFast->setEnabled(false);
        videoChannelAutoFocusButtonPAF->setEnabled(false);
        BAutoFocus2->setEnabled(false);
        BAutoFocus->setEnabled(false);

        disconnect(m, SIGNAL(motorStatusChanged(int, bool, bool)), v, SLOT(autoFocusStages(int, bool, bool)));
        v->autoFocusMode->setCurrentIndex(0);
        v->autoFocusStage = 1;
        v->autoFocusMagicMultiplyCurrentId = 0;
        v->autoFocusDirection = 0;
        v->autoFocusStopped = false;
        v->isInterrupted = false;
        //m->goStop(mZfocus);
        connect(v, SIGNAL(AutoFocusEnded(bool)), this, SLOT(autoFocusUpDownEnded(bool)));
        //connect(m, SIGNAL(motorStatusChanged(int, bool, bool)), v, SLOT(autoFocusStages(int, bool, bool)));
        m->goUp(mZfocus, 9999, v->autoFocusExitDistance);
        autoFocusUpDownBtnWaiting();
    }

    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
    {
        mob->setPropIdVar(DataID::videoFocusUpDown, newSt);
    }
}
void ScanPlot::autoFocusUpDownEnded(bool sucsess)
{
    qDebug()<<"AutoFocusUpDown ended"<<sucsess;
    auto v = VideoPlayer::GetIt();
    disconnect(v, SIGNAL(AutoFocusEnded(bool)), this, SLOT(autoFocusUpDownEnded(bool)));
    BAutoFocus->change_state_man(false);
    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
    {
        mob->setPropIdVar(DataID::videoMafPrecise, false);
        mob->setPropIdVar(DataID::videoMafFast, false);
    }
    videoChannelAutoFocusButtonPrec->change_state_man(false);
    videoChannelAutoFocusButtonFast->change_state_man(false);
    videoChannelAutoFocusButtonFast->setEnabled(true);
    videoChannelAutoFocusButtonPAF->setEnabled(true);
    BAutoFocus->setEnabled(true);
    BAutoFocus2->setEnabled(true);
    if (videoChannelAutoFocusUpDownPrevMode != -1)
        v->autoFocusMode->setCurrentIndex(videoChannelAutoFocusUpDownPrevMode);
    videoChannelAutoFocusUpDownPrevMode = -1;
    auto spm = SPMTunning::GetIt();
    spm->BLaserOnOff->change_state_man_with_sig(false);
    //if (sucsess)
        //videoChannelAutoFocusUpDownBtn->change_state_man(videoChannelAutoFocusUpDownBtn->state + 1);
    //else
        //autoFocusUpDownBtn(2, 3);
}
void ScanPlot::autoFocusPrec_ChangedState(bool state)
{
    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
        mob->setPropIdVar(DataID::videoMafPrecise, state);
    videoChannelAutoFocusButtonFast->setEnabled(!state);
    videoChannelAutoFocusButtonPAF->setEnabled(!state);
    BAutoFocus2->setEnabled(!state);
    if (state)
    {
        auto v = VideoPlayer::GetIt();
        v->autoFocusDirection = 0;
//        if (videoChannelAutoFocusUpDownBtn->isVisible())
//        {
//            videoChannelAutoFocusUpDownBtn->setEnabled(false);
//            videoChannelAutoFocusUpDownBtn->setVisible(false);
//        }
        if (!videoChannelPlayCamera->isOn)
            videoChannelPlayCamera->change_state_man_with_sig(true);
        videoChannelAutoFocusPrevMode = v->autoFocusScanVariant;
        v->autoFocusMode->setCurrentIndex(1);
    }
    BAutoFocus->change_state_man_with_sig(state);
}
void ScanPlot::autoFocusFast_ChangedState(bool state)
{
    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
        mob->setPropIdVar(DataID::videoMafFast, state);
    videoChannelAutoFocusButtonPrec->setEnabled(!state);
    videoChannelAutoFocusButtonPAF->setEnabled(!state);
    BAutoFocus2->setEnabled(!state);
    if (state)
    {
        auto v = VideoPlayer::GetIt();
        v->autoFocusDirection = 0;
//        if (videoChannelAutoFocusUpDownBtn->isVisible())
//        {
//            videoChannelAutoFocusUpDownBtn->setEnabled(false);
//            videoChannelAutoFocusUpDownBtn->setVisible(false);
//        }
        if (!videoChannelPlayCamera->isOn)
            videoChannelPlayCamera->change_state_man_with_sig(true);
        videoChannelAutoFocusPrevMode = v->autoFocusScanVariant;
        v->autoFocusMode->setCurrentIndex(0);
    }
    BAutoFocus->change_state_man_with_sig(state);
}
void ScanPlot::autoFocus2_ChangedState(bool on)
{
    qDebug()<<"autoFocus2_ChangedState"<<on;
    videoChannelAutoFocusButtonPAF->change_state_man(on);
    BAutoFocus2->change_state_man(on);
    videoChannelAutoFocusButtonFast->setEnabled(!on);
    videoChannelAutoFocusButtonPrec->setEnabled(!on);
    BAutoFocus->setEnabled(!on);
    auto v = VideoPlayer::GetIt();
    if (on)
    {
//        if (videoChannelAutoFocusUpDownBtn->isVisible())
//        {
//            videoChannelAutoFocusUpDownBtn->setEnabled(false);
//            videoChannelAutoFocusUpDownBtn->setVisible(false);
//        }
        videoChannelAutoFocusPrevCameraState = BMakeScreen->isOn;
        v->BAutoFocus->click();
    }
    else {
        v->cancelOperation();
    }

    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
        mob->setPropIdVar(DataID::videoPaf, on);
}

void ScanPlot::motorXStageLeft(bool pressed)
{
    auto m = Motors::GetIt();
    auto v = VideoPlayer::GetIt();
    auto mXstage = m->getRoleMotor(Motors::MotorRoles::RoleStageX);
    if (pressed)
        m->goUpE(mXstage, v->motorXYStageSpeed);
    else
        m->goStop(mXstage);
}
void ScanPlot::motorXStageRight(bool pressed)
{
    auto m = Motors::GetIt();
    auto v = VideoPlayer::GetIt();
    auto mXstage = m->getRoleMotor(Motors::MotorRoles::RoleStageX);
    if (pressed)
        m->goDownE(mXstage, v->motorXYStageSpeed);
    else
        m->goStop(mXstage);
}
void ScanPlot::motorYStageTop(bool pressed)
{
    auto m = Motors::GetIt();
    auto v = VideoPlayer::GetIt();
    auto mYstage = m->getRoleMotor(Motors::MotorRoles::RoleStageY);
    if (pressed)
        m->goDownE(mYstage, v->motorXYStageSpeed);
    else
        m->goStop(mYstage);
}
void ScanPlot::motorYStageBottom(bool pressed)
{
    auto m = Motors::GetIt();
    auto v = VideoPlayer::GetIt();
    auto mYstage = m->getRoleMotor(Motors::MotorRoles::RoleStageY);
    if (pressed)
        m->goUpE(mYstage, v->motorXYStageSpeed);
    else
        m->goStop(mYstage);
}
void ScanPlot::motorZFocusUp(bool pressed)
{
    auto m = Motors::GetIt();
    auto v = VideoPlayer::GetIt();
    auto mZfocus = m->getRoleMotor(Motors::MotorRoles::RoleZ2Focus);
    if (pressed)
        m->goDownE(mZfocus, v->motorZFocusSpeed);
    else
        m->goStop(mZfocus);
}
void ScanPlot::motorZFocusDown(bool pressed)
{
    auto m = Motors::GetIt();
    auto v = VideoPlayer::GetIt();
    auto mZfocus = m->getRoleMotor(Motors::MotorRoles::RoleZ2Focus);
    if (pressed)
        m->goUpE(mZfocus, v->motorZFocusSpeed);
    else
        m->goStop(mZfocus);
}

void ScanPlot::receiveCameraImage(QImage img)
{
    Plot->ThreeDPlots.at(videoChannelId)->SetCurrentXScales(0, videoChannelAxisScaleXMax[1]);
    Plot->ThreeDPlots.at(videoChannelId)->SetCurrentYScales(0, videoChannelAxisScaleYMax[1]);
    ritem->image = img;
    Plot->ThreeDPlots.at(videoChannelId)->Replot();
}

void ScanPlot::startMAFPrcs()
{
    videoChannelAutoFocusButtonPrec->change_state_man_with_sig(true);
    delete sender();
}

void ScanPlot::startPAFFast()
{
    videoChannelAutoFocusButtonFast->change_state_man_with_sig(true);
    delete sender();
}

void ScanPlot::startPAF()
{
    videoChannelAutoFocusButtonPAF->change_state_man_with_sig(true);
    delete sender();
}

void ScanPlot::updateVideoChannel()
{
    auto video = VideoPlayer::GetIt();
    if (!video)
        return;

    auto videoPlot = Plot->ThreeDPlots.at(videoChannelId);
    if (showingVideoFrame)
    {
        videoChannelAxisScaleXMax[1] = video->sl_get_rangeX() * 1e-6;
        videoChannelAxisScaleYMax[1] = video->sl_get_rangeY() * 1e-6;
        videoPlot->SetCurrentXScales(0, videoChannelAxisScaleXMax[1]);
        videoPlot->SetCurrentYScales(0, videoChannelAxisScaleYMax[1]);
        plotLegends[videoChannelId] = comboVideoProfile->currentText();
        Plot->SetTitle(plotLegends[videoChannelId], videoChannelId);
    }
    else
    {
        videoPlot->BFixXIsChecked = videoPlot->BFixYIsChecked = false;
        videoPlot->SetCurrentXScales(0, videoChannelAxisScaleXMax[0]);
        videoPlot->SetCurrentYScales(0, videoChannelAxisScaleYMax[0]);
        plotLegends[videoChannelId] = CBChannel[videoChannelId]->currentText();
        Plot->SetTitle(plotLegends[videoChannelId] + getLegendCurrPoint(), videoChannelId);
    }

    videoPlot->Replot();

    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
    {
        QList<QVariant> list;
        list << video->sl_get_rangeX()
             << video->sl_get_rangeY();

        QStringList sList;
        for (int i=0; i<videoChannelMagnification->count(); i++)
            sList << videoChannelMagnification->itemText(i);
        list << sList
             << videoChannelMagnification->currentIndex();

        list << video->mafPrcsEnabled->isChecked()
             << video->mafFastEnabled->isChecked()
             << video->pafEnabled->isChecked();

        mob->setPropIdVar(DataID::videoChangedMagnification, list);
    }
}
void ScanPlot::make_screen_videocam(bool start)
{
    auto video = VideoPlayer::GetIt();
    if (!video)
        return;

    showingVideoFrame = start;

    if (start)
    {
        connect(video, SIGNAL(ImgReceivedAfter(QImage)), this, SLOT(receiveCameraImage(QImage)));

        ritem->attach(Plot->ThreeDPlots.at(videoChannelId)->QPlot);
        if(!video->StartStop(true))
        {
            BMakeScreen->change_state_man(false);
            make_screen_videocam(false);
        }
    }
    else
    {
        video->StartStop(false);
        disconnect(video, SIGNAL(ImgReceivedAfter(QImage)), this, SLOT(receiveCameraImage(QImage)));
        //ritem->detach();
    }

    videoChannelPlayCamera->change_state_man(start);
    BMakeScreen->change_state_man(start);
    auto mob = MobileServer::GetIt();
    if (mob && mob->need_send())
        mob->setPropIdVar(DataID::VideoOn, start);

    updateVideoChannel();
}
